/* 
  MPHELL-4.0 
  Author(s): The MPHELL team

 (C) Copyright 2015-2018 - Institut Fourier / Univ. Grenoble Alpes (France) 

 This file is part of the MPHELL Library.
 MPHELL is free software: you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation, version 3 of the License.
 
 MPHELL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with MPHELL.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
  * \file  mphell-define.h
  * \brief Wrap the config.h generated by configure.ac.
  */

#ifndef MPHELL_DEFINE_H
#define MPHELL_DEFINE_H

#include <stdint.h>

#include "mphell-config.h"

#if MPHELL_USE_DEBUG == 1
#define MPHELL_USE_ASSERT 1
#else
#define MPHELL_USE_ASSERT 0
#endif

#define PROC X86_64

#if MPHELL_HAVE_GMP == 1
#if MPHELL_USE_GMP == 1
#include <gmp.h>
#define INIT_SIZE_MAIN_STACK 1000
#define INIT_SIZE_OTHERS_STACKS 1000
#define BLOCK_SIZE GMP_LIMB_BITS
#endif
#endif

#if MPHELL_HAVE_IPP == 1
#if MPHELL_USE_IPP == 1
#include <ippcp.h>
#define INIT_SIZE_MAIN_STACK 1000
#define INIT_SIZE_OTHERS_STACKS 1000
#define BLOCK_SIZE 32
#endif
#endif

#if MPHELL_HAVE_MBEDTLS == 1
#if MPHELL_USE_MBEDTLS == 1
#include "mbedtls/bignum.h"
#include "mbedtls/bn_mul.h"
#define INIT_SIZE_MAIN_STACK 1000
#define INIT_SIZE_OTHERS_STACKS 1000
#if defined(MBEDTLS_HAVE_INT64)
#define BLOCK_SIZE 64
#else
#define BLOCK_SIZE 32
#endif
#endif
#endif

#define STACK_1 1
#define STACK_2 2

#define POOL_SIZE_FP 40
#define POOL_SIZE_FIELD 100

#if BLOCK_SIZE == 64
#define UMAX (~(uint64_t)0)
typedef uint64_t block;
#else
#define UMAX (~(uint32_t)0)
typedef uint32_t block;
#endif

#define BYTES_PER_LIMB (BLOCK_SIZE/8)

#endif
