/* 
  MPHELL-4.0 
  Author(s): The MPHELL team

 (C) Copyright 2015-2018 - Institut Fourier / Univ. Grenoble Alpes (France) 

 This file is part of the MPHELL Library.
 MPHELL is free software: you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation, version 3 of the License.
 
 MPHELL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with MPHELL.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
  * \file  mphell-entropy.h
  * \brief Declaration of entropy functions.
  */

#ifndef ENTROPY_H
#define ENTROPY_H

#include <string.h>
#include <math.h>

#include "mphell-errors.h"

/**
 * @enum  entropy_source
 * @brief Define the entropy source
 */
enum entropy_source {
    INTEL,          /*!< Use the RDSEED instruction, available since Broadwell ("5th-generation Core" of the Intel Core processors) */
    DEVURANDOM,     /*!< Non blocant entropy source, can be of poor quality                                                         */
    DEVRANDOM,      /*!< Blocant entropy source, consider installing HAVEGED to increase the entropy throughput                     */
    ENTROPY_UNSET   /*!< To use when you provide a RNG which use its own entropy source (ex RANDOM_INTEL or RANDOM_STM32)           */
};

/**
 * @typedef  entropy_type
 * @brief    Define the entropy source
 */
typedef enum entropy_source entropy_type;

/**
 * @fn    get_entropy_input(mphell_status ret, uint8_t *data, const entropy_type entropy_src, const uint16_t length);
 * @brief Get length bit of entropy from the entropy source selected
 * @param ret         Set to MPHELL_CATASTROPHIC_ERROR_RANDOM if the entropy source is not available, MPHELL_ERROR_RANDOM if no random seed was available, MPHELL_SUCCESS if everything worked !
 * @param data        To store the entropy
 * @param entropy_src To select the entropy source
 * @param length      Length (in bytes) of data
 */
void
get_entropy_input(mphell_status ret, uint8_t *data,
        const entropy_type entropy_src, const uint16_t length);

#endif
