/* 
  MPHELL-4.0 
  Author(s): The MPHELL team

 (C) Copyright 2015-2018 - Institut Fourier / Univ. Grenoble Alpes (France) 

 This file is part of the MPHELL Library.
 MPHELL is free software: you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation, version 3 of the License.
 
 MPHELL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with MPHELL.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
  * \file  mphell-init.h
  * \brief Declaration of MPHELL initialisation function.
  */

#ifndef MPHELL_INIT_ARITH_H
#define MPHELL_INIT_ARITH_H

#include <inttypes.h>

#include "mphell-tmp.h"
#include "mphell-random.h"

extern random_state_t *random_working_state;
extern uint16_t random_security_strength;

/**
 * @fn    void init_mphell(const uint16_t security_strength, const random_type type, const entropy_type entropy);
 * @brief Initialise MPHELL with security_strength bits of security (for random number only).
 * @param security_strength Minimum security strength required in bits for random number
 * @param type              Random process to use
 * @param entropy           Entropy source to use
 */
void 
init_mphell(const uint16_t security_strength, const random_type type,
        const entropy_type entropy);

#if BLOCK_SIZE == 64
/**
 * @fn     uint64_t rand64();
 * @brief  Return a 64 bits random number using MPHELL random process
 * @return A 64 bits random number
 */
uint64_t 
rand64();
#endif

/**
 * @fn     uint64_t rand32();
 * @brief  Return a 32 bits random number using MPHELL random process
 * @return A 32 bits random number
 */
uint32_t 
rand32();

/**
 * @fn     int mphell_rng(void * param, unsigned char * res, size_t n);
 * @brief  Set res to n random bytes
 * @param  param For compatibility, set NULL
 * @param  res   Result byte string
 * @param  n     Size (in byte) of res
 * @return 0 on success, abort the program otherwise.
 */
int 
mphell_rng(void * param, unsigned char * res, size_t n);

/**
 * @fn    free_mphell();
 * @brief Free MPHELL memory, especially the big amount of temporary memory.
 */
void 
free_mphell();

#endif
