/* 
  MPHELL-4.0 
  Author(s): The MPHELL team

 (C) Copyright 2015-2018 - Institut Fourier / Univ. Grenoble Alpes (France) 

 This file is part of the MPHELL Library.
 MPHELL is free software: you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation, version 3 of the License.
 
 MPHELL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with MPHELL.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
  * \file  mphell-sha1.h
  * \brief Declaration of sha1 functions.
  */

#ifndef SHA1_H 
#define SHA1_H

#include <string.h>
#include <stdint.h>

/**
 * @union _buffer
 * @brief Contain one block of 512 bits
 */
union _buffer
{
    uint8_t b[64];  /*!< One block of 64 bytes = 512 bits of the message */
    uint32_t w[16]; /*!< 16 x 32-bits words = 512 bits of the message    */

};

/**
 * @struct sha1_state_t
 * @brief  State of sha1
 */
struct sha1_state_t
{
    union _buffer buffer;  /*!< buffer to contain 512 bits block.           */
    uint8_t buffer_offset; /*!< buffer length.                              */
    uint32_t state[5];     /*!< stores the value H0, H1, H2, H3, H4.        */
    uint64_t byte_count;   /*!< counts the number of byte in the message.   */
};

/**
 * @typedef sha1_state
 * @brief   State of sha1
 */
typedef struct sha1_state_t sha1_state;

/**
 * @fn    void sha1(uint8_t *hashvalue, const uint8_t* data, const uint64_t data_len);
 * @brief Compute the sha1 hash of data
 * @param hashvalue Resulting hash (160 bits)
 * @param data      Message to hash
 * @param data_len  Length of data (in bits)
 */
void
sha1(uint8_t *hashvalue, const uint8_t* data, const uint64_t data_len);

#endif
