dnl This file is part of MPHELL.
dnl MPHELL_CHECK_GMP
dnl Test for the GNU Multiprecision library and define GMP_CFLAGS and GMP_LIBS

AC_DEFUN([MPHELL_CHECK_GMP],
[
AC_ARG_WITH([gmp],
            [AC_HELP_STRING([--with-gmp=<path>|yes], 
                            [Use GMP library.
                             If argument is yes or <empty> that means
                             the library is reachable with the standard search path
                             "/usr" or "/usr/local" (set as default). Otherwise you
                             give the <path> to the directory which contain the
                             library.])],
            [if test "$withval" = no ; then
                GMP_HOME_PATH="${DEFAULT_CHECKING_PATH}"
                use_gmp="no"
             elif test "$withval" = yes ; then
                GMP_HOME_PATH="${DEFAULT_CHECKING_PATH}"
                use_gmp="yes"
             elif test "$withval" != no ; then
                GMP_HOME_PATH="$withval ${DEFAULT_CHECKING_PATH}"
                use_gmp="yes"
             fi
             ],
             [
                GMP_HOME_PATH="${DEFAULT_CHECKING_PATH}"
                use_gmp="yes"
             ])

if test "x$use_gmp" != "xno"; then

min_gmp_version=ifelse([$1], ,6.0.0, $1)

dnl Check for existence
BACKUP_CFLAGS=${CFLAGS}
BACKUP_LIBS=${LIBS}

gmp_found="no"
AC_MSG_CHECKING(for GMP >= $min_gmp_version)

for GMP_HOME in ${GMP_HOME_PATH}
  do
    if test -r "${GMP_HOME}/include/gmp.h"; then
        GMP_CFLAGS="-I${GMP_HOME}/include"
        GMP_LIBS="-L${GMP_HOME}/lib -Wl,-rpath=${GMP_HOME}/lib -lgmp "

        CFLAGS="${GMP_CFLAGS} ${CFLAGS}"
        LIBS="${GMP_LIBS} ${LIBS}"

        AC_TRY_LINK([#ifdef __PATHCC__
                     #include <rw/_defs.h>
                     #include <ansi/_cstddef.h>
                     #endif
                     #include "stddef.h"
                     #include <gmp.h> 
                     ],
                    [mpz_t a; mpz_init (a);],
                    [AC_TRY_RUN([#ifdef __PATHCC__
                                 #include <rw/_defs.h>
                                 #include <ansi/_cstddef.h>
                                 #include "stddef.h"
                                 #endif
                                 #include <gmp.h>
                                 int main () {  if (__GNU_MP_VERSION < 6) return -1; else return 0; }],
                                 [AC_MSG_RESULT(found)
                                  AC_SUBST(GMP_CFLAGS)
                                  AC_SUBST(GMP_LIBS)
                                  AC_DEFINE(HAVE_GMP,1,[Define if GMP is installed])
                                  gmp_found="yes"
                                  break;],
                                 [gmp_problem="$gmp_problem $GMP_HOME"
                                  unset GMP_CFLAGS
                                  unset GMP_LIBS],
                                 [AC_MSG_RESULT(unknown)
                                  echo "WARNING: You appear to be cross compiling, so there is no way to determine"
                                  echo "whether your GMP version is new enough. I am assuming it is."
                                  AC_SUBST(GMP_CFLAGS)
                                  AC_SUBST(GMP_LIBS)
                                  AC_DEFINE(HAVE_GMP,1,[Define if GMP is installed])
                                  gmp_found="yes"
                                  break;])],
                    [gmp_found="no"
                     unset GMP_CFLAGS
                     unset GMP_LIBS])

    else
        gmp_found="no"
    fi
done

if test "x$gmp_found" != "xyes"; then
    if test -n "$gmp_problem"; then
        echo "Sorry, your GMP version is too old. Disabling."
        AC_MSG_RESULT(problem)
        AC_MSG_ERROR(Gmp problem)
    elif test "x$gmp_found" == "xno"; then
        AC_MSG_RESULT(not found)
        AC_MSG_ERROR(Gmp not found)
    fi
fi

CFLAGS=${BACKUP_CFLAGS}
LIBS=${BACKUP_LIBS}

fi

])
