dnl This file is part of MPHELL.
dnl MPHELL_CHECK_IPP
dnl Test for the Intel IPPCP library and define IPP_CFLAGS and IPP_LIBS

AC_DEFUN([MPHELL_CHECK_IPP],
[
AC_ARG_WITH([ipp],
            [AC_HELP_STRING([--with-ipp=<path>|no], 
                            [Use IPP library.
                             If argument is yes or <empty> that means
                             the library is reachable with the standard search path
                             "/usr" or "/usr/local" (set as default). Otherwise you
                             give the <path> to the directory which contain the
                             library.])],
            [if test "$withval" = no ; then
                IPP_HOME_PATH="${DEFAULT_CHECKING_PATH}"
                use_ipp="no"
             elif test "$withval" = yes ; then
                IPP_HOME_PATH="${DEFAULT_CHECKING_PATH}"
                use_ipp="yes"
             elif test "$withval" != no ; then
                IPP_HOME_PATH="$withval ${DEFAULT_CHECKING_PATH}"
                use_ipp="yes"
             fi
             ],
             [
                IPP_HOME_PATH="${DEFAULT_CHECKING_PATH}"
                use_ipp="yes"
             ])

if test "x$use_ipp" != "xno"; then

min_ipp_version=ifelse([$1], ,2020.0.0, $1)

dnl Check for existence
BACKUP_CFLAGS=${CFLAGS}
BACKUP_LIBS=${LIBS}

ipp_found="no"
AC_MSG_CHECKING(for IPP >= $min_ipp_version)

for IPP_HOME in ${IPP_HOME_PATH}
  do
    if test -r "${IPP_HOME}/include/ippcp.h"; then
        IPP_CFLAGS="-Wno-unknown-pragmas -I${IPP_HOME}/build/.build/RELEASE/include"
        IPP_LIBS="-L${IPP_HOME}/build/.build/RELEASE/lib -Wl,-rpath=${IPP_HOME}/build/.build/RELEASE/lib -lippcp"

        CFLAGS="${IPP_CFLAGS} ${CFLAGS}"
        LIBS="${IPP_LIBS} ${LIBS}"

        AC_TRY_LINK([#ifdef __PATHCC__
                     #include <rw/_defs.h>
                     #include <ansi/_cstddef.h>
                     #endif
                     #include "stddef.h"
                     #include <ippcp.h>
                     ],
                    [int ctxSize; ippsECCPGetSize(8, &ctxSize); ],
                    [AC_TRY_RUN([#ifdef __PATHCC__
                                 #include <rw/_defs.h>
                                 #include <ansi/_cstddef.h>
                                 #include "stddef.h"
                                 #endif
                                 #include <ippversion.h>
                                 int main () {  if (IPP_VERSION_MAJOR < 2020) return -1; else return 0; }],
                                 [AC_MSG_RESULT(found)
                                  AC_SUBST(IPP_CFLAGS)
                                  AC_SUBST(IPP_LIBS)
                                  AC_DEFINE(HAVE_IPP,1,[Define if IPP is installed])
                                  ipp_found="yes"
                                  break;],
                                 [ipp_problem="$ipp_problem $IPP_HOME"
                                  unset IPP_CFLAGS
                                  unset IPP_LIBS],
                                 [AC_MSG_RESULT(unknown)
                                  echo "WARNING: You appear to be cross compiling, so there is no way to determine"
                                  echo "whether your IPP version is new enough. I am assuming it is."
                                  AC_SUBST(IPP_CFLAGS)
                                  AC_SUBST(IPP_LIBS)
                                  AC_DEFINE(HAVE_IPP,1,[Define if IPP is installed])
                                  ipp_found="yes"
                                  break;])],
                    [ipp_found="no"
                     unset IPP_CFLAGS
                     unset IPP_LIBS])

    else
        ipp_found="no"
    fi
done

if test "x$ipp_found" != "xyes"; then
    if test -n "$ipp_problem"; then
        echo "Sorry, your IPP version is too old. Disabling."
        AC_MSG_RESULT(problem)
        AC_MSG_ERROR(IPP problem)
    elif test "x$ipp_found" == "xno"; then
        AC_MSG_RESULT(not found)
        AC_MSG_ERROR(IPP not found)
    fi
fi

CFLAGS=${BACKUP_CFLAGS}
LIBS=${BACKUP_LIBS}

fi

])
