dnl This file is part of MPHELL.
dnl MPHELL_CHECK_MBEDTLS
dnl Test for the MbedTLS library and define MBEDTLS_CFLAGS and MBEDTLS_LIBS

AC_DEFUN([MPHELL_CHECK_MBEDTLS],
[
AC_ARG_WITH([mbedtls],
            [AC_HELP_STRING([--with-mbedtls=<path>|yes], 
                            [Use MBEDTLS library.
                             If argument is yes or <empty> that means
                             the library is reachable with the standard search path
                             "/usr" or "/usr/local" (set as default). Otherwise you
                             give the <path> to the directory which contain the
                             library.])],
            [if test "$withval" = no ; then
                MBEDTLS_HOME_PATH="${DEFAULT_CHECKING_PATH}"
                use_mbedtls="no"
             elif test "$withval" = yes ; then
                MBEDTLS_HOME_PATH="${DEFAULT_CHECKING_PATH}"
                use_mbedtls="yes"
             elif test "$withval" != no ; then
                MBEDTLS_HOME_PATH="$withval ${DEFAULT_CHECKING_PATH}"
                use_mbedtls="yes"
             fi
             ],
             [
                MBEDTLS_HOME_PATH="${DEFAULT_CHECKING_PATH}"
                use_mbedtls="yes"
             ])

if test "x$use_mbedtls" != "xno"; then

min_mbedtls_version=ifelse([$1], ,2.11.0, $1)

dnl Check for existence
BACKUP_CFLAGS=${CFLAGS}
BACKUP_LIBS=${LIBS}

mbedtls_found="no"
AC_MSG_CHECKING(for MBEDTLS >= $min_mbedtls_version)

for MBEDTLS_HOME in ${MBEDTLS_HOME_PATH}
  do
    if test -r "${MBEDTLS_HOME}/include/mbedtls/bignum.h"; then
        MBEDTLS_CFLAGS="-I${MBEDTLS_HOME}/include"
        MBEDTLS_LIBS="-L${MBEDTLS_HOME}/lib -lmbedcrypto"

        CFLAGS="${MBEDTLS_CFLAGS} ${CFLAGS}"
        LIBS="${MBEDTLS_LIBS} ${LIBS}"

        AC_TRY_LINK([#ifdef __PATHCC__
                     #include <rw/_defs.h>
                     #include <ansi/_cstddef.h>
                     #endif
                     #include "stddef.h"
                     #include "mbedtls/bignum.h"
                     ],
                    [mbedtls_mpi a; mbedtls_mpi_init(&a);],
                    [AC_TRY_RUN([#ifdef __PATHCC__
                                 #include <rw/_defs.h>
                                 #include <ansi/_cstddef.h>
                                 #include "stddef.h"
                                 #endif
                                 #include "mbedtls/version.h"
                                 int main () {  if (((MBEDTLS_VERSION_MAJOR == 2) && (MBEDTLS_VERSION_MINOR < 11)) || (MBEDTLS_VERSION_MAJOR < 2) ) return -1; else return 0; }],
                                 [AC_MSG_RESULT(found)
                                  AC_SUBST(MBEDTLS_CFLAGS)
                                  AC_SUBST(MBEDTLS_LIBS)
                                  AC_DEFINE(HAVE_MBEDTLS,1,[Define if MBEDTLS is installed])
                                  mbedtls_found="yes"
                                  break;],
                                 [mbedtls_problem="$mbedtls_problem $MBEDTLS_HOME"
                                  unset MBEDTLS_CFLAGS
                                  unset MBEDTLS_LIBS],
                                 [AC_MSG_RESULT(unknown)
                                  echo "WARNING: You appear to be cross compiling, so there is no way to determine"
                                  echo "whether your MBEDTLS version is new enough. I am assuming it is."
                                  AC_SUBST(MBEDTLS_CFLAGS)
                                  AC_SUBST(MBEDTLS_LIBS)
                                  AC_DEFINE(HAVE_MBEDTLS,1,[Define if MBEDTLS is installed])
                                  mbedtls_found="yes"
                                  break;])],
                    [mbedtls_found="no"
                     unset MBEDTLS_CFLAGS
                     unset MBEDTLS_LIBS])

    else
        mbedtls_found="no"
    fi
done

if test "x$mbedtls_found" != "xyes"; then
    if test -n "$mbedtls_problem"; then
        echo "Sorry, your MBEDTLS version is too old. Disabling."
        AC_MSG_RESULT(problem)
        AC_MSG_ERROR(MbedTLS problem)
    elif test "x$mbedtls_found" == "xno"; then
        AC_MSG_RESULT(not found)
        AC_MSG_ERROR(MbedTLS not found)
    fi
fi

CFLAGS=${BACKUP_CFLAGS}
LIBS=${BACKUP_LIBS}

fi

])
