/* 
  MPHELL-4.0 
  Author(s): The MPHELL team

 (C) Copyright 2015-2018 - Institut Fourier / Univ. Grenoble Alpes (France) 

 This file is part of the MPHELL Library.
 MPHELL is free software: you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation, version 3 of the License.
 
 MPHELL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with MPHELL.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
  * \file  mphell-curve.c
  * \brief Definition of ECC functions.
  */

#include "mphell-curve.h"

/**************************************TMP**********************************/

/* void ec_point_get_pool_elt (ec_point_ptr P, field_ptr k, uint8_t stack){} is inline */

/* void ec_point_relax_pool_elt (ec_point_ptr P, field_ptr k, uint8_t stack){} is inline */

/************************************SETTERS**********************************/

void 
ec_alloc (ec_curve_ptr E, field_srcptr k)
{
    E->k = (field_t*)k;
    number_init(&(E->q), k->size);    
    field_elt_alloc(&E->a, k);
    field_elt_alloc(&E->b, k);
    ec_point_alloc(E->G, k);
    number_init(&(E->n), k->size);
    number_init(&(E->h), k->size);
    field_elt_alloc(&E->D, k);
    field_elt_alloc(&E->disc, k);
    E->id_curve=malloc(100*sizeof(char));
}

void 
ec_init (ec_curve_ptr E, field_srcptr k)
{ 
    field_elt_init(E->a, k);
    field_elt_init(E->b, k);
    ec_point_init(E->G, k);
    field_elt_init(E->D, k);
    field_elt_init(E->disc, k);
}

void 
ec_create (ec_curve_ptr E, const char *id_curve, field_srcptr k, fe_srcptr a,
        fe_srcptr b, ec_point_srcptr G, number_srcptr h, number_srcptr n, 
        const ec_type type, const ec_formula f, uint8_t stack)
{
    E->k = (field_t*)k;
    field_get_size(E->q, k);
    field_elt_copy(E->a, a, k);
    field_elt_copy(E->b, b, k);
    
    if(G != NULL)
    {
        ec_point_copy(E->G, G, k);
    }
    else
    {
        field_elt_set_ui(E->G->x, 0, true, k, stack);
        field_elt_set_ui(E->G->y, 0, true, k, stack);
        field_elt_set_ui(E->G->z, 0, true, k, stack);
        field_elt_set_ui(E->G->t, 0, true, k, stack);
    }
    if(h != NULL)
    {
        number_copy(E->h, h);
    }
    else
    {
        number_set_ui(E->h, 1);
    }

    if(n != NULL)
    {
        number_copy(E->n, n);
    }
    else
    {
        number_set_ui(E->n, 0);
    }
    E->type = type;
    E->f = f;
    if((id_curve != NULL) && (id_curve[0] !='\0'))
    {
        strcpy(E->id_curve, id_curve);
    }
    else
    {
        strcpy(E->id_curve, "no_name");
    }
    ec_compute_disc(E, stack);
    ec_test_spec(E, stack);
}

void 
ec_use_curve (ec_curve_ptr E, field_ptr k, const ec_known_curve id_curve, const ec_formula f, uint8_t stack)
{
    number tmp;
    switch(id_curve)
    {
        case NIST_192:
            /* Create k */
            number_tmp_alloc(&tmp, bits_to_nblock(192), stack);
            number_set_str(tmp, "fffffffffffffffffffffffffffffffeffffffffffffffff", 16);
            field_create(k, "p192r1", stack, 1, tmp);
            number_copy(E->q, tmp);
            number_tmp_free(&tmp, bits_to_nblock(192), stack);
            E->k = (field_t*)k;
            
            /* Initialise the curve, now that k is set up */
            ec_init(E, k);
            
            /* Set curve parameters*/
            field_elt_set_str(E->a, "fffffffffffffffffffffffffffffffefffffffffffffffc", 16, false, (field_t*)k, stack);
            field_elt_set_str(E->b, "64210519e59c80e70fa7e9ab72243049feb8deecc146b9b1", 16, false, (field_t*)k, stack);
            ec_point_set_aff_str(E->G, "188da80eb03090f67cbf20eb43a18800f4ff0afd82ff1012", "07192b95ffc8da78631011ed6b24cdd573f977a11e794811", false, 16, WEIERSTRASS, k, stack);
            number_set_str(E->n, "ffffffffffffffffffffffff99def836146bc9b1b4d22831", 16);
            number_set_str(E->h, "000000000000000000000000000000000000000000000001", 16);
            E->type = WEIERSTRASS;
            E->f = f;
            strcpy(E->id_curve, "nist_p192");
            ec_compute_disc(E, stack);
            ec_test_spec(E, stack);
            break;
            
        case NIST_224:
            /* Create k */
            number_tmp_alloc(&tmp, bits_to_nblock(224), stack);
            number_set_str(tmp, "ffffffffffffffffffffffffffffffff000000000000000000000001", 16);
            field_create(k, "p224r1", stack, 1, tmp);
            number_copy(E->q,tmp);
            number_tmp_free(&tmp, bits_to_nblock(224), stack);
            E->k = (field_t*)k;
            
            /* Initialise the curve, now that k is set up */
            ec_init(E, k);
            
            /* Set curve parameters*/
            field_elt_set_str(E->a, "fffffffffffffffffffffffffffffffefffffffffffffffffffffffe", 16, false, (field_t*)k, stack);
            field_elt_set_str(E->b, "b4050a850c04b3abf54132565044b0b7d7bfd8ba270b39432355ffb4", 16, false, (field_t*)k, stack);
            ec_point_set_aff_str(E->G, "b70e0cbd6bb4bf7f321390b94a03c1d356c21122343280d6115c1d21", "bd376388b5f723fb4c22dfe6cd4375a05a07476444d5819985007e34", false, 16, WEIERSTRASS, k, stack);
            number_set_str(E->n, "ffffffffffffffffffffffffffff16a2e0b8f03e13dd29455c5c2a3d", 16);
            number_set_str(E->h, "00000000000000000000000000000000000000000000000000000001", 16);
            E->type = WEIERSTRASS;
            E->f = f;
            strcpy(E->id_curve, "nist_p224");
            ec_compute_disc(E, stack);
            ec_test_spec(E, stack);
            break;
            
        case NIST_256:
            /* Create k */
            number_tmp_alloc(&tmp, bits_to_nblock(256), stack);
            number_set_str(tmp, "ffffffff00000001000000000000000000000000ffffffffffffffffffffffff", 16);
            field_create(k, "p256r1", stack, 1, tmp);
            number_copy(E->q,tmp);
            number_tmp_free(&tmp, bits_to_nblock(256), stack);
            E->k = (field_t*)k;
            
            /* Initialise the curve, now that k is set up */
            ec_init(E, k);
            
            /* Set curve parameters*/
            field_elt_set_str(E->a, "ffffffff00000001000000000000000000000000fffffffffffffffffffffffc", 16, false, (field_t*)k, stack);
            field_elt_set_str(E->b, "5ac635d8aa3a93e7b3ebbd55769886bc651d06b0cc53b0f63bce3c3e27d2604b", 16, false, (field_t*)k, stack);
            ec_point_set_aff_str(E->G, "6b17d1f2e12c4247f8bce6e563a440f277037d812deb33a0f4a13945d898c296", "4fe342e2fe1a7f9b8ee7eb4a7c0f9e162bce33576b315ececbb6406837bf51f5", false, 16, WEIERSTRASS, k, stack);
            number_set_str(E->n, "ffffffff00000000ffffffffffffffffbce6faada7179e84f3b9cac2fc632551", 16);
            number_set_str(E->h, "0000000000000000000000000000000000000000000000000000000000000001", 16);
            E->type = WEIERSTRASS;
            E->f = f;
            strcpy(E->id_curve, "nist_p256");
            ec_compute_disc(E, stack);
            ec_test_spec(E, stack);
            break;
            
        case NIST_384:
            /* Create k */
            number_tmp_alloc(&tmp, bits_to_nblock(384), stack);
            number_set_str(tmp, "fffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffeffffffff0000000000000000ffffffff", 16);
            number_copy(E->q,tmp);
            field_create(k, "p384r1", stack, 1, tmp);
            number_tmp_free(&tmp, bits_to_nblock(384), stack);
            E->k = (field_t*)k;
            
            /* Initialise the curve, now that k is set up */
            ec_init(E, k);
            
            /* Set curve parameters*/
            field_elt_set_str(E->a, "fffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffeffffffff0000000000000000fffffffc", 16, false, (field_t*)k, stack);
            field_elt_set_str(E->b, "b3312fa7e23ee7e4988e056be3f82d19181d9c6efe8141120314088f5013875ac656398d8a2ed19d2a85c8edd3ec2aef", 16, false, (field_t*)k, stack);
            ec_point_set_aff_str(E->G, "aa87ca22be8b05378eb1c71ef320ad746e1d3b628ba79b9859f741e082542a385502f25dbf55296c3a545e3872760aB7", "3617de4a96262c6f5d9e98bf9292dc29f8f41dbd289a147ce9da3113b5f0b8c00a60b1ce1d7e819d7a431d7c90ea0e5F", false, 16, WEIERSTRASS, k, stack);
            number_set_str(E->n, "ffffffffffffffffffffffffffffffffffffffffffffffffc7634d81f4372ddf581a0db248b0a77aecec196accc52973", 16);
            number_set_str(E->h, "0000000000000000000000000000000000000000000000000000000000000000000000000000000000000001", 16);
            E->type = WEIERSTRASS;
            E->f = f;
            strcpy(E->id_curve, "nist_p384");
            ec_compute_disc(E, stack);
            ec_test_spec(E, stack);
            break;
            
        case NIST_521:
            /* Create k */
            number_tmp_alloc(&tmp, bits_to_nblock(521), stack);
            number_set_str(tmp, "000001ffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffff", 16);
            number_copy(E->q,tmp);
            field_create(k, "p521r1", stack, 1, tmp);
            number_tmp_free(&tmp, bits_to_nblock(521), stack);
            E->k = (field_t*)k;
            
            /* Initialise the curve, now that k is set up */
            ec_init(E, k);
            
            /* Set curve parameters*/
            field_elt_set_str(E->a, "000001fffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffc", 16, false, (field_t*)k, stack);
            field_elt_set_str(E->b, "00000051953eb9618e1c9a1f929a21a0b68540eea2da725b99b315f3b8b489918ef109e156193951ec7e937b1652c0bd3bb1bf073573df883d2c34f1ef451fd46b503f00", 16, false, (field_t*)k, stack);
            ec_point_set_aff_str(E->G, "000000c6858e06b70404e9cd9e3ecb662395b4429c648139053fb521f828af606b4d3dbaa14b5e77efe75928fe1dc127a2ffa8de3348b3c1856a429bf97e7e31c2e5bd66", "0000011839296a789a3bc0045c8a5fb42c7d1bd998f54449579b446817afbd17273e662c97ee72995ef42640c550b9013fad0761353c7086a272c24088be94769fd16650", false, 16, WEIERSTRASS, k, stack);
            number_set_str(E->n, "000001fffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffa51868783bf2f966b7fcc0148f709a5d03bb5c9b8899c47aebb6fb71e91386409", 16);
            number_set_str(E->h, "0000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000001", 16);
            E->type = WEIERSTRASS;
            E->f = f;
            strcpy(E->id_curve, "nist_p521");
            ec_compute_disc(E, stack);
            ec_test_spec(E, stack);
            break;
            
        case BRAINPOOL_160:
            /* Create k */
            number_tmp_alloc(&tmp, bits_to_nblock(160), stack);
            number_set_str(tmp, "E95E4A5F737059DC60DFC7AD95B3D8139515620F", 16);
            number_copy(E->q,tmp);
            field_create(k, "ZZ_160", stack, 1, tmp);
            number_tmp_free(&tmp, bits_to_nblock(160), stack);
            E->k = (field_t*)k;
            
            /* Initialise the curve, now that k is set up */
            ec_init(E, k);
            
            /* Set curve parameters*/
            field_elt_set_str(E->a, "340E7BE2A280EB74E2BE61BADA745D97E8F7C300", 16, false, (field_t*)k, stack);
            field_elt_set_str(E->b, "1E589A8595423412134FAA2DBDEC95C8D8675E58", 16, false, (field_t*)k, stack);
            ec_point_set_aff_str(E->G, "BED5AF16EA3F6A4F62938C4631EB5AF7BDBCDBC3", "1667CB477A1A8EC338F94741669C976316DA6321", false, 16, WEIERSTRASS, k, stack);
            number_set_str(E->n, "E95E4A5F737059DC60DF5991D45029409E60FC09", 16);
            number_set_str(E->h, "0000000000000000000000000000000000000001", 16);
            E->type = WEIERSTRASS;
            E->f = f;
            strcpy(E->id_curve, "brainpool_p160r1");
            ec_compute_disc(E, stack);
            ec_test_spec(E, stack);
            break;
            
        case BRAINPOOL_192:
            /* Create k */
            number_tmp_alloc(&tmp, bits_to_nblock(192), stack);
            number_set_str(tmp, "C302F41D932A36CDA7A3463093D18DB78FCE476DE1A86297", 16);
            number_copy(E->q,tmp);
            field_create(k, "ZZ_192", stack, 1, tmp);
            number_tmp_free(&tmp, bits_to_nblock(192), stack);
            E->k = (field_t*)k;
            
            /* Initialise the curve, now that k is set up */
            ec_init(E, k);
            
            /* Set curve parameters*/
            field_elt_set_str(E->a, "6A91174076B1E0E19C39C031FE8685C1CAE040E5C69A28EF", 16, false, (field_t*)k, stack);
            field_elt_set_str(E->b, "469A28EF7C28CCA3DC721D044F4496BCCA7EF4146FBF25C9", 16, false, (field_t*)k, stack);
            ec_point_set_aff_str(E->G, "C0A0647EAAB6A48753B033C56CB0F0900A2F5C4853375FD6", "14B690866ABD5BB88B5F4828C1490002E6773FA2FA299B8F", false, 16, WEIERSTRASS, k, stack);
            number_set_str(E->n, "C302F41D932A36CDA7A3462F9E9E916B5BE8F1029AC4ACC1", 16);
            number_set_str(E->h, "000000000000000000000000000000000000000000000001", 16);
            E->type = WEIERSTRASS;
            E->f = f;
            strcpy(E->id_curve, "brainpool_p192r1");
            ec_compute_disc(E, stack);
            ec_test_spec(E, stack);
            break;
            
        case BRAINPOOL_224:
            /* Create k */
            number_tmp_alloc(&tmp, bits_to_nblock(224), stack);
            number_set_str(tmp, "D7C134AA264366862A18302575D1D787B09F075797DA89F57EC8C0FF", 16);
            number_copy(E->q,tmp);
            field_create(k, "ZZ_224", stack, 1, tmp);
            number_tmp_free(&tmp, bits_to_nblock(224), stack);
            E->k = (field_t*)k;
            
            /* Initialise the curve, now that k is set up */
            ec_init(E, k);
            
            /* Set curve parameters*/
            field_elt_set_str(E->a, "68A5E62CA9CE6C1C299803A6C1530B514E182AD8B0042A59CAD29F43", 16, false, (field_t*)k, stack);
            field_elt_set_str(E->b, "2580F63CCFE44138870713B1A92369E33E2135D266DBB372386C400B", 16, false, (field_t*)k, stack);
            ec_point_set_aff_str(E->G, "D9029AD2C7E5CF4340823B2A87DC68C9E4CE3174C1E6EFDEE12C07D", "58AA56F772C0726F24C6B89E4ECDAC24354B9E99CAA3F6D3761402CD", false, 16, WEIERSTRASS, k, stack);
            number_set_str(E->n, "D7C134AA264366862A18302575D0FB98D116BC4B6DDEBCA3A5A7939F", 16);
            number_set_str(E->h, "00000000000000000000000000000000000000000000000000000001", 16);
            E->type = WEIERSTRASS;
            E->f = f;
            strcpy(E->id_curve, "brainpool_p224r1");
            ec_compute_disc(E, stack);
            ec_test_spec(E, stack);
            break;
            
        case BRAINPOOL_256:
            /* Create k */
            number_tmp_alloc(&tmp, bits_to_nblock(256), stack);
            number_set_str(tmp, "A9FB57DBA1EEA9BC3E660A909D838D726E3BF623D52620282013481D1F6E5377", 16);
            number_copy(E->q,tmp);
            field_create(k, "ZZ_256", stack, 1, tmp);
            number_tmp_free(&tmp, bits_to_nblock(256), stack);
            E->k = (field_t*)k;
            
            /* Initialise the curve, now that k is set up */
            ec_init(E, k);
            
            /* Set curve parameters*/
            field_elt_set_str(E->a, "7D5A0975FC2C3057EEF67530417AFFE7FB8055C126DC5C6CE94A4B44F330B5D9", 16, false, (field_t*)k, stack);
            field_elt_set_str(E->b, "26DC5C6CE94A4B44F330B5D9BBD77CBF958416295CF7E1CE6BCCDC18FF8C07B6", 16, false, (field_t*)k, stack);
            ec_point_set_aff_str(E->G, "8BD2AEB9CB7E57CB2C4B482FFC81B7AFB9DE27E1E3BD23C23A4453BD9ACE3262", "547EF835C3DAC4FD97F8461A14611DC9C27745132DED8E545C1D54C72F046997", false, 16, WEIERSTRASS, k, stack);
            number_set_str(E->n, "A9FB57DBA1EEA9BC3E660A909D838D718C397AA3B561A6F7901E0E82974856A7", 16);
            number_set_str(E->h, "0000000000000000000000000000000000000000000000000000000000000001", 16);
            E->type = WEIERSTRASS;
            E->f = f;
            strcpy(E->id_curve, "brainpool_p256r1");
            ec_compute_disc(E, stack);
            ec_test_spec(E, stack);
            break;
            
        case BRAINPOOL_320:
            /* Create k */
            number_tmp_alloc(&tmp, bits_to_nblock(320), stack);
            number_set_str(tmp, "D35E472036BC4FB7E13C785ED201E065F98FCFA6F6F40DEF4F92B9EC7893EC28FCD412B1F1B32E27", 16);
            number_copy(E->q,tmp);
            field_create(k, "ZZ_320", stack, 1, tmp);
            number_tmp_free(&tmp, bits_to_nblock(320), stack);
            E->k = (field_t*)k;
            
            /* Initialise the curve, now that k is set up */
            ec_init(E, k);
            
            /* Set curve parameters*/
            field_elt_set_str(E->a, "3EE30B568FBAB0F883CCEBD46D3F3BB8A2A73513F5EB79DA66190EB085FFA9F492F375A97D860EB4", 16, false, (field_t*)k, stack);
            field_elt_set_str(E->b, "520883949DFDBC42D3AD198640688A6FE13F41349554B49ACC31DCCD884539816F5EB4AC8FB1F1A6", 16, false, (field_t*)k, stack);
            ec_point_set_aff_str(E->G, "43BD7E9AFB53D8B85289BCC48EE5BFE6F20137D10A087EB6E7871E2A10A599C710AF8D0D39E20611", "14FDD05545EC1CC8AB4093247F77275E0743FFED117182EAA9C77877AAAC6AC7D35245D1692E8EE1", false, 16, WEIERSTRASS, k, stack);
            number_set_str(E->n, "D35E472036BC4FB7E13C785ED201E065F98FCFA5B68F12A32D482EC7EE8658E98691555B44C59311", 16);
            number_set_str(E->h, "00000000000000000000000000000000000000000000000000000000000000000000000000000001", 16);
            E->type = WEIERSTRASS;
            E->f = f;
            strcpy(E->id_curve, "brainpool_p320r1");
            ec_compute_disc(E, stack);
            ec_test_spec(E, stack);
            break;
            
        case BRAINPOOL_384:
            /* Create k */
            number_tmp_alloc(&tmp, bits_to_nblock(384), stack);
            number_set_str(tmp, "8CB91E82A3386D280F5D6F7E50E641DF152F7109ED5456B412B1DA197FB71123ACD3A729901D1A71874700133107EC53", 16);
            number_copy(E->q,tmp);
            field_create(k, "ZZ_384", stack, 1, tmp);
            number_tmp_free(&tmp, bits_to_nblock(384), stack);
            E->k = (field_t*)k;
            
            /* Initialise the curve, now that k is set up */
            ec_init(E, k);
            
            /* Set curve parameters*/
            field_elt_set_str(E->a, "7BC382C63D8C150C3C72080ACE05AFA0C2BEA28E4FB22787139165EFBA91F90F8AA5814A503AD4EB04A8C7DD22CE2826", 16, false, (field_t*)k, stack);
            field_elt_set_str(E->b, "4A8C7DD22CE28268B39B55416F0447C2FB77DE107DCD2A62E880EA53EEB62D57CB4390295DBC9943AB78696FA504C11", 16, false, (field_t*)k, stack);
            ec_point_set_aff_str(E->G, "1D1C64F068CF45FFA2A63A81B7C13F6B8847A3E77EF14FE3DB7FCAFE0CBD10E8E826E03436D646AAEF87B2E247D4AF1E", "8ABE1D7520F9C2A45CB1EB8E95CFD55262B70B29FEEC5864E19C054FF99129280E4646217791811142820341263C5315", false, 16, WEIERSTRASS, k, stack);
            number_set_str(E->n, "8CB91E82A3386D280F5D6F7E50E641DF152F7109ED5456B31F166E6CAC0425A7CF3AB6AF6B7FC3103B883202E9046565", 16);
            number_set_str(E->h, "000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000001", 16);
            E->type = WEIERSTRASS;
            E->f = f;
            strcpy(E->id_curve, "brainpool_p384r1");
            ec_compute_disc(E, stack);
            ec_test_spec(E, stack);
            break;
            
        case BRAINPOOL_512:
            /* Create k */
            number_tmp_alloc(&tmp, bits_to_nblock(512), stack);
            number_set_str(tmp, "AADD9DB8DBE9C48B3FD4E6AE33C9FC07CB308DB3B3C9D20ED6639CCA703308717D4D9B009BC66842AECDA12AE6A380E62881FF2F2D82C68528AA6056583A48F3", 16);
            number_copy(E->q,tmp);
            field_create(k, "ZZ_512", stack, 1, tmp);
            number_tmp_free(&tmp, bits_to_nblock(512), stack);
            E->k = (field_t*)k;
            
            /* Initialise the curve, now that k is set up */
            ec_init(E, k);
            
            /* Set curve parameters*/
            field_elt_set_str(E->a, "7830A3318B603B89E2327145AC234CC594CBDD8D3DF91610A83441CAEA9863BC2DED5D5AA8253AA10A2EF1C98B9AC8B57F1117A72BF2C7B9E7C1AC4D77FC94CA", 16, false, (field_t*)k, stack);
            field_elt_set_str(E->b, "3DF91610A83441CAEA9863BC2DED5D5AA8253AA10A2EF1C98B9AC8B57F1117A72BF2C7B9E7C1AC4D77FC94CADC083E67984050B75EBAE5DD2809BD638016F723", 16, false, (field_t*)k, stack);
            ec_point_set_aff_str(E->G, "81AEE4BDD82ED9645A21322E9C4C6A9385ED9F70B5D916C1B43B62EEF4D0098EFF3B1F78E2D0D48D50D1687B93B97D5F7C6D5047406A5E688B352209BCB9F822", "7DDE385D566332ECC0EABFA9CF7822FDF209F70024A57B1AA000C55B881F8111B2DCDE494A5F485E5BCA4BD88A2763AED1CA2B2FA8F0540678CD1E0F3AD80892", false, 16, WEIERSTRASS, k, stack);
            number_set_str(E->n, "AADD9DB8DBE9C48B3FD4E6AE33C9FC07CB308DB3B3C9D20ED6639CCA70330870553E5C414CA92619418661197FAC10471DB1D381085DDADDB58796829CA90069", 16);
            number_set_str(E->h, "00000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000001", 16);
            E->type = WEIERSTRASS;
            E->f = f;
            strcpy(E->id_curve, "brainpool_p512r1");
            ec_compute_disc(E, stack);
            ec_test_spec(E, stack);
            break;
            
        case FR_256:
            /* Create k */
            number_tmp_alloc(&tmp, bits_to_nblock(256), stack);
            number_set_str(tmp, "F1FD178C0B3AD58F10126DE8CE42435B3961ADBCABC8CA6DE8FCF353D86E9C03", 16);
            number_copy(E->q,tmp);
            field_create(k, "ZZ_256", stack, 1, tmp);
            number_tmp_free(&tmp, bits_to_nblock(256), stack);
            E->k = (field_t*)k;
            
            /* Initialise the curve, now that k is set up */
            ec_init(E, k);
            
            /* Set curve parameters*/
            field_elt_set_str(E->a, "F1FD178C0B3AD58F10126DE8CE42435B3961ADBCABC8CA6DE8FCF353D86E9C00", 16, false, (field_t*)k, stack);
            field_elt_set_str(E->b, "EE353FCA5428A9300D4ABA754A44C00FDFEC0C9AE4B1A1803075ED967B7BB73F", 16, false, (field_t*)k, stack);
            ec_point_set_aff_str(E->G, "B6B3D4C356C139EB31183D4749D423958C27D2DCAF98B70164C97A2DD98F5CFF", "6142E0F7C8B204911F9271F0F3ECEF8C2701C307E8E4C9E183115A1554062CFB", false, 16, WEIERSTRASS, k, stack);
            number_set_str(E->n, "F1FD178C0B3AD58F10126DE8CE42435B53DC67E140D2BF941FFDD459C6D655E1", 16);
            number_set_str(E->h, "0000000000000000000000000000000000000000000000000000000000000001", 16);
            E->type = WEIERSTRASS;
            E->f = f;
            strcpy(E->id_curve, "frp256v1");
            ec_compute_disc(E, stack);
            ec_test_spec(E, stack);
            break;

        case JQ_256_3:
            /* Create k */
            number_tmp_alloc(&tmp, bits_to_nblock(256), stack);
            number_set_str(tmp, "b09f075797da89f57ec8c0265b014c5ba37bcb84208a4c4d83199a31d17cb1af", 16);
            number_copy(E->q,tmp);
            field_create(k, "ZZ_256", stack, 1, tmp);
            number_tmp_free(&tmp, bits_to_nblock(256), stack);
            E->k = (field_t*)k;
            
            /* Initialise the curve, now that k is set up */
            ec_init(E, k);
            
            /* Set curve parameters*/
            field_elt_set_str(E->a, "1468727718e87fae1d94287a36205fa0e3b70bbaad31a097c1ffa5a5db263683", 16, false, (field_t*)k, stack);
            ec_point_set_str(E->G, "7e99fa90a889802939e8704d3cc8abc5c0c879540dea85075eabfb6c73d515c5", "5d5db0901ca66a952304e220fcf6581c55fb7fb7986522096911c09d1e76de9b", "86eccefd9aa9174b1e61505ba174452525f7fd2989304574b298978e79690c20", "77d5d37af2ee21e362a68ec5e277b40538434a020c6d1a3c885b6eb8fec53dd3", false, 16, k, stack);
            number_set_str(E->n, "2c27c1d5e5f6a27d5fb2300996c0531746f66dec9a2e0bd13eabeafa863611f7", 16);
            number_set_str(E->h, "0000000000000000000000000000000000000000000000000000000000000004", 16);
            E->type = JACOBI_QUARTIC;
            E->f = f;
            strcpy(E->id_curve, "jq256_3");
            ec_compute_disc(E, stack);
            ec_test_spec(E, stack);
            break;

        case ED_25519:
            /* Create k */
            number_tmp_alloc(&tmp, bits_to_nblock(256), stack);
            number_set_str(tmp, "7fffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffed", 16);
            number_copy(E->q,tmp);
            field_create(k, "ZZ_256", stack, 1, tmp);
            number_tmp_free(&tmp, bits_to_nblock(256), stack);
            E->k = (field_t*)k;
            
            /* Initialise the curve, now that k is set up */
            ec_init(E, k);
            
            /* Set curve parameters*/
            field_elt_set_str(E->a, "0000000000000000000000000000000000000000000000000000000000000001", 16, false, (field_t*)k, stack);
            field_elt_set_str(E->b, "2dfc9311d490018c7338bf8688861767ff8ff5b2bebe27548a14b235eca6874a", 16, false, (field_t*)k, stack);
            ec_point_set_aff_str(E->G, "159a6849e44c3c7f061b3d570fc4ed5b5d14c8ba4253df49cc7edf80f533ad9b", "6666666666666666666666666666666666666666666666666666666666666658", false, 16, EDWARDS, k, stack);
            number_set_str(E->n, "1000000000000000000000000000000014def9dea2f79cd65812631a5cf5d3ed", 16);
            number_set_str(E->h, "0000000000000000000000000000000000000000000000000000000000000008", 16);
            E->type = EDWARDS;
            E->f = f;
            strcpy(E->id_curve, "ed_25519");
            ec_compute_disc(E, stack);
            ec_test_spec(E, stack);
            break;
    }
}

void ec_random(ec_curve_ptr E, const char *id_curve, char * seed_res, field_srcptr k, const ec_type type, const ec_formula f, uint8_t stack)
{
    field_elt a, b;
    field_elt_get_pool_elt(&a, (field_t*)k, stack);
    field_elt_get_pool_elt(&b, (field_t*)k, stack);
    switch(type)
    {
        case JACOBI_QUARTIC :
            jacobi_quartic_curve_random_generation(a, seed_res, k, stack);
            field_elt_set_ui(b, 0, false, k, stack);
            ec_create(E, id_curve, k, a, b, NULL, NULL, NULL, type, f, stack);
            break;

        case EDWARDS :
            edwards_curve_random_generation(b, seed_res, k, stack);
            field_elt_set_one(a, k);
            ec_create(E, id_curve, k, a, b, NULL, NULL, NULL, type, f, stack);
            break;

        case WEIERSTRASS :
            weierstrass_curve_random_generation(a, b, seed_res, k, stack);
            ec_create(E, id_curve, k, a, b, NULL, NULL, NULL, type, f, stack);
            break;
    }
    field_elt_relax_pool_elt(&a, (field_t*)k, stack);
    field_elt_relax_pool_elt(&b, (field_t*)k, stack);
}

bool ec_verify_random_generation(ec_curve_ptr E, const char * seed, uint8_t stack)
{
    bool res = false;
    switch(E->type)
    {
        case JACOBI_QUARTIC :
            res = jacobi_quartic_verify_random_generation(E, seed, stack);
            break;

        case EDWARDS :
            res = edwards_verify_random_generation(E, seed, stack);
            break;

        case WEIERSTRASS :
            res = weierstrass_verify_random_generation(E, seed, stack);
            break;
    }
    return res;
}

void 
ec_set_fast_unified_coordinates(ec_curve_ptr E)
{
    switch(E->type)
    {
        case JACOBI_QUARTIC :
            break;

        case EDWARDS :
            E->f = EXTENDED_EDWARDS;
            break;

        case WEIERSTRASS :
            E->f = JACOBIAN;
            break;
    }
}

void 
ec_set_fast_dedicated_coordinates(ec_curve_ptr E)
{
    switch(E->type)
    {
        case JACOBI_QUARTIC :
            break;

        case EDWARDS :
            /* Hard to tell which one is the better so no choice made here */
            break;

        case WEIERSTRASS :
            E->f = JACOBIAN;
            break;
    }
}

void
ec_compute_disc (ec_curve_ptr E, uint8_t stack)
{
    switch(E->type)
    {
        case JACOBI_QUARTIC :
            jacobi_quartic_compute_disc(E, stack);
            break;

        case EDWARDS :
            edwards_compute_disc(E, stack);
            break;

        case WEIERSTRASS :
            weierstrass_compute_disc(E, stack);
            break;
    }
}

void
ec_test_spec (ec_curve_ptr E, uint8_t stack)
{
    field_elt tmp1;
    field_elt_get_pool_elt(&tmp1, E->k, stack);
    field_elt_set_ui(tmp1, 3, false, E->k, stack);
    field_elt_neg(tmp1, tmp1, E->k);

    if(field_elt_cmp(tmp1, E->a, E->k) == 0)
    {
        E->ec_spec1 = true;
    }
    else
    {
        E->ec_spec1 = false;
    }
    field_elt_relax_pool_elt(&tmp1, E->k, stack);
}

void
ec_copy (ec_curve_ptr E_res, ec_curve_srcptr E)
{
    field_t *k = E->k;
    number_copy(E_res->q, E->q);    
    E_res->k = k;
    field_elt_copy(E_res->a, E->a, k);
    field_elt_copy(E_res->b, E->b, k);
    ec_point_copy(E_res->G, E->G, k);
    number_copy(E_res->h, E->h);
    number_copy(E_res->n, E->n);
    field_elt_copy(E_res->D, E->D, k);
    field_elt_copy(E_res->disc, E->disc, k);
    E_res->type=E->type;
    if(E->id_curve != NULL)
    {
        strcpy(E_res->id_curve, E->id_curve);
    }
    else
    {
        strcpy(E_res->id_curve, "no_name");
    }
    E_res->f=E->f;
}

void 
ec_clear (ec_curve_ptr E)
{
    field_t *k = E->k;
    field_elt_clear(&E->a, k);
    field_elt_clear(&E->b, k);
    ec_point_clear(E->G, k);
    field_elt_clear(&E->D, k);
    field_elt_clear(&E->disc, k);

}

void 
ec_free (ec_curve_ptr E)
{
    field_t *k = E->k;
    number_free(&E->q);
    field_elt_free(&E->a, k);
    field_elt_free(&E->b, k);
    ec_point_free(E->G, k);
    number_free(&E->h);
    number_free(&E->n);
    field_elt_free(&E->D, k);
    field_elt_free(&E->disc, k);
    if(E->id_curve != NULL)
    {
        free(E->id_curve);
    }
}

void 
ec_point_alloc (ec_point_ptr P, field_srcptr k)
{
    field_elt_alloc(&(P->x), k);
    field_elt_alloc(&(P->y), k);
    field_elt_alloc(&(P->z), k);
    field_elt_alloc(&(P->t), k);
}

void 
ec_point_init (ec_point_ptr P, field_srcptr k)
{
    field_elt_init(P->x, k);
    field_elt_init(P->y, k);
    field_elt_init(P->z, k);
    field_elt_init(P->t, k);
}

void 
ec_point_clear (ec_point_ptr P, field_srcptr k)
{
    field_elt_clear(&(P->x), k);
    field_elt_clear(&(P->y), k);
    field_elt_clear(&(P->z), k);
    field_elt_clear(&(P->t), k);
}

void 
ec_point_free (ec_point_ptr P, field_srcptr k)
{
    field_elt_free(&(P->x), k);
    field_elt_free(&(P->y), k);
    field_elt_free(&(P->z), k);
    field_elt_free(&(P->t), k);
}

void
ec_point_copy (ec_point_ptr P3, ec_point_srcptr P, field_srcptr k)
{
    field_elt_copy(P3->x, P->x, k);
    field_elt_copy(P3->y, P->y, k);
    field_elt_copy(P3->z, P->z, k);
    field_elt_copy(P3->t, P->t, k);
}

void 
ec_point_set (ec_point_ptr P, fe_srcptr x, fe_srcptr y, 
        fe_srcptr z, fe_srcptr t, field_srcptr k)
{
    field_elt_copy(P->x, x, k);
    field_elt_copy(P->y, y, k);
    field_elt_copy(P->z, z, k);
    field_elt_copy(P->t, t, k);
}

void 
ec_point_set_str (ec_point_ptr P, const char *str_x, const char *str_y, 
        const char *str_z, const char *str_t, const bool is_reduced, 
        const uint8_t base, field_srcptr k, uint8_t stack)
{
    field_elt_set_str(P->x, str_x, base, is_reduced, k, stack);
    field_elt_set_str(P->y, str_y, base, is_reduced, k, stack);
    field_elt_set_str(P->z, str_z, base, is_reduced, k, stack);
    field_elt_set_str(P->t, str_t, base, is_reduced, k, stack);
}

void 
ec_point_set_aff (ec_point_ptr P, fe_srcptr x, fe_srcptr y,
        const ec_type type, field_srcptr k, uint8_t stack)
{
    switch(type)
    {
        case JACOBI_QUARTIC :
            jacobi_quartic_point_set_aff(P, x, y, k, stack);
            break;

        case EDWARDS :
            edwards_point_set_aff(P, x, y, k, stack);
            break;

        case WEIERSTRASS :
            weierstrass_point_set_aff(P, x, y, k);
            break;
    }
}

void 
ec_point_set_aff_str (ec_point_ptr P, const char *str_x, const char *str_y,
        const bool is_reduced, const uint8_t base, const ec_type type, 
        field_srcptr k, uint8_t stack)
{
    switch(type)
    {
        case JACOBI_QUARTIC :
            jacobi_quartic_point_set_aff_str(P, str_x, str_y, is_reduced, base, k, stack);
            break;

        case EDWARDS :
            edwards_point_set_aff_str(P, str_x, str_y, is_reduced, base, k, stack);
            break;

        case WEIERSTRASS :
            weierstrass_point_set_aff_str(P, str_x, str_y, is_reduced, base, k, stack);
            break;
    }
}

void 
ec_point_set_neutral (ec_point_ptr dst, ec_curve_srcptr E, uint8_t stack)
{
    switch(E->type)
    {
        case JACOBI_QUARTIC :
            jacobi_quartic_point_set_neutral(dst, E, stack);
            break;

        case EDWARDS :
            edwards_point_set_neutral(dst, E, stack);
            break;

        case WEIERSTRASS :
            weierstrass_point_set_neutral(dst, E, stack);
            break;
    }
}

void
ec_point_norm (ec_point_ptr P, ec_curve_srcptr E, uint8_t stack)
{
    switch(E->type)
    {
        case JACOBI_QUARTIC :
            jacobi_quartic_point_norm(P, E, stack);
            break;

        case EDWARDS :
            edwards_point_norm(P, E, stack);
            break;

        case WEIERSTRASS :
            weierstrass_point_norm(P, E, stack);
            break;
    }
}

void
ec_point_get_x_affine (field_elt x, ec_point_ptr P, ec_curve_srcptr E, uint8_t stack)
{
    switch(E->type)
    {
        case JACOBI_QUARTIC :
            jacobi_quartic_point_get_x_affine(x, P, E, stack);
            break;

        case EDWARDS :
            edwards_point_get_x_affine(x, P, E, stack);
            break;

        case WEIERSTRASS :
            weierstrass_point_get_x_affine(x, P, E, stack);
            break;
    }
}

void
ec_point_get_y_affine (field_elt y, ec_point_ptr P, ec_curve_srcptr E, uint8_t stack)
{
    switch(E->type)
    {
        case JACOBI_QUARTIC :
            jacobi_quartic_point_get_y_affine(y, P, E, stack);
            break;

        case EDWARDS :
            edwards_point_get_y_affine(y, P, E, stack);
            break;

        case WEIERSTRASS :
            weierstrass_point_get_y_affine(y, P, E, stack);
            break;
    }
}

void
ec_point_lift (ec_point_ptr P, field_srcptr k, uint8_t stack)
{
    field_elt_lift(P->x, P->x, k, stack);
    field_elt_lift(P->y, P->y, k, stack);
    field_elt_lift(P->z, P->z, k, stack);
    field_elt_lift(P->t, P->t, k, stack);
}

bool
ec_belongs (ec_point_srcptr P, ec_curve_srcptr E, uint8_t stack)
{
    switch(E->type)
    {
        case JACOBI_QUARTIC :
            return jacobi_quartic_belongs(P, E, stack);
            break;

        case EDWARDS :
            return edwards_belongs(P, E, stack);
            break;

        case WEIERSTRASS :
            return weierstrass_belongs(P, E, stack);
            break;
    }
    return false;
}

void 
ec_point_random (ec_point_ptr P, ec_curve_srcptr E, uint8_t stack)
{
    switch(E->type)
    {
        case JACOBI_QUARTIC :
            jacobi_quartic_point_random(P, E, stack);
            break;

        case EDWARDS :
            edwards_point_random(P, E, stack);
            break;

        case WEIERSTRASS :
            weierstrass_point_random(P, E, stack);
            break;
    }
}

/**
 * @fn    void ec_type_str(char** str, const ec_type type)
 * @brief Allocate *str and write in it the type of E
 * @param str  Pointer to a string, allocated by the function.
 * @param type Type of the elliptic curve
 */
void
ec_type_str(char** str, const ec_type type)
{
    switch(type)
    {
        case JACOBI_QUARTIC :
            *str = (char*)malloc(strlen("JACOBI QUARTIC") + 1);
            strcpy(*str, "JACOBI QUARTIC");
            break;

        case EDWARDS :
            *str = (char*)malloc(strlen("EDWARDS") + 1);
            strcpy(*str, "EDWARDS");
            break;

        case WEIERSTRASS :
            *str = (char*)malloc(strlen("WEIERSTRASS") + 1);
            strcpy(*str, "WEIERSTRASS");
            break;
    }
}

void
ec_curve_str (char **str, ec_curve_srcptr E, const uint8_t base, uint8_t stack)
{
    field_t *k = E->k;
    
    char *a_str, *b_str, *G_str, *h_str, *n_str, *k_str, *type_str, *disc_str, *D_str;
    
    ec_point P;
    ec_point_get_pool_elt(P, E->k, stack);
    ec_point_copy(P, E->G, k);
    if(!field_elt_iszero(E->G->z, k))
    {
        ec_point_norm(P, E, stack);
    }
    
    switch(E->type)
    {
        case JACOBI_QUARTIC :
            ec_type_str(&type_str, E->type);
            field_str(&k_str, E->k, base, stack);
            field_elt_str(&a_str, E->a, base, true, k, stack);
            number_str(&h_str, E->h, base);
            number_str(&n_str, E->n, base);
            field_elt_str(&disc_str, E->disc, base, true, k, stack);
            ec_point_str(&G_str, P, base, true, k, stack);

            *str = (char*)malloc(strlen(a_str) + strlen(k_str) +
                    strlen(h_str) + strlen(n_str) + strlen(G_str) + 
                    strlen(type_str) + strlen(disc_str) + 100);

            sprintf(*str,"id:%s\ntype:%s\nK:%s\nA:%s\nG:%s\nh:%s\nn:%s\ndisc:%s\n", 
                    E->id_curve, type_str, k_str, a_str, G_str, h_str, n_str, disc_str);

            free(type_str);
            free(k_str);
            free(a_str);
            free(G_str);
            free(h_str);
            free(n_str);
            free(disc_str);
            ec_point_relax_pool_elt(P, E->k, stack);
            break;

        case EDWARDS :
            ec_type_str(&type_str, E->type);
            field_str(&k_str, E->k, base, stack);
            field_elt_str(&a_str, E->a, base, true, k, stack);
            field_elt_str(&b_str, E->b, base, true, k, stack);
            number_str(&h_str, E->h, base);
            number_str(&n_str, E->n, base);
            field_elt_str(&disc_str, E->disc, base, true, k, stack);
            ec_point_str(&G_str, P, base, true, k, stack);

            *str = (char*)malloc(strlen(a_str) + strlen(b_str) + strlen(k_str) +
                    strlen(h_str) + strlen(n_str) + strlen(G_str) + 
                    strlen(type_str) + strlen(disc_str) + 100);

            sprintf(*str,"id:%s\ntype:%s\nK:%s\nA:%s\nD:%s\nG:%s\nh:%s\nn:%s\ndisc:%s\n", 
                    E->id_curve, type_str, k_str, a_str, b_str, G_str, h_str, n_str, disc_str);

            free(type_str);
            free(k_str);
            free(a_str);
            free(b_str);
            free(G_str);
            free(h_str);
            free(n_str);
            free(disc_str);
            ec_point_relax_pool_elt(P, E->k, stack);
            break;

        case WEIERSTRASS :
            ec_type_str(&type_str, E->type);
            field_str(&k_str, E->k, base, stack);
            field_elt_str(&a_str, E->a, base, true, k, stack);
            field_elt_str(&b_str, E->b, base, true, k, stack);
            number_str(&h_str, E->h, base);
            number_str(&n_str, E->n, base);
            field_elt_str(&D_str, E->D, base, true, k, stack);
            field_elt_str(&disc_str, E->disc, base, true, k, stack);
            ec_point_str(&G_str, P, base, true, k, stack);

            *str = (char*)malloc(strlen(a_str) + strlen(b_str) + strlen(k_str) +
                    strlen(n_str) + strlen(n_str) + strlen(G_str) + strlen(type_str) + 
                    strlen(D_str) + strlen(disc_str) + 100);

            sprintf(*str,"id:%s\ntype:%s\nK:%s\nA:%s\nB:%s\nG:%s\nh:%s\nn:%s\ndisc:%s\nD:%s", 
                    E->id_curve, type_str, k_str, a_str, b_str, G_str, h_str, n_str, disc_str, 
                    D_str);

            free(type_str);
            free(k_str);
            free(a_str);
            free(b_str);
            free(G_str);
            free(h_str);    
            free(n_str);
            free(disc_str);
            free(D_str);
            ec_point_relax_pool_elt(P, E->k, stack);
            break;
    }
}

void
ec_curve_print (ec_curve_srcptr E, const uint8_t base, uint8_t stack)
{
    char *str;
    ec_curve_str(&str, E, base, stack);
    printf("%s\n", str);
    free(str);
}

void
ec_point_str (char **str, ec_point_srcptr P, const uint8_t base, 
        const bool lift, field_srcptr k, uint8_t stack)
{
    char *x_str, *y_str, *z_str, *t_str;
    field_elt_str(&x_str, P->x, base, lift, k, stack);
    field_elt_str(&y_str, P->y, base, lift, k, stack);
    field_elt_str(&z_str, P->z, base, lift, k, stack);
    field_elt_str(&t_str, P->t, base, lift, k, stack);

    *str = (char*)malloc(strlen(x_str) + strlen(y_str) + strlen(z_str) + 
            strlen(t_str) + 20);
    sprintf(*str, "[%s,\n%s,\n%s,\n%s]", x_str, y_str, z_str, t_str);
    free(x_str);
    free(y_str);
    free(z_str);
    free(t_str);
}

void
ec_point_print (ec_point_srcptr P, const uint8_t base, const bool lift, field_srcptr k, uint8_t stack)
{
    char *str;
    ec_point_str(&str, P, base, lift, k, stack);
    printf("%s\n", str);
    free(str);
}

/*******************************COMPARISON************************************/

bool 
ec_point_is_neutral (ec_point_srcptr P, ec_curve_srcptr E, uint8_t stack)
{
    switch(E->type)
    {
        case JACOBI_QUARTIC :
            return jacobi_quartic_point_is_neutral(P, E);
            break;

        case EDWARDS :
            return edwards_point_is_neutral(P, E);
            break;

        case WEIERSTRASS :
            return weierstrass_point_is_neutral(P, E, stack);
            break;
    }
    return false;
}

bool
ec_point_are_equal (ec_point_srcptr P1, ec_point_srcptr P2, ec_curve_srcptr E, uint8_t stack)
{
    switch(E->type)
    {
        case JACOBI_QUARTIC :
            return jacobi_quartic_point_are_equal(P1, P2, E, stack);
            break;

        case EDWARDS :
            return edwards_point_are_equal(P1, P2, E, stack);
            break;

        case WEIERSTRASS :
            return weierstrass_point_are_equal(P1, P2, E, stack);
            break;
    }
    return false;
}

/*******************************OPERATIONS************************************/

void 
ec_point_neg (ec_point_ptr P3, ec_point_srcptr P1, ec_curve_srcptr E)
{
    switch(E->type)
    {
        case JACOBI_QUARTIC :
            jacobi_quartic_point_neg(P3, P1, E);
            break;

        case EDWARDS :
            edwards_point_neg(P3, P1, E);
            break;

        case WEIERSTRASS :
            weierstrass_point_neg(P3, P1, E);
            break;
    }
}

/* UNIFIED => Resistant to SPA */

void 
ec_point_add_unified (ec_point_ptr P3, ec_point_srcptr P1, ec_point_srcptr P2, 
        ec_curve_srcptr E, uint8_t stack)
{
    switch(E->type)
    {
        case JACOBI_QUARTIC :
            jacobi_quartic_point_add_unified(P3, P1, P2, E, stack);
            break;

        case EDWARDS :
            /* Only EXTENDED_EDWARDS coordinate provide unified addition */
            MPHELL_ASSERT(E->f==EXTENDED_EDWARDS, "\nOnly Extended Edwards coordinates are supported \n");
            edwards_point_add_unified(P3, P1, P2, E, stack);
            break;

        case WEIERSTRASS :
            /* Only PROJECTIVE coordinates provide unified addition, JACOBIAN coordinates provide unified multiplication through COZ */
            MPHELL_ASSERT(E->f==PROJECTIVE, "\nOnly Projective coordinates are supported \n");
            weierstrass_point_add_unified(P3, P1, P2, E, stack);
            break;
    }
}

void 
ec_point_dbl_unified (ec_point_ptr P3, ec_point_srcptr P1, ec_curve_srcptr E, uint8_t stack)
{
    switch(E->type)
    {
        case JACOBI_QUARTIC :
            jacobi_quartic_point_add_unified(P3, P1, P1, E, stack);
            break;

        case EDWARDS :
            /* Only EXTENDED_EDWARDS coordinates provide unified addition */
            MPHELL_ASSERT(E->f==EXTENDED_EDWARDS, "\nOnly Extended Edwards coordinates are supported \n");
            edwards_point_add_unified(P3, P1, P1, E, stack);
            break;

        case WEIERSTRASS :
            /* Only PROJECTIVE coordinates provide unified addition, JACOBIAN coordinates provide unified multiplication through COZ */
            MPHELL_ASSERT(E->f==PROJECTIVE, "\nOnly Projective coordinates are supported \n");
            weierstrass_point_add_unified(P3, P1, P1, E, stack);
            break;
    }
}

void 
ec_point_sub_unified (ec_point_ptr P3, ec_point_srcptr P1, ec_point_srcptr P2, 
        ec_curve_srcptr E, uint8_t stack)
{
    field_t *k = (field_t*)E->k;
    ec_point tmp;
    ec_point_get_pool_elt(tmp, k, stack);

    ec_point_neg(tmp, P2, E);
    ec_point_add_unified(P3, P1, tmp, E, stack);

    ec_point_relax_pool_elt(tmp, k, stack);
}

/**
 * @fn    void ec_point_mul_naive_unified (ec_point_ptr P3, number_srcptr n, ec_point_srcptr P1, ec_curve_srcptr E, uint8_t stack)
 * @brief Set P3 to n * P1 using naive double and add method, Montgomery and Joye multiplication are also available for Weierstrass elliptic curve, using unified formulae (protection against SPA)
 * @param P3    Destination point
 * @param n     Scalar to multiply P1 with
 * @param P1    Point
 * @param E     Elliptic curve
 * @param stack Temporary memory stack to use
 */
void 
ec_point_mul_naive_unified (ec_point_ptr P3, number_srcptr n, ec_point_srcptr P1,
        ec_curve_srcptr E, uint8_t stack)
{
    ec_point_set_neutral(P3, E, stack);
    if(!number_iszero(n))
    {
        field_t *k = (field_t*)E->k;
        ec_point tmp;
        ec_point_get_pool_elt(tmp, k, stack);
        int16_t l = number_log2(n);
        int16_t i;
        
        /* Set n in base 2: n_str=[n_l-1, ..., n_0] */
        char n_str[k->bit_size+4];
        number_to_bit_string(n_str, n);
        
        ec_point_copy(tmp, P1, k);
        
        i = l-1;
        
        while (i>=0) 
        {
            if (n_str[i]=='1') 
            {
                ec_point_add_unified(P3, P3, tmp, E, stack);
            }
            ec_point_add_unified(tmp, tmp, tmp, E, stack);
            i--;
        }
        ec_point_relax_pool_elt(tmp, k, stack);
    }
}

void
ec_point_mul_unified(ec_point_ptr P3, number_srcptr n, ec_point_srcptr P1, 
        ec_curve_srcptr E, uint8_t stack)
{
    switch(E->type)
    {
        case JACOBI_QUARTIC :
            ec_point_mul_naive_unified(P3, n, P1, E, stack);
            break;

        case EDWARDS :
            switch(E->f)
            {
                case EXTENDED_EDWARDS:
                    /* Only EXTENDED_EDWARDS coordinates provide unified addition */
                    ec_point_mul_naive_unified(P3, n, P1, E, stack);
                    break;
                default:
                    mphell_error("ec_point_mul_unified, the only available coordinate system is EXTENDED_EDWARDS for unified multiplication over Edwards curves \n");
                    break;
            }
            break;

        case WEIERSTRASS :
            switch(E->f)
            {
                case PROJECTIVE:
                    /* Only PROJECTIVE coordinates provide unified addition */
                    ec_point_mul_naive_unified(P3, n, P1, E, stack);
                    break;
                case JACOBIAN:
                    /* But Jacobian coordinates provide unified multiplication through COZ */
                    weierstrass_Zmontgomery_Kim(P3, n, P1, E, stack);
                    /* weierstrass_Zmontgomery_ladder(P3, n, P1, E, stack); */
                    break;
                default :
                    mphell_error("ec_point_mul_unified, the only available coordinates are PROJECTIVE and JACOBIAN\n");
                    break;
            }
            break;
            
        default :
            mphell_error("ec_point_mul_unified, the only available curves are WEIERSTRASS, JACOBI_QUARTIC and EDWARDS\n");
            break;
    }
}

/* DEDICATED => Not resistant to SPA, but faster */

void 
ec_point_add (ec_point_ptr P3, ec_point_srcptr P1, ec_point_srcptr P2, 
        ec_curve_srcptr E, uint8_t stack)
{
    switch(E->type)
    {
        case JACOBI_QUARTIC :
            /* Only the unified addition is implemented because it is faster than the dedicated addition */
            jacobi_quartic_point_add_unified(P3, P1, P2, E, stack);
            break;

        case EDWARDS :
            edwards_point_add_dedicated(P3, P1, P2, E, stack);
            break;

        case WEIERSTRASS :
            weierstrass_point_add_dedicated(P3, P1, P2, E, stack);
            break;
    }
}

void 
ec_point_dbl (ec_point_ptr P3, ec_point_srcptr P1, ec_curve_srcptr E, uint8_t stack)
{
    switch(E->type)
    {
        case JACOBI_QUARTIC :
            /* The dedicated doubling save time compared to the unified doubling */
            jacobi_quartic_point_dbl_dedicated(P3, P1, E, stack);
            break;

        case EDWARDS :
            edwards_point_dbl_dedicated(P3, P1, E, stack);
            break;

        case WEIERSTRASS :
            weierstrass_point_dbl_dedicated(P3, P1, E, stack);
            break;
    }
}

void 
ec_point_sub (ec_point_ptr P3, ec_point_srcptr P1, ec_point_srcptr P2, 
        ec_curve_srcptr E, uint8_t stack)
{
    field_t *k = (field_t*)E->k;
    ec_point tmp;
    ec_point_get_pool_elt(tmp, k, stack);

    ec_point_neg(tmp, P2, E);
    ec_point_add(P3, P1, tmp, E, stack);

    ec_point_relax_pool_elt(tmp, k, stack);
}

/**
 * @fn    void ec_point_mul_naive (ec_point_ptr P3, number_srcptr n, ec_point_srcptr P1, ec_curve_srcptr E, uint8_t stack)
 * @brief Set P3 to n * P1 using naive double and add method, Montgomery and Joye multiplication are also available for Weierstrass elliptic curve, using dedicated formulae (not protected against SPA, but faster)
 * @param P3    Destination point
 * @param n     Scalar to multiply P1 with
 * @param P1    Point
 * @param E     Elliptic curve
 * @param stack Temporary memory stack to use
 */
void 
ec_point_mul_naive (ec_point_ptr P3, number_srcptr n, ec_point_srcptr P1,
        ec_curve_srcptr E, uint8_t stack)
{
    ec_point_set_neutral(P3, E, stack);
    if(!number_iszero(n))
    {
        field_t *k = (field_t*)E->k;
        ec_point tmp;
        ec_point_get_pool_elt(tmp, k, stack);
        int16_t l = number_log2(n);
        int16_t i;
        
        /* Set n in base 2: n_str=[n_l-1, ..., n_0] */
        char n_str[k->bit_size+4];
        number_to_bit_string(n_str, n);
        
        ec_point_copy(tmp, P1, k);
        
        i = l-1;
        
        while (i>=0) 
        {
            if (n_str[i]=='1') 
            {
                ec_point_add(P3, P3, tmp, E, stack);
            }
            ec_point_dbl(tmp, tmp, E, stack);
            i--;
        }
        ec_point_relax_pool_elt(tmp, k, stack);
    }
}

/**
 * @fn     int8_t binary_to_indice(char *str, int16_t beg, int16_t end)
 * @brief  Compute the indice for the sliding window multiplication
 * @param  str Binary string
 * @param  beg Where we start to read str
 * @param  end Where we end to read str
 * @return The indice to use in the precomputed table
 */
int16_t binary_to_indice(char *str, int16_t beg, int16_t end)
{
    /* str = [n_(l-1), ..., n_beg, ..., n_end, ..., n_0] */
    /*        0             3           6           l-1  */
    MPHELL_ASSERT(str[beg]=='1',  "Sliding window error, str[beg]!=1\n");
    MPHELL_ASSERT(str[end]=='1',  "Sliding window error, str[end]!=1\n");
    int16_t i;
    uint16_t j = 1;
    uint16_t res = 1;
    for (i = end-1; i >= beg; i--)
    {
        if (str[i]=='1')
        {
            res+=((uint16_t)1<<j);
        }
        j++;
    }
    return ((res-1)/2);
}

/**
 * @fn     int8_t binary_to_indice_new(char *str, int16_t beg, int16_t end)
 * @brief  Compute the indice for the sliding window multiplication
 * @param  str Binary string
 * @param  beg Where we start to read str
 * @param  end Where we end to read str
 * @return The indice to use in the precomputed table
 */
int16_t binary_to_indice_new(char *str, int16_t beg, int16_t end)
{
    /* str = [n_(l-1), ..., n_beg, ..., n_end, ..., n_0] */
    /*        0             3           6           l-1  */
    int16_t i;
    uint16_t j = 0;
    uint16_t res = 0;
    for (i = end; i >= beg; i--)
    {
        if (str[i]=='1')
        {
            res+=((uint16_t)1<<j);
        }
        j++;
    }
    return res;
}

/**
 * @fn    void ec_point_mul_sliding_window(ec_point_ptr P3, number_srcptr n, ec_point_srcptr P1, ec_curve_srcptr E, uint8_t stack);
 * @brief Set P3 to n * P1 using naive double and add method, Montgomery and Joye multiplication are also available for Weierstrass elliptic curve, using dedicated formulae (not protected against SPA, but faster)
 * @param P3    Destination point
 * @param n     Scalar to multiply P1 with
 * @param P1    Point
 * @param E     Elliptic curve
 * @param stack Temporary memory stack to use
 */
void 
ec_point_mul_sliding_window (ec_point_ptr P3, number_srcptr n, ec_point_srcptr P1,
        ec_curve_srcptr E, uint8_t stack)
{
    ec_point_set_neutral(P3, E, stack);

    field_t *k = (field_t*)E->k;
    ec_point tmp;
    ec_point_get_pool_elt(tmp, k, stack);

    int16_t e;
    int16_t pr;
    int16_t l = number_log2(n);
    int16_t i, s, h;
    
    /* Set n in base 2: n_str=[n_l-1, ..., n_0] */
    char n_str[k->bit_size+4];
    number_to_bit_string(n_str, n);
    
    /* Constant e */
    if(l<70)
    {
        /* Precompute 3P1, 5P1, 7P1 */
        e = 3;
        pr = 3;
    }
    else if(l<197)
    {
        /* Precompute 3P1, 5P1, 7P1, 9P1, 11P1, 13P1, 15P1 */
        e = 4;
        pr = 7;
    }
    else if(l<539)
    {
        /* Precompute 3P1, 5P1, 7P1, 9P1, 11P1, 13P1, 15P1, 17P1, 19P1, 21P1, 23P1, 25P1, 27P1, 29P1, 31P1 */
        e = 5;
        pr = 15;
    }
    else
    {
        /* Precompute 3P1, ... , 63P1 (31 values) */
        e = 6;
        pr = 31;
    }
    
    /* Precomputes value 3P1, 5P1, ..., (2^e - 1)P1 */
    
    ec_point tab[pr+1];
    
    for(i = 0; i <= pr; i++)
    {
        ec_point_get_pool_elt(tab[i], k, stack);
    }
        
    ec_point_dbl(tmp, P1, E, stack);        /* tmp = 2P1    */
    
    ec_point_copy(tab[0], P1, k);           /* tab[0] = P1  */
    for(i = 1; i <= pr; i++)
    {
        /* tab[i]=(2*i + 1) P1 */
        ec_point_add(tab[i], tmp, tab[i-1], E, stack);
    }
    
    /* Sliding method */
    i = 0; /* l-1 */
    
    while(i < l)
    {
        if (n_str[i]=='0')
        {
            ec_point_dbl(P3, P3, E, stack);
            i++;
        }
        else
        {
            s = i+e-1; /* l-1 -> l-4 for e = 4 */
            if (s > l-1)
            {
                s = l-1;
            }
            while(n_str[s]=='0')
            {
                s--;
            }
            if(i!=0)
            {
                for(h=1; h<=s-i+1; h++)
                {
                    ec_point_dbl(P3, P3, E, stack);
                }
                ec_point_add(P3, P3, tab[binary_to_indice(n_str, i, s)], E, stack);
            }
            else
            {
                ec_point_copy(P3, tab[binary_to_indice(n_str, i, s)], k);
            }
            i=s+1;
        }
    }

    for(i = 0; i <= pr; i++)
    {
        ec_point_relax_pool_elt(tab[i], k, stack);
    }
    ec_point_relax_pool_elt(tmp, k, stack);
}

void 
ec_point_mul_with_precomp (ec_point_ptr P3, number_srcptr n, ec_point * tab_P1, 
        int16_t win_size, ec_curve_srcptr E, uint8_t stack)
{
    ec_point_set_neutral(P3, E, stack);
    
    int16_t e = win_size;
    int16_t l = number_log2(n);
    int16_t i, s, h;
    
    /* Set n in base 2: n_str=[n_l-1, ..., n_0] */
    char n_str[E->k->bit_size+4];
    number_to_bit_string(n_str, n);
    
    /* Sliding method */
    i = 0; /* l-1 */
    
    while(i < l)
    {
        if (n_str[i]=='0')
        {
            ec_point_dbl(P3, P3, E, stack);
            i++;
        }
        else
        {
            s = i+e-1; /* l-1 -> l-4 for e = 4 */
            if (s > l-1)
            {
                s = l-1;
            }
            while(n_str[s]=='0')
            {
                s--;
            }
            if(i!=0)
            {
                for(h=1; h<=s-i+1; h++)
                {
                    ec_point_dbl(P3, P3, E, stack);
                }
                ec_point_add(P3, P3, tab_P1[binary_to_indice(n_str, i, s)], E, stack);
            }
            else
            {
                ec_point_copy(P3, tab_P1[binary_to_indice(n_str, i, s)], E->k);
            }
            i=s+1;
        }
    }
}

void 
ec_point_2mul_with_precomp (ec_point_ptr P3, number_srcptr n1, ec_point * tab_P1,
        number_srcptr n2, ec_point * tab_P2, 
        int16_t win_size, ec_curve_srcptr E, uint8_t stack)
{
    ec_point_set_neutral(P3, E, stack);

    int16_t i, s, h, l, i1, i2, a, b;
    int16_t e = win_size;
    int16_t l1 = number_log2(n1);
    int16_t l2 = number_log2(n2);
    
    /* Set n1, n2 in base 2: n1_str=[n_l-1, ..., n_0] */
    char n1_str[E->k->bit_size+4];
    char n2_str[E->k->bit_size+4];
    number_to_bit_string(n1_str, n1);
    number_to_bit_string(n2_str, n2);
    
    /* Sliding method */
    i = 0; /* l-1 */
    i1 = 0;
    i2 = 0;
    bool init = false;
    
    if(l1>l2)
    {
        l=l2;
        while(i1<l1-l2)
        {
            if (n1_str[i1]=='0')
            {
                ec_point_dbl(P3, P3, E, stack);
                i1++;
            }
            else
            {
                s = i1+e-1;
                if(s>l1-l2-1)
                {
                    s = l1-l2-1;
                }
                while(n1_str[s]=='0')
                {
                    s--;
                }
                if(init)
                {
                    for(h=1; h<=s-i1+1; h++)
                    {
                        ec_point_dbl(P3, P3, E, stack);
                    }
                    ec_point_add(P3, P3, tab_P1[binary_to_indice_new(n1_str, i1, s)], E, stack);
                }
                else
                {
                    ec_point_copy(P3, tab_P1[binary_to_indice_new(n1_str, i1, s)], E->k);
                    init = true;
                }
                i1=s+1;
            }
        }
    }
    else if(l2>l1)
    {
        l=l1;
        while(i2<l2-l1)
        {
            if (n2_str[i2]=='0')
            {
                ec_point_dbl(P3, P3, E, stack);
                i2++;
            }
            else
            {
                s = i2+e-1;
                if(s>l2-l1-1)
                {
                    s = l2-l1-1;
                }
                while(n2_str[s]=='0')
                {
                    s--;
                }
                if(init)
                {
                    for(h=1; h<=s-i2+1; h++)
                    {
                        ec_point_dbl(P3, P3, E, stack);
                    }    
                    ec_point_add(P3, P3, tab_P2[binary_to_indice_new(n2_str, i2, s)], E, stack);
                }
                else
                {
                    ec_point_copy(P3, tab_P2[binary_to_indice_new(n2_str, i2, s)], E->k);
                    init = true;
                }
                i2=s+1;
            }
        }
    }
    else
    {
        l=l1;
    }
    while(i < l)
    {
        if ((n1_str[i+i1]=='0') && (n2_str[i+i2]=='0'))
        {
            ec_point_dbl(P3, P3, E, stack);
            i++;
        }
        else
        {
            s = i+e-1; /* l-1 -> l-4 for e = 4 */
            if (s > l-1)
            {
                s = l-1;
            }
            while((n1_str[s+i1]=='0') && (n2_str[s+i2]=='0'))
            {
                s--;
            }
            if(init)
            {
                for(h=1; h<=s-i+1; h++)
                {
                    ec_point_dbl(P3, P3, E, stack);
                }
                a = binary_to_indice_new(n1_str, i+i1, s+i1);
                b = binary_to_indice_new(n2_str, i+i2, s+i2);
                if(a == 0)
                {
                    ec_point_add(P3, P3, tab_P2[b], E, stack);
                }
                else if (b == 0)
                {
                    ec_point_add(P3, P3, tab_P1[a], E, stack);
                }
                else
                {
                    ec_point_add(P3, P3, tab_P1[a], E, stack);
                    ec_point_add(P3, P3, tab_P2[b], E, stack);
                }
            }
            else
            {
                a = binary_to_indice_new(n1_str, i+i1, s+i1);
                b = binary_to_indice_new(n2_str, i+i2, s+i2);
                if(a == 0)
                {
                    ec_point_copy(P3, tab_P2[b], E->k);
                }
                else if (b == 0)
                {
                    ec_point_copy(P3, tab_P1[a], E->k);
                }
                else
                {
                    ec_point_copy(P3, tab_P1[a], E->k);
                    ec_point_add(P3, P3, tab_P2[b], E, stack);
                }
                init = true;
            }
            i=s+1;
        }
    }
}

void
ec_point_mul(ec_point_ptr P3, number_srcptr n, ec_point_srcptr P1, 
        ec_curve_srcptr E, uint8_t stack)
{
    switch(E->type)
    {
        case JACOBI_QUARTIC :
            ec_point_mul_sliding_window(P3, n, P1, E, stack);
            break;

        case EDWARDS :
            ec_point_mul_sliding_window(P3, n, P1, E, stack);
            break;

        case WEIERSTRASS :
            switch(E->f)
            {
                case PROJECTIVE:
                    ec_point_mul_sliding_window(P3, n, P1, E, stack);
                    break;
                case JACOBIAN:
                    ec_point_mul_sliding_window(P3, n, P1, E, stack);
                    break;
                default :
                    mphell_error("ec_point_mul, the only available coordinates are PROJECTIVE and JACOBIAN\n");
                    break;
            }
            break;
            
        default :
            mphell_error("ec_point_mul, the only available curves are WEIERSTRASS, JACOBI_QUARTIC and EDWARDS\n");
            break;
    }
}

/***********************************CONVERSION JACOBI WEIERSTRASS********************************/

void
jacobi_quartic_point_to_weierstrass_point (ec_point_ptr dst, ec_point_srcptr P1, 
        ec_curve_srcptr E, uint8_t stack)
{
    /* E: y^2 = z^2 + 2ax^2 + t^2; x^2 = zt; a != 1 */
    /* dst = (X3,X3,Z3,T3)                          */
    /* P = (X1,Y1,Z1,T1)                            */
    
    field_t *k = E->k;
    
    ec_point P;
    ec_point_get_pool_elt(P, E->k, stack);
    ec_point_copy(P, P1, k);
    ec_point_norm(P, E, stack);
    
    if(jacobi_quartic_point_is_neutral(P,E))
    {
        weierstrass_point_set_neutral(dst, E, stack);
    }
    else 
    {
        field_elt t2, t1;
        field_elt_get_pool_elt(&t2, k, stack);
        field_elt_get_pool_elt(&t1, k, stack);
        
        field_elt_set_one(t1, k);                               /* t1 = 1                                   */
        field_elt_neg(t1, t1, k);                               /* t1 = -1                                  */
            
        if (field_elt_iszero(P->x, k) && field_elt_cmp(P->y, t1, k)==0 && field_elt_isone(P->z, k))
        {
            /* Point of order 2 */
            field_elt_set_ui(t1, 4, false, k, stack);           /* t1 = 4                                   */
            field_elt_set_ui(t2, 3, false, k, stack);           /* t2 = 3                                   */
            field_elt_div(t1, t1, t2, k, stack);                /* t1 = 4 / 3                               */
            field_elt_neg(t1, t1, k);                           /* t1 = -(4 / 3)                            */
            field_elt_mul(dst->x, t1, E->a, k, stack);          /* X3 = -(4 / 3).a                          */
            field_elt_set_ui(dst->y, 0, false, k, stack);       /* Y3 = 0                                   */
        }
        else
        {
            field_elt_sqr(t1, P->z, k, stack);                  /* t1 = Z1^2                                */
            field_elt_add(dst->x, P->y, t1, k);                 /* X3 = Y1 + Z1^2                           */
            field_elt_copy(dst->y, dst->x, k);                  /* Y3 = Y1 + Z1^2                           */
            field_elt_add(dst->x, dst->x, dst->x, k);           /* X3 = 2.(Y1 + Z1^2)                       */
            field_elt_sqr(t1, P->x, k, stack);                  /* t1 = X1^2                                */
            field_elt_div(dst->x, dst->x, t1, k, stack);        /* X3 = 2.(Y1 + Z1^2)/(X1^2)                */
            field_elt_add(t1, E->a, E->a, k);                   /* t1 = 2.a                                 */
            field_elt_set_ui(t2, 3, false, k, stack);           /* t2 = 3                                   */
            field_elt_div(t1, t1, t2, k, stack);                /* t1 = 2.a/3                               */
            field_elt_add(dst->x, dst->x, t1, k);               /* X3 = 2.(Y1+Z1^2)/(X1^2) + 2.a/3          */

            field_elt_add(dst->y, dst->y, dst->y, k);           /* Y3 = 2.(Y1 + Z1^2)                       */
            field_elt_add(dst->y, dst->y, dst->y, k);           /* Y3 = 4.(Y1 + Z1^2)                       */
            field_elt_add(t1, E->a, E->a, k);                   /* t1 = 2.a                                 */
            field_elt_add(t1, t1, t1, k);                       /* t1 = 4.a                                 */
            field_elt_sqr(t2, P->x, k, stack);                  /* t2 = X1^2                                */
            field_elt_mul(t1, t1, t2, k, stack);                /* t1 = 4.a.X1^2                            */
            field_elt_mul(t2, t2, P->x, k, stack);              /* t2 = X1^3                                */
            field_elt_add(dst->y, dst->y, t1, k);               /* Y3 = 4.(Y1 + Z1^2) + 4.a.X1^2            */
            field_elt_div(dst->y, dst->y, t2, k, stack);        /* Y3 = (4.(Y1 + Z1^2) + 4.a.X1^2)/X1^3     */
            field_elt_mul(dst->y, dst->y, P->z, k, stack);      /* Y3 = Z1.(4.(Y1 + Z1^2) + 4.a.X1^2)/X1^3  */
        }
        
        field_elt_set_one(dst->z, k);                           /* Z3 = 1                                   */
        field_elt_set_one(dst->t, k);                           /* T3 = 1                                   */
        
        ec_point_relax_pool_elt(P, k, stack);
        field_elt_relax_pool_elt(&t2, k, stack);
        field_elt_relax_pool_elt(&t1, k, stack);
    }
}

void
jacobi_quartic_to_weierstrass (ec_curve_ptr E_res, ec_curve_srcptr E, uint8_t stack)
{
    field_t *k = E->k;
    field_elt t, t1, t2;
    field_elt_get_pool_elt(&t, k, stack);
    field_elt_get_pool_elt(&t1, k, stack);
    field_elt_get_pool_elt(&t2, k, stack);

    field_elt_sqr(t2, E->a, k, stack);                          /* t2 = a^2                                     */
    field_elt_set_ui(t1, 3, false, k, stack);                   /* t1 = 3                                       */
    field_elt_div(t, t2, t1, k, stack);                         /* t = a^2 / 3                                  */
    field_elt_set_one(t1, k);                                   /* t1 = 1                                       */
    field_elt_add(t, t, t1, k);                                 /* t = (a^2)/3 + 1                              */
    field_elt_set_ui(t1, 4, false, k, stack);                   /* t1 = 4                                       */
    field_elt_neg(t1, t1, k);                                   /* t1 = -4                                      */
    field_elt_mul(E_res->a, t, t1, k, stack);                   /* Aw = -4((a^2)/3 + 1)                         */
                                                                                                                
    field_elt_set_ui(t, 64, false, k, stack);                   /* t = 64                                       */
    field_elt_set_ui(t1, 27, false, k, stack);                  /* t1 = 27                                      */
    field_elt_div(t, t, t1, k, stack);                          /* t = 64 / 27                                  */
    field_elt_mul(t2, t2, E->a, k, stack);                      /* t2 = a^3                                     */
    field_elt_mul(E_res->b, t2, t, k, stack);                   /* Bw = (64/27).a^3                             */
    field_elt_set_ui(t, 4, false, k, stack);                    /* t = 4                                        */
    field_elt_set_ui(t1, 3, false, k, stack);                   /* t1 = 3                                       */
    field_elt_div(t, t, t1, k, stack);                          /* t = 4 / 3                                    */
    field_elt_mul(t, t, E->a, k, stack);                        /* t = (4/3)a                                   */
    field_elt_mul(t, t, E_res->a, k, stack);                    /* t = (4/3)a.Aw                                */
    field_elt_add(E_res->b, E_res->b, t, k);                    /* Bw = (64/27).a^3 + (4/3)a.Aw                 */
    
    number_copy(E_res->h, E->h);    
    number_copy(E_res->n, E->n);
    E_res->k = (field_t*)(E->k);                                /* The same field is used, do not free 2 times  */
    
    jacobi_quartic_point_to_weierstrass_point(E_res->G, E->G, E, stack);
    
    E_res->type = WEIERSTRASS;
    E_res->f = PROJECTIVE;
    ec_compute_disc(E_res, stack);
    ec_test_spec(E_res, stack);
    
    if(!weierstrass_belongs(E_res->G, E_res, stack))
    {
        mphell_error("ERROR: jacobi_quartic_to_weierstrass\n");
        mphell_error("G do not belong to E_res\n");
    }

    if(E->id_curve != NULL)
    {
        strcpy(E_res->id_curve, E->id_curve);
    }
    else
    {
        strcpy(E_res->id_curve, "no_name");
    }
    
    field_elt_relax_pool_elt(&t, k, stack);
    field_elt_relax_pool_elt(&t1, k, stack);
    field_elt_relax_pool_elt(&t2, k, stack);
}

void
weierstrass_point_to_jacobi_quartic_point (ec_point_ptr dst, ec_point_srcptr P1, ec_curve_srcptr E, uint8_t stack)
{
    /* E: y^2 = z^2 + 2ax^2 + t^2; x^2 = zt; a != 1 */
    /* dst = (X3,X3,T3,Z3)                          */
    /* P = (X1,Y1,Z1,T1)                            */
    
    field_t *k = E->k;
    
    ec_point P;
    ec_point_get_pool_elt(P, k, stack);
    ec_point_copy(P, P1, k);
    ec_point_norm(P, E, stack);
    
    if(weierstrass_point_is_neutral(P, E, stack))
    {
        jacobi_quartic_point_set_neutral(dst, E, stack);
    }
    else
    {
        field_elt t1, t2;
        field_elt_get_pool_elt(&t1, k, stack);
        field_elt_get_pool_elt(&t2, k, stack);

        field_elt_set_ui(t1, 3, false, k, stack);               /* t1 = 3                                           */
        field_elt_div(t1, E->a, t1, k, stack);                  /* t1 = a / 3                                       */
        field_elt_set_ui(t2, 4, false, k, stack);               /* t2 = 4                                           */
        field_elt_mul(t1, t1, t2, k, stack);                    /* t1 = (4/3).a                                     */
        
        field_elt_neg(t2, t1, k);                               /* t2 = -(4/3).a                                    */
        
        if(field_elt_cmp(t2, P->x, k)==0 && field_elt_iszero(P->y, k))
        {
            /* dst = (0, -1, 1, 0) */
            field_elt_set_ui(dst->x, 0, false, k, stack);
            field_elt_set_one(dst->y, k);
            field_elt_neg(dst->y, dst->y, k);
            field_elt_set_one(dst->z, k);
            field_elt_set_ui(dst->t, 0, false, k, stack);
            
        }
        else
        {
            /* Same formula but we multiply each X3,Y3,Z3 coordinates by Y1 */
            field_elt_add(t2, t1, t1, k);                       /* t2 = (8/3).a                                     */
            field_elt_add(dst->x, P->x, P->x, k);               /* X3 = 2.X1                                        */
            field_elt_copy(dst->y, dst->x, k);                  /* Y3 = 2.X1                                        */
            field_elt_add(dst->x, dst->x, t2, k);               /* X3 = 2.X1 + (8/3).a                              */
            field_elt_mul(dst->x, dst->x, P->y, k, stack);      /* X3 = (2.X1 + (8/3).a).Y1                         */

            field_elt_sub(dst->y, dst->y, t1, k);               /* Y3 = 2.X1 - (4/3).a                              */
            field_elt_add(t2, P->x, t1, k);                     /* t2 = X1 + (4/3).a                                */
            field_elt_sqr(t2, t2, k, stack);                    /* t2 = (X1 + (4/3).a)^2                            */
            field_elt_mul(dst->y, dst->y, t2, k, stack);        /* Y3 = (2.X1 - (4/3).a).(X1 + (4/3).a)^2           */
            field_elt_sqr(t2, P->y, k, stack);                  /* t2 = Y1^2                                        */
            field_elt_sub(dst->y, dst->y, t2, k);               /* Y3 = (2.X1 - (4/3).a).(X1 + (4/3).a)^2 - Y1^2    */
            
            field_elt_copy(dst->z, t2, k);                      /* Z3 = Y1^2                                        */
            field_elt_sqr(dst->t, dst->x, k, stack);            /* T3 = X3^2                                        */
            field_elt_div(dst->t, dst->t, dst->z, k, stack);    /* T3 = (X3^2)/Z3                                   */
            
            ec_point P_test;
            ec_point_get_pool_elt(P_test, E->k, stack);
            jacobi_quartic_point_to_weierstrass_point(P_test, dst, E, stack);
            ec_point_norm(P_test, E, stack);
            
            if(!ec_point_are_equal(P_test, P, E, stack))
            {
                field_elt_neg(dst->y, dst->y, k);               /* Y3 = -(X3^4 + 2.a.X3^2 + 1)^(1/2)                */
            }
            ec_point_relax_pool_elt(P_test, k, stack);
        }

        field_elt_relax_pool_elt(&t1, k, stack);
        field_elt_relax_pool_elt(&t2, k, stack);
        ec_point_relax_pool_elt(P, k, stack);
    }
}

void
weierstrass_to_jacobi_quartic (ec_curve_ptr E_res, ec_curve_srcptr E, const bool determined, fe_ptr teta, uint8_t stack)
{
    /* E: y^2 = y^2 = x^3 + a.x + b                  */
    /* E_res: y^2 = bj.x^4 + 2aj.x^2 + 1             */
    
    field_t *k = E->k;
    field_elt t1, t2;
    field_elt_get_pool_elt(&t1, k, stack);
    field_elt_get_pool_elt(&t2, k, stack); 

    if(!determined)  
    {
        /* We compute the (3 possible candidates for) teta */
        ec_point P1,P2,P3;
        ec_point_get_pool_elt(P1, E->k, stack);
        ec_point_get_pool_elt(P2, E->k, stack);
        ec_point_get_pool_elt(P3, E->k, stack);
        weierstrass_points_of_order_2(P1, P2, P3, E, stack);
        
        /* And we test if we took the good one */
        if(!weierstrass_point_are_equal(P1, P2, E, stack))
        {
            do
            {
                field_elt_copy(teta,P1->x, k);
                field_elt_sqr(t1, teta, k, stack);                  /* t1 = teta^2                      */
                field_elt_set_ui(t2, 3, false, k, stack);           /* t2 = 3                           */
                field_elt_mul(t1, t1, t2, k, stack);                /* t1 = 3.teta^2                    */
                field_elt_set_ui(t2, 4, false, k, stack);           /* t2 = 4                           */
                field_elt_mul(t2, t2, E->a, k, stack);              /* t2 = 4.a                         */
                field_elt_add(t1, t1, t2, k);                       /* t1 = 3.teta^2 + 4.a              */
                field_elt_set_ui(t2, 16, false, k, stack);          /* t2 = 16                          */
                field_elt_neg(t2, t2, k);                           /* t2 = -16                         */
                field_elt_div(E_res->b, t1, t2, k, stack);          /* bj = -(3.teta^2 + 4.a)/16        */
                ec_point_copy(P1,P2, k);
                ec_point_copy(P2,P3, k);
            }while(!field_elt_isone(E_res->b, k));
            /* At this point theta has been computed and checked. */
        }
        else
        {
            field_elt_copy(teta,P1->x, k);
            field_elt_sqr(t1, teta, k, stack);                      /* t1 = teta^2                      */
            field_elt_set_ui(t2, 3, false, k, stack);               /* t2 = 3                           */
            field_elt_mul(t1, t1, t2, k, stack);                    /* t1 = 3.teta^2                    */
            field_elt_set_ui(t2, 4, false, k, stack);               /* t2 = 4                           */
            field_elt_mul(t2, t2, E->a, k, stack);                  /* t2 = 4.a                         */
            field_elt_add(t1, t1, t2, k);                           /* t1 = 3.teta^2 + 4.a              */
            field_elt_set_ui(t2, 16, false, k, stack);              /* t2 = 16                          */
            field_elt_neg(t2, t2, k);                               /* t2 = -16                         */
            field_elt_div(E_res->b, t1, t2, k, stack);              /* bj = -(3.teta^2 + 4.a)/16        */
        }

        ec_point_relax_pool_elt(P1, k, stack);
        ec_point_relax_pool_elt(P2, k, stack);
        ec_point_relax_pool_elt(P3, k, stack);
    }
    else
    {
        field_elt_sqr(t1, teta, k, stack);                          /* t1 = teta^2                      */
        field_elt_set_ui(t2, 3, false, k, stack);                   /* t2 = 3                           */
        field_elt_mul(t1, t1, t2, k, stack);                        /* t1 = 3.teta^2                    */
        field_elt_set_ui(t2, 4, false, k, stack);                   /* t2 = 4                           */
        field_elt_mul(t2, t2, E->a, k, stack);                      /* t2 = 4.a                         */
        field_elt_add(t1, t1, t2, k);                               /* t1 = 3.teta^2 + 4.a              */
        field_elt_set_ui(t2, 16, false, k, stack);                  /* t2 = 16                          */
        field_elt_neg(t2, t2, k);                                   /* t2 = -16                         */
        field_elt_div(E_res->b, t1, t2, k, stack);                  /* bj = -(3.teta^2 + 4.a)/16        */
        /* We could eventually add an MPHELL ASSERT to check that b_j is indeed equals to one   */
    }
    field_elt_set_ui(t1, 3, false, k, stack);                       /* t1 = 3                           */
    field_elt_neg(t1, t1, k);                                       /* t1 = -3                          */
    field_elt_mul(t1,t1,teta, k, stack);                            /* t1 = -3.teta                     */
    field_elt_set_ui(t2, 4, false, k, stack);                       /* t2 = 4                           */
    field_elt_div(E_res->a, t1, t2, k, stack);                      /* aj = -(3.teta)/4                 */
    
    E_res->k = (field_t*)(E->k);                                    /* The same field is used, do not free 2 times */
    E_res->type = JACOBI_QUARTIC;
    E_res->f = PROJECTIVE;
    ec_compute_disc(E_res, stack);
    ec_test_spec(E_res, stack);
    
    number_copy(E_res->h, E->h);
    number_copy(E_res->n, E->n);
    
    weierstrass_point_to_jacobi_quartic_point(E_res->G, E->G, E_res, stack);
    
    if(!jacobi_quartic_belongs(E_res->G, E_res, stack))
    {
        mphell_error("ERROR: weierstrass_to_jacobi_quartic\n");
        mphell_error("G do not belong to E_res\n");
    }

    if(E->id_curve != NULL)
    {
        strcpy(E_res->id_curve, E->id_curve);
    }
    else
    {
        strcpy(E_res->id_curve, "no_name");
    }

    field_elt_relax_pool_elt(&t1, k, stack);
    field_elt_relax_pool_elt(&t2, k, stack);
}

/***********************************CONVERSION EDWARDS WEIERSTRASS********************************/

/*  The main reference for this part are [BBJ+08,Theorem3.2] for the conversion
 *  between Montgomery and Twisted Edwards Curves and then the [CS17,§2.4] 
 *  for the conversion between short Weierstrass and Montgomery. There is also
 *  in [CS17,§2.5] formulas for conversion between Montgomery and Twisted 
 *  Edwards curves
 *
 */
void
edwards_to_weierstrass (ec_curve_ptr E_res, ec_curve_srcptr E, uint8_t stack)
{
    ec_point P;
    field_elt c,d,e,f; 
    field_t *k = E->k;

    E_res->k = (field_t*)(E->k);
    E_res->type = WEIERSTRASS;
    number_copy(E_res->h, E->h);    
    number_copy(E_res->n, E->n);
    E_res->f=PROJECTIVE;  

    field_elt_get_pool_elt(&c, k, stack);
    field_elt_get_pool_elt(&d, k, stack);
    field_elt_get_pool_elt(&e, k, stack);
    field_elt_get_pool_elt(&f, k, stack);
    ec_point_get_pool_elt(P, k, stack);
    ec_point_copy(P,E->G, k);

    /* First we normalise the point P */
    if(!field_elt_isone(P->z, k))
    {
        edwards_point_norm(P,E, stack);
    }
    
    /* First step Edwards to Montgomery 
     * (here a d should be understood as coefficients of the  
     * twisted Edwards curve ) 
     */
    field_elt_sub(c,E->a,E->b, k);                  /* c = a -d                                     */
    field_elt_inv(c,c, k, stack);                   /* c = 1 / (a - d)                              */
    field_elt_add(c,c,c, k);                        /* c = 2 / (a - d)                              */
    field_elt_add(d,c,c, k);                        /* B = 4 / (a - d) (stored in d)                */
    field_elt_add(e,E->a,E->b, k);                  /* e = a + d                                    */
    field_elt_mul(e,e,c, k, stack);                 /* A = 2 ( a + d ) / ( a - d ) (stored in e)    */
    /* We have computed the coefficients of the birationnally equivalent 
     * Montgomery curve we have also to compute the image of E->G 
     */
    field_elt_set_one(c,k);
    field_elt_sub(c,c,P->y, k);
    field_elt_inv(c,c, k, stack);                   /* c = 1 /(1-v)                                 */
    field_elt_inc(f,P->y, k);
    field_elt_mul(f,c,f, k, stack);                 /* f = (1 + v)/(1 -v)                           */
    field_elt_div(P->y,f,P->x, k, stack);           /* y = (1 + v)/((1 -v) * u)                     */
    field_elt_copy(P->x,f, k);                      /* x = (1 + v)/(1 -v)                           */
        

    /* Second step Montgomery to short Weierstras
     * At this step we have a Montgomery curve with coefficients (e,d) 
     * instead of the usual notation A,B 
     */
    field_elt_set_ui(f, 3, false, k, stack);
    field_elt_div(c,e,f, k, stack);                 /* c = A / 3                                    */
    field_elt_add(P->x, P->x, c, k);
    field_elt_div(P->x, P->x, d, k, stack);         /* u = ( x + A / 3) / B                         */
    field_elt_copy(E_res->b,d, k);
    field_elt_div(P->y, P->y,d, k, stack);          /* v = y / B                                    */
    field_elt_copy(E_res->a,e, k); 
    field_elt_set_one(P->z,k);
    field_elt_set_one(P->t,k);
    ec_point_copy(E_res->G,P, k);
    
    /* After the computation of the image of E->G we compute the short
    *  Weierstrass coefficients we have a=1/B**2(1-A^2/3) b = (A/(3*B**3))*(2 * A**2/9 -1)
    */
    field_elt_sqr(e,E_res->a, k, stack);            /* e = A^2                                      */
    field_elt_div(c,e,f, k, stack);                 /* c = A^2 /3                                   */
    field_elt_set_one(e,k);    
    field_elt_sub(e,e,c, k);                        /* e = 1 - A^2 /3                               */
    field_elt_inv(d,E_res->b, k, stack);            /* d = 1 / B                                    */
    field_elt_sqr(c,d, k, stack);                   /* c = 1 / B^2                                  */
    field_elt_mul(e,c,e, k, stack);                 /* e = ( 1 - A^2 /3 ) / B^2                     */
    field_elt_div(f,E_res->a,f, k, stack);          /* f = A / 3                                    */
    field_elt_sqr(c,f, k, stack);                   /* c = A^2 /9                                   */
    field_elt_add(c,c,c, k);                        /* c = 2 A^2 /9                                 */
    field_elt_dec(c,c, k);                          /* c = 2 A^2 /9 -1 */
    field_elt_mul(c,c,f, k, stack);                 /* c = ( A / 3 ) (2 A^2 /9 - 1)                 */
    field_elt_sqr(f,d, k, stack);                   /* f = 1 / B^2                                  */
    field_elt_mul(c,c,f, k, stack);                 /* c = ( A / 3 ) (2 A^2 /9 - 1) / B^2           */
    field_elt_mul(E_res->b,d,c, k, stack);          /* c = ( A / 3 ) (2 A^2 /9 - 1) / B^3           */
    field_elt_copy(E_res->a,e, k);
    
    if(E->id_curve != NULL)
    {  
        strcpy(E_res->id_curve, E->id_curve);
    }
    else
    {
        strcpy(E_res->id_curve, "no_name");
    }
    
    ec_compute_disc(E_res, stack);
    ec_test_spec(E_res, stack);
    
    /* We now free the memory  */
    field_elt_relax_pool_elt(&c, k, stack);
    field_elt_relax_pool_elt(&d, k, stack);
    field_elt_relax_pool_elt(&e, k, stack);
    field_elt_relax_pool_elt(&f, k, stack);
    ec_point_relax_pool_elt(P, k, stack);
}

void
edwards_point_to_weierstrass_point (ec_point_ptr dst, ec_point_srcptr P, 
        ec_curve_srcptr E, uint8_t stack)
{
    field_elt c,d,e,f; 
    field_t *k = E->k;
   
    field_elt_get_pool_elt(&c, k, stack);
    field_elt_get_pool_elt(&d, k, stack);
    field_elt_get_pool_elt(&e, k, stack);
    field_elt_get_pool_elt(&f, k, stack);
    ec_point_copy(dst,P, k);

    /* First we normalise the point dst */
    if(!field_elt_isone(dst->z, k))
    {
        edwards_point_norm(dst,E, stack);
    }
    
    /* First step Edwards to Montgomery 
     * (here a d should be understood as coefficients of the  
     * twisted Edwards curve ) 
     */
    field_elt_sub(c,E->a,E->b, k);                  /* c = a -d                                     */
    field_elt_inv(c, c, k, stack);                  /* c = 1 / (a - d)                              */
    field_elt_add(c,c,c, k);                        /* c = 2 / (a - d)                              */
    field_elt_add(d,c,c, k);                        /* B = 4 / (a - d) (stored in d)                */ 
    field_elt_add(e,E->a,E->b, k);                  /* e = a + d                                    */
    field_elt_mul(e,e,c, k, stack);                 /* A = 2 ( a + d ) / ( a - d ) (stored in e)    */
    
    /* We have computed the coefficients of the birationnally equivalent 
    *  Montgomery curve we have also to compute the image of dst
    *  Those Montgomery coefficients were necessary for the next step. 
    */
    
    field_elt_set_one(c,k);
    field_elt_sub(c,c,dst->y, k);
    field_elt_inv(c,c, k, stack);                   /* c = 1 /(1-v)                                 */
    field_elt_inc(f,dst->y, k);
    field_elt_mul(f,c,f, k, stack);                 /* f = (1 + v)/(1 -v)                           */
    field_elt_div(dst->y,f,dst->x, k, stack);       /* y = (1 + v)/((1 -v) u )                      */
    field_elt_copy(dst->x,f, k);                    /* x = (1 + v)/(1 -v)                           */
        
    /* Second step Montgomery to short Weierstras */
    /* At this step we have a Montgomery curve with coefficients (e,d) 
    *  instead of the usual notation A,B 
    */
    field_elt_set_ui(f, 3, false, k, stack);
    field_elt_div(f,e,f, k, stack);                 /* f = A / 3                                    */
    field_elt_add(dst->x, dst->x, f, k);
    field_elt_div(dst->x, dst->x, d, k, stack);     /* u = ( x + A / 3) /B                          */
    field_elt_div(dst->y,dst->y,d, k, stack);       /* v = y /B                                     */
    field_elt_set_one(dst->t,k);
    
    /* We now free the memory  */
    field_elt_relax_pool_elt(&c, k, stack);
    field_elt_relax_pool_elt(&d, k, stack);
    field_elt_relax_pool_elt(&e, k, stack);
    field_elt_relax_pool_elt(&f, k, stack);
}

void
weierstrass_to_edwards (ec_curve_ptr E_res, ec_curve_srcptr E, const uint8_t n, fe_ptr alpha, fe_ptr beta, uint8_t stack)
{
    ec_point S,P,Q,R;
    field_elt c,d,e,f,a; 
    field_t *k = E->k;

    E_res->k = (field_t*)(E->k);
    E_res->type = EDWARDS;
    number_copy(E_res->h, E->h);
    number_copy(E_res->n, E->n);
    E_res->f=EXTENDED_EDWARDS; 
    field_elt_get_pool_elt(&a, k, stack);
    field_elt_get_pool_elt(&c, k, stack);
    field_elt_get_pool_elt(&d, k, stack);
    field_elt_get_pool_elt(&e, k, stack);
    field_elt_get_pool_elt(&f, k, stack);
    ec_point_get_pool_elt(S, k, stack);
    ec_point_get_pool_elt(P, k, stack);
    ec_point_get_pool_elt(Q, k, stack);
    ec_point_get_pool_elt(R, k, stack);
    ec_point_copy(P,E->G, k);

    field_elt_set_one(a,k);
    field_elt_dec(a,a, k);
    field_elt_copy(E_res->D,a, k);

    /* First we normalise the point P */
    if(!field_elt_isone(P->z, k))
    {
        edwards_point_norm(P, E, stack);
    }

    /* First step short Weierstrass to  Montgomery */
    field_elt_set_ui(f, 3, false, k, stack);
    /* First we compute the abscissa of a point of order 2 of the Curve */
    switch(n)
    {
        case 1:
            /* We suppose here that the user gives us as input alpha aka the abscissa of a point of order 2 which is good for a mapping */
            field_elt_copy(c, alpha, k);
            field_elt_sqr(a, c, k, stack);
            field_elt_mul(d, a, f, k, stack);                      /* d = 3 alpha^2                                */
            field_elt_add(e, d, E->a, k);                          /* d = 3 alpha^2 + a                            */
            MPHELL_ASSERT( field_elt_issquare(e, k, stack), "This curve do not seem to be isomorphic to an edwards curve \n We do not have 3*alpha^2 + a = beta^2 \n");
            field_elt_sqrt(e, e, k, stack);                        /* beta = (3 alpha^2 + a)^(1/2)  (stored in e)  */
            field_elt_copy(beta, e, k); 
            break;

        case 2:
            /* We suppose here that the user gives us as input alpha (aka the abscissa of a point of order 2) and beta ((3*alpha^2+a)^(1/2)) which are good for a mapping */
            field_elt_copy(c, alpha, k);
            field_elt_copy(e, beta, k);
            break;

        default:
            /* No data specified for the correspondance we want to use */
            /* First we compute the abscissa of a point of order 2 of the Curve */
            weierstrass_points_of_order_2(Q, R, S, E, stack);
            /* If we are here this means that Q is a point of order 2 */
            do
            {
                field_elt_copy(c, Q->x, k);                        /* alpha = Qx (stored in c)                     */
                field_elt_copy(alpha, c, k);
                field_elt_sqr(a, c, k, stack);
                field_elt_mul(d, a, f, k, stack);                  /* d = 3 alpha^2                                */
                field_elt_add(e, d, E->a, k);                      /* d = 3 alpha^2 + a                            */
                ec_point_copy(Q, R, k);
                ec_point_copy(R, S, k);
            }while(!field_elt_issquare(e, k, stack));
            MPHELL_ASSERT( field_elt_issquare(e, k, stack), "This curve do not seem to be isomorphic to an edwards curve \n We do not have 3*alpha^2 + a = beta^2 \n");
            field_elt_sqrt(e, e, k, stack);                        /* beta = (3 alpha^2 + a)^(1/2)  (stored in e)  */
            field_elt_copy(beta, e, k);                
            break;
    }
    
    field_elt_mul(d, f, c, k, stack);                              /* d = 3 alpha                                  */
    field_elt_inv(E_res->b, e, k, stack);                          /* B = 1 / beta                                 */
    field_elt_mul(E_res->a, d, E_res->b, k, stack);                /* A = 3 alpha / beta                           */
    field_elt_sub(d, P->x, c, k);
    field_elt_mul(P->x, d, E_res->b, k, stack);                    /* x = (u - alpha) / beta                       */
    field_elt_mul(P->y, P->y, E_res->b, k, stack);                 /* y = v / beta                                 */

    /* Second step Montgomery to Edwards */
    /* We compute the coefficients of the twisted edwards curve */
    field_elt_dec(a, f, k);
    field_elt_sub(c, E_res->a, a, k);                              /* c = A - 2                                    */
    field_elt_add(e, E_res->a, a, k);                              /* e = A + 2                                    */
    field_elt_inv(d, E_res->b, k, stack);                          /* d = 1 / B                                    */
    field_elt_mul(E_res->a, e, d, k, stack);                       /* a = (A + 2) / B                              */
    field_elt_mul(E_res->b, c, d, k, stack);                       /* d = (A - 2) / B                              */

    /* We compute the mapping of the point P */
    field_elt_dec(d, P->x, k);
    field_elt_inc(e, P->x, k);
    field_elt_div(P->x, P->x, P->y, k, stack);                     /* u = x / y                                    */
    field_elt_div(P->y, d, e, k, stack);                           /* v = (x - 1)/(x + 1)                          */
    edwards_point_set_aff(E_res->G, P->x, P->y, k, stack);
    if(E->id_curve != NULL)
    {  
        strcpy(E_res->id_curve, E->id_curve);
    }
    else
    {
        strcpy(E_res->id_curve, "no_name");
    }
    
    ec_compute_disc(E_res, stack);
    ec_test_spec(E_res, stack);

    /*We now free the memory  */
    field_elt_relax_pool_elt(&a, k, stack);    
    field_elt_relax_pool_elt(&c, k, stack);
    field_elt_relax_pool_elt(&d, k, stack);
    field_elt_relax_pool_elt(&e, k, stack);
    field_elt_relax_pool_elt(&f, k, stack);
    ec_point_relax_pool_elt(S, k, stack);
    ec_point_relax_pool_elt(P, k, stack);
    ec_point_relax_pool_elt(Q, k, stack);
    ec_point_relax_pool_elt(R, k, stack);
}

void
weierstrass_point_to_edwards_point (ec_point_ptr dst, ec_point_srcptr P, 
        ec_curve_srcptr E, const uint8_t n, fe_ptr alpha, fe_ptr beta, uint8_t stack)
{
    ec_point Q;
    field_elt c,d,e,f,a; 
    field_t *k = E->k;

    /* Allocation */ 
    field_elt_get_pool_elt(&a, k, stack);    
    field_elt_get_pool_elt(&c, k, stack);
    field_elt_get_pool_elt(&d, k, stack);
    field_elt_get_pool_elt(&e, k, stack);
    field_elt_get_pool_elt(&f, k, stack);
    ec_point_get_pool_elt(Q, k, stack);
    
    ec_point_copy(dst,P, k);

    /* First we normalise the point dst */
    if(!field_elt_isone(dst->z, k))
    {
        edwards_point_norm(dst,E, stack);
    }
    field_elt_set_ui(f, 3, false, k, stack);
    /* First step short Weierstrass to  Montgomery */
    switch(n)
    {
        case 1:
            /* We suppose here that the user gives us as input alpha aka the abscissa of a point of order 2 */
            field_elt_copy(c,alpha, k);
            
            field_elt_sqr(a, c, k, stack);
            field_elt_mul(d, a, f, k, stack);                    /* d = 3 alpha^2                                */
            field_elt_add(e, d, E->a, k);                        /* d = 3 alpha^2 + a                            */
            MPHELL_ASSERT( field_elt_issquare(e, k, stack), "This curve do not seem to be isomorphic to an edwards curve \n We do not have 3*alpha^2 + a = beta^2 \n");
            field_elt_sqrt(e, e, k, stack);                      /* beta = (3 alpha^2 + a)^(1/2)  (stored in e)  */
            field_elt_copy(beta, e, k); 
            break;

        case 2:
            /* We suppose here that the user gives us as input alpha (aka the abscissa of a point of order 2) and beta ((3*alpha^2+a)^(1/2)) */
            field_elt_copy(c, alpha, k);
            field_elt_copy(e, beta, k);
            break;

        default:
            /* No data specified for the correspondance we want to use */
            /* First we compute the abscissa of a point of order 2 of the Curve */
            weierstrass_point_of_order_2(Q, E, stack);
            /* If we are here this means that Q is a point of order 2 */
            field_elt_copy(c, Q->x, k);                         /* alpha = Qx (stored in c)                     */
            field_elt_copy(alpha, c, k);                
            field_elt_sqr(a, c, k, stack);
            field_elt_mul(d, a, f, k, stack);                    /* d = 3 alpha^2                                */
            field_elt_add(e, d, E->a, k);                        /* d = 3 alpha^2 + a                            */
            MPHELL_ASSERT( field_elt_issquare(e, k, stack), "This curve do not seem to be isomorphic to an edwards curve \n We do not have 3*alpha^2 + a = beta^2 \n");
            field_elt_sqrt(e, e, k, stack);                     /* beta = (3 alpha^2 + a)^(1/2)  (stored in e)  */
            field_elt_copy(beta, e, k);
            break;
    }
    field_elt_inv(a, e, k, stack);                              /* a = 1 / beta                                 */
    field_elt_sub(d, dst->x, c, k);
    field_elt_mul(dst->x, d, a, k, stack);                       /* x = (u - alpha) / beta                       */
    field_elt_mul(dst->y, dst->y, a, k, stack);                  /* y = v / beta                                 */
    
    /* Second step Montgomery to Edwards */
    /* We compute the mapping of the point P */
    field_elt_dec(d,dst->x, k);
    field_elt_inc(e,dst->x, k);
    field_elt_div(dst->x,dst->x,dst->y, k, stack);             /* u = x / y                                    */
    field_elt_div(dst->y,d,e, k, stack);                       /* v = (x - 1)/(x + 1)                          */
    field_elt_mul(dst->t,dst->x,dst->y, k, stack);

        
    /* We now free the memory  */
    field_elt_relax_pool_elt(&a, k, stack);    
    field_elt_relax_pool_elt(&c, k, stack);
    field_elt_relax_pool_elt(&d, k, stack);
    field_elt_relax_pool_elt(&e, k, stack);
    field_elt_relax_pool_elt(&f, k, stack);
    ec_point_relax_pool_elt(Q, k, stack);
}

