/* 
  MPHELL-4.0 
  Author(s): The MPHELL team

 (C) Copyright 2015-2018 - Institut Fourier / Univ. Grenoble Alpes (France) 

 This file is part of the MPHELL Library.
 MPHELL is free software: you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation, version 3 of the License.
 
 MPHELL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with MPHELL.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
  * \file  mphell-errors.c
  * \brief Definition of error functions.
  */

#include "mphell-init.h"

void
mphell_assert_error (const char *filename, int line, const char *expr)
{
    if (filename != NULL && filename[0] != '\0')
    {
        fprintf (stderr, "%s:", filename);
        if (line != -1)
        {
            fprintf (stderr, "%d: ", line);
        }
    }
    fprintf (stderr, "MPHELL assertion failed: %s\n", expr);
    free_mphell();
    abort();
}

void
mphell_error (char *expr)
{
    if (MPHELL_ASSERT_FILE != NULL && MPHELL_ASSERT_FILE[0] != '\0')
    {
        fprintf (stderr, "%s:", MPHELL_ASSERT_FILE);
        if (MPHELL_ASSERT_LINE != -1)
        {
            fprintf (stderr, "%d: ", MPHELL_ASSERT_LINE);
        }
    }
    fprintf (stderr, "MPHELL error: %s\n", expr);
    free_mphell();
    abort();
}

void
mphell_error_free (char *expr)
{
    if (MPHELL_ASSERT_FILE != NULL && MPHELL_ASSERT_FILE[0] != '\0')
    {
        fprintf (stderr, "%s:", MPHELL_ASSERT_FILE);
        if (MPHELL_ASSERT_LINE != -1)
        {
            fprintf (stderr, "%d: ", MPHELL_ASSERT_LINE);
        }
    }
    fprintf (stderr, "MPHELL error free: %s\n", expr);
    free(expr);
    free_mphell();
    abort();
}
