/* 
  MPHELL-4.0 
  Author(s): The MPHELL team

 (C) Copyright 2015-2018 - Institut Fourier / Univ. Grenoble Alpes (France)

 This file is part of the MPHELL Library.
 MPHELL is free software: you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation, version 3 of the License.
 
 MPHELL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with MPHELL.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
  * \file  mphell_perf_field.c
  * \brief Benchmark for the field functions
  */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>
#include <string.h>

#include "mphell.h"

int main()
{
    init_mphell(128, RANDOM_AES128, DEVURANDOM);
    int i;
    int nb_iteration = 20000;
    
    struct timespec start, end;
    
    /* Allocate fields */
    
    /* 128 bits */
    
    field k2;
    field_alloc(k2, FP, bits_to_nblock(128), NULL);
    number p;
    number_init(&p, bits_to_nblock(128));
    number_set_str(p, "170141183460469231731687303715884105757", 10);
    field_create(k2, "", STACK_1, 1, p);
    number_free(&p);
    
    /* 192 bits */
    
    field k3;
    field_alloc(k3, FP, bits_to_nblock(192), NULL);
    number_init(&p, bits_to_nblock(192));
    number_set_str(p, "fffffffffffffffffffffffffffffffeffffffffffffffff", 16);
    field_create(k3, "", STACK_1, 1, p);
    number_free(&p);
    
    /* 256 bits */
    
    field k4;
    field_alloc(k4, FP, bits_to_nblock(256), NULL);
    number_init(&p, bits_to_nblock(256));
    number_set_str(p, "57896044618658097711785492504343953926634992332820282019728792003956564819949", 10);
    field_create(k4, "", STACK_1, 1, p);
    number_free(&p);
    
    /* 384 bits */
    
    field k6;
    field_alloc(k6, FP, bits_to_nblock(384), NULL);
    number_init(&p, bits_to_nblock(384));
    number_set_str(p, "fffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffeffffffff0000000000000000ffffffff", 16);
    field_create(k6, "", STACK_1, 1, p);
    number_free(&p);
    
    /* 512 bits */
    
    field k8;
    field_alloc(k8, FP, bits_to_nblock(512), NULL);
    number_init(&p, bits_to_nblock(512));
    number_set_str(p, "AADD9DB8DBE9C48B3FD4E6AE33C9FC07CB308DB3B3C9D20ED6639CCA703308717D4D9B009BC66842AECDA12AE6A380E62881FF2F2D82C68528AA6056583A48F3", 16);
    field_create(k8, "", STACK_1, 1, p);
    number_free(&p);
    
    field_elt x[nb_iteration];
    field_elt y[nb_iteration];
    field_elt dst, dst1, dst2, dst3, dst4, dst5;
    
    printf("Enter TEST:\n");
    
    /* 128 bits */
    
    for (i=0; i<nb_iteration; i++)
    {
        field_elt_alloc(&x[i], k2);
        field_elt_init(x[i], k2);
    }
    for (i=0; i<nb_iteration; i++)
    {
        field_elt_alloc(&y[i], k2);
        field_elt_init(y[i], k2);
    }
    field_elt_alloc(&dst, k2);
    field_elt_init(dst, k2);
    
    for(i=0; i<nb_iteration; i++)
    {
        /* Set element random */
        
        field_elt_random(x[i], k2, STACK_1);
        field_elt_random(y[i], k2, STACK_1);
    }
    
    /* Add */
    
    clock_gettime(CLOCK_MONOTONIC_RAW, &start);
    for (i=0; i<nb_iteration; i++)
    {
        field_elt_add(dst, x[i], y[i], k2);
    }
    clock_gettime(CLOCK_MONOTONIC_RAW, &end);
    printf("Add 128 bits | %lu s %04u ns \n", get_s(&start, &end), get_ns(&start, &end));
    
    /* Sub */
    
    clock_gettime(CLOCK_MONOTONIC_RAW, &start);
    for (i=0; i<nb_iteration; i++)
    {
        field_elt_sub(dst, x[i], y[i], k2);
    }
    clock_gettime(CLOCK_MONOTONIC_RAW, &end);
    printf("Sub 128 bits | %lu s %04u ns \n", get_s(&start, &end), get_ns(&start, &end));
    
    /* Mul */
    
    clock_gettime(CLOCK_MONOTONIC_RAW, &start);
    for (i=0; i<nb_iteration; i++)
    {
        field_elt_mul(dst, x[i], y[i], k2, STACK_1);
    }
    clock_gettime(CLOCK_MONOTONIC_RAW, &end);
    printf("Mul 128 bits | %lu s %04u ns \n", get_s(&start, &end), get_ns(&start, &end));
    
    ///* Inv */
    
    //clock_gettime(CLOCK_MONOTONIC_RAW, &start);
    //for (i=0; i<nb_iteration; i++)
    //{
        //field_elt_inv(dst, x[i], k2, STACK_1);
    //}
    //clock_gettime(CLOCK_MONOTONIC_RAW, &end);
    //printf("Inv 128 bits | %lu s %04u ns \n\n", get_s(&start, &end), get_ns(&start, &end));
    
    /* Free allocated field elts */
    
    for (i=0; i<nb_iteration; i++)
    {
        field_elt_free(&x[i], k2);
        field_elt_free(&y[i], k2);
    }
    field_elt_free(&dst, k2);
    
    /* 192 bits */
    
    for (i=0; i<nb_iteration; i++)
    {
        field_elt_alloc(&x[i], k3);
        field_elt_init(x[i], k3);
    }
    for (i=0; i<nb_iteration; i++)
    {
        field_elt_alloc(&y[i], k3);
        field_elt_init(y[i], k3);
    }
    field_elt_alloc(&dst, k3);
    field_elt_init(dst, k3);
    
    for(i=0; i<nb_iteration; i++)
    {
        /* Set element random */
        
        field_elt_random(x[i], k3, STACK_1);
        field_elt_random(y[i], k3, STACK_1);
    }
    
    /* Add */
    
    clock_gettime(CLOCK_MONOTONIC_RAW, &start);
    for (i=0; i<nb_iteration; i++)
    {
        field_elt_add(dst, x[i], y[i], k3);
    }
    clock_gettime(CLOCK_MONOTONIC_RAW, &end);
    printf("Add 192 bits | %lu s %04u ns \n", get_s(&start, &end), get_ns(&start, &end));
    
    /* Sub */
    
    clock_gettime(CLOCK_MONOTONIC_RAW, &start);
    for (i=0; i<nb_iteration; i++)
    {
        field_elt_sub(dst, x[i], y[i], k3);
    }
    clock_gettime(CLOCK_MONOTONIC_RAW, &end);
    printf("Sub 192 bits | %lu s %04u ns \n", get_s(&start, &end), get_ns(&start, &end));
    
    /* Mul */
    
    clock_gettime(CLOCK_MONOTONIC_RAW, &start);
    for (i=0; i<nb_iteration; i++)
    {
        field_elt_mul(dst, x[i], y[i], k3, STACK_1);
    }
    clock_gettime(CLOCK_MONOTONIC_RAW, &end);
    printf("Mul 192 bits | %lu s %04u ns \n", get_s(&start, &end), get_ns(&start, &end));
    
    ///* Inv */
    
    //clock_gettime(CLOCK_MONOTONIC_RAW, &start);
    //for (i=0; i<nb_iteration; i++)
    //{
        //field_elt_inv(dst, x[i], k3, STACK_1);
    //}
    //clock_gettime(CLOCK_MONOTONIC_RAW, &end);
    //printf("Inv 192 bits | %lu s %04u ns \n\n", get_s(&start, &end), get_ns(&start, &end));
    
    /* Free allocated field elts */
    
    for (i=0; i<nb_iteration; i++)
    {
        field_elt_free(&x[i], k3);
        field_elt_free(&y[i], k3);
    }
    field_elt_free(&dst, k3);

    /* 256 bits */
    
    for (i=0; i<nb_iteration; i++)
    {
        field_elt_alloc(&x[i], k4);
        field_elt_init(x[i], k4);
    }
    for (i=0; i<nb_iteration; i++)
    {
        field_elt_alloc(&y[i], k4);
        field_elt_init(y[i], k4);
    }
    field_elt_alloc(&dst, k4);
    field_elt_init(dst, k4);
    
    for(i=0; i<nb_iteration; i++)
    {
        /* Set element random */
        field_elt_random(x[i], k4, STACK_1);
        field_elt_random(y[i], k4, STACK_1);
    }
    
    /* Add */
    
    clock_gettime(CLOCK_MONOTONIC_RAW, &start);
    for (i=0; i<nb_iteration; i++)
    {
        field_elt_add(dst, x[i], y[i], k4);
    }
    clock_gettime(CLOCK_MONOTONIC_RAW, &end);
    printf("Add 256 bits | %lu s %04u ns \n", get_s(&start, &end), get_ns(&start, &end));
    
    /* Sub */
    
    clock_gettime(CLOCK_MONOTONIC_RAW, &start);
    for (i=0; i<nb_iteration; i++)
    {
        field_elt_sub(dst, x[i], y[i], k4);
    }
    clock_gettime(CLOCK_MONOTONIC_RAW, &end);
    printf("Sub 256 bits | %lu s %04u ns \n", get_s(&start, &end), get_ns(&start, &end));
    
    /* Mul */
    
    clock_gettime(CLOCK_MONOTONIC_RAW, &start);
    for (i=0; i<nb_iteration; i++)
    {
        field_elt_mul(dst, x[i], y[i], k4, STACK_1);
    }
    clock_gettime(CLOCK_MONOTONIC_RAW, &end);
    printf("Mul 256 bits | %lu s %04u ns \n", get_s(&start, &end), get_ns(&start, &end));
    
     ///* Inv */
    
    //clock_gettime(CLOCK_MONOTONIC_RAW, &start);
    //for (i=0; i<nb_iteration; i++)
    //{
        //field_elt_inv(dst, x[i], k4, STACK_1);
    //}
    //clock_gettime(CLOCK_MONOTONIC_RAW, &end);
    //printf("Inv 256 bits | %lu s %04u ns \n\n", get_s(&start, &end), get_ns(&start, &end));
    
    /* Free allocated field elts */
    
    for (i=0; i<nb_iteration; i++)
    {
        field_elt_free(&x[i], k4);
        field_elt_free(&y[i], k4);
    }
    field_elt_free(&dst, k4);

    /* 384 bits */
    
    for (i=0; i<nb_iteration; i++)
    {
        field_elt_alloc(&x[i], k6);
        field_elt_init(x[i], k6);
    }
    for (i=0; i<nb_iteration; i++)
    {
        field_elt_alloc(&y[i], k6);
        field_elt_init(y[i], k6);
    }
    field_elt_alloc(&dst, k6);
    field_elt_init(dst, k6);
    
    for(i=0; i<nb_iteration; i++)
    {
        /* Set element random */
        field_elt_random(x[i], k6, STACK_1);
        field_elt_random(y[i], k6, STACK_1);
    }
    
    /* Add */
    
    clock_gettime(CLOCK_MONOTONIC_RAW, &start);
    for (i=0; i<nb_iteration; i++)
    {
        field_elt_add(dst, x[i], y[i], k6);
    }
    clock_gettime(CLOCK_MONOTONIC_RAW, &end);
    printf("Add 384 bits | %lu s %04u ns \n", get_s(&start, &end), get_ns(&start, &end));
    
    /* Sub */
    
    clock_gettime(CLOCK_MONOTONIC_RAW, &start);
    for (i=0; i<nb_iteration; i++)
    {
        field_elt_sub(dst, x[i], y[i], k6);
    }
    clock_gettime(CLOCK_MONOTONIC_RAW, &end);
    printf("Sub 384 bits | %lu s %04u ns \n", get_s(&start, &end), get_ns(&start, &end));
    
    /* Mul */
    
    clock_gettime(CLOCK_MONOTONIC_RAW, &start);
    for (i=0; i<nb_iteration; i++)
    {
        field_elt_mul(dst, x[i], y[i], k6, STACK_1);
    }
    clock_gettime(CLOCK_MONOTONIC_RAW, &end);
    printf("Mul 384 bits | %lu s %04u ns \n", get_s(&start, &end), get_ns(&start, &end));
    
    ///* Inv */
    
    //clock_gettime(CLOCK_MONOTONIC_RAW, &start);
    //for (i=0; i<nb_iteration; i++)
    //{
        //field_elt_inv(dst, x[i], k6, STACK_1);
    //}
    //clock_gettime(CLOCK_MONOTONIC_RAW, &end);
    //printf("Inv 384 bits | %lu s %04u ns \n\n", get_s(&start, &end), get_ns(&start, &end));
    
    /* Free allocated field elts */
    
    for (i=0; i<nb_iteration; i++)
    {
        field_elt_free(&x[i], k6);
        field_elt_free(&y[i], k6);
    }
    field_elt_free(&dst, k6);

    /* 512 bits */
    
    for (i=0; i<nb_iteration; i++)
    {
        field_elt_alloc(&(x[i]), k8);
        field_elt_init(x[i], k8);
    }
    for (i=0; i<nb_iteration; i++)
    {
        field_elt_alloc(&(y[i]), k8);
        field_elt_init(y[i], k8);
    }
    field_elt_alloc(&dst, k8);
    field_elt_init(dst, k8);
    field_elt_alloc(&dst1, k8);
    field_elt_init(dst1, k8);
    field_elt_alloc(&dst2, k8);
    field_elt_init(dst2, k8);
    field_elt_alloc(&dst3, k8);
    field_elt_init(dst3, k8);
    field_elt_alloc(&dst4, k8);
    field_elt_init(dst4, k8);
    field_elt_alloc(&dst5, k8);
    field_elt_init(dst5, k8);
    
    for(i=0; i<nb_iteration; i++)
    {
        /* Set element random */
        field_elt_random(x[i], k8, STACK_1);
        field_elt_random(y[i], k8, STACK_1);
    }
    
    /* Add */
    
    clock_gettime(CLOCK_MONOTONIC_RAW, &start);
    for (i=0; i<nb_iteration; i++)
    {
        field_elt_add(dst, x[i], y[i], k8);
    }
    clock_gettime(CLOCK_MONOTONIC_RAW, &end);
    printf("Add 512 bits | %lu s %04u ns \n", get_s(&start, &end), get_ns(&start, &end));
    
    /* Sub */
    
    clock_gettime(CLOCK_MONOTONIC_RAW, &start);
    for (i=0; i<nb_iteration; i++)
    {
        field_elt_sub(dst, x[i], y[i], k8);
    }
    clock_gettime(CLOCK_MONOTONIC_RAW, &end);
    printf("Sub 512 bits | %lu s %04u ns \n", get_s(&start, &end), get_ns(&start, &end));
    
    /* Mul */
    
    clock_gettime(CLOCK_MONOTONIC_RAW, &start);
    for (i=0; i<nb_iteration; i++)
    {
        field_elt_mul(dst, x[i], y[i], k8, STACK_1);
    }
    clock_gettime(CLOCK_MONOTONIC_RAW, &end);
    printf("Mul 512 bits | %lu s %04u ns \n", get_s(&start, &end), get_ns(&start, &end));
    
    ///* Inv */
    
    //clock_gettime(CLOCK_MONOTONIC_RAW, &start);
    //for (i=0; i<nb_iteration; i++)
    //{
        //field_elt_inv(dst, x[i], k8, STACK_1);
    //}
    //clock_gettime(CLOCK_MONOTONIC_RAW, &end);
    //printf("Inv 512 bits | %lu s %04u ns \n\n", get_s(&start, &end), get_ns(&start, &end));
    
    /* Free allocated field elts */
    
    for (i=0; i<nb_iteration; i++)
    {
        field_elt_free(&x[i], k8);
        field_elt_free(&y[i], k8);
    }
    field_elt_free(&dst, k8);
    field_elt_free(&dst1, k8);
    field_elt_free(&dst2, k8);
    field_elt_free(&dst3, k8);
    field_elt_free(&dst4, k8);
    field_elt_free(&dst5, k8);
    
    /* Free allocated fields */
    
    field_free(k2);
    field_free(k3);
    field_free(k4);
    field_free(k6);
    field_free(k8);
    
    free_mphell();
}
