/* 
  MPHELL-4.0 
  Author(s): The MPHELL team

 (C) Copyright 2015-2018 - Institut Fourier / Univ. Grenoble Alpes (France)

 This file is part of the MPHELL Library.
 MPHELL is free software: you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation, version 3 of the License.
 
 MPHELL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with MPHELL.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
  * \file  mphell_perf_number.c
  * \brief Benchmark for the number functions
  */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>
#include <string.h>

#include "mphell.h"

int main()
{
    init_mphell(128, RANDOM_AES128, DEVURANDOM);
    int i;
    int nb_iteration = 20000;
    
    struct timespec start, end;
    
    /* Allocate number elements */
    
    number x[nb_iteration];
    number y[nb_iteration];
    number dst;
    
    /* ----------------- 192 bits ------------------ */
    
    for (i=0; i<nb_iteration; i++)
    {
        number_init(&x[i], bits_to_nblock(192));
        number_random(x[i], bits_to_nblock(192));
    }
    for (i=0; i<nb_iteration; i++)
    {
        number_init(&y[i], bits_to_nblock(192));
        number_random(y[i], bits_to_nblock(192));
    }
    number_init(&dst, 2*bits_to_nblock(192));
    
    printf("Enter TEST:\n");
    
    /* Add */
    
    clock_gettime(CLOCK_MONOTONIC_RAW, &start);
    for (i=0; i<nb_iteration; i++)
    {
        number_add(dst, x[i], y[i]);
    }
    clock_gettime(CLOCK_MONOTONIC_RAW, &end);
    printf("Add 192 bits | %lu s %04u ns \n", get_s(&start, &end), get_ns(&start, &end));
    
    /* Sub */
    
    clock_gettime(CLOCK_MONOTONIC_RAW, &start);
    for (i=0; i<nb_iteration; i++)
    {
        number_sub(dst, x[i], y[i]);
    }
    clock_gettime(CLOCK_MONOTONIC_RAW, &end);
    printf("Sub 192 bits | %lu s %04u ns \n", get_s(&start, &end), get_ns(&start, &end));
    
    /* Mul */
    
    clock_gettime(CLOCK_MONOTONIC_RAW, &start);
    for (i=0; i<nb_iteration; i++)
    {
        number_mul(dst, x[i], y[i]);
    }
    clock_gettime(CLOCK_MONOTONIC_RAW, &end);
    printf("Mul 192 bits | %lu s %04u ns \n", get_s(&start, &end), get_ns(&start, &end));
    
    /* Div */
    
    clock_gettime(CLOCK_MONOTONIC_RAW, &start);
    for (i=0; i<nb_iteration; i++)
    {
        number_div(dst, x[i], y[i]);
    }
    clock_gettime(CLOCK_MONOTONIC_RAW, &end);
    printf("Div 192 bits | %lu s %04u ns \n", get_s(&start, &end), get_ns(&start, &end));
    
    /* Free */
    for (i=0; i<nb_iteration; i++)
    {
        number_free(&x[i]);
        number_free(&y[i]);
    }
    number_free(&dst);
    
    /* ----------------- 256 bits ------------------ */
    
    for (i=0; i<nb_iteration; i++)
    {
        number_init(&x[i], bits_to_nblock(256));
        number_random(x[i], bits_to_nblock(256));
    }
    for (i=0; i<nb_iteration; i++)
    {
        number_init(&y[i], bits_to_nblock(256));
        number_random(y[i], bits_to_nblock(256));
    }
    number_init(&dst, 2*bits_to_nblock(256));
    
    printf("Enter TEST:\n");
    
    /* Add */
    
    clock_gettime(CLOCK_MONOTONIC_RAW, &start);
    for (i=0; i<nb_iteration; i++)
    {
        number_add(dst, x[i], y[i]);
    }
    clock_gettime(CLOCK_MONOTONIC_RAW, &end);
    printf("Add 256 bits | %lu s %04u ns \n", get_s(&start, &end), get_ns(&start, &end));
    
    /* Sub */
    
    clock_gettime(CLOCK_MONOTONIC_RAW, &start);
    for (i=0; i<nb_iteration; i++)
    {
        number_sub(dst, x[i], y[i]);
    }
    clock_gettime(CLOCK_MONOTONIC_RAW, &end);
    printf("Sub 256 bits | %lu s %04u ns \n", get_s(&start, &end), get_ns(&start, &end));
    
    /* Mul */
    
    clock_gettime(CLOCK_MONOTONIC_RAW, &start);
    for (i=0; i<nb_iteration; i++)
    {
        number_mul(dst, x[i], y[i]);
    }
    clock_gettime(CLOCK_MONOTONIC_RAW, &end);
    printf("Mul 256 bits | %lu s %04u ns \n", get_s(&start, &end), get_ns(&start, &end));
    
    /* Div */
    
    clock_gettime(CLOCK_MONOTONIC_RAW, &start);
    for (i=0; i<nb_iteration; i++)
    {
        number_div(dst, x[i], y[i]);
    }
    clock_gettime(CLOCK_MONOTONIC_RAW, &end);
    printf("Div 256 bits | %lu s %04u ns \n", get_s(&start, &end), get_ns(&start, &end));

    /* Shift */  
    clock_gettime(CLOCK_MONOTONIC_RAW, &start);
    for (i=0; i<nb_iteration; i++)
    {
        number_lshift(dst, x[i], 1);
        //number_rshift(y[i],dst, 1);
    }
    clock_gettime(CLOCK_MONOTONIC_RAW, &end);
    printf("shift of 1   bit  for 256 bits numbers | %lu s %04u ns \n", get_s(&start, &end), get_ns(&start, &end));
    
    //for (i=0; i<nb_iteration; i++)
    //{
        //if(number_cmp(x[i],y[i])!=0)
        //{
            //printf("Error in 1 bit shift[%d]\n",i);
        //}
    //}
    
    clock_gettime(CLOCK_MONOTONIC_RAW, &start);
    for (i=0; i<nb_iteration; i++)
    {
        number_lshift(dst, x[i], 40);
        //number_rshift(y[i],dst, 40);
    }
    clock_gettime(CLOCK_MONOTONIC_RAW, &end);
    printf("shift of 40  bits for 256 bits numbers | %lu s %04u ns \n", get_s(&start, &end), get_ns(&start, &end));
    
    //for (i=0; i<nb_iteration; i++)
    //{
        //if(number_cmp(x[i],y[i])!=0)
        //{
            //printf("Error in 40 bits shift[%d]\n",arithmétique i);
        //}
    //}

    clock_gettime(CLOCK_MONOTONIC_RAW, &start);
    for (i=0; i<nb_iteration; i++)
    {
        number_lshift(dst, x[i], 255);
        //number_rshift(y[i],dst, 255);
    }
    clock_gettime(CLOCK_MONOTONIC_RAW, &end);
    printf("shift of 255 bits for 256 bits numbers | %lu s %04u ns \n", get_s(&start, &end), get_ns(&start, &end));
    
    //for (i=0; i<nb_iteration; i++)
    //{
        //if(number_cmp(x[i],y[i])!=0)
        //{
            //printf("Error in 255 bits shift[%d]\n",i);
        //}
    //} 
    /* Powm*/
    clock_gettime(CLOCK_MONOTONIC_RAW, &start);
    for (i=0; i<nb_iteration; i++)
    {
        number_powm(dst, x[i], y[i],x[((i+1)%nb_iteration)], STACK_1);
    }
    clock_gettime(CLOCK_MONOTONIC_RAW, &end);
    printf("Powm 256 bits | %lu s %04u ns \n", get_s(&start, &end), get_ns(&start, &end));

    /* Free */
    for (i=0; i<nb_iteration; i++)
    {
        number_free(&x[i]);
        number_free(&y[i]);
    }
    number_free(&dst);
    
    /* ----------------- 320 bits ------------------ */
    
    for (i=0; i<nb_iteration; i++)
    {
        number_init(&x[i], bits_to_nblock(320));
        number_random(x[i], bits_to_nblock(320));
    }
    for (i=0; i<nb_iteration; i++)
    {
        number_init(&y[i], bits_to_nblock(320));
        number_random(y[i], bits_to_nblock(320));
    }
    number_init(&dst, 2*bits_to_nblock(320));
    
    printf("Enter TEST:\n");
    
    /* Add */
    
    clock_gettime(CLOCK_MONOTONIC_RAW, &start);
    for (i=0; i<nb_iteration; i++)
    {
        number_add(dst, x[i], y[i]);
    }
    clock_gettime(CLOCK_MONOTONIC_RAW, &end);
    printf("Add 320 bits | %lu s %04u ns \n", get_s(&start, &end), get_ns(&start, &end));
    
    /* Sub */
    
    clock_gettime(CLOCK_MONOTONIC_RAW, &start);
    for (i=0; i<nb_iteration; i++)
    {
        number_sub(dst, x[i], y[i]);
    }
    clock_gettime(CLOCK_MONOTONIC_RAW, &end);
    printf("Sub 320 bits | %lu s %04u ns \n", get_s(&start, &end), get_ns(&start, &end));
    
    /* Mul */
    
    clock_gettime(CLOCK_MONOTONIC_RAW, &start);
    for (i=0; i<nb_iteration; i++)
    {
        number_mul(dst, x[i], y[i]);
    }
    clock_gettime(CLOCK_MONOTONIC_RAW, &end);
    printf("Mul 320 bits | %lu s %04u ns \n", get_s(&start, &end), get_ns(&start, &end));
    
    /* Div */
    
    clock_gettime(CLOCK_MONOTONIC_RAW, &start);
    for (i=0; i<nb_iteration; i++)
    {
        number_div(dst, x[i], y[i]);
    }
    clock_gettime(CLOCK_MONOTONIC_RAW, &end);
    printf("Div 320 bits | %lu s %04u ns \n", get_s(&start, &end), get_ns(&start, &end));
    
    /* Free */
    for (i=0; i<nb_iteration; i++)
    {
        number_free(&x[i]);
        number_free(&y[i]);
    }
    number_free(&dst);
    
    /* ----------------- 384 bits ------------------ */
    
    for (i=0; i<nb_iteration; i++)
    {
        number_init(&x[i], bits_to_nblock(384));
        number_random(x[i], bits_to_nblock(384));
    }
    for (i=0; i<nb_iteration; i++)
    {
        number_init(&y[i], bits_to_nblock(384));
        number_random(y[i], bits_to_nblock(384));
    }
    number_init(&dst, 2*bits_to_nblock(384));
    
    printf("Enter TEST:\n");
    
    /* Add */
    
    clock_gettime(CLOCK_MONOTONIC_RAW, &start);
    for (i=0; i<nb_iteration; i++)
    {
        number_add(dst, x[i], y[i]);
    }
    clock_gettime(CLOCK_MONOTONIC_RAW, &end);
    printf("Add 384 bits | %lu s %04u ns \n", get_s(&start, &end), get_ns(&start, &end));
    
    /* Sub */
    
    clock_gettime(CLOCK_MONOTONIC_RAW, &start);
    for (i=0; i<nb_iteration; i++)
    {
        number_sub(dst, x[i], y[i]);
    }
    clock_gettime(CLOCK_MONOTONIC_RAW, &end);
    printf("Sub 384 bits | %lu s %04u ns \n", get_s(&start, &end), get_ns(&start, &end));
    
    /* Mul */
    
    clock_gettime(CLOCK_MONOTONIC_RAW, &start);
    for (i=0; i<nb_iteration; i++)
    {
        number_mul(dst, x[i], y[i]);
    }
    clock_gettime(CLOCK_MONOTONIC_RAW, &end);
    printf("Mul 384 bits | %lu s %04u ns \n", get_s(&start, &end), get_ns(&start, &end));
    
    /* Div */
    
    clock_gettime(CLOCK_MONOTONIC_RAW, &start);
    for (i=0; i<nb_iteration; i++)
    {
        number_div(dst, x[i], y[i]);
    }
    clock_gettime(CLOCK_MONOTONIC_RAW, &end);
    printf("Div 384 bits | %lu s %04u ns \n", get_s(&start, &end), get_ns(&start, &end));
    
    /* Free */
    for (i=0; i<nb_iteration; i++)
    {
        number_free(&x[i]);
        number_free(&y[i]);
    }
    number_free(&dst);
    
    /* ----------------- 448 bits ------------------ */
    
    for (i=0; i<nb_iteration; i++)
    {
        number_init(&x[i], bits_to_nblock(448));
        number_random(x[i], bits_to_nblock(448));
    }
    for (i=0; i<nb_iteration; i++)
    {
        number_init(&y[i], bits_to_nblock(448));
        number_random(y[i], bits_to_nblock(448));
    }
    number_init(&dst, 2*bits_to_nblock(448));
    
    printf("Enter TEST:\n");
    
    /* Add */
    
    clock_gettime(CLOCK_MONOTONIC_RAW, &start);
    for (i=0; i<nb_iteration; i++)
    {
        number_add(dst, x[i], y[i]);
    }
    clock_gettime(CLOCK_MONOTONIC_RAW, &end);
    printf("Add 448 bits | %lu s %04u ns \n", get_s(&start, &end), get_ns(&start, &end));
    
    /* Sub */
    
    clock_gettime(CLOCK_MONOTONIC_RAW, &start);
    for (i=0; i<nb_iteration; i++)
    {
        number_sub(dst, x[i], y[i]);
    }
    clock_gettime(CLOCK_MONOTONIC_RAW, &end);
    printf("Sub 448 bits | %lu s %04u ns \n", get_s(&start, &end), get_ns(&start, &end));
    
    /* Mul */
    
    clock_gettime(CLOCK_MONOTONIC_RAW, &start);
    for (i=0; i<nb_iteration; i++)
    {
        number_mul(dst, x[i], y[i]);
    }
    clock_gettime(CLOCK_MONOTONIC_RAW, &end);
    printf("Mul 448 bits | %lu s %04u ns \n", get_s(&start, &end), get_ns(&start, &end));
    
    /* Div */
    
    clock_gettime(CLOCK_MONOTONIC_RAW, &start);
    for (i=0; i<nb_iteration; i++)
    {
        number_div(dst, x[i], y[i]);
    }
    clock_gettime(CLOCK_MONOTONIC_RAW, &end);
    printf("Div 448 bits | %lu s %04u ns \n", get_s(&start, &end), get_ns(&start, &end));
    
    /* Free */
    for (i=0; i<nb_iteration; i++)
    {
        number_free(&x[i]);
        number_free(&y[i]);
    }
    number_free(&dst);
    
    /* ----------------- 512 bits ------------------ */
    
    for (i=0; i<nb_iteration; i++)
    {
        number_init(&x[i], bits_to_nblock(512));
        number_random(x[i], bits_to_nblock(512));
    }
    for (i=0; i<nb_iteration; i++)
    {
        number_init(&y[i], bits_to_nblock(512));
        number_random(y[i], bits_to_nblock(512));
    }
    number_init(&dst, 2*bits_to_nblock(512));
    
    printf("Enter TEST:\n");
    
    /* Add */
    
    clock_gettime(CLOCK_MONOTONIC_RAW, &start);
    for (i=0; i<nb_iteration; i++)
    {
        number_add(dst, x[i], y[i]);
    }
    clock_gettime(CLOCK_MONOTONIC_RAW, &end);
    printf("Add 512 bits | %lu s %04u ns \n", get_s(&start, &end), get_ns(&start, &end));
    
    /* Sub */
    
    clock_gettime(CLOCK_MONOTONIC_RAW, &start);
    for (i=0; i<nb_iteration; i++)
    {
        number_sub(dst, x[i], y[i]);
    }
    clock_gettime(CLOCK_MONOTONIC_RAW, &end);
    printf("Sub 512 bits | %lu s %04u ns \n", get_s(&start, &end), get_ns(&start, &end));
    
    /* Mul */
    
    clock_gettime(CLOCK_MONOTONIC_RAW, &start);
    for (i=0; i<nb_iteration; i++)
    {
        number_mul(dst, x[i], y[i]);
    }
    clock_gettime(CLOCK_MONOTONIC_RAW, &end);
    printf("Mul 512 bits | %lu s %04u ns \n", get_s(&start, &end), get_ns(&start, &end));
    
    /* Div */
    
    clock_gettime(CLOCK_MONOTONIC_RAW, &start);
    for (i=0; i<nb_iteration; i++)
    {
        number_div(dst, x[i], y[i]);
    }
    clock_gettime(CLOCK_MONOTONIC_RAW, &end);
    printf("Div 512 bits | %lu s %04u ns \n", get_s(&start, &end), get_ns(&start, &end));
    
    /* Free */
    for (i=0; i<nb_iteration; i++)
    {
        number_free(&x[i]);
        number_free(&y[i]);
    }
    number_free(&dst);
    
    /* ----------------- 576 bits ------------------ */
    
    for (i=0; i<nb_iteration; i++)
    {
        number_init(&x[i], bits_to_nblock(576));
        number_random(x[i], bits_to_nblock(576));
    }
    for (i=0; i<nb_iteration; i++)
    {
        number_init(&y[i], bits_to_nblock(576));
        number_random(y[i], bits_to_nblock(576));
    }
    number_init(&dst, 2*bits_to_nblock(576));
    
    printf("Enter TEST:\n");
    
    /* Add */
    
    clock_gettime(CLOCK_MONOTONIC_RAW, &start);
    for (i=0; i<nb_iteration; i++)
    {
        number_add(dst, x[i], y[i]);
    }
    clock_gettime(CLOCK_MONOTONIC_RAW, &end);
    printf("Add 576 bits | %lu s %04u ns \n", get_s(&start, &end), get_ns(&start, &end));
    
    /* Sub */
    
    clock_gettime(CLOCK_MONOTONIC_RAW, &start);
    for (i=0; i<nb_iteration; i++)
    {
        number_sub(dst, x[i], y[i]);
    }
    clock_gettime(CLOCK_MONOTONIC_RAW, &end);
    printf("Sub 576 bits | %lu s %04u ns \n", get_s(&start, &end), get_ns(&start, &end));
    
    /* Mul */
    
    clock_gettime(CLOCK_MONOTONIC_RAW, &start);
    for (i=0; i<nb_iteration; i++)
    {
        number_mul(dst, x[i], y[i]);
    }
    clock_gettime(CLOCK_MONOTONIC_RAW, &end);
    printf("Mul 576 bits | %lu s %04u ns \n", get_s(&start, &end), get_ns(&start, &end));
    
    /* Div */
    
    clock_gettime(CLOCK_MONOTONIC_RAW, &start);
    for (i=0; i<nb_iteration; i++)
    {
        number_div(dst, x[i], y[i]);
    }
    clock_gettime(CLOCK_MONOTONIC_RAW, &end);
    printf("Div 576 bits | %lu s %04u ns \n", get_s(&start, &end), get_ns(&start, &end));
    
    /* Free */
    for (i=0; i<nb_iteration; i++)
    {
        number_free(&x[i]);
        number_free(&y[i]);
    }
    number_free(&dst);
    
    free_mphell();
}
