/* 
  MPHELL-4.0 
  Author(s): The MPHELL team

 (C) Copyright 2015-2018 - Institut Fourier / Univ. Grenoble Alpes (France)

 This file is part of the MPHELL Library.
 MPHELL is free software: you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation, version 3 of the License.
 
 MPHELL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with MPHELL.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
  * \file  test_aes.c
  * \brief Self check tests for AES module.
  */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "mphell-aes.h"
#include "mphell-util.h"

int main()
{
    char *files[9] = {
        DATADIR "/vector_test_aes/ECBKeySbox128.rsp",
        DATADIR "/vector_test_aes/ECBKeySbox192.rsp",
        DATADIR "/vector_test_aes/ECBKeySbox256.rsp",
        DATADIR "/vector_test_aes/ECBVarKey128.rsp",
        DATADIR "/vector_test_aes/ECBVarKey192.rsp",
        DATADIR "/vector_test_aes/ECBVarKey256.rsp",
        DATADIR "/vector_test_aes/ECBVarTxt128.rsp",
        DATADIR "/vector_test_aes/ECBVarTxt192.rsp",
        DATADIR "/vector_test_aes/ECBVarTxt256.rsp"
    };
    FILE *fp;
    char plaintext[16];
    char encrypted[16];
    char encryptedref[16];
    char marker[20];
    char key[32];

    int i, j, k, index_key, lenkey;
    int count;
    int success = 0;
    for(i = 0; i < 8; i++)
    {
        int cmp = 0;
        switch(i)
        {   
            case 0 :
                count = 21;
                break;
            case 1 : 
                count = 24;
                break;
            case 2 :
                count = 16;
                break;
            case 3 :
                count = 128;
                break;
            case 4 :
                count = 192;
                break;
            case 5 :
                count = 256;
                break;
            case 6 :
            case 7 :
            case 8 :
                count = 128;
                break;
            default :
                abort();
        }
        
        index_key = i % 3;
        if(index_key == 0)
        {
            lenkey = 128;
        }
        else if(index_key == 1)
        {
            lenkey = 192;
        }  
        else
        {
            lenkey = 256;
        }

        fp = fopen(files[i], "r");
        if(fp == NULL)
        {
            fprintf(stderr, "Opening file failed %d %s \n", i, files[i]);
            abort();
        }
        
        printf("test %s\n", files[i]);       
   
        for (j = 0; j < count; j++)
        {
            sprintf( marker, "COUNT = %u", j);
            if (!FindMarker(fp, marker) )
            {
                printf("ERROR : no test vector found ( count %u)\n", j);
                break;
            }

            if (!ReadHex(fp, key, lenkey/4, "KEY = ") ) 
            {
                printf("ERROR : unable to read 'KEY' (count %u)\n", j);
                break;
            }
 
            if (!ReadHex(fp, plaintext, 32, "PLAINTEXT = ") ) 
            {
                printf("ERROR : unable to read 'PLAINTEXT' (count %u)\n", j);
                break;
            }
            
            if(lenkey == 128)
            {
                aes128_cipher((uint8_t*)encrypted, (uint8_t*)key,
                        (uint8_t*)plaintext);
            }
            else if(lenkey == 192)
            {
                aes192_cipher((uint8_t*)encrypted, (uint8_t*)key, 
                        (uint8_t*)plaintext);
            }
            else
            {
                aes256_cipher((uint8_t*)encrypted, (uint8_t*)key,
                        (uint8_t*)plaintext);
            }
                        
            if (!ReadHex(fp, encryptedref, 32, "CIPHERTEXT = ") )
            {
                printf("ERROR : no ciphertext value found (count %u)\n", j);
                break;
            }

            for(k = 0; k < 16; k++)
            {
                
                if(encrypted[k] != encryptedref[k])
                {
                    cmp = 1;
                    success ++;
                    printf("ERROR : no correct ciphertext value computed \
                            (count %u )\n", j);
                    break;
                }
            }
        }
        fclose(fp);
        if(! cmp)
        {
            printf("SUCCESS : %s\n", files[i]);
        }
    }
    if (success == 0)
    {
        return 0;
    }
    else
    {
        return -1;
    }
}

