/* 
  MPHELL-4.0 
  Author(s): The MPHELL team

 (C) Copyright 2015-2018 - Institut Fourier / Univ. Grenoble Alpes (France)

 This file is part of the MPHELL Library.
 MPHELL is free software: you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation, version 3 of the License.
 
 MPHELL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with MPHELL.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
  * \file  mphell_test_edwards.c
  * \brief Self check tests for Edwards Elliptic Curves.
  */

#include "mphell_test_weierstrass.h"
#include "mphell_test_edwards.h"


void
test_curve_edwards_read(const char *id_curve, const ec_formula f, data_t *data)
{
    char* curve_file = (char*)malloc(sizeof(char) * 200);
    /*sprintf(curve_file, "%s.txt", id_curve);*/
    sprintf(curve_file, DATADIR "/elliptic_curves/%s.txt", id_curve); 
    printf("Chemin:    %s \n", curve_file);
    FILE *fp = NULL;
    char line[160];
    fp = fopen(curve_file, "r");
    free(curve_file);

    if(fp == NULL)
    {
        fprintf(stderr, "Error : no curve data2 file found \n");
        exit(EXIT_FAILURE);
    }
    char *str = (char*)malloc(sizeof(char) * 200);
    char *str1 = (char*)malloc(sizeof(char) * 200);
    number tmp;
    number_init(&tmp, bits_to_nblock(data->nbits));

    while(fgets(line, sizeof(line), fp) != NULL)
    { 
        if(strncmp(line, "p = ", 4)== 0)
        {
            strcpy(str, line + 4);
            number_set_str(tmp, str, 16);
            field_create(data->k, "", STACK_1, 1, tmp);
            /* Initialise the curve, now that k is set up */
            ec_init(data->E, data->k);
        }
        if(strncmp(line, "a = ", 4)== 0)
        {
            strcpy(str, line + 4);
            number_set_str(tmp, str, 16);
            field_elt_set_number(data->E->a, false, data->k, STACK_1, 1, tmp);
        }
        if(strncmp(line, "b = ", 4)== 0)
        {
            strcpy(str, line + 4);
            number_set_str(tmp, str, 16);
            field_elt_set_number(data->E->b, false, data->k, STACK_1, 1, tmp);
        }
        if(strncmp(line, "xG = ", 5)== 0)
        {
            strcpy(str, line + 5);
        }
        if(strncmp(line, "yG = ", 5)== 0)
        {
            strcpy(str1, line + 5);
            edwards_point_set_aff_str(data->E->G, str, str1, false, 16, data->k, STACK_1);
        }
        if(strncmp(line, "h = ", 4) == 0)
        {
            strcpy(str, line + 4);
            number_set_str(data->E->h, str, 16);
        }
        if(strncmp(line, "n = ", 4) == 0)
        {
            strcpy(str, line + 4);
            number_set_str(data->E->n, str, 16);
            break;
        }
    }
    data->E->k = data->k;
    data->E->type = EDWARDS;
    data->E->f = f;
    strcpy(data->E->id_curve, id_curve);
    ec_compute_disc(data->E, STACK_1); 
    fclose(fp);
    free(str);
    free(str1);
    number_free(&tmp);
}

void
test_curve_edwards_read_point(FILE *filep, ec_point P, data_t *data)
{
    char line[180];

    if(filep == NULL)
    {
        fprintf(stderr, "Error : no test data file found \n");
        exit(EXIT_FAILURE);
    }

    char *str = malloc(sizeof(char) * 170);
    field_elt tmp, tmp1;
    field_elt_alloc(&tmp, data->k);
    field_elt_init(tmp, data->k);
    field_elt_alloc(&tmp1, data->k);
    field_elt_init(tmp1, data->k);

    while(fgets(line, sizeof(line), filep) != NULL)
    { 
        if(strncmp(line, "x = ", 4) == 0)
        {
            strcpy(str, line + 4);
            if(str[strlen(str)-2] == '\\')
            {
                if(fgets(line, sizeof(line), filep) == NULL)
                {
                    exit(EXIT_FAILURE);
                }
                strcpy(str + strlen(str)-2, line);
            }
            field_elt_set_str(tmp, str, 16, false, data->k, STACK_1);
        }
        if(strncmp(line, "y = ", 4) == 0)
        {
            strcpy(str, line + 4);
            if(str[strlen(str)-2] == '\\')
            {
                if(fgets(line, sizeof(line), filep) == NULL)
                {
                    exit(EXIT_FAILURE);
                }
                strcpy(str + strlen(str)-2, line);
            }
            field_elt_set_str(tmp1, str, 16, false, data->k, STACK_1);
            edwards_point_set_aff(P, tmp, tmp1, data->k, STACK_1);
            break;
        }
    }
    free(str);
    field_elt_free(&tmp, data->k);
    field_elt_free(&tmp1, data->k);
}

void
edwards_computation_of_curve_from_alpha_beta_and_field(ec_curve E_Edbis, fe_srcptr alpha, fe_srcptr beta, field_t *k, const char *id_curve)
{
    field_elt a,c,d,f; 
    ec_curve E_Ed;  
    
    /* Allocation */
    ec_alloc(E_Ed, k);
    ec_init(E_Ed, k);
    field_elt_alloc(&c, k);
    field_elt_init(c, k);
    field_elt_alloc(&f, k);
    field_elt_init(f, k);
    field_elt_alloc(&a, k);
    field_elt_init(a, k);
    field_elt_alloc(&d, k);
    field_elt_init(d, k);

    #if MPHELL_USE_PARI == 0
        field_elt_set_str(f,"3",10,false,k, STACK_1);
    #else
        field_elt_set_str(f,"3",10,true,k, STACK_1);
    #endif
    field_elt_inv(a,beta, k, STACK_1);              /* B = 1 / beta (stored in a) */
    field_elt_mul(c,alpha,a, k, STACK_1);
    field_elt_mul(d,c,f, k, STACK_1);               /* A = 3 alpha / beta (stored in d) */
    #if MPHELL_USE_PARI == 0
        field_elt_set_str(f,"2",10,false,k, STACK_1);
    #else
        field_elt_set_str(f,"2",10,true,k, STACK_1);
    #endif
    field_elt_add(c,d,f, k);
    field_elt_sub(f,d,f, k);
    field_elt_div(d,f,a, k, STACK_1);               /* d = (A - 2) / B */
    field_elt_div(a,c,a, k, STACK_1);               /* a = (A + 2) / B */
    

    field_elt_copy(E_Ed->a,a, k);
    field_elt_copy(E_Ed->b,d, k);
    E_Ed->k = k;
    field_elt_dec(a,f, k);
    field_elt_dec(d,a, k);
    
    ec_point_set_aff(E_Ed->G, d, a, EDWARDS, k, STACK_1); 
    number_set_str(E_Ed->h, "1", 16);     
    number_set_str(E_Ed->n, "1", 16);
    E_Ed->type = EDWARDS;
    E_Ed->f = EXTENDED_EDWARDS;
    strcpy(E_Ed->id_curve, id_curve);
    ec_compute_disc(E_Ed, STACK_1);    
    ec_copy(E_Edbis,E_Ed);

    /* We now free the memory */
    field_elt_free(&a, k);
    field_elt_free(&c, k);
    field_elt_free(&d, k);
    field_elt_free(&f, k);
    ec_free(E_Ed);
}
