/* 
  MPHELL-4.0 
  Author(s): The MPHELL team

 (C) Copyright 2015-2018 - Institut Fourier / Univ. Grenoble Alpes (France)

 This file is part of the MPHELL Library.
 MPHELL is free software: you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation, version 3 of the License.
 
 MPHELL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with MPHELL.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
  * \file  mphell_test_edwards.h
  * \brief Functions for self check tests for Edwards Elliptic Curves.
  */

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include "mphell.h"

/**
 * @fn    void test_curve_edwards_read(const char *id_curve, const ec_formula f, data_t *data);
 * @brief Set E to the curve defined by the file id_curve.txt
 * @param id_curve Name of the curve, read the file data/elliptic_curves/id_curve.txt
 * @param f        Coordinates system
 * @param data     Data for the tests
 */
void
test_curve_edwards_read(const char *id_curve, const ec_formula f, data_t *data);

/**
 * @fn    void test_curve_edwards_read_point(FILE *filep, ec_point P, data_t *data);
 * @brief Read a point P=(x,y) in a vector test file
 * @param filep Name of the vector test file
 * @param P 	Destination point
 * @param data  Data for the tests
 */
void
test_curve_edwards_read_point(FILE *filep, ec_point P, data_t *data);

void
edwards_computation_of_curve_from_alpha_beta_and_field(ec_curve E_Ed, fe_srcptr alpha, fe_srcptr beta, field_t *k, const char *id_curve);


