/* 
  MPHELL-4.0 
  Author(s): The MPHELL team

 (C) Copyright 2015 - Institut Fourier / Université Grenoble Alpes (Grenoble) 

 This file is part of the MPHELL Library.
 MPHELL is free software: you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation, version 3 of the License.
 
 MPHELL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with MPHELL.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
  * \file  mphell_test_edwards_belongs.c
  * \brief Self check tests for the function: bool edwards_belongs (ec_point_srcptr P, ec_curve_srcptr E)
  */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>
#include "mphell.h"

int main()
{
    init_mphell(128, RANDOM_AES128, DEVURANDOM);
    bool test = true;
    bool test1, test3;  
    int i;
    int nb_iteration = 100;
    /* Create base prime field of characteristic p */
    field k;
    field_elt b, c, x1, x2, y1, y2, z1, z2, t1, t2, a, d;
    ec_curve E,  Etri; 
    ec_point P, Q; /* The two points that we will test */
    number p;

    /* Allocation */
    field_alloc(k, FP, bits_to_nblock(384), NULL);
    number_init(&p, bits_to_nblock(384));
    
    /*Setting variables*/
    number_set_str(p, "fffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffeffffffff0000000000000000ffffffff", 16);
    field_create(k, "", STACK_1, 1, p);
    
    /* Allocation  of field elts */
    field_elt_alloc(&a, k);
    field_elt_init(a, k);
    field_elt_alloc(&d, k);
    field_elt_init(d, k);
    field_elt_alloc(&b, k);
    field_elt_init(b, k);
    field_elt_alloc(&c, k);
    field_elt_init(c, k);
    field_elt_alloc(&x1, k);
    field_elt_init(x1, k);
    field_elt_alloc(&x2, k);
    field_elt_init(x2, k);
    field_elt_alloc(&y1, k);
    field_elt_init(y1, k);
    field_elt_alloc(&y2, k);
    field_elt_init(y2, k);
    field_elt_alloc(&z1, k);
    field_elt_init(z1, k);
    field_elt_alloc(&z2, k);
    field_elt_init(z2, k);
    field_elt_alloc(&t1, k);
    field_elt_init(t1, k);
    field_elt_alloc(&t2, k);
    field_elt_init(t2, k);
    
    /* We work on the base field of NIST P 384 */
    ec_point_alloc(P, k);
    ec_point_init(P, k);
    ec_point_alloc(Q, k);
    ec_point_init(Q, k);
    /* We make the assumption that the point have been correctly created that 
    *  is that in extended Edwards we have t=x*y/z
    */
    
    ec_alloc(E, k);
    ec_init(E, k);
    ec_alloc(Etri, k);
    ec_init(Etri, k);
    /* We create elliptic curves for the test    */
    const char *array_alea[4]; /* random for "projective coodinate"*/
    array_alea[0]="20995147416391166677980721228885546002736842251288287164126872820502391670380279263134112406390918551866357280904409";
    array_alea[1]="15742133033998992807823582249555265577773631874428909893729468807030212686629179530841549589979116091838995361331401";
    array_alea[2]="10641385055334172958167632272894720786081789938450255301242446722215388684623784557888416886572546676935038402790360";
    array_alea[3]="6025087504798599993064568591109256057249347913734633918277714546258781288865070391066231780844231520436406032831501";
    
    field_elt_set_str(a,array_alea[0], 10, false , k, STACK_1);
    field_elt_set_str(b,array_alea[1], 10, false , k, STACK_1);
    field_elt_set_str(c,array_alea[2], 10, false , k, STACK_1);
    field_elt_set_str(d,array_alea[3], 10, false , k, STACK_1);

    ec_create(E, "Test", k, b, c, NULL, NULL, NULL, EDWARDS, PROJECTIVE, STACK_1);
    ec_create(Etri, "Test", k, a, d, NULL, NULL, NULL, EDWARDS, EXTENDED_EDWARDS, STACK_1);
    
    printf("Enter TEST:\n");
    
    const char *array_src1[100]; /* x coordinate */
    const char *array_src2[100]; /* y coordinate */
    
    
    array_src1[0]="8836876423990597525185985849087506203670912370650625196294750491369110257283813278643389029445491429289618236123675";
    array_src2[0]="5538261695075426701924248999069997694255869813665062557415656065060237641172988027057640046333505225158928793956826";
    array_src1[1]="30615257316772645513292729374059818676468401409099778773470469686587224755066978006850928191669011693058832179294402";
    array_src2[1]="14442003175052569413704122829398343751312322194687516447462569527484491996980689965399893111522743115122147466636482";
    array_src1[2]="8662863942207277758065186944817799170922426142715118782585184476367741571746417582883688146665147919376533907298390";
    array_src2[2]="17877563245008119326545813317358229709635658874369017442751652201338783873830627854464095803227938009561718535927258";
    array_src1[3]="27794222460220158434313983274376230663142911508230290919890020963928908504100768536754836988875628111238622333891293";
    array_src2[3]="16481034250361532227180871648083580006670666638293315722138926959546857211823711987463929101914465672349465685034343";
    array_src1[4]="26425962976173669098831345098948597239392084895592803972263991734755693773498508473120199920123301170882538064192416";
    array_src2[4]="5392317041863070581880108388790659982079256798402689400570804088617568062275279872923269946817902596451480680313124";
    array_src1[5]="3699389340554361660233009466773070172277441078451975552943598809999549499935467204306764304250466222973276854667052";
    array_src2[5]="9738523983088187258431936105667620386117213569175780304032032233770771018132751828480090755148429642443175577927868";
    array_src1[6]="7467171983293197913528079463488161616883945214890689565472701387772621943517817427843434425475935438493634651212216";
    array_src2[6]="14012572530087638153447704402604889738263146786059205505873004380566385824206485000767400000823526618544170841764050";
    array_src1[7]="36342880402910745490915542933315650815009069292199089487648225971870177623051060261313699459130689011713887615798586";
    array_src2[7]="11510772187022710225306869541983336415074471520859671466532384230339523855855478333414782787985387257178097171705393";
    array_src1[8]="27337099173294436720982036427723672980206854505281353025631139427024429427571445825465975504367205801714084002516889";
    array_src2[8]="10212284679499809396129046448467870201929358504056129247701457183179304059041241915107144666192196551839428509340371";
    array_src1[9]="27057815009402921730537485382374133730268188864193718246712536179279863441647036612253058379555375358425673520447897";
    array_src2[9]="7697031677722646029834182921939566299122019919283063281677185644471130463199794295140942865002307804662916518748928";
    array_src1[10]="38118194200726920636020555030198431888159264610651531452714549044545840762989235823046189545940198946486051820339654";
    array_src2[10]="14023063207327833408203060149861530782439586969850957773054019112954373303777192169409756180219149129226463280408870";
    array_src1[11]="5538086858693878964786398111098019136299923772564046973527261145645342920784452856134476037714074867035989036405405";
    array_src2[11]="15892358019558384810612498562612612093138658610612038430109471024240148551835757463054475661853137010658048810093";
    array_src1[12]="13317310153348909563708397569927002607781475177769938666164068341127453891948786136219872981534835484247165158342416";
    array_src2[12]="10771069290771244361518382764251988667806290518266988953405823058281464529693196939151692947535968907757728833661231";
    array_src1[13]="28088591713819020215565829130870864463121950446908925828614746901349043591770010591382618515617211363463382510214086";
    array_src2[13]="5879839121737368982451468197869573175455441813608427957218548565898703206917464236484051315409634355313617080051693";
    array_src1[14]="23267668596443058245998733611571251904716066036267419423338273543381387975928880993919011787099639634931073058240007";
    array_src2[14]="866786083143685985695904206999376305114637398984706617615354947920950437691669477005999917910631952973956398097727";
    array_src1[15]="16575565071150120632957165606648030553907572776726267921270474912166397706021363713252927410236171829459266177273116";
    array_src2[15]="15694999491769789377008718521274228431466149943195358655612107251813087221079562073195337523804316690827026775360780";
    array_src1[16]="38386901937275927869525894471947079034812618440512086173073667170075633576906053807990686654279808327023180879503495";
    array_src2[16]="1738376529177009413400403504390028087752820629126312572037072985923520102095854850447087409343612280218777221449679";
    array_src1[17]="24226794740288975727375691373945298839279268456197158635286481316606268416479745591803801821252860341243949743598411";
    array_src2[17]="11846804452598835930783496686036725914087236649206373839934556408203120966849854761072612538985354388937028608950244";
    array_src1[18]="4837831649214809594064201669978306528887362489537419377842594883691436452780930635964356231812870981198142226618884";
    array_src2[18]="3352823223115633741876696357273351554484723421071201270256806148392579295745540694373828404519133711856220777046486";
    array_src1[19]="27001214566460831894824326476343555213312442544372098482597534722666906604807182473320697273116852678863198363655468";
    array_src2[19]="6233134414978500963492595843952862261574699581790003719169991401148455763970149347453660666762054358813369675311185";
    array_src1[20]="2603854745622284695145683730531885126127215784772655883757095116870132882450681160295373569509272991451959155438531";
    array_src2[20]="19672952667612351363096600975667788340640870720791121177999226453825765266835985233224332941406075490002414615101254";
    array_src1[21]="35815317171228004279332184232456728477415659672987400006060385965748604363249363055946948452289317674016797875097940";
    array_src2[21]="8274567585129947625161967964720282521205778358663174807096448211422670371487110702414565466611706299350313014160257";
    array_src1[22]="37236342664127529158832621349855566645014962039746965974841358062847622364656248136472367893253430993442274169570557";
    array_src2[22]="69421434634750039920734273274343027097717300160079300440720137696262621911129555647594667325729131475743534831679";
    array_src1[23]="11698970190698397708181608356497680868652686797993746225839824400747005813874756505068484562446682553209673246127940";
    array_src2[23]="7090586005382638356969201725444432635122711551596063626888769543968234297035541058643454106162061724270477834419980";
    array_src1[24]="2824102552583350938252277557477360822062035854827866346460854338688641669572631323115417976701532540920248363688424";
    array_src2[24]="16304639248154876279418432979362994855431844808945071589336880518750688412276247167957751829348180461228336568782651";
    array_src1[25]="7515043317234082002738545437061461908471150520536006482918801889532845851883889751952720170701550944875942363206897";
    array_src2[25]="458335656058075435832691647156609340329900505733302619869070449924093992687538600368583354268245116641655712231128";
    array_src1[26]="33118373500861704397808076298577246395705336585288908102058732267036794895180916779732396564788833105462868728344234";
    array_src2[26]="6774867898577522400297304698766633073178386037096912429920315354988605815703503231041100689209920282143922913007376";
    array_src1[27]="22694882501051729281976007195651379029607932988290490522044022712121132979172383433881235756137854651912957061692449";
    array_src2[27]="7962752094126155011264541291547580072515013577559892085337543271639288371448468756299455976600860313954462772072075";
    array_src1[28]="6262178847473358950155374627186899953489099070463923086523621644848031076593810005293355093524361930694446317578020";
    array_src2[28]="6931788994872782976614665955308678309068776022071029169995967166288261929629175738106588881866598878433614394947708";
    array_src1[29]="3965821052972297211768323102525079768816980549391660968435094112474271910995550912116737307175996158662990582679057";
    array_src2[29]="7447342533487442291068645265681527651849397563660228048345183836143611809313444508619307897828823704725085591795421";
    array_src1[30]="29056103777849961274162956440137693564058296153171216174361061397281424537973364770396427655650725793954697780196029";
    array_src2[30]="19548026891970264035465798461936881016485891842237086834363038007390432463109884630690973804618354629023891464265899";
    array_src1[31]="14808786980781979672744057994825458784467027951923313533256859506927390406989309772075511719587281662209355400414644";
    array_src2[31]="7041192222531681146601942750883578664326248949300333477040854952360547074118039422923437865545256360422331603239087";
    array_src1[32]="31031021150183912015541274165190376690298325681458164513492825642698076469142254819820267273754961196024378431372705";
    array_src2[32]="15414739089716203101924328614793719344699394537672287873018526385092483199461944430215444503251537174822964505665781";
    array_src1[33]="7176555717965521823019894978543545877086705120020958539979744780961332829847799615546934319885837320041299323893748";
    array_src2[33]="6167796664768298153525571444280688447294624585134600347659560266016689337097346122899841883528011647625830305595379";
    array_src1[34]="3028178811159178883003175524417522700324281571143032999502782228865552703650246248688138900635024173488127206955287";
    array_src2[34]="19634930696778635261842703520338072052961659170128981876943581219358368879986302711206087003008035744922537708074702";
    array_src1[35]="1346290250815044258649884068545766606558621394699046288435644988897758377414262223981185250215275753055023385364835";
    array_src2[35]="4625692593202236827688606822820509447931791522709402282790621138692266111814489557998905403898245735547095386659024";
    array_src1[36]="21597945324962556709787945963317401163455689914355233186343534835075810467917458923749720433945849032788620448225126";
    array_src2[36]="3646965623566831175654375838394886075332432497873546461221571108118354342730598910358853025309739322415193812589359";
    array_src1[37]="6596403469665695114757379350136799378786254202847315704470707173503610377083257565612129692434025531893118951866726";
    array_src2[37]="13744074390289810643496319170385518923108400832552332909502561009522645948423694179983974350382361237336229843874380";
    array_src1[38]="13477623271407953979102545190270993960706420566751822887169337302546963316948919067281989385985890387921068771074644";
    array_src2[38]="2014771765533809495548218520775439879221292656487310928310484324695783791380243252663677431745515129948243960777569";
    array_src1[39]="16155402137430655914086570247959073817751489111023249848667135719962004677996734566326006373424979725756215430262324";
    array_src2[39]="16681981744469594175897477265681965768436615669911711674172341119784144688871802077594394786251597285650895096111985";
    array_src1[40]="23578962428728044522832467658374903826927575693167757274321046013435071497968416817885432272666317361064571973190326";
    array_src2[40]="7783696908060404184117262801032149495879326265323792402884571572116233263807823977493506236809193761586554629451239";
    array_src1[41]="37481723966543149051248759665967559412380184748346056751414030075711502055838582560637888131642398835085999288036898";
    array_src2[41]="15965311847627048072261268996033890013209150887906923320034844648498957743746935974863218788073834940843287404537508";
    array_src1[42]="33313610901230363380455978738620555992808312201116133566481370541071003601957536580728969769603366544566288348478266";
    array_src2[42]="6413274595669275282297957627848986824770764671746622157963332541812052978734119886351124998761632782769412255212050";
    array_src1[43]="24687507667875786178952147544914066093240939870344096923679303686866395231096080537602283204560199912628831029158552";
    array_src2[43]="17190357868031790652051210651126747902358383236005440644155902612055948078155893070462944493921172501948474099144831";
    array_src1[44]="25077400386431233231583769654846323057574173569695587245732581380306374133655765349124741388725447449725718008160761";
    array_src2[44]="14487505569177211776678610952782364739701297531655496909045108744780893934985749709477088591724845659152807632472698";
    array_src1[45]="15182353165802961542364314715969954083742255045864395113605256882600015307348240986579479748949047586237250949473459";
    array_src2[45]="3711212433561937408444961312483793632953106711440204856962076329474727734508465009515871767160150386941099537110966";
    array_src1[46]="12771455040735294479409316222310174617406719469616863419657572775527852464486783029939524568144635632372917641519952";
    array_src2[46]="4578358328341373664006709760202013312821069303951874964544023524968566259566534266484858660936352935274819777948434";
    array_src1[47]="21843880831852063810853695233149255646651589073595615269573992596120213039848127220935042329725306519330527843940131";
    array_src2[47]="4712662866061305644805065845719225897200311093255822071114803328774089124858234720642894336262566971849635103449814";
    array_src1[48]="23686918768533834482129017137321971030024979425933441716095003353051187910249362540384976583688713779890500081933210";
    array_src2[48]="19371218631334595336114587666036647600947219659661486833384398886711737487009344445738193366560991037382258445576334";
    array_src1[49]="23286768618469927478580246510179148821132787298142443218628158802685335131403751407778447057314406573179071094507179";
    array_src2[49]="2129676498349394490009639176618080525272354516781259927941465199965307999488870399169174246386969170749965121858227";
    array_src1[50]="9566892616156632564319109999059539554684085106466818460151992630491634332923058677361308758256536274654919575998449";
    array_src2[50]="10180525968392540974764703305528463777333761615854457229728182857172484909665750728258552474537236869355200681591121";
    array_src1[51]="36951818210296277864425393464492697698680643437955029076980127809020112661253173671923419151969725143998518508905815";
    array_src2[51]="10675257768750204206413748980067509497289533261674040326209682003328107737655151667845880780815408832972100333446879";
    array_src1[52]="33959703702032837906534701465320853419105206042182173852784189864020524340617771347763886045964255951414188194126187";
    array_src2[52]="9206003409694910827910997449941525453659920966605485031167118195407691457593068989686129525053465508032926136018258";
    array_src1[53]="30163472968718669240472636566380205998913632256703172398703001199666407253151168672260337647841776444974478234938357";
    array_src2[53]="2094047289892796551647541292689977261900472276211223178365785455722133551961943462653458138390596974838338750796953";
    array_src1[54]="12831643411996715883359755923671830382289125897504483602180906896642290950484609522006430842872845574875109173582316";
    array_src2[54]="19305750215470993697410249662231958675290029501420925906244697057114416205504421568222038453224178003396934613399102";
    array_src1[55]="26670483612549467398287249094681832443416669595462913479260127539570293252776501792183138022381889518591609862258646";
    array_src2[55]="17063386063845208097838774097131426855213791201533077665777722732078962090472093847966759120655705859394846588448072";
    array_src1[56]="12752432711161940250234746090766468198192649553904575858086921740833821266033783367791240086837113348816783902051045";
    array_src2[56]="14260317920303456584135423023596379193420421190509570133740225563108922623024791150129272780193250991759901519862695";
    array_src1[57]="21956637631667992193423812491524869738230071229928067254315250546311147067599630447848461432885960049639328974123846";
    array_src2[57]="13876378379300536827245571747297780425369704601532725920320114158073275416671033025122664301395572054635965501144721";
    array_src1[58]="26036188998266843780536527654063027668685959499623521335917592768808115861319050888816235262262486556856907480505340";
    array_src2[58]="19005409848613928136528074897885593027665267183668382046114481826836971336770720690528512873793962990397672878362908";
    array_src1[59]="6037742074164703497073441387225204746170607192302178722500667188257498662400116814669323521063194431731036854976469";
    array_src2[59]="18827785067022379860467210870210155569292172434757497617171696836152624722709972516544064219526748024133271049652513";
    array_src1[60]="38843295797277617873044580022227619379280289134601967380725820116967916565403064271018063892635860034477980487813756";
    array_src2[60]="9394434588145961352883027190258575694523774271645121765058275303717564467266367032594753817316570820918360036872354";
    array_src1[61]="17667559382260576574365352182691972983012272091001811633777354904250194718908334593556752512705799732319465193349936";
    array_src2[61]="1238002197964127202521226683602491262280198604436413764685641803508565003978509459562928475765653600748778159588505";
    array_src1[62]="20021031946924432906244916495556711361555516769711693434635304597084509301426419648090574789846566508312335790386811";
    array_src2[62]="15910946195983638935587280955097579303189121428647868967749171674736876160856712931183942391651177035346869756319654";
    array_src1[63]="19422614697872934900569098409533003931105196079166917082086600409967495327163113640016898378665228452612784819488851";
    array_src2[63]="4647997110186295116693798350146561831309247504926236303770622569575065657699310143979111808386716416559604895292061";
    array_src1[64]="11788817228482907771583301046005227899515155429453568498921252238902750082082220294538410616606213596971525460576155";
    array_src2[64]="1597659120459680785103243307585708281881985976445459391991869739735827899531148786464204563044545890444331065614343";
    array_src1[65]="19056358048158276698450366718981883691492375629024679854512124120082790487286168768138849217354483814890150853516183";
    array_src2[65]="12029750386927494809224759532106392811395450086170248537247962955785179960761374112505311164027075099503680329582948";
    array_src1[66]="10218666844121630982455033680535586132298060003484520823694433105900892420304771177961931104489715704546452432047844";
    array_src2[66]="701248067915689252859534615826812061139296234705272373342260566306514155214452380743397001109346419407538913681067";
    array_src1[67]="29387578488873999544323569671127288042306393326160307244748215731086486894080942481959429269689390468476620310893669";
    array_src2[67]="16513481173663216126444128437252180660468153056173522703914999200747840616506130520575909417505072462080119276518494";
    array_src1[68]="27671157141752338640737708997524597312606333467414836987424457085019748886668246190023486661520935891823936910096543";
    array_src2[68]="12972579701557174370976068041006101446005293597577828269844189611128581345119066993632146127278537773006450385866754";
    array_src1[69]="2403344984863170403351198948346533075363340475671894421723230765393785196609210284574458072511040755492978818201170";
    array_src2[69]="8355884005484473398557070000047678064926305616783738710420682263978720034170321653001380427647240174921265485696141";
    array_src1[70]="15879956316723287907691720231421767061806757539000038479113779137576885979355379963381063795680481971723531331174554";
    array_src2[70]="2898313638739060531473855103568075065565287822762941295498032374180261572543875143872021152466929128081221061982513";
    array_src1[71]="25211262798586986223874155217046765187530841661449123062289589546338719333999219952438653060058126120268039905254681";
    array_src2[71]="2851600779353746204974930588064647136693730037884159266834307747357006206020936635706849182346461695508609919951546";
    array_src1[72]="34594007922752306779060840181625162714028232373964967930366102715271905128826081100370359340805183818029852261207406";
    array_src2[72]="16241295254897781073637336658996680337267988811597078315648239866567490454093737855745893649375725902673966788889451";
    array_src1[73]="17549048077318504861957360762088167675036846811940299506166340719389593436497818551355150752530394426540520810004371";
    array_src2[73]="2755236527944024182665538744358928852617026633091693487188489797281484581980949582907790519691144497660661350767461";
    array_src1[74]="39044903731410682949431824198878776925859855001817554392760936825118753788635179339213118013168530136280725175965059";
    array_src2[74]="12046288912923662508176687366370855236065348514103262039749408789023073262827915615869409621523455457906891251859164";
    array_src1[75]="16162511506871572708605276883432409266174759095141171041552234529054150673187112518725535471872219793104666155739898";
    array_src2[75]="408383520467324477982267572055885921462679087402136433896989535021257422279219977369787026922396826526351066160248";
    array_src1[76]="15040453928659189432759225347395968498622958751839683315485090086668073203461787953923043036802855293054770161939362";
    array_src2[76]="11838064425710235248173810322043212700534037156037956636463018041245136715206788392489032764939621428939420302253289";
    array_src1[77]="30362954200959000910311253167861703979557314635389995254682386309638687537711626216987824277100855055556336152135520";
    array_src2[77]="11341151331491967312215471517625802457605318958480255503681393553476235417028428855130381458865721526156356369955007";
    array_src1[78]="36740871968196964453796725442223566161879733457887487522593753671179318497002899349843279882445641365647886445784693";
    array_src2[78]="11930194669744345737006682317844810133023424873365450003149622646359915537071621921782075618849598710751650227355624";
    array_src1[79]="38671501825140444328387163246959812181561460510037996864963292609457275021966002183029742980171412216193433045609747";
    array_src2[79]="501789880460175821490730465091739369030931601871207645424592885118535594381783817813908310325379234097793023712003";
    array_src1[80]="28520471046431029111956863917963191047521299342680508404019156568322560438136592324196485111791961047463366738579649";
    array_src2[80]="18593602872761787954597206900246092956366556500550852434837484518722304787995540898620712503093928405993531602811936";
    array_src1[81]="4583758404484326141609093887340460847088925867786341022440882280785607644903045050710879476209069925928019413375350";
    array_src2[81]="10033282494517365481594663396538880690141655773432195274630131805654749069233075020344781157130312652228587723693654";
    array_src1[82]="17855008464755239123568488917298637017422235627824739489145145795556506231046493027061705649925085768083446886279754";
    array_src2[82]="2610452202842066457011007643830264215875810494079510340757931172748543855873676194542991814889000528873327934621583";
    array_src1[83]="2199016057675522511034866022161726379057059353486913101444630624004120578743224172139860787881285833267887895822578";
    array_src2[83]="13763637485463920857935055547787148209662476071606872504203642215252226315512957803951676316777035659549752512863295";
    array_src1[84]="1773428026109102996195101358723720274391005456714572096204954641508606404187402678395342510632826097237585470403621";
    array_src2[84]="17355326174828604539637768272304005466892915424385285608865224382269468308850143143020268880663991863743165495484565";
    array_src1[85]="35921073440815100311939946975110458647250789405812029311210203625646175353587082414504739291410996904546934430264763";
    array_src2[85]="6936748260126148817137965739114780819862723412561753582707621067379129506073186598508771652247949319088167476180328";
    array_src1[86]="26559095994466783315810023487733475571658544288994426550979209405883872561666895470298966495745205441696772577907943";
    array_src2[86]="19383206694678055625155161968534458357737084732013544050630043891465665069038793380499746052324878479392170692644627";
    array_src1[87]="20190105340804577842237196825666883579189089661222664416796550646290285898939545469051047138339630683945034999404881";
    array_src2[87]="15146281372832670159460654562890630077947851932342044797726213463571455808744317900693185544174835843365136639285835";
    array_src1[88]="15980055823363162310041579067646515858571288926948269071173971540729936997599154987692076845841375449391559464038738";
    array_src2[88]="8672624943809512523266727739780082776650749470377818037424897426327603073587916015802683321626281524158842974116946";
    array_src1[89]="1631087243137281681707807941189211255433284526517007673620953866635766101228564439938028763663628525324768523348160";
    array_src2[89]="8412946931783035039177710279676845285597156841168261979136022604384503866323954737124825108436508712987086524269546";
    array_src1[90]="28258592817524059058454379839585431261267363655193675112103769732361316757559343077344925926491240571685725908101977";
    array_src2[90]="19588769083286956526596042128423323735623089563647485832018779060304666228191799833228072669443907357857289636206847";
    array_src1[91]="22907259320614704220303996989582351765839151985242945087794954428090575854876248282223020187617985091158264309466768";
    array_src2[91]="6861507208429663645487088599156640407978101626833082243895800975950130287863975648940209234402571868166626058443418";
    array_src1[92]="16242942483184037385723165074286993972375485577349257094181718938703166480185153023777291326483275800139275417133259";
    array_src2[92]="19570128087089600764024525688337475656020389664435851252954692342523764464225102364779522475778114077372301678658560";
    array_src1[93]="22645704553047198906155266549124039271854505469092773365906104989967940386811042372138231911760486881848256874838587";
    array_src2[93]="13827672232242679224102976930472842183467511074649638186383222429852860608534292982326960517984338560455821086771557";
    array_src1[94]="11113608305474280836889263370989835954788999532447407269627956023427238079344327472939155175535285303623095397333844";
    array_src2[94]="14524438471450514438337530412701373078403078968601411393768072275595814986360878184868746303572459432925904080916485";
    array_src1[95]="27277548459954396269843827989481756777895423581826291082710282823517080691429952955843210706176436280834525791432033";
    array_src2[95]="11019295037036206407874133382247845389461786139400065331532887707373174367858476990775872248874437306941983346537904";
    array_src1[96]="30271703195716515541181027370416015325441850583576134533002546904579872442864281438240645839001225019601569739536189";
    array_src2[96]="8843354121258489709902844983037078201289347265164272516542193427995815195777426158729389427493866089397575054934384";
    array_src1[97]="19064831356915357284093914352650423130866226647341214542589824973796397611165562154427901986357124630578350378996415";
    array_src2[97]="1996230898157357017726439485598405373458723222121179908690974191460954870490082183932557843626088304570970847312377";
    array_src1[98]="27620720846773246324543289613150478724485743974218893119863178544531214093884969756519924367384103966471425658558189";
    array_src2[98]="14664585008272728275810273876120238944695897424081870730778628506961090001202862435451619991760891073478420196323769";
    array_src1[99]="16849637796739976864850518568464276259075000695136223394061448973159919018879356117064801102566262418119770128787769";
    array_src2[99]="18526001416576642487324442859348234959364505321784831678846742012899467488219887607985155663478005627382639306656372";
    /* All those coordinates are coordinates of random points of the elliptic curve
    * obtained by the python scrypt below used in sagemath 8.1
    */  

    const char *darray_src1[100]; /* x coordinate */
    const char *darray_src2[100]; /* y coordinate */
    darray_src1[0]="26090191723284819538456389599597879902554185171899178652909139443215042877351545135095151961628600472323144025691472";
    darray_src2[0]="16202919490439535928192167861330532222816756742072455869133735629713699520149632855542061891674458588041672567308959";
    darray_src1[1]="23967895705536519174948424621879807260348854957557173414108253897263829141587282504769782311547454463813782031191311";
    darray_src2[1]="18285272607043641447290665999321956216510459829260508524196781023400197824035400586022715954032654803910165639072520";
    darray_src1[2]="16567194000473229627438343714139086228189039570779673162610348553701034004803394867971513913496067378120163714412491";
    darray_src2[2]="5670191295022526366294692210706623836215103090815176420275709444135181132660333604190037726509852465856181572637108";
    darray_src1[3]="19704112690567426947086798451701577071663699570251656032234748828421900484490141044913155858647014594910430333211106";
    darray_src2[3]="11721542252925818027341744049519879981457985868161505809948171985036346154036741188422394113947515040909211814890663";
    darray_src1[4]="32944407010237131088964645195243173498349354058544581884826465398867326015437721500497009588199965468210165440075475";
    darray_src2[4]="19320193315364359577629254298530327835096542694066604918502080486999823985969742424514752175401257266473018851483045";
    darray_src1[5]="6235065350146476366347739590682006412071337081021081401546701684094445602194027015722496615939596527437481581928329";
    darray_src2[5]="1627999652989743949696821596449085276173925152714260373848050681844946097286343389659235483366350635754063687076517";
    darray_src1[6]="21302319581269170352761841366740427100264255894466656827967189387875557533154745330020184244105390530853447359362916";
    darray_src2[6]="1283543802956660869850750913661253996064870220667252602754791059106969314597850840166009110748396511573507243438518";
    darray_src1[7]="9155490474091914877323063496136970603660895622775221291164992090342511122812335152912369683313866670135160974140829";
    darray_src2[7]="18557146962828260753257112888868237711898317420508314392518106254687715022657589144823295734279345995412128335058498";
    darray_src1[8]="35066962592064487396132663829384936602669505031338955301898235719745693007471481292657659578560802331200760585321529";
    darray_src2[8]="2762721567655553558146417962992455221969380378295765574590462997745477910375806425326135946449717210537710016077318";
    darray_src1[9]="18886915399875407981068619522260279698553245972099766058355741245328070662283928621761842821455032031314434410113918";
    darray_src2[9]="9541583868494745999598111131400734296463911462291415911665117905596900337372802695588227264371326040589748650586164";
    darray_src1[10]="15067502265017238648926928425712166278907087958809777568156414033637196370899139700729850522463420519126482694115243";
    darray_src2[10]="8934611432814555769390852878876771968004797529315943191185291572982121117103294338040238171308459714846242091089434";
    darray_src1[11]="24988437691319889447332901053531682858465708104729592769733373938331093273855779710668632133170903885808287249038992";
    darray_src2[11]="9622095219645116230465292082561096249929120092173210855755863836810730584612576955711293134047681022150132764565013";
    darray_src1[12]="21897815702952437910819325742062896028159740233266543601267781997315837031186667901441703332135521233222923992023065";
    darray_src2[12]="18537634685497217815638832534885181511607145418078581749184134741434409828867091289227849640554983964564429823936331";
    darray_src1[13]="37791829785841905715384368378442107379513888775730116451032238189763943683929290927460962440991243113047040633084347";
    darray_src2[13]="10411066027899904679119938718010551693031208660644847204446996654483590617115811776951640790354835494722889615269321";
    darray_src1[14]="30825900906465053932871414518474302487175761310186852845224265739770768989204311649188617879785016018194535563873574";
    darray_src2[14]="14960445058498094901379167335516167220246344408766473067772801195645435611571773562335947071762463072201498035749754";
    darray_src1[15]="24336237596586173819879018740432789616493007341757076224197891812101576518867780347593400521722955687299528154651332";
    darray_src2[15]="19216453221377612925747640728556462442376250942594488187320313823554749438010712945043162988009658761832339263413511";
    darray_src1[16]="15808669927773631868952815682070647736291078007643304344525254405162268572384390756488647193556851558262140270509172";
    darray_src2[16]="8887337673236525422244919851508208688831090512146595238964934635624022610668289805190142789622931240222953389731002";
    darray_src1[17]="29492649684128882336615704906880675148472411845589287531579226712274686208246438411777830449423903646110039391367059";
    darray_src2[17]="9991918621125019799078370175056422633190703833400193456245690069161468240542173784936058505124349254885946983732455";
    darray_src1[18]="13243546029027319537628782377001562662520885070565100610063222881442723020277304643257411142843096610031023588219917";
    darray_src2[18]="16591920918829476787021668400595159122217219810536773853679311411708068588475300922451147023383325871996659753077577";
    darray_src1[19]="24430249518855058771394475599859414068528649284877742648072781597732787466563401099379217700742068334575078841710068";
    darray_src2[19]="7494616418706693220281816211592639382812952758229749541484986619115534722557113660098534514156474638628756761667870";
    darray_src1[20]="28087075885522604981572263594183281622283408610813164444923649250442178412361036905371344275571865604806859856866480";
    darray_src2[20]="5701365766005475338131333669539247794466542794735974354087621101913444160140634747883140440912412077767544981115781";
    darray_src1[21]="10994145920061823939015168715973478807807502740212513345981799206377603195824769379736581794580255595409149294515898";
    darray_src2[21]="14971756209751048275456561779885278451558289291744617822108161674885982595212886407100439724356204290759684429102884";
    darray_src1[22]="33920541941471027651386505496863009637609196308019978326627813588422448591937651184358571682154653688023033107446260";
    darray_src2[22]="2824046072669607797571859682564632001853318742070740475046527705176850992924104530432306294344377722901759651250272";
    darray_src1[23]="35677150428831792483484588147213743254994207318033812051956822665829895100985900954730541637000542821661910433438435";
    darray_src2[23]="321833944641360670423641388956530738672363003257476068402109511961918539637751796774640654140490142487608867710049";
    darray_src1[24]="3123914154855929008121386494004804456940227473776516958593420988629649426014744143818848598253438231834945398394044";
    darray_src2[24]="2057795954206907845055787902749886477602998493515132100099593846232660660901339676370910513417196415486385039284348";
    darray_src1[25]="20930456857177424666100820541981728447535258999076379814215770994865336255872707817759639559024269981119392403133139";
    darray_src2[25]="5420102678785825378024471547037842487540483638843272762695887345115815793155537379451707330887062723784667357597315";
    darray_src1[26]="1733919271585243497207266712933223013335536960459867507170321015461821738738805086465399081598605220553160371697364";
    darray_src2[26]="1347260571195854876164928761175581473145629235443190356731723342647924512202964583451559682775400975153951721492403";
    darray_src1[27]="16391453546604923957368556659931302938411411449448077829768642224810656559059141038757872883501353261755220747481206";
    darray_src2[27]="5439415456944155348414600791550124932029579062762114916604071240578721686540698370476570154987813221731785276188143";
    darray_src1[28]="10963616939542507278709199495086807291589433711572055626884705621684201993424337802449918384782202463129749966393003";
    darray_src2[28]="12333323290358633891576616917367423799928943291111999259030782015450925969800771948927646450100864764521630017994476";
    darray_src1[29]="22540347213175966271317847212355118308574403851873007184434147723212993678316354936903573891590425491695475904027987";
    darray_src2[29]="15520320200285552505411805879392447129446032996538445862590570415624756506674990233431234761582775372648767473364724";
    darray_src1[30]="5278915941166596322426733689963891091534600974587668092754031628232970515214093420590226545877131645462746939130988";
    darray_src2[30]="10765656393121803984331232727649640904658067035694162475384456232199758531479248372676985137203839616495556721199374";
    darray_src1[31]="24757846816308595694446103934641677040495241033695971146165412030611330222611982944115772325756365093561361914641468";
    darray_src2[31]="13525971420200538997297883997838898390847518234253749891784620175999438373041131815807771826531456512701549652079822";
    darray_src1[32]="10053795442010801199133400701637630710677567527434619860303958272006384162050186286406545883630654310406094019766203";
    darray_src2[32]="11264869081399166152326850615264460747056536205377415554539725826157745660930871218841158409738635239554483681452502";
    darray_src1[33]="27975577797630761563865482836365683231557138711328284928463922917118727959136910580398169297696636775059796846199531";
    darray_src2[33]="5669447280263558919186166324159525401632230751575405308517320913206278561694637193900291748595238663159497316445963";
    darray_src1[34]="16553262813523786727500537669921739806321005129621868558285878793248243239870399152358983716362747597698003088185347";
    darray_src2[34]="12580436335481148340466573626556638079077783940983325806809910459691486334625812805871308832262760856320300658332436";
    darray_src1[35]="23328815034757422893652055127152078723664293851650326088904288587405398395490388664882706478893849365113761580732957";
    darray_src2[35]="1254505179890798570357947562171789108233021665585132703038563576306008054898243577040996954632878503442062016910705";
    darray_src1[36]="26684883830894865882303953243411175383072208325746131234048299186778189705964312740969663810358273192788339148177002";
    darray_src2[36]="1754081479810357772454275507126993051998707051000226235054125508070672881900142585119372662981612483480889590976568";
    darray_src1[37]="27797159196378192492513813978116899188737947128336286712696598409762946216431483056179356883422896870326520769274352";
    darray_src2[37]="2189821177952422718415268317953319403690455286668263050570584928793625554041986335359188564038039990271751824424005";
    darray_src1[38]="38160345675238706448699086263434221087476788233439839968856669132720716573983283383086896804876322455766484094154690";
    darray_src2[38]="14879335051809135072708364037365181774151619312661391058706369227754512586379442336962149836141593017576306017962970";
    darray_src1[39]="3942180127459379399520708726621013652892305973712885951397017823057749063743129634762581573623970692388186181859986";
    darray_src2[39]="6376385906023090832941598041666272295039855333683399877687829182011712635789159631733817516209955210277635089420326";
    darray_src1[40]="32899658393210854176194450481238646450552864212192909383222805724544367470895213576432259038289445454642639330240375";
    darray_src2[40]="16837702488690448336480495187337073669820912297503961474555749766958765972207340359141026305134462536636049129916537";
    darray_src1[41]="12778827629011722430273737514044792344266875650991036252420213179336976004020920976243541540760504116089037565737414";
    darray_src2[41]="8321158634852849392486765742736800635605218869314088599591241151194677790635293355170743170324218210811757776532587";
    darray_src1[42]="25893779209529117038060584392185479699264360920048711468912169953046516555866178320404239395225358754686765221856552";
    darray_src2[42]="5917166076790483191141974740300416188846174918666016622786851643693130969897352728265181324871636930240018905383346";
    darray_src1[43]="3084113237733573522630119067262628740881800217854516054932473370122026251794192885502479954890366476210991797094122";
    darray_src2[43]="937167458261055416375813536917356038862028500446623392618142652541289762485924337628727829970332374986733478541340";
    darray_src1[44]="17024169559773654077471766337496435727905416317015621532849187099435345199238204225186691782812258144899031636338281";
    darray_src2[44]="6567623119744116825139961686094074677256318402942081421260725263724338925685625796853377994826544830282261497128946";
    darray_src1[45]="8995676008181285497130517332754237063878045216967139908313536823081552522591613195613085692913406004495429617522771";
    darray_src2[45]="986422086394869598460013823099728336729499681580819543187829415761856984173556520868736630832084150198680932354145";
    darray_src1[46]="8526493000456690681590677284054777415116907737894148629704032637830565278389697519128514268023624505928144011840139";
    darray_src2[46]="10844734808851837051824180996152135280025445374645767789555078790904617223689124663246123350342803196864669115757032";
    darray_src1[47]="34060798640423397777284163138511924873336128498990448086258426927609083807057595499422044936198566569449731316653476";
    darray_src2[47]="14849122583832119375558878760042193666905849501439078308931316591532624560513199007897725925972880440555923024686579";
    darray_src1[48]="32212234296436160283919774083071300644763387291112060706859118609197928137111891303803890609266209262570736819616556";
    darray_src2[48]="10978128425292001232309279988173355587764388575581492550194952429576840036704569876235391588930809930034146534938395";
    darray_src1[49]="30243303429680520725303241648452198176210056786848256522918948326843320086946873512658125295636004968804584784570702";
    darray_src2[49]="2377478841204611241551241861501938247615328889964467466764967540760570638014872756057488235861127875368220012471145";
    darray_src1[50]="32291312222187264805702310559257226617699591377800489028086836394435017688884452122070866109945190371527946504652500";
    darray_src2[50]="18380735324688671939789465798720021292857280645548019957168012517505677396693233537442711825054366820496961229700449";
    darray_src1[51]="17997655806873870209072360788566114873033239033671264549886568910816681986235249554715065927507946638352364990691590";
    darray_src2[51]="7669148712638172356689773848651862772138130852777948594319527932192192513735951796287361019064137331007580175133205";
    darray_src1[52]="3359564083203689636519184729923578440398631930441177321350674542409084344147801166486362066863446401123027712468609";
    darray_src2[52]="2932448362991657423130623800204686675955230979502238858551125773383584913652277270387254053325426703131146808842323";
    darray_src1[53]="13722795663962815991134671815874045014436135985445918575894165868575002798154542096541987152615545321158651305743026";
    darray_src2[53]="4794631849903355665917241641015513933090293738510391106575640049001753692615606615268294920217762398723509714090969";
    darray_src1[54]="3237615726513880743577769819223159978016993118226906497168796373470594187570974446254039486355596122927003288115032";
    darray_src2[54]="6873432046054153890610899808269904754007233791361581830297794744121380589781904044231835444636821116272323673596121";
    darray_src1[55]="18536271719349618918771414499816933165879775277301270300516567286308092820093918182657393154942660948268515930659812";
    darray_src2[55]="8551190308460410343517712301538736549584909821446374726775906691338022182050552334774877525138589997106467891032654";
    darray_src1[56]="27242141002965812792683317206152837558495878115305408403987528024072927124463788746502770610832585967058550736773598";
    darray_src2[56]="6109858400943054877300012513965119478666684558096367394961099167349916289902177184951028359457078101978061620984982";
    darray_src1[57]="13519360690145203743956233485217180748500013122064834318448951769619212410173207116710746939821059188934189718173082";
    darray_src2[57]="213536603843933754635492127346530266752123886178518703885552788786228358959855321551219037641865955195159233135170";
    darray_src1[58]="30873338063061552347570821944573186049243221068819622977255433133773977366001795119667089624775510686994800645695686";
    darray_src2[58]="10427384919014687265259605090494510598147032246618877472342647563642501667142112423341167096795931072499009099724161";
    darray_src1[59]="11972789833204298623567925834852336847509169054616163704606420541995928034186648037253208875821706475031472529159225";
    darray_src2[59]="5524912551773601750531434763511220837059573194781538623841848398398687799959981454536784485294548748311504324380214";
    darray_src1[60]="8539495005516518261520507876217362652437541274707128739545163742801166145308047560615698128849181121193171381321405";
    darray_src2[60]="10197067855914291383722507379519336231851415276360989317649814239070573425863195192606980966696411768584012461276495";
    darray_src1[61]="11712736182257195540959279308829422425091728108846924219821517098936908451808812139946405392792883836077142175556413";
    darray_src2[61]="18964579342495998468162402514274051620952406374147122937749498956004072899214499175525806858746236082478273260817116";
    darray_src1[62]="9228860874957777259223711119722629994933550679646710278325989059146018275321761191267564694177367811319704952679436";
    darray_src2[62]="18779253835747662110723509069880822533610133118573310217016278313135043613073215886841267960347293091424839916362611";
    darray_src1[63]="30239274140262361177653099866258620242704683564623016509607010545039681014585911418398529602076242886029260232253311";
    darray_src2[63]="7508176484687610906889135920911829890604907786849786353202989065984882659752429949419656469489583253730553697111142";
    darray_src1[64]="23326130801757755511804072192285387958884404822482251589300698827586564630702181838978934804953207536941743542132463";
    darray_src2[64]="14187392535733742265420784317426834052651302937031763306633386149221838132247878335177628082364047295203611491212983";
    darray_src1[65]="16603166314458704620896377866276790429517605948206046400069731783934957461646559324077673545201667820957834499116314";
    darray_src2[65]="11530972581055289525064525515529613597902980562750083597922195805089810582448412410684169989435742430106926624753282";
    darray_src1[66]="10878608953243428416959946173944891674380229949089308723667943889567573040546814591293027579224383443774420258547525";
    darray_src2[66]="16554405082027571419165511058719236109816175152935444298845037123370367048872772444019062418982508490601765996429517";
    darray_src1[67]="15894111129269274801759102302139329463790855804375106709839723419501569358136055027967310139200315351133362310924484";
    darray_src2[67]="7541170500725810547067027745964675798310572016387407066116764721059045748260578879686888648168926821105600077650473";
    darray_src1[68]="6611682208427248052501205861261567619791835956859531281041008288345136087900275972132832632683420166691585520180436";
    darray_src2[68]="19438913696293495690403319107752248403670525920173930284534023862640692673523811311935979690438348356830141840227547";
    darray_src1[69]="2268975541486550755328072246954619750483034595529040932709143310083509521404042942161258708424504400376134028377708";
    darray_src2[69]="13234150093229572262082856795781814168398315994335203121291054302765774045371206806716154212102934523151482332520458";
    darray_src1[70]="19892582974543231415649644986015614811171206143475766683636607534501491153251868987482576339933569904801848552520494";
    darray_src2[70]="19563663589826560728944238324611344587028719249117416927432283705183589016277284737634847160102978047409176386725250";
    darray_src1[71]="14964355799698047807793211790182920503689261822722357793717727519567123268970777166496361824709197339649871512904646";
    darray_src2[71]="19162914179723996410556084982536108945597405367902817417982720317125828846938823686342848927135551820532117359992444";
    darray_src1[72]="11487055548981242053456425843007333189459819320422282527045693775163052515363184590722448149333772243954421791923937";
    darray_src2[72]="14797141528270361544597577512009374738916018932295462075061755246289512969616151826992144573760233449366272118399539";
    darray_src1[73]="12611851919500358740596298985277377158853742253434805295679623287926166861228951548730121024389039782702115396293019";
    darray_src2[73]="4285933748022622095560154031865533270090162862088787464510078766564595130039832443134698695913448617384392380336393";
    darray_src1[74]="35610108467544351967795734726888641046179184618559304085608204025153309215092591850135637416778113749866167958193975";
    darray_src2[74]="1191432924519261632354885773113029832416128098345479905948925173875827538365430241095201069067476192793808198935889";
    darray_src1[75]="30695038333035369702597996247780601734906986143918795807504705008444956455852359065336971682552922186472616685336484";
    darray_src2[75]="6425714447303932301430468276902237028611488164141475252905561620114481135307265551420898297914396162072698396143037";
    darray_src1[76]="28682325739399599850601981172714870795202346208804662714081246302135031231003770430640330895422148768037788738659670";
    darray_src2[76]="15396511014302253788935171084004161757906040380442038456514818456748317844312116877911755723051541712795021330919064";
    darray_src1[77]="19720557896951152887742697371960822383200808481073388289512886844191374457702610486424640677360934363021601327544109";
    darray_src2[77]="7410405254215791089191151971831044286646669361449555579441054353242639968836746754616658773844313662107615523093802";
    darray_src1[78]="32390697127218835243969189723133920202375151764037269258589694521606168904250247713361037832483778671146448481161444";
    darray_src2[78]="14125736253711443158791140460107568400962596251311341163401918323173091520913380751399156593226186179255340264006429";
    darray_src1[79]="5897595676706636754669776826214044530469094940399219657281510731926562078763731001088399098469528867233365214799995";
    darray_src2[79]="4456543583393889917372134503623696485803419197492390376441423728365426097710714706793050246245403529536093011187231";
    darray_src1[80]="16270502900589756944013619160653247646385296700511024655986051668492499253079931656180077123268498552706124175692297";
    darray_src2[80]="10839936809982618517817839273173073351562471683188166484850438456600731242619451994202173313083317868050517655803002";
    darray_src1[81]="38849462969377747378319918065784870539023105565820509313983465898258235574223145238663314693298477828318793604630531";
    darray_src2[81]="12327440444624294179212489843936152591381757330599444174013882596324714346407762770708072413729738862668821822848010";
    darray_src1[82]="30686120362939191013697400976917943944554219044738119745747054579669467132426534058673511151133177265908431578038144";
    darray_src2[82]="18084054828608614531493404174814946949603652171100758268056225678170576578641860032045449828979786930482824308359686";
    darray_src1[83]="11270702865941964927666882462543985987005560123230899705291326796129818223969524908050886697099534870507956662534137";
    darray_src2[83]="4397997943702141544014595780960742721175839558007099689068021735945625850242535859009082881436642369099793288799795";
    darray_src1[84]="10554563969685666942329024032283553635833569469761271218741232074287314903113868331798695829399171077317416822259588";
    darray_src2[84]="16321321651028870676175786773242660737019993718427577012603476549670746166643123156194580286085041065019284189727184";
    darray_src1[85]="22407584270570295465144914071753912515979766396504789126166629808588478869238030627208369623561634134785734334196779";
    darray_src2[85]="17025025612136937663090251544578800737090066587425847029969034848957567377634568647707581524069917007970713059501787";
    darray_src1[86]="4944087514251779623068801322920720516795800684963512993353265191003849362149457297030986281606173800652721557058030";
    darray_src2[86]="3451512276906899004646923115817846978798698510195266758959699299359299743663786427513847223766093912803978097523468";
    darray_src1[87]="9687215106040844967661055991453851970185273475400685377214268995016211770630677171617342955881484750105012834423630";
    darray_src2[87]="8031974492898786105560860601978899307122619985433142966903194075533650581347756701147390718475907532138953534679817";
    darray_src1[88]="32758558157959363907105471279881790490706870579015840042215914215139614057283532752752629747769940605516224019531880";
    darray_src2[88]="4932209106830237436182904923956259361207546308161950858468637562545595103490331255703325655343690786005222546873195";
    darray_src1[89]="38296600097718517606905811990566554648695292285783803145628931576454477066573693276317949445339088640445317119288402";
    darray_src2[89]="12780458304812238264104519163139230473073161949497737428170080802347976629263192168049888364321851094350616420780105";
    darray_src1[90]="39103325147676159426347541634541085529865175952674778519209889384260480970466512179411195106397118158206388755325953";
    darray_src2[90]="6667417319943443691928298166967572606678473015633420605938837188205812949217065840595841238401343781800703821532444";
    darray_src1[91]="22206218224763311566268220678548350621064331103595698041632226239935733835971424330136081608165428486859358051969848";
    darray_src2[91]="13281928503267446785897950668420155658491374485056616987015882548061953513730700866739557970685247315540349070928701";
    darray_src1[92]="4083948941315667817057149681373080362392321106319879826502272601148326493655184708746007890995307116065152155882513";
    darray_src2[92]="17277894647898584797474469418158192748276495969292171316528193681846728047022837002093517127414426201364528389479097";
    darray_src1[93]="25208758243053106559808700637561369028573761742316713087671109526263726462045035188135408371311707823560451030570627";
    darray_src2[93]="3732982771202803114164936804419898280502036459739448666075709846021093721068272178712368667694946490114050736803739";
    darray_src1[94]="32906299296330114926691997110969634747379270291367873326855462090957197133274321164728678666917108299762461084536062";
    darray_src2[94]="3306947568550225718386771066381501403824260433567400946271635749395529993324447204017971034091627906046283224930872";
    darray_src1[95]="11589322588298091613018328540812440601398397484311154610720650012368989443652189516824427333647097464295365767263306";
    darray_src2[95]="6889861123453273949222752054101106374715170845394724979976919293198536578727302259867273624178165702593572082720821";
    darray_src1[96]="2799994095234242605907521366345838579810191463939235166155808316780988149379620090903193010024467586675051924831934";
    darray_src2[96]="9220698280480521415543866778225543423612182430765916036556843067258705642596382543138522930112848058932751525223189";
    darray_src1[97]="31629753898502625113508742386138328334958826303113032808233281015801167284257647156671720485456496010121251800717716";
    darray_src2[97]="270694786589140620695484908057499973170033690303001024934604346330453035310789726730659570278529899192764591999953";
    darray_src1[98]="23982379125306698564205244160463093276194986968695693366545446658843331490476090668961277239287849207668247478987591";
    darray_src2[98]="19489077743059547484050307155868440833656400104378525463022814392388785354983814315368074957821637790164816121455332";
    darray_src1[99]="24295039121220204554598155834013952010261196895204359195522021025200030393215042242487094826360140433061844523400495";
    darray_src2[99]="16203011683826544906528448186560162301402794206219719859643343796836685116378440653765879171611755658974362796386669";

    /* All those coordinates are coordinates of random points of the elliptic curve
    * obtained by the python scrypt below used in sagemath 8.1
    */    
    
    for(i=0;i<nb_iteration; i++)
    {
        /* first we test projective coordinates */
        field_elt_set_str(x1,array_src1[i], 10, false , k, STACK_1);
        field_elt_set_str(y1,array_src2[i], 10, false , k, STACK_1);
        field_elt_set_str(z1,array_src1[(i+1)%100], 10, false, k, STACK_1);
        field_elt_set_ui(t1, (block)0, false, k, STACK_1);
        field_elt_mul(x1,x1,z1, k, STACK_1);
        field_elt_mul(y1,y1,z1, k, STACK_1);                        
        ec_point_set (P, x1, y1, z1, t1, k);        
        test1=edwards_belongs(P,E, STACK_1);  
    
        /* secondly we test Extended Edwards coordinates */   
        field_elt_set_str(x1,darray_src1[i], 10, false , k, STACK_1);
        field_elt_set_str(y1,darray_src2[i], 10, false , k, STACK_1);
        field_elt_set_str(z1,array_src1[(i+37)%100], 10, false , k, STACK_1); 
        field_elt_mul(x1,x1,z1, k, STACK_1);
        field_elt_mul(t1,x1,y1, k, STACK_1);
        field_elt_mul(y1,y1,z1, k, STACK_1);      
        ec_point_set(P, x1, y1, z1, t1, k);
        test3=edwards_belongs(P,Etri, STACK_1);       
        if(!test1 || !test3 )
        {
            test=false;            
            printf("we are in the loop %d \n",i);
            if(!test3)
            {
                printf("test 3 failed \n");            
                printf("\n P : ");
                ec_point_print(P,10,true, k, STACK_1);
            }
                        
            if(!test1)
            {
                printf("test 1 failed \n");            
                printf("P : ");
                field_elt_set_str(x1,array_src1[i], 10, false , k, STACK_1);
                field_elt_set_str(y1,array_src2[i], 10, false , k, STACK_1);
                field_elt_set_str(z1,array_src1[(i+1)%100], 10, false , k, STACK_1);
                field_elt_set_one(t1,k);
                field_elt_mul(x1,x1,z1, k, STACK_1);
                field_elt_mul(y1,y1,z1, k, STACK_1);                        
                ec_point_set(P, x1, y1, z1, t1, k);
                ec_point_print(P,10,true, k, STACK_1);
            }
            
        }
    }
    /* We do a second loop to test if we have false positive */
    for(i=0;i<nb_iteration; i++)
    {
        /* first we test Projective coordinates */
        field_elt_set_str(x1,array_src1[i], 10, false , k, STACK_1);
        field_elt_set_str(y1,array_src2[i], 10, false , k, STACK_1);
        field_elt_set_str(z1,array_src1[(i+1)%100], 10, false , k, STACK_1);
        field_elt_set_one(t1,k);
        field_elt_mul(x1,x1,z1, k, STACK_1);
        field_elt_mul(y1,y1,z1, k, STACK_1); 
        if(i%2==0)
        {
            field_elt_inc(y1,y1, k);
        }
        else
        {
            field_elt_inc(z1,z1, k);
        }                       
        ec_point_set(P, x1, y1, z1, t1, k); 
        
        test1=edwards_belongs(P,E, STACK_1); 

        /* secondly we test Extended Edwards coordinates */
        field_elt_set_str(x1,darray_src1[i], 10, false , k, STACK_1);
        field_elt_set_str(y1,darray_src2[i], 10, false , k, STACK_1);
        field_elt_set_str(z1,array_src1[(i+37)%100], 10, false , k, STACK_1); 
        field_elt_mul(x1,x1,z1, k, STACK_1);
        field_elt_mul(t1,x1,y1, k, STACK_1);
        field_elt_mul(y1,y1,z1, k, STACK_1);
        if(i%3==0)
        {
            field_elt_inc(y1,y1, k);
        }
        else if(i%3==1)
        {
            field_elt_inc(z1,z1, k);
        }
        else
        {
            field_elt_inc(t1,t1, k);
        }      
        ec_point_set(P, x1, y1, z1, t1, k);
        test3=edwards_belongs(P, Etri, STACK_1);
        if(test1 || test3) 
        {
            test=false;            
            printf("we are in the «false» loop %d \n",i);
            if(test3)
            {
                printf("test 3 failed \n");            
                printf("\n P : ");
                ec_point_print(P,10,true, k, STACK_1);
            }
                        
            if(test1)
            {
                printf("test 1 failed \n");            
                printf("P : ");
                field_elt_set_str(x1,array_src1[i], 10, false , k, STACK_1);
                field_elt_set_str(y1,array_src2[i], 10, false , k, STACK_1);
                field_elt_set_str(z1,array_src1[(i+1)%100], 10, false , k, STACK_1);
                field_elt_set_one(t1,k);
                field_elt_mul(x1,x1,z1, k, STACK_1);
                field_elt_mul(y1,y1,z1, k, STACK_1); 
                if(i%2==0)
                {
                    field_elt_inc(y1,y1, k);
                }
                else
                {
                    field_elt_inc(z1,z1, k);
                }                       
                ec_point_set(P, x1, y1, z1, t1, k); 
                ec_point_print(P,10,true, k, STACK_1); 
            }
            
        }
    }

    ec_point_free(P, k);
    ec_point_free(Q, k);
    ec_free(E);
    ec_free(Etri); 
    number_free(&p);
    field_elt_free(&a, k);
    field_elt_free(&d, k);
    field_elt_free(&b, k);
    field_elt_free(&c, k);
    field_elt_free(&x1, k);
    field_elt_free(&x2, k);
    field_elt_free(&y1, k);
    field_elt_free(&y2, k);
    field_elt_free(&z1, k);
    field_elt_free(&z2, k);
    field_elt_free(&t1, k);
    field_elt_free(&t2, k);
    field_free(k);
    free_mphell();

    if (test)
    {
        printf("SUCCESS \n");
        return 0;
    }
    else
    {
        printf("ERROR \n");
        return -1;
    }
}

/*  Python scrypt used for the tests in Sagemath 8.1
*   (some points were tested with PARI GP 2.9.4)
*   p=39402006196394479212279040100143613805079739270465446667948293404245721771496870329047266088258938001861606973112319;
*   F=FiniteField(p);
*   a=20995147416391166677980721228885546002736842251288287164126872820502391670380279263134112406390918551866357280904409;
*   d=6025087504798599993064568591109256057249347913734633918277714546258781288865070391066231780844231520436406032831501;
*   i=0;
*   fichier1='edwards1.tsv'
*   while(i<100):
*       x=F.random_element();
*       u=(1-a*x**2)/(1-d*x**2);
*       if(u.is_square()):
*           y=u.sqrt()
*           test1=a*x**2+y**2;
*           test2=1+d*x**2*y**2;
*           if(test1==test2):
*               i=i+1
*               Fichier = open(fichier1,'a')    
*               Fichier.write('darraysrc1['+str(i)+']="'+ str(x) +'";'+'\n'+'darraysrc2['+str(i)+']="'+ str(y) +'";'+'\n')
*               Fichier.close() 
*   fichier2='edwards2.tsv'
*   a2=15742133033998992807823582249555265577773631874428909893729468807030212686629179530841549589979116091838995361331401;
*   d2=10641385055334172958167632272894720786081789938450255301242446722215388684623784557888416886572546676935038402790360;
*   i=0;
*   fichier1='edwards2.tsv'
*   while(i<100):
*       x=F.random_element();
*       u=(1-a2*x**2)/(1-d2*x**2);
*       if(u.is_square()):
*           y=u.sqrt()
*           test1=a2*x**2+y**2;
*           test2=1+d2*x**2*y**2;
*           if(test1==test2):
*               i=i+1
*               Fichier = open(fichier2,'a')    
*               Fichier.write('arraysrc1['+str(i)+']="'+ str(x) +'";'+'\n'+'arraysrc2['+str(i)+']="'+ str(y) +'";'+'\n')
*               Fichier.close() 
*
*/
