/* 
  MPHELL-4.0 
  Author(s): The MPHELL team

 (C) Copyright 2015-2018 - Institut Fourier / Univ. Grenoble Alpes (France)

 This file is part of the MPHELL Library.
 MPHELL is free software: you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation, version 3 of the License.
 
 MPHELL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with MPHELL.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
  * \file  mphell_test_edwards_conversion_curve.c
  * \brief Self check tests for the functions: edwards_to_weierstrass,  weierstrass_to_edwards in Extended Edwards coordinates
  */
#include "mphell.h"

bool 
test_curve_edwards_conversion(field k, const char *id_curve, const uint16_t nbits, 
        const uint32_t num_point)
{
    printf("Validity tests on %d curves %s : ", num_point , id_curve);
    uint8_t n = bits_to_nblock(nbits);
    uint32_t i = 0;
    uint8_t j = 0;
    bool test=true;
    ec_curve E_Ed,E_W,E_T,E_T2;
    field_elt alpha,beta;
    field_elt c,e; 
    field_elt_alloc(&alpha, k);
    field_elt_init(alpha, k);
    field_elt_alloc(&beta, k);
    field_elt_init(beta, k);
    field_elt_alloc(&c, k);
    field_elt_init(c, k);
    field_elt_alloc(&e, k); 
    field_elt_init(e, k); 
    field_elt_set_ui(c, 0, false, k, STACK_1);
    field_elt_set_ui(e, 0, false, k, STACK_1);
    number tmp;
    number_init(&tmp, n);
    uint8_t check[num_point][5];
    FILE *filep1, *filep2;
    char test_file1[200];
    char test_file2[200];
    char str[200];
    char str1[200];
    char line[160];
    for(i = 0; i < num_point; i++)
    {
        for(j = 0; j < 2; j++)
        {
            check[i][j] = (uint8_t)0;
        }
    }
    sprintf(test_file1, DATADIR "/vector_test_elliptic_curves/random_curve%s_conversion_test.txt", id_curve); 
    filep1 = fopen(test_file1,"r"); /* 1st one for Edwards */
    if(filep1 == NULL)
    {
        printf(" \n Chemin:    %s \n", test_file1);
        fprintf(stderr, "Error : no data file found for curve conversion (Edwards) \n");        
        exit(EXIT_FAILURE);
    }    
    sprintf(test_file2, DATADIR "/vector_test_elliptic_curves/rand_curve_W%s_conversion_test.txt", id_curve); 
    filep2 = fopen(test_file2,"r"); /* 2nd one for Weierstrass */
    if(filep2 == NULL)
    {
        printf(" \n Chemin:    %s \n", test_file2);
        fprintf(stderr, "Error : no curve data file found for curve conversion (Weierstrass) \n");        
        exit(EXIT_FAILURE);
    }
    for(i = 0; i < num_point; i++)
    {
        ec_alloc(E_Ed, k);                  /* We allocate space for an Edwards curve we will read */
        ec_init(E_Ed, k);
        ec_alloc(E_W, k);                   /* We allocate space for a Weierstrass curve we will read */
        ec_init(E_W, k);
        ec_alloc(E_T, k);                   /* We allocate space for a curve we will compute */
        ec_init(E_T, k);
        ec_alloc(E_T2, k);                  /* We allocate space for a 2nd curve we will compute */
        ec_init(E_T2, k);
        /* First we read the two curves */
        /* We read Weierstrass curve first */
        while(fgets(line, sizeof(line), filep2) != NULL)
        { 
            E_W->k=k;
            if(strncmp(line, "a = ", 4)== 0)
            {
                strcpy(str, line + 4);
                number_set_str(tmp, str, 16);
                field_elt_set_number(E_W->a, false, E_W->k, STACK_1, 1, tmp);
            }
            if(strncmp(line, "b = ", 4)== 0)
            {
                strcpy(str, line + 4);
                number_set_str(tmp, str, 16);
                field_elt_set_number(E_W->b, false, E_W->k, STACK_1, 1, tmp);
            }
            if(strncmp(line, "xG = ", 5)== 0)
            {
                strcpy(str, line + 5);
            }
            if(strncmp(line, "yG = ", 5)== 0)
            {
                strcpy(str1, line + 5);
                weierstrass_point_set_aff_str(E_W->G, str, str1, false, 16, E_W->k, STACK_1);
            }
            if(strncmp(line, "h = ", 4) == 0)
            {
                strcpy(str, line + 4);
                number_set_str(E_W->h, str, 16);
                break;
            }
            if(strncmp(line, "n = ", 4) == 0)
            {
                strcpy(str, line + 4);
                number_set_str(E_W->n, str, 16);
                break;
            }
        }
        E_W->type = WEIERSTRASS;
        E_W->f = PROJECTIVE;
        strcpy(E_W->id_curve, id_curve);
        ec_compute_disc(E_W, STACK_1);
        if(i%3==0) 
        {
            weierstrass_to_edwards(E_T, E_W, 0, alpha, beta, STACK_1);
        }
        else if(i%3==1)
        {
            /* We read the alpha we will use for the mapping */
           while(fgets(line, sizeof(line), filep1) != NULL)
            {         
                if(strncmp(line, "alpha = ", 8)== 0)
                {   
                    strcpy(str, line + 8);
                    number_set_str(tmp, str, 16);
                    field_elt_set_number(alpha, false, k, STACK_1, 1, tmp);
                    break;
                }
            } 
            weierstrass_to_edwards(E_T, E_W, 1, alpha, beta, STACK_1);
        }
        else
        {
            /* We read the alpha and beta we will use for the mapping */
            while(fgets(line, sizeof(line), filep1) != NULL)
            { 
                           
                if(strncmp(line, "alpha = ", 8)== 0)
                {
                    strcpy(str, line + 8);
                    number_set_str(tmp, str, 16);
                    field_elt_set_number(alpha, false, k, STACK_1, 1, tmp);
                }
                if(strncmp(line, "beta = ", 7)== 0)
                {
                    strcpy(str, line + 7);
                    number_set_str(tmp, str, 16);
                    field_elt_set_number(beta, false, k, STACK_1, 1, tmp);
                    break;
                }
            }
            weierstrass_to_edwards(E_T, E_W, 2, alpha, beta, STACK_1);
        }
        /* Now we have to test the computed values */
        if(i%3==0)
        {
            /* We read alpha and beta */
            while((fgets(line, sizeof(line), filep1) != NULL))
            {                 
                if(strncmp(line, "alpha = ", 8)== 0)
                {
                    strcpy(str, line + 8);
                    number_set_str(tmp, str, 16);
                    field_elt_set_number(c, false, k, STACK_1, 1, tmp);
                }
                if(strncmp(line, "beta = ", 7)== 0)
                {
                    strcpy(str, line + 7);
                    number_set_str(tmp, str, 16);
                    field_elt_set_number(e, false, k, STACK_1, 1, tmp);
                }
                if((field_elt_cmp(alpha, c, k)==0)&&(field_elt_cmp(beta, e, k)==0))
                {
                    break;
                }
            }
        }
        else if(i%3==1)
        {
            /* We read beta */
            while((fgets(line, sizeof(line), filep1) != NULL))
            {                 
                if(strncmp(line, "beta = ", 7)== 0)
                {
                    strcpy(str, line + 7);
                    number_set_str(tmp, str, 16);
                    field_elt_set_number(e, false, k, STACK_1, 1, tmp);
                }
                if((field_elt_cmp(beta,e, k)==0))
                {
                    break;
                }
            }    
        }
        /* In the last case there is nothing to do we are at the right place */
        /* We first read the values of a curve of Edwards for the reverse test */
        while(fgets(line, sizeof(line), filep1) != NULL)
        { 
            E_Ed->k=k;
            if(strncmp(line, "a = ", 4)== 0)
            {
                strcpy(str, line + 4);
                number_set_str(tmp, str, 16);
                field_elt_set_number(E_Ed->a, false, E_Ed->k, STACK_1, 1, tmp);
            }
            if(strncmp(line, "b = ", 4)== 0)
            {
                strcpy(str, line + 4);
                number_set_str(tmp, str, 16);
                field_elt_set_number(E_Ed->b, false, E_Ed->k, STACK_1, 1, tmp);
            }
            if(strncmp(line, "xG = ", 5)== 0)
            {
                strcpy(str, line + 5);
                field_elt_set_str(c,str,16,false,k, STACK_1);
            }
            if(strncmp(line, "yG = ", 5)== 0)
            {
                strcpy(str1, line + 5);
                field_elt_set_str(e,str1,16,false,k, STACK_1);
                edwards_point_set_aff(E_Ed->G, c, e, k, STACK_1); 
        
            }
            if(strncmp(line, "h = ", 4) == 0)
            {
                strcpy(str, line + 4);
                number_set_str(E_Ed->h, str, 16);
                break;
            }
            if(strncmp(line, "n = ", 4) == 0)
            {
                strcpy(str, line + 4);
                number_set_str(E_Ed->n, str, 16);
                break;
            }
        }
        E_Ed->type = EDWARDS;
        E_Ed->f = EXTENDED_EDWARDS;
        strcpy(E_Ed->id_curve, id_curve);
        ec_compute_disc(E_Ed, STACK_1); 
        /* Now we compare the values */
        if(field_elt_cmp(E_T->a, E_Ed->a, k)==0 && field_elt_cmp(E_T->b, E_Ed->b, k)==0 && edwards_point_are_equal(E_T->G, E_Ed->G, E_Ed, STACK_1)) 
        {
            check[i][0]=true;
        }
        else
        {
            printf("\nProbleme in the morphism Weierstrass -> Edwards \n");
            printf("\nWe are in the loop i: %d \n",i);  
            printf("\nWe start from the curve:\n");
            ec_curve_print(E_W,16, STACK_1);
            printf("\nWe hoped this curve:\n");
            ec_curve_print(E_Ed,16, STACK_1);            
            printf("\nE->a computed \n");
            field_elt_print(E_T->a,16,true, k, STACK_1);
            printf("\nE->b computed \n");
            field_elt_print(E_T->b,16,true, k, STACK_1);
            printf("\nE->a hoped \n");
            field_elt_print(E_Ed->a,16,true, k, STACK_1);
            printf("\nE->b hoped \n");
            field_elt_print(E_Ed->b,16,true, k, STACK_1);
            printf("\nalpha computed \n");
            field_elt_print(alpha,16,true, k, STACK_1);
            printf("\nbeta computed \n");
            field_elt_print(beta,16,true, k, STACK_1);
            printf("\npoint E->G computed \n");
            ec_point_print(E_T->G,16,true, k, STACK_1);
            printf("\npoint E->G hoped \n");
            ec_point_print(E_Ed->G,16,true, k, STACK_1);             
            check[i][0]=false;
        }   
        /* We Now test the revers Edwards -> Weierstrass */
        
        edwards_to_weierstrass(E_T2, E_Ed, STACK_1);
        
        if(field_elt_cmp(E_T2->a, E_W->a, k)==0 && field_elt_cmp(E_T2->b, E_W->b, k)==0 && weierstrass_point_are_equal(E_T2->G, E_W->G, E_W, STACK_1)) 
        {
            check[i][1]=true;
        }
        else
        {
            printf("\nProbleme in the morphism Weierstrass <- Edwards \n");
            printf("\nWe are in the loop i: %d \n",i);
            printf("\nE->a computed \n");
            field_elt_print(E_T2->a,16,true, k, STACK_1);
            printf("\nE->b computed \n");
            field_elt_print(E_T2->b,16,true, k, STACK_1);
            printf("\nE->a hoped \n");
            field_elt_print(E_W->a,16,true, k, STACK_1);
            printf("\nE->b hoped \n");
            field_elt_print(E_W->b,16,true, k, STACK_1);
            printf("\npoint E->G computed \n");
            ec_point_print(E_T2->G,16,true, k, STACK_1);
            printf("\npoint E->G hoped \n");
            ec_point_print(E_W->G,16,true, k, STACK_1); 
            printf("\nWe start from the curve:\n");
            ec_curve_print(E_Ed,16, STACK_1);
            printf("\nWe hoped this curve:\n");
            ec_curve_print(E_W,16, STACK_1);
            check[i][1]=false;
        }
        
        /* Now we read in each file to be at the next curve for the next loop */ 
        while(fgets(line, sizeof(line), filep2) != NULL)
        { 
            if(strncmp(line, "New curve of rank ", 18)== 0)
            {
                break;
            }
        }
        
        while(fgets(line, sizeof(line), filep1) != NULL)
        { 
            if(strncmp(line, "New curve of rank ", 18)== 0)
            {
                break;
            }
        }
        
        ec_free(E_Ed); 
        ec_free(E_W); 
        ec_free(E_T); 
        ec_free(E_T2); 
          
    }
    fclose(filep1);
    fclose(filep2);
    number_free(&tmp);
    field_elt_free(&alpha, k);
    field_elt_free(&beta, k);
    field_elt_free(&c, k);
    field_elt_free(&e, k);
    for(i = 0; i < num_point; i++)
    {
        for(j = 0; j < 2; j++)
        {
            if(check[i][j] == false)
            {
                printf("ERROR i %d j %d\n",i,j);
                test=false;
            }
        }
    }
    if(test)
    {
        printf("OK\n");
    }
    return test;
}

bool
test_curve_edwards(const char *id_curve, const uint16_t nbits, 
        const uint32_t num_point)
{
    bool res;
    field k;
    FILE *filep1 = NULL;
    char line[160];
    char* test_file1 = (char*)malloc(sizeof(char) * 200);
    number tmp;
    number_init(&tmp, bits_to_nblock(nbits));
    field_alloc(k, FP, bits_to_nblock(nbits), NULL);
    
    /* We first read the field we work on */
    sprintf(test_file1, DATADIR "/vector_test_elliptic_curves/random_curve%s_conversion_test.txt", id_curve); 
    filep1 = fopen(test_file1,"r"); /* 1st one for Edwards */
    if(filep1 == NULL)
    {
        printf(" \n Chemin:    %s \n", test_file1);
        fprintf(stderr, "Error : no data file found for point conversion (Edwards) \n");        
        exit(EXIT_FAILURE);
    }

    while(fgets(line, sizeof(line), filep1) != NULL)
    { 
        if(strncmp(line, "p = ", 4)== 0)
        {
            strcpy(test_file1, line + 4);
            number_set_str(tmp, test_file1, 16);
            field_create(k, "", STACK_1, 1, tmp);
            break;
        }
    }

    fclose(filep1);

    /* We free the variables not anymore necessary */
    free(test_file1);
    number_free(&tmp);

    res=test_curve_edwards_conversion(k, id_curve, nbits, num_point);
    
    /* We now free the memory */
    field_free(k);
    return res;
}

int main()
{
    init_mphell(128, RANDOM_AES128, DEVURANDOM);
    int nb_points=100;
    if (!test_curve_edwards("P25519rank2", 256, nb_points))
    {
        free_mphell();
        return -1;
    }
    if (!test_curve_edwards("P25519rank1", 256, nb_points))
    {
        free_mphell();
        return -1;
    }
    free_mphell();
    return 0;
}
