/* 
  MPHELL-4.0 
  Author(s): The MPHELL team

 (C) Copyright 2015-2018 - Institut Fourier / Univ. Grenoble Alpes (France)

 This file is part of the MPHELL Library.
 MPHELL is free software: you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation, version 3 of the License.
 
 MPHELL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with MPHELL.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
  * \file  mphell_test_edwards_conversion_point.c
  * \brief Self check tests for the functions: edwards_point_to_weierstrass_point,  weierstrass_point_to_edwards_point in Extended Edwards coordinates
  */

#include "mphell_test_weierstrass.h"
#include "mphell_test_edwards.h"


bool 
test_curve_edwards_conversion(data_t *data)
{
    printf("Validity tests on curve %s with %d points: \n", data->E->id_curve, data->nb_points);
    uint8_t n = data->k->size;
    uint32_t i = 0;
    uint8_t j = 0;
    bool test=true;
    ec_point Res;
    ec_curve E_Ed;
    field_elt a,alpha,beta,c,d,e;
    field_elt_alloc(&alpha, data->k);    
    field_elt_init(alpha, data->k);    
    field_elt_alloc(&beta, data->k);
    field_elt_init(beta, data->k);
    field_elt_alloc(&a, data->k);
    field_elt_init(a, data->k);
    field_elt_alloc(&c, data->k);    
    field_elt_init(c, data->k);    
    field_elt_alloc(&d, data->k);
    field_elt_init(d, data->k);
    field_elt_alloc(&e, data->k);
    field_elt_init(e, data->k);
    field_elt_set_ui(a, 0, false, data->k, STACK_1);
    field_elt_set_ui(c, 0, false, data->k, STACK_1);
    field_elt_set_ui(d, 0, false, data->k, STACK_1);
    field_elt_set_ui(e, 0, false, data->k, STACK_1);
    number n1,tmp;
    number_init(&tmp, n);
    uint8_t check[data->nb_points][5];
    FILE *filep1, *filep2;
    char* test_file1 = malloc(sizeof(char) * 200);
    char* test_file2 = malloc(sizeof(char) * 200);
    char *str = (char*)malloc(sizeof(char) * 200);
    char *str1 = (char*)malloc(sizeof(char) * 200);
    char line[160];
    ec_point_alloc(Res, data->k);
    ec_point_init(Res, data->k);
    for(i = 0; i < data->nb_points; i++)
    {
        for(j = 0; j < 2; j++)
        {
            check[i][j] = (uint8_t)0;
        }
    }
    sprintf(test_file1, DATADIR "/vector_test_elliptic_curves/random_point_%s_conversion_test.txt", data->E->id_curve); 
    filep1 = fopen(test_file1,"r"); /* 1st one for Edwards */
    if(filep1 == NULL)
    {
        printf(" \n Chemin:    %s \n", test_file1);
        fprintf(stderr, "Error : no data file found for point conversion (Edwards) \n");        
        exit(EXIT_FAILURE);
    }    

    sprintf(test_file2, DATADIR "/vector_test_elliptic_curves/rand_point_W_%s_conversion_test.txt", data->E->id_curve); 
    filep2 = fopen(test_file2,"r"); /* 2nd one for Weierstrass */
    if(filep2 == NULL)
    {
        printf(" \n Chemin:    %s \n", test_file2);
        fprintf(stderr, "Error : no curve data file found for point conversion (Weierstrass) \n");        
        exit(EXIT_FAILURE);
    }
    number_init(&n1, n);
    for(i = 0; i < data->nb_points; i++)
    {
        ec_alloc(E_Ed, data->k);
        ec_init(E_Ed, data->k);
        test_curve_weierstrass_read_point(filep2, data->P1[i], data);     /* We read P1 */  
        data->E->f = PROJECTIVE;
        /* We test if the point read belongs to the weierstrass curve */
        if(!weierstrass_belongs(data->P1[i], data->E, STACK_1))
        {
            printf("(P1) does not belongs to the weierstrass elliptic curve E\n");
            printf("Point (P1) i:=%d: \n",i);
            ec_point_print(data->P1[i],16,true, data->k, STACK_1);
            return false;       
        }
        /* We make sure to read the good point */
        while(fgets(line, sizeof(line), filep1) != NULL)
        {         
            if(strncmp(line, "New Point", 9)== 0)
            {   
                break;
            }
        }
        if((i%3)==0)
        {
            /* We first read alpha */
            while(fgets(line, sizeof(line), filep1) != NULL)
            {         
                if(strncmp(line, "alpha = ", 8)== 0)
                {   
                    strcpy(str, line + 8);
                    number_set_str(tmp, str, 16);
                    field_elt_set_number(alpha, false, data->k, STACK_1, 1, tmp);
                    break;
                }
            }
            /* We compute the image of the point */
            weierstrass_point_to_edwards_point(Res, data->P1[i], data->E, 1, alpha, beta, STACK_1);
            /* We read the point corresponding to our choice of beta */
            while(fgets(line, sizeof(line), filep1) != NULL)
            { 
                if(strncmp(line, "beta = ", 7)== 0)
                {
                    strcpy(str, line + 7);
                    number_set_str(tmp, str, 16);
                    field_elt_set_number(c, false, data->k, STACK_1, 1, tmp);
                    if(field_elt_cmp(c, beta, data->k)==0) 
                    {
                        test=true;
                        /* We read the pre computed point and compare it */
                        /* We compute the corresponding coefficient of the edwards curve */
                        
                        /* We create the Edwards E_ed curve on which the point should belongs */ 
                        edwards_computation_of_curve_from_alpha_beta_and_field(E_Ed, alpha, beta, data->k, data->E->id_curve);
                        
                        /* We test if the computed point belongs to the Edwards curve */
                        if(edwards_belongs(Res, E_Ed, STACK_1))
                        {
                            /* We read the point that we sould have obtained */
                            test_curve_edwards_read_point(filep1, data->P2[i], data);
                            if(!edwards_belongs(data->P2[i], E_Ed, STACK_1))
                            {
                                printf("\nThe point read does not belong to the curve\n");
                                ec_point_print(data->P2[i],16,true, data->k, STACK_1);
                                printf("\n The curve is: \n");
                                ec_curve_print(E_Ed,16, STACK_1);
                                return false;
                            }
                            if(edwards_point_are_equal(data->P2[i], Res, E_Ed, STACK_1))
                            {
                                check[i][0] = true;
                            }
                            else
                            {
                                printf("\nThe Edwards point computed is not the good one\n");
                                check[i][0] = false;
                                printf("\nPoint whished %d :\n",i);
                                ec_point_print(data->P2[i],16,true, data->k, STACK_1);
                                printf("\nPoint obtained :\n");
                                ec_point_print(Res,16,true, data->k, STACK_1); 
                                return false;
                            }
                        }
                        else
                        {
                            printf("\nThe point doesn't belong to the Edwards curve1\n");
                            printf("\n The curve: \n");                       
                            ec_curve_print(E_Ed,16, STACK_1);
                            printf("\n The point : \n");
                            ec_point_print(Res,16,true, data->k, STACK_1);
                            printf("\n The point read : \n");
                            test_curve_edwards_read_point(filep1, data->P2[i], data);
                            ec_point_print(data->P2[i],16,true, data->k, STACK_1); 
                            return false;                           
                        } 
                        break;
                    }
                    
                }
            }
        }
        else if((i%3)==1)
        {
            /* We read alpha and beta */
            while(fgets(line, sizeof(line), filep1) != NULL)
            { 
                           
                if(strncmp(line, "alpha = ", 8)== 0)
                {
                    strcpy(str, line + 8);
                    number_set_str(tmp, str, 16);
                    field_elt_set_number(alpha, false, data->k, STACK_1, 1, tmp);
                }
                if(strncmp(line, "beta = ", 7)== 0)
                {
                    strcpy(str, line + 7);
                    number_set_str(tmp, str, 16);
                    field_elt_set_number(beta, false, data->k, 1, STACK_1, tmp);
                    break;
                }
            }
            /* We compute the image of the point */
            weierstrass_point_to_edwards_point (Res, data->P1[i], data->E, 2, alpha, beta, STACK_1);
            /* We compute the corresponding coefficient of the edwards curve */
            /* We create the Edwards E_ed curve on which the point should belongs */ 
            edwards_computation_of_curve_from_alpha_beta_and_field(E_Ed, alpha, beta, data->k, data->E->id_curve);
            /* We test if the computed point belongs to the Edwards curve */
            if(edwards_belongs(Res, E_Ed, STACK_1))
            {
                /* We read the point corresponding to the alpha and beta we read in input */
                /* We read the point that we sould have obtained */
                test_curve_edwards_read_point(filep1, data->P2[i], data);
                if(!edwards_belongs(data->P2[i], E_Ed, STACK_1))
                {
                    printf("\nThe point read does not belong to the curve\n");
                    ec_point_print(data->P2[i],16,true, data->k, STACK_1);
                    printf("\n The curve is: \n");
                    ec_curve_print(E_Ed,16, STACK_1);
                    return false;
                }
                if(edwards_point_are_equal(data->P2[i],Res,E_Ed, STACK_1))
                {
                    check[i][0] = true;    
                }
                else
                {
                    printf("\nThe Edwards point computed is not the good one\n");
                    check[i][0] = false;
                    printf("\nPoint whished %d :\n",i);
                    ec_point_print(data->P2[i],16,true, data->k, STACK_1);
                    printf("\nPoint obtained :\n");
                    ec_point_print(Res,16,true, data->k, STACK_1); 
                    return false;
                }
            }
            else
            {
                printf("\nThe point doesn't belong to the Edwards curve2\n");
                printf("\n The curve: \n");                       
                ec_curve_print(E_Ed,16, STACK_1);
                printf("\n The point : \n");
                ec_point_print(Res,16,true, data->k, STACK_1);
                printf("\n The point read : \n");
                test_curve_edwards_read_point(filep1, data->P2[i], data);
                ec_point_print(data->P2[i],16,true, data->k, STACK_1); 
                return false;               
            }   
            
        }
        else
        {
            /* We compute alpha and beta */ 
            weierstrass_point_to_edwards_point (Res, data->P1[i], data->E, 0, alpha, beta, STACK_1);
            /* We compute the corresponding coefficient of the edwards curve */
            /* We create the Edwards E_ed curve on which the point should belongs */ 
            edwards_computation_of_curve_from_alpha_beta_and_field(E_Ed, alpha, beta, data->k, data->E->id_curve);
            /* We read alpha and beta */
            while((fgets(line, sizeof(line), filep1) != NULL))
            { 
                if(strncmp(line, "alpha = ", 8)== 0)
                {
                    strcpy(str, line + 8);
                    number_set_str(tmp, str, 16);
                    field_elt_set_number(c, false, data->k, STACK_1, 1, tmp);
                }
                if(strncmp(line, "beta = ", 7)== 0)
                {
                    strcpy(str, line + 7);
                    number_set_str(tmp, str, 16);
                    field_elt_set_number(e, false, data->k, STACK_1, 1, tmp);
                }
                if((field_elt_cmp(alpha, c, data->k)==0)&&(field_elt_cmp(beta, e, data->k)==0))
                {
                    break;
                }
            }
            while(fgets(line, sizeof(line), filep1) != NULL)
            { 
                if(strncmp(line, "x = ", 4) == 0)
                {
                    strcpy(str, line + 4);
                    if(str[strlen(str)-2] == '\\')
                    {
                        if(fgets(line, sizeof(line), filep1) == NULL)
                        {
                            exit(EXIT_FAILURE);
                        }
                        strcpy(str + strlen(str)-2, line);
                    }
                    field_elt_set_str(c, str, 16, false, data->k, STACK_1);
                }
                if(strncmp(line, "y = ", 4) == 0)
                {
                    strcpy(str, line + 4);
                    if(str[strlen(str)-2] == '\\')
                    {
                        if(fgets(line, sizeof(line), filep1) == NULL)
                        {
                            exit(EXIT_FAILURE);
                        }
                        strcpy(str + strlen(str)-2, line);
                    }
                    field_elt_set_str(e, str, 16, false, data->k, STACK_1);
                    edwards_point_set_aff(data->P2[i], c, e, data->k, STACK_1);
                    break;
                }
            }  
            /* We read the point that we sould have obtained */
            if(edwards_belongs(Res, E_Ed, STACK_1))
            {
                /* We read the point corresponding to the alpha and beta we read in input */
                /* We read the point that we sould have obtained */
                if(!edwards_belongs(data->P2[i], E_Ed, STACK_1))
                {
                    printf("\nThe point read does not belong to the curve\n");
                    ec_point_print(data->P2[i],16,true, data->k, STACK_1);
                    printf("\n The curve is: \n");
                    ec_curve_print(E_Ed,16, STACK_1);
                    printf("\n Computed alpha \n");
                    field_elt_print(alpha,16,true, data->k, STACK_1);
                    printf("\n Computed beta \n");
                    field_elt_print(beta,16,true, data->k, STACK_1);
                    printf("\n Read alpha \n");
                    field_elt_print(c,16,true, data->k, STACK_1);
                    printf("\n Read beta \n");
                    field_elt_print(e,16,true, data->k, STACK_1);
                    return false;
                }
                if(edwards_point_are_equal(data->P2[i],Res,E_Ed, STACK_1))
                {
                    check[i][0] = true;    
                }
                else
                {
                    printf("\nThe Edwards point computed is not the good one\n");                    
                    check[i][0] = false;
                    printf("\nPoint whished %d :\n",i);
                    ec_point_print(data->P2[i],16,true, data->k, STACK_1);
                    printf("\nPoint obtained :\n");
                    ec_point_print(Res,16,true, data->k, STACK_1); 
                    return false;
                }
            }
            else
            {
                printf("\nThe point doesn't belong to the Edwards curve3\n");
                printf("\n The curve: \n");                       
                ec_curve_print(E_Ed,16, STACK_1);
                printf("\n The point : \n");
                ec_point_print(Res,16,true, data->k, STACK_1);
                printf("\n The point read : \n");
                test_curve_edwards_read_point(filep1, data->P2[i], data);
                ec_point_print(data->P2[i],16,true, data->k, STACK_1);
                
            }
        }
        /* Now we test the conversion edwards to weierstrass */
        edwards_point_to_weierstrass_point (Res, data->P2[i], E_Ed, STACK_1);
        
        if(!weierstrass_belongs(Res, data->E, STACK_1))
        {
            printf("\ndata->P2[i] does not belongs to data->E\n");
            printf("\n alpha: \n");
            field_elt_print(alpha,16,true, data->k, STACK_1);
            printf("\n beta: \n");
            field_elt_print(beta,16,true, data->k, STACK_1);
            printf("\nPoint data->P2[i] i:=%d: \n",i);
            ec_point_print(data->P2[i],16,true, data->k, STACK_1);
            printf("\nPoint computed i:=%d: \n",i);
            ec_point_print(Res,16,true, data->k, STACK_1);
            printf("\nPoint wished \n");
            ec_point_print(data->P1[i],16,true, data->k, STACK_1);            
            printf("\n The curve is: \n");
            ec_curve_print(data->E,16, STACK_1);
            return false;
        }
        if(!ec_point_are_equal(Res,data->P1[i],data->E, STACK_1))
        {
            check[i][1] = false;
            printf("\nEdwards -> Weierstrass did not work \n");
            printf("\nPoint obtained: \n");
            ec_point_print(Res,16,true, data->k, STACK_1);
            printf("\nPoint wished: \n");
            ec_point_print(data->P1[i],16,true, data->k, STACK_1);
            return false;
        }
        else
        {
            check[i][1] = true;
        }
        ec_free(E_Ed);    
    }
    
    fclose(filep1);
    fclose(filep2);
    ec_point_free(Res, data->k);
    number_free(&n1);
    free(str);
    free(str1);
    number_free(&tmp);
    free(test_file1);
    free(test_file2);
    field_elt_free(&alpha, data->k);
    field_elt_free(&beta, data->k);
    field_elt_free(&a, data->k);
    field_elt_free(&c, data->k);
    field_elt_free(&d, data->k);
    field_elt_free(&e, data->k);
    for(i = 0; i < data->nb_points; i++)
    {
        for(j = 0; j < 2; j++)
        {
            if(check[i][j] == false)
            {
                printf("ERROR i %d j %d\n",i,j);
                test=false;
            }
        }
    }
    return test;
}

bool
test_curve_edwards(const char *id_curve, const uint16_t nbits, 
        const uint32_t num_point, data_t *data)
{
    bool res;
    test_curve_weierstrass_alloc(nbits, num_point, data);       /* We can re use this function */
    test_curve_weierstrass_read(id_curve, PROJECTIVE, data);    /* We read the Weierstrass curve we will work with */
    test_curve_weierstrass_init(data);
    res=test_curve_edwards_conversion(data);
    test_curve_weierstrass_free(data);                          /* We can re use this function */
    return res;
}

int main()
{
    data_t data;
    init_mphell(128, RANDOM_AES128, DEVURANDOM);
    int nb_points=100;
    if (!test_curve_edwards("randomP25519rank1", 256, nb_points, &data))
    {
        free_mphell();
        return -1;
    }
    if (!test_curve_edwards("randomP25519rank2", 256, nb_points, &data))
    {
        free_mphell();
        return -1;
    }
    free_mphell();
    return 0;
}
