/* 
  MPHELL-4.0 
  Author(s): The MPHELL team

 (C) Copyright 2015 - Institut Fourier / Université Grenoble Alpes (Grenoble) 

 This file is part of the MPHELL Library.
 MPHELL is free software: you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation, version 3 of the License.
 
 MPHELL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with MPHELL.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
  * \file  mphell_test_edwards_point_is_neutral.c
  * \brief Self check tests for the function: bool edwards_point_is_neutral (ec_point_srcptr P, ec_curve_srcptr E)
  */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>
#include "mphell.h"

int main()
{
    init_mphell(128, RANDOM_AES128, DEVURANDOM);
    bool test = true;
    bool test1, test3;
    int i;
    int nb_iteration = 100;
    /* Create base prime field of characteristic p */
    field k;
    field_elt b,c,x1,x2,y1,y2,z1,z2,t1,t2;
    ec_curve E, Etri; 
    ec_point P,Q; /* Les deux points que l on va tester */
    number p,a,d;

    /* Allocation */
    field_alloc(k, FP, bits_to_nblock(384), NULL);
    number_init(&p, bits_to_nblock(384));
    number_init(&a, bits_to_nblock(384));
    number_init(&d, bits_to_nblock(384));
    
    /*Setting variables*/
    number_set_str(p, "fffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffeffffffff0000000000000000ffffffff", 16);
    field_create(k, "", STACK_1, 1, p);
    
    /* Allocation of field elt */
    field_elt_alloc(&b, k);
    field_elt_init(b, k);
    field_elt_alloc(&c, k);
    field_elt_init(c, k);
    field_elt_alloc(&x1, k);
    field_elt_init(x1, k);
    field_elt_alloc(&x2, k);
    field_elt_init(x2, k);
    field_elt_alloc(&y1, k);
    field_elt_init(y1, k);
    field_elt_alloc(&y2, k);
    field_elt_init(y2, k);
    field_elt_alloc(&z1, k);
    field_elt_init(z1, k);
    field_elt_alloc(&z2, k);
    field_elt_init(z2, k);
    field_elt_alloc(&t1, k);
    field_elt_init(t1, k);
    field_elt_alloc(&t2, k);
    field_elt_init(t2, k);
    
    /* We work on the base field of NIST P 384 */
    number_set_str(a, "ffffffffffffff0ffffffffffffffffffffffffffffffffffffffffffffffffeffffffff0000000000000000ffffffff", 16);
    number_set_str(d, "ffffffffffffffffff0ffffffffffffffffffffffffffffffffffffffffffffeffffffff0000000000000000ffffffff", 16);
    field_elt_set_number(b,true, k, STACK_1, 1 ,a);
    field_elt_set_number(c,true, k, STACK_1, 1 ,d);
    ec_point_alloc(P, k);
    ec_point_init(P, k);
    ec_point_alloc(Q, k);
    ec_point_init(Q, k);
    /* We make the assumption that the point have been correctly created that 
    *  is that in extended Edwards we have t=x*y/z
    */
    
    ec_alloc(E, k);
    ec_init(E, k);
    ec_alloc(Etri, k);
    ec_init(Etri, k);
    /* We create elliptic curves for the test    */
    ec_create(E, "Test", k, b, c, NULL, NULL, NULL, EDWARDS,  PROJECTIVE, STACK_1);
    ec_create(Etri, "Test", k, b, c, NULL, NULL, NULL, EDWARDS, EXTENDED_EDWARDS, STACK_1);
    printf("After creation of extended edwards elliptic curves \n");
    
    printf("Enter TEST:\n");
    
    const char *array_src2[100]; /* x coordinate */
    const char *array_alea[100]; /* random for "projective coodinate"*/
    
    array_src2[0]="30874124709894201564133090287070310313999922710881447306624021047767305731597224050053119518530145909117253502987511";
    array_src2[1]="36313637477488811160496316706818892582703909792777009178732940368631759685122503660022091369279461163271833919785883";
    array_src2[2]="33208754320113925180898559737799296455872559207570414956152728459720810574797087457433696865467776644845385109910828";
    array_src2[3]="739497375006341856106610668352569644291535569369058550733219599170690323303486939051047312145233768680091964835189";
    array_src2[4]="30520436367638092268259496762897358501646868542703972993574308500263794812187327959697475719293200229717412427393694";
    array_src2[5]="27418934597355459752334430993575319400948618379563932155700099937273236535790389938200353898450511846536527664352633";
    array_src2[6]="17372865002616809805756232172384163074020245767057199577893919515036566574752286781914191470750597915697762140465791";
    array_src2[7]="19424626673850655250908613546306298968664842125412013225802088467720131922547313083848517048193592457362059027502051";
    array_src2[8]="24296542165652774862029479973058099088316081551481310360390039912432751719628369824407858993886267254596941028070303";
    array_src2[9]="28279926515394337982267583127326731341263795685044761957234671188581343830892462104258917200750783111394248084522124";
    array_src2[10]="12087452654394335643225551932333006528638669644503390986818033574686316075828013311277998725909102848268177433750393";
    array_src2[11]="2455299098526175034251334028976153505704412877349830451026487702167779667553932783298646620441870181854790094626771";
    array_src2[12]="6576546339780667481149891665893873677214472226428573376956455397004066513779860477013658076134932890457972575672997";
    array_src2[13]="10839291933327992746261923859491873199676289271344479104256125352880320213025822709938163573333914956408015808081151";
    array_src2[14]="30858595901047432190866961063288034452676947127548760478543861283192358123029295681939024494563183580523735572337249";
    array_src2[15]="13200180296220794606437919631135461340917848358637307061850267604138737741838960518992358238541748646592564278585927";
    array_src2[16]="2974900259923471905799429574846014663368023676195227438455859442331838283531768788565511534859742370608083639342703";
    array_src2[17]="29202788942585501450182435250780942115129492109446498705274348986040830648464055853072476950408265941698340343327301";
    array_src2[18]="18113696161170277552896954154931687673343900001782101895114032846831625171626492435655223956679060506649117096285358";
    array_src2[19]="20669176171951538872824061143207373627315613455618878903134911993336262258869248312055459214295267859180230661874898";
    array_src2[20]="29201064351882168507833972956896114225157103423233746680101688269562725257733944017327949842307851582828102085480792";
    array_src2[21]="35793191803598974423405332067528356472687408785622122655857823094536258226243081418546835320057611433415896881293587";
    array_src2[22]="5705225853563739821893559913029948233786272399657547744182884825395090501662737554384399661215812236868520478242532";
    array_src2[23]="17390790018062264352298791361588891085053440573199568750068595577913244739399286039934664226868773129851247109057239";
    array_src2[24]="8459256091543489579460758525944124676873807038146743836543825872129689231605482278755301653138210781487770409279175";
    array_src2[25]="19040976716262926779655419212623760170703141268451275593991797867404713440692706175985276775695493122270513907138486";
    array_src2[26]="39089678197945845477177609523928772106216991556406332724264573275717081881107444572146650244962787621169894730699626";
    array_src2[27]="38127703385595719750302027658143322633545850149293899430647233227791028481326973811532841129432892025536162295567681";
    array_src2[28]="5475572248108493465003214072104395782367122109262573173801072271405127632961538597647774467382058193448061161452964";
    array_src2[29]="16250904375064482180474273942934682287976538372890126500510659003755308188660295834308806536374871628811578671565704";
    array_src2[30]="9509753172525227984816128579490867808991052831079167373219432546814892901386977414140681098855464060834504171573800";
    array_src2[31]="22083609218127162539658912786639048882920305348039362279602479026652813384233337527532390594979325733921445158104284";
    array_src2[32]="27728763195795774077169007312714632277227547435499155241487999756037005749423433763225095409954390935125707584442043";
    array_src2[33]="27183611456891429567968343134565881177624521441638021703335926111908334533824585870028153582037841591425619540940290";
    array_src2[34]="34926093273068558738720687079940340848922771972318824716046076540944692569144731254562421701958993411463822951896985";
    array_src2[35]="15596369159821027706208656082371773228422651825041143712932702342484478615291511066933011724876536285575834408167068";
    array_src2[36]="15412257938786795657117596075801349317228954984594765905943698959064495727272942887327338489897061474177456480794411";
    array_src2[37]="37447307896871004647361674101952408672558839849220571611786585609177919890030234953275555363992176535859637667770751";
    array_src2[38]="13540486437005677489523005584550457876194773172269653121020262663049419401861830546431856475728006874443265219198852";
    array_src2[39]="31281837823461526149778692408716743272238417714215482258016000021231468931884601228399494738223510893635689891271158";
    array_src2[40]="11723145365275178150230930558742437433077856366315915405941145460166377591885591458912463422336992026626789277757713";
    array_src2[41]="20885464890389961904785477607834685112325415036192351687124021031003178888176562035727764047308554268250542811469255";
    array_src2[42]="1851529126330115824242838602178719049995315659185315925691908098735413011780348900757643666658781982967925308663656";
    array_src2[43]="4075882106035227280782227819726193602793830155863008397577331653532323814673518927550536662486898035660284157760815";
    array_src2[44]="7332439438475649007310714817071138823723531937216635072381817237192188280124138173616555835268593593579109313047247";
    array_src2[45]="6822649163421333164010568727823189959619585628080952665856905223959910369330954821523216325608143146273827469087323";
    array_src2[46]="32702623899090113431359607516417664466833831209027354413649400855878677209787697590488640222987559882112242246951588";
    array_src2[47]="3706715642131626608660104039173040141959537332998646731476181818249988484640127199953248327998115545101279941572762";
    array_src2[48]="35476541000571081705526638386303522747507953186349294005737747401598294725768417329646414966598638810314196525074562";
    array_src2[49]="23033134661136911995195041162229474563455541791316025597319825278613646150304966030925962552749897573353646828493526";
    array_src2[50]="13895145819828134154564711136217891487629613095748701402116918041653252029248895498353516482444879588001500894205337";
    array_src2[51]="24817206871552713501985368565465727017688411384373689377178945719479907193530761458800587899006564681937950734549667";
    array_src2[52]="31746143390100514764665136474672576523390038214847464982872275892738176636421747751554371010817669983471366764666306";
    array_src2[53]="10773444459417172143244466431241033360866579154340996374320226917568274597070091046659850022827505445421572935957838";
    array_src2[54]="19728073241967201881369368940794511629454246433959598878612195745409636007177254106478967680798738471382272551181018";
    array_src2[55]="8586318533088328884179388188283774629986476021551733284419593226496043949863403616899095086875290955915397942363033";
    array_src2[56]="17902041152462702370605103276338209862452970893452710161224172200351345481086082738724200725494079889427987483272817";
    array_src2[57]="31983853194755101888704968872064038654200645795316474401862734017023044796938642699370510594071597151465134485844048";
    array_src2[58]="2235200249090507596454124819777140924108673386850209382823837891075715376003210837640852815925311840862749935924730";
    array_src2[59]="17550514557658981264602206559943313555322563727544200777878970396995220695698714776930671993407340759432025826134411";
    array_src2[60]="23899839145392689273882496154131679535893992568450428638784398071943752323384290244300538843747162142063168542058070";
    array_src2[61]="31033741333272465135964434846643393985440918812083368929921483676886595137152086063637135950267318457078162736966541";
    array_src2[62]="23009120513741473667379617892130851204832798775226597715370957897885964150390385760866416000985173722730834204560693";
    array_src2[63]="11502782546068970378179936706074420307438299017512964241680150337135292816936434622221869628016734771993990326529636";
    array_src2[64]="12283274849885979981907773944048684132047944589843648148596402405001491878696160738311976064984630673680337999951443";
    array_src2[65]="20440573541961472094994274409539980712583285538477514356397722871344118344028169374680998987969613219444201476123241";
    array_src2[66]="20128643100453473403577058878878689009663854221086383389839736286154415635757522974785423340221955463614194144831361";
    array_src2[67]="33799491796158199216772286004385786552911297837116613357185509352894791197297400901639931310758046845828414275831194";
    array_src2[68]="25786622281218428329735960226477943997076100286813779926703725274322389470518255339773178486491170093407922074522126";
    array_src2[69]="38711779461918305572714932175742912752230398422265774232702379793466427472747244107679868199368306047221506176286235";
    array_src2[70]="25731484215284761937060413156800323484650712644831766992473755324264565033893626321444926093694711152925168491845680";
    array_src2[71]="19927899650939464314242245812212564684153917381415175039821898026520475919466933573466242278438251159015119351842832";
    array_src2[72]="16809221219399219366177034646772868036240139954267564701415953357854172083598559513046870730070380989401735957966379";
    array_src2[73]="13021793491905886092993376682793086304090716212335258282566133008889622820642979532834647663788344572406443317865275";
    array_src2[74]="30904092893716797419210295458349979469497455055694119144540369291213048734521520200670873950487216620186014338117265";
    array_src2[75]="26122420021028397700137323576314089751211099086570427798504911216686551475280018092848727279459154535560525103442745";
    array_src2[76]="26704048108378672386979231779820071622511814865190275821003822495302919403679284745274754588844108327003019907960246";
    array_src2[77]="11331703356967777843954674824510442948755533386405370445119945406995261386916801834934432282061017040934590780254127";
    array_src2[78]="28839546765121199545370900321030963330586887397504095500287917655874236781949275441019825412006360787776904069487863";
    array_src2[79]="1844596038275890651493657192326774389629904263754480850035351624089487045314513321344145857413713401184813505321752";
    array_src2[80]="3550195077122829517333513114154598763586277391625550710544671394776165408005541757255238056319508414579192065765177";
    array_src2[81]="14595643072268736762304395506872739493951628617138356048908156128490976576868767802318618858817012816260794231987330";
    array_src2[82]="18626032070550953530326173282154894356656274405606361809689790517445022677320910958264858616046295133795419473891698";
    array_src2[83]="37296429869669798838958404082450596766008386549670903937734656699665297258211924783192118329609569636107891003201807";
    array_src2[84]="9498133477656077381574500972630479064780284400723446716958456063524749090687343527973404542675519209298049549029324";
    array_src2[85]="8652479748297886976140243172033080759760502676173620983797241467651084642062299645105488274580166899316584912313934";
    array_src2[86]="2817249622432427675766843960464127078713955350011110054434615929532493521020906314764503942176014465516614357920810";
    array_src2[87]="31641126296493019259597046383153057127112828725782208733994827555130168103271911507298489960010779011045183028923158";
    array_src2[88]="38392220145518884122289680996670122477404930756132272339210409652893303329773345458380216572276040697838444990637215";
    array_src2[89]="21756106404757770115494116646741508945182892856512935411554054840120353899155932976418809124407905167519520369670645";
    array_src2[90]="16788878198613118854752736828210203088801179037731915887052368311609574266681501642372552292166052274571743591436335";
    array_src2[91]="23055080704319315422576380314109897127289601499771792370069800214185914702186390752210439786097520084509803201587515";
    array_src2[92]="5923322596576113928454543145332276130649394711037054004945055434841527629009878534918898108828340029951224251499090";
    array_src2[93]="12813482489359694829226565303376310830522653944576523579469920836243833070483872869726728774378541802150599024020611";
    array_src2[94]="32112923078416041654189446155621861580659431937716990449453579674989037525790901230368472738179806625606737716394708";
    array_src2[95]="6578708232495671324946894164600569910183781167180036595659943265052472704365964556326791626326896083901305860271081";
    array_src2[96]="26525251063968776934087352856028745025952385964636481700698204693810938810437115399985334395339357686629486172331487";
    array_src2[97]="19366999853950886570486074947962968949059939256361991864371270917374003393215442337578843233977827565933932588784240";
    array_src2[98]="4783100403267299459681892282753781132195585579451669375588186132295594394462016958503625429795028715189692236976259";
    array_src2[99]="21898639081159638645822892965456292144025963611119878737806107903677452717803830177330037702391729831902292414616883";
    /* All those coordinates are coordinates of random points of the elliptic curve
    * obtained by E=ellinit(ellfromeqn(a*x^2+y^2 - (1+d*x^2*y^2)),p)
      */    
    
    array_alea[0]="4692364054341501777484795144939626250685470964878886329217690616430863549694526821379682834482890539803809181151507";
    array_alea[1]="19724835407434812769515963856992565292302547417914533688416654119833260297363094680355284442421622339767839399663922";
    array_alea[2]="27470455932030092876302261698612049168305669638483499875532319784966004988370370230074597366970545099054786264383782";
    array_alea[3]="8157516754797368684039476144156572407981947803665707193332672177773553876569271276506453238887647986768082026719999";
    array_alea[4]="6786192380584162935885717846740054989501742613039448881159498949441111709308959453234556905421953177448733945244227";
    array_alea[5]="26121916768129639938284159862708508134260286599228589014586189439129997815893205602575770889443908656504361926824038";
    array_alea[6]="3881969765437264413399858482317374256075240398174198559815704459977099218840318212890688853205132374837838764360909";
    array_alea[7]="25452438119107127727805130784077141960471324088614747091371319544421898082130578097951643968810328616897706830085811";
    array_alea[8]="17096475077388591939712222185690181911342228570931592028763106326847497149532205163219734545593386098964081708049614";
    array_alea[9]="7279677033716924080314787890327515354224396162550839087840820939250115474330551819797730527833955002287981862155730";
    array_alea[10]="29869982572370403193389375060877331913070018084798880719560282559397658966466143294326822052512330771797381092989224";
    array_alea[11]="29249220596171628268411830846134588163954450536535996602606799139957237012106321653562536653509972559384420370523813";
    array_alea[12]="12675150932760089796496518612026664071839302028425896788392156732252188178510560418132063550795143866865679331668469";
    array_alea[13]="9784278566229219036854351736468497038102548499796660457783431045816559565013651861486644033096401999035614076025139";
    array_alea[14]="33486162708841168714999007235764408168500240180070259063322609557681179255229197146041894349826490839349913808976809";
    array_alea[15]="32761655410266865972097461410130485382755108117437537063165750002826601048465288823197582735967059377185262366082973";
    array_alea[16]="26982819223282615322151576998586613491603443233836616374746067250584529860034602464635867510182274344214550243873850";
    array_alea[17]="13347320994579095582445559911940399582570937395909695187018401969200852565170110283721010086053857851953103708002392";
    array_alea[18]="14542042386510793571531843575843831837847991507328319757252538108285583479981477989579238915961352342210696521888933";
    array_alea[19]="18482336425874261840813163040402686169871232605776792493120883624353701416213290707358466326033717398859479585482020";
    array_alea[20]="8779164873416503757112076708830688626894369568566085061280483504599733140996671351985163056831748800001194423674358";
    array_alea[21]="17386504341104395703847794054319908001102576351499728546797283955797344443040795468482100477322892923609491753188034";
    array_alea[22]="13043430377954565653105599403471996427833086176931571244064453405440975270611781250354942210383212210790300927655652";
    array_alea[23]="19106431638866252417176993510284114854651734127133631290119855905753562573462191501282100189730271379196012882753923";
    array_alea[24]="18204122972965481774166943406399320088710717922096901363122188328610903977101136803630433992818516853067430196187200";
    array_alea[25]="24587714761805427601217829658490240673960099166699993491770472635162907188698852288481766814845867162595776820200530";
    array_alea[26]="26997262758192053829628440595426583237848577977615324269164440550481706925417900682699137209272243290845084613142506";
    array_alea[27]="35656938137806185071040796915887563582059477251604584849605049120855623603025525537626723753819274829725429141640097";
    array_alea[28]="37211516123858055895957654537510251264009737665707117532054212651418247254427214939789334234928594875930504495589128";
    array_alea[29]="7441953229669274882720702208172276754769474212379874398112727576731631247569536461991945103271572040362362891039061";
    array_alea[30]="37831630170198253763257788069225132004252289312457247790635859215096046531032168302804246128058885851379095510343762";
    array_alea[31]="4815091940098053059859332347977995429715865592447189539136382210447111505551509970661311453205801102364299208542259";
    array_alea[32]="37784470666864906516242025362449689709530059867585150368099104418689826868849239288977194118640300317548564120160520";
    array_alea[33]="9566090638853063396664759068613757405303874976609137313631504030073060110402528520294088617037470060694378580344533";
    array_alea[34]="38107566390263778306445904219708929586888222055049939941122185352827818969025711681953889461361962498912414310618685";
    array_alea[35]="14803760651762084166518156242591146462954729403203224782660874194934172178494895510821588004648125592739939885560978";
    array_alea[36]="24307499959565249284258866525956851490112259613402905925388730895368973513750562497092544138478034034027258664166598";
    array_alea[37]="17492099229152781289886881741250582914432347243948328538936085020623621024161951091891456387693930620631770099178250";
    array_alea[38]="21692228419354832181134792926796098611719983996371673258142868650334289723897220519298345871850884726380523476068878";
    array_alea[39]="9482280848026293095952737855741865317354683942642124772900437455592623898639266421867945575620458130021705123490121";
    array_alea[40]="31829027968395555754094266299507558746070978718073841184970755713613420689282092583693935354781004600106168464862861";
    array_alea[41]="28654846296410446891981428239195595518897566320787898285114818170344085011365169111244108654197097483825404385305918";
    array_alea[42]="36882600883666399474439986137651266074845836800520099217087776064818847517599977644658963340684676400663311118866002";
    array_alea[43]="19738044048258460468788718037608607770885464389239805564718283368858170590478525545165507581184518613046309890206550";
    array_alea[44]="14722053830182779282607004614482939099895644389653183790152360429634335647646313300143552715186358591188735919679880";
    array_alea[45]="12209849006290642745735149249945118453752904749257533027778985492905642241556914171684121892602286559850058668279528";
    array_alea[46]="15784203958115933079680335796822981616970724008276405878365173803490238907451900949396665040840506179518295721704996";
    array_alea[47]="27391117969426507218084361956366592456423566141465538280440258717319555044696348403527268509642991653088583209537590";
    array_alea[48]="9939901295283038008639078545511876682065311971860152945889030896954379047317045712084041436466449234256402030891733";
    array_alea[49]="7468971494930205533255295566722500869409947834556171751565292967838717722978175658219788756019079347580814819661066";
    array_alea[50]="30403745923478500344304425316072238899593766770335851775475664751713453153756720337206632515244546292149883103010551";
    array_alea[51]="6673879805974905143197090147716620888706731775138103327517985594767813831103629984811743517481575799549837042811655";
    array_alea[52]="28916596504164173191794201318487702098537324734234111016084446257668958827769137202760625174959209710926272934884125";
    array_alea[53]="5374703998523779877668814347087078229484744080768928436056691061595276571035132716695777581248585894110804172595660";
    array_alea[54]="37094571471732066241746786633947177139428527417859806986712901053160226199120502858708133908553995494754797799826182";
    array_alea[55]="8952280803077042771560225401068315613701864990317137431473421194251972197395306874529941231662959631463217231529529";
    array_alea[56]="9223337831243173319655248983786315157378520901434304923023851496243704532939168240737328136906103404771470820840457";
    array_alea[57]="23183084941706163864322437699255313248299773959705464904646749784763121293384432835504953172477011583257107585995245";
    array_alea[58]="26644402859350422103195410962540914951344860070834717965285846719694197212279458235805438884661928573797566469133197";
    array_alea[59]="20555944280897915320797989429411192611553750125805830251171935274961797717593280982105472137680857626525886528769039";
    array_alea[60]="3300378883059543484258328305118280321133452213091681260278072353894569481405926966082218398675083561947642871554868";
    array_alea[61]="20018212592403416409138225009617812988965020663971281964971529196493705955047421927624816469144790254284962539373981";
    array_alea[62]="14745081550539762176575827005097209641524434569867627164444867447556128463799756517771163244715597578405821381428941";
    array_alea[63]="38340358784833275379272552101372653529941276035197998669645392720313846850970813263493914416263154625514785499594094";
    array_alea[64]="1207851693609052930820264934677123176248286916467301325803225962468040960445562951783688068361932382918965426187407";
    array_alea[65]="37126473301151089279044322030167612520923459372796439942707205332612145961595275439355017175837161347796879299808062";
    array_alea[66]="2539079358602235274639536561532528029052109852058346966921647142168036563602925520585698708116431330849920129743495";
    array_alea[67]="33413438940270683798221600752382288060859241335656370426814771585191340889130704134914493040609203271760567195388349";
    array_alea[68]="418128461610981593785493832794684816337671284686456548452355912355473792469407317995905305609194364500483967826591";
    array_alea[69]="7744545237301752168452373055106360408462945287159506757525652496645891130851141232855334761567256050194314659566270";
    array_alea[70]="33671858057228584985328993640188244843506236467926643965770204929218365389577931784630527017226337979065405293377429";
    array_alea[71]="14693990126566723239741209085371970794272337543749439798221546697396518981740288790778793159088463103472703163080469";
    array_alea[72]="31073868724597544372287260245686549943627942150361766124965618969285448430815560600488560439548686241959680401775650";
    array_alea[73]="4711238783299487169839233708394803731564015469711006855452005375487186462302611871258546993332652297885384673951747";
    array_alea[74]="27975057678473636064596887617390950835069099654043710740987103970778526220521796285173676104091841590199958979843092";
    array_alea[75]="12407909872648171771870415610742896677705230779364283731611740561869258785038929304221184561228671462765182614368358";
    array_alea[76]="114967238866291666949642191578350943644284225128854580082211992660039508837652626953363505504334233554329815871813";
    array_alea[77]="34625262688018544097724640367565937595534923609592186554728723838322921892301785683849177716109850939869486769610736";
    array_alea[78]="27050804249463125554258763618325833930599860935986323321351779103612778543005221756370954309976276612700509374710178";
    array_alea[79]="24127270375791733355360680183156323974699288034920915192919074798337415050253444876959762882673396487452925132669441";
    array_alea[80]="25584133073809474402331589719186377181875095938701150952871637735322261237755827260564473139147793592074762327550173";
    array_alea[81]="1758594778750278395142649838744698089508028292506610480180848546531836860438682678000451262353009291315174108978828";
    array_alea[82]="4951347563633952489328982390882686713282170903382370652945374745353208416169386746093997470919687009215382619408054";
    array_alea[83]="10358489261625356163935889234872793088403130195200632218946935458955797427222703587620409820688520929195822853421094";
    array_alea[84]="10245058769083106830862035076539032490788314669788972232081938516379478681083568922620166534118714377837878503146563";
    array_alea[85]="34961421455447242163339253022353874255430434239806054360445972805883356322943658912459508471900220189633497983299514";
    array_alea[86]="34909927619095153875899368295422310370837137083666515483295151563141251612883999489984665912373845803702676228823639";
    array_alea[87]="29495790398699001680983490189271681786269913166554243570852493686809942614197648725715840660481065046970852280664020";
    array_alea[88]="7648338008123556247218479795671491103827934910566006385095867138612945298525142627704014153176786802290235774266954";
    array_alea[89]="6101945166645906411774929510708727685575044638723220564361325474700518476491154435275304986281572962175859749080721";
    array_alea[90]="24955829176927879306024831011157515402381808030476248665156367914655725112058095316253540199223346275352551416474111";
    array_alea[91]="34201573967283613678790511553475708131426326976222703176008871014758820677610486591060022559777540710365063586257394";
    array_alea[92]="23568693791776666267796877028436717046259192675854344537678205212999656588545494442821278495101215447608290733957079";
    array_alea[93]="23820750314913895851286738826863938682593624733006709990979930087902012413379416565034864796823944442503311743504827";
    array_alea[94]="38252898300094384081407683391744194756844526215928390298053757368299301165823692100455528776766396722879330582144912";
    array_alea[95]="30200753567943460741731514275344959756911876576352771311209676930826552145808914063544258978440694355063941495571860";
    array_alea[96]="38403122881909311581752508694456217818962160461480451610569769901848567370967454181461384516174097311547519837155143";
    array_alea[97]="762460890485477657997646235411587539530659245637302734581007652895914511816032920908905568876776307513662942403118";
    array_alea[98]="39016364790203415320347918331856763238313294988209084934356709470658118424540970798744422488820588405360762749046597";
    array_alea[99]="23880641744487629072628645460087238420984912387737803963692368178894625842498942806178870064840546690261684190941601";
    
    for(i=0;i<nb_iteration; i++)
    {
        
        /* first we test projective coordinates */
        field_elt_set_ui(x1, 0, true, k, STACK_1);
        field_elt_set_str(y1,array_src2[i], 10, false , k, STACK_1);
        field_elt_set_one(t1,k);
        field_elt_set_str(z1, array_src2[i], 10, false , k, STACK_1);
        ec_point_set(P, x1, y1, z1, t1, k);        
    
        test1=edwards_point_is_neutral(P,E);        
        
        /* secondly we test Extended Edwards coordinates */
        field_elt_set_str(z2,array_alea[i], 10, false , k, STACK_1);
        field_elt_set_str(y2,array_alea[i], 10, false , k, STACK_1);
        field_elt_set_ui(t2, 0, true, k, STACK_1);
        field_elt_set_ui(x2, 0, true, k, STACK_1);
        ec_point_set (Q, x2, y2, z2, t2, k);
        test3=edwards_point_is_neutral(Q,Etri);        
        if(!test1 || !test3) 
        {
            test=false;            
            printf("we are in the loop %d \n",i);
            if(!test3)
            {
                printf("test 3 failed \n");            
                printf("\n Q : ");
                ec_point_print(Q,10,true, k, STACK_1);
            }
                        
            if(!test1)
            {
                printf("test 1 failed \n");            
                printf("P : ");
                field_elt_set_ui(x1, 0, true, k, STACK_1);
                field_elt_set_str(y1,array_src2[i], 10, false , k, STACK_1);
                field_elt_set_one(t1,k);
                field_elt_set_ui(z1, 0, true, k, STACK_1);
                ec_point_set(P, x1, y1, z1, t1, k);
                ec_point_print(P,10,true, k, STACK_1);
                printf("test sur Z %d, test sur x %d \n Z && X %d \n", field_elt_iszero(P->z, k), field_elt_iszero(P->x, k), field_elt_iszero(P->z, k) && field_elt_iszero(P->x, k));
                printf("test fonction %d \n",edwards_point_is_neutral(P,E) );
            }
            
        }
    }
    /* We do a second loop to test if we have false positive */
    for(i=0;i<nb_iteration; i++)
    {
        /* first we test projective coordinates */
        field_elt_set_ui(x1, 0, true, k, STACK_1);
        field_elt_set_str(y1,array_src2[i], 10, false , k, STACK_1);
        field_elt_set_one(t1,k);
        field_elt_set_str(z1,array_alea[i], 10, false , k, STACK_1);
        ec_point_set(P, x1, y1, z1, t1, k);        
    
        test1=edwards_point_is_neutral(P,E);  

        /* secondly we test Extended Edwards coordinates */
        field_elt_set_str(z2,array_alea[i], 10, false , k, STACK_1);
        field_elt_set_str(y2,array_alea[(i+1)%100], 10, false , k, STACK_1);
        field_elt_set_ui(t2, 0, true, k, STACK_1);
        field_elt_set_ui(x2, 0, true, k, STACK_1);  
        ec_point_set(Q, x2, y2, z2, t2, k);
        test3=edwards_point_is_neutral(Q,Etri);         
        if(test1 || test3)
        {
            test=false;            
            printf("we are in the «false» loop %d \n",i);
            if(test3)
            {
                printf("test 3 failed \n");            
                printf("\n Q : ");
                ec_point_print(Q,10,true, k, STACK_1);
            }
                        
            if(test1)
            {
                printf("test 1 failed \n");            
                printf("P : ");
                field_elt_set_ui(x1, 0, true, k, STACK_1);
                field_elt_set_str(y1,array_src2[i], 10, false , k, STACK_1);
                field_elt_set_one(t1,k);
                field_elt_set_str(z1,array_alea[i], 10, false , k, STACK_1);
                ec_point_set(P, x1, y1, z1, t1, k);
                ec_point_print(P,10,true, k, STACK_1);
            }
            
        }
    }

    ec_point_free(P, k);
    ec_point_free(Q, k);
    number_free(&p);
    number_free(&a);
    number_free(&d);
    field_elt_free(&b, k);
    field_elt_free(&c, k);
    field_elt_free(&x1, k);
    field_elt_free(&x2, k);
    field_elt_free(&y1, k);
    field_elt_free(&y2, k);
    field_elt_free(&z1, k);
    field_elt_free(&z2, k);
    field_elt_free(&t1, k);
    field_elt_free(&t2, k);
    ec_free(E);
    ec_free(Etri); 
    field_free(k);
    free_mphell();

    if (test)
    {
        printf("SUCCESS \n");
        return 0;
    }
    else
    {
        printf("ERROR \n");
        return -1;
    }
}
