/* 
  MPHELL-4.0 
  Author(s): The MPHELL team

 (C) Copyright 2015 - Institut Fourier / Université Grenoble Alpes (Grenoble) 

 This file is part of the MPHELL Library.
 MPHELL is free software: you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation, version 3 of the License.
 
 MPHELL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with MPHELL.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
  * \file  mphell_test_edwards_point_random.c
  * \brief Self check tests for the function: void edwards_point_random (ec_point P, ec_curve_srcptr E)
  */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>
#include "mphell.h"
#include "mphell_test_weierstrass.h"
#include "mphell_test_edwards.h"


bool 
test_curve_edwards_random(data_t *data)
{
    printf("Test random: ");
    int i;
    for (i=0; i<100; i++)
    {
        edwards_point_random (data->P1[i], data->E, STACK_1);
        if(!edwards_belongs(data->P1[i], data->E, STACK_1))
        {
            printf("Error, P3[%d] does not belong to E\n", i);
            return false;
        }
    }
    printf("tests for collisions \n");
    int j;
    int nb_collision = 0;
    for (j=0; j<100; j++)
    {
        for (i=0; i<100; i++)
        {
            if (i != j)
            {
                if (edwards_point_are_equal(data->P1[i], data->P1[j], data->E, STACK_1))
                {
                    nb_collision++;
                }
            }
        }
    }
    if(nb_collision > 1)
    {
        printf("Error, to many collision: %d\n", nb_collision);
        return false;
    }
    printf("number of collision: %d\n", nb_collision);
    
    return true;
}

bool
test_curve_edwards(const char *id_curve, const uint16_t nbits,
        const uint32_t num_point, data_t *data)
{
    bool res1,res2;
    test_curve_weierstrass_alloc(nbits, num_point, data);       /* We can re use this function */
    test_curve_edwards_read(id_curve, EXTENDED_EDWARDS, data);
    test_curve_weierstrass_init(data);
    res1=test_curve_edwards_random(data);
    test_curve_weierstrass_free(data);
    test_curve_weierstrass_alloc(nbits, num_point, data);       /* We can re use this function */
    test_curve_edwards_read(id_curve, PROJECTIVE, data);
    test_curve_weierstrass_init(data);
    res2=test_curve_edwards_random(data);
    test_curve_weierstrass_free(data);
    return res1 && res2 ;
}



int main()
{
    data_t data;
    init_mphell(128, RANDOM_AES128, DEVURANDOM);
    int nb_points=100;
    if (!test_curve_edwards("ed_25519", 256, nb_points, &data))
    {
        free_mphell();
        return -1;
    }
    free_mphell();
    printf("SUCCESS \n");
    return 0;
} 
