/* 
  MPHELL-4.0 
  Author(s): The MPHELL team

 (C) Copyright 2015-2018 - Institut Fourier / Univ. Grenoble Alpes (France)

 This file is part of the MPHELL Library.
 MPHELL is free software: you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation, version 3 of the License.
 
 MPHELL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with MPHELL.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
  * \file  mphell_test_field_elt_conversion_montgomery.c
  * \brief Self check tests for the conversion between Montgomery and classical modular form.
  */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>
#include <string.h>

#include "mphell.h"

int main()
{
    init_mphell(128, RANDOM_AES128, DEVURANDOM);
    bool test = true;
    
    field k;
    field k1;
    field k2;
    field_alloc(k, FP, bits_to_nblock(384), NULL);
    field_alloc(k1, FP, bits_to_nblock(224), NULL);
    field_alloc(k2, FP, bits_to_nblock(160), NULL);
    
    number p;
    number p1;
    number p2;
    number_init(&p, bits_to_nblock(384));
    number_init(&p1, bits_to_nblock(224));
    number_init(&p2, bits_to_nblock(160));
    number_set_str(p, "fffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffeffffffff0000000000000000ffffffff", 16);
    number_set_str(p1, "ffffffffffffffffffffffffffffffff000000000000000000000001", 16);
    number_set_str(p2, "E95E4A5F737059DC60DFC7AD95B3D8139515620F", 16);
    field_create(k, "", STACK_1, 1, p);
    field_create(k1, "", STACK_1, 1, p1);
    field_create(k2, "", STACK_1, 1, p2);
    
    printf("k: \n");
    field_print(k, 16, STACK_1);
    
    printf("k1: \n");
    field_print(k1, 16, STACK_1);
    
    printf("k2: \n");
    field_print(k2, 16, STACK_1);
    
    field_elt u;
    field_elt u1;
    field_elt u2;
    
    field_elt_alloc(&u, k);
    field_elt_init(u, k);
    field_elt_alloc(&u1, k1);
    field_elt_init(u1, k1);
    field_elt_alloc(&u2, k2);
    field_elt_init(u2, k2);
    
    char * dst_str;
    
    printf("Enter TEST:\n");
    
    /* Test k */
    printf("TEST k:\n");
        
    field_elt_set_str(u, "28822474035011734998840637310067332146104893076943838244941897319458526249053465950091908758716210117533603076599931", 10, false, k, STACK_1);
    printf("mon(u):"); field_elt_print(u, 10, false, k, STACK_1); printf("\n");
    field_elt_str(&dst_str, u, 10, true, k, STACK_1);
    if (strcmp(dst_str, "28822474035011734998840637310067332146104893076943838244941897319458526249053465950091908758716210117533603076599931") !=0)
    {
        test = false;
        printf("ERROR k \n");
    }
    free(dst_str);
    
    /* Test k1 */
    printf("TEST k1:\n");
    
    field_elt_set_str(u1, "8065891156419130626512280800099061435264494856603834915288544685563", 10, false, k1, STACK_1);
    printf("mon(u1):"); field_elt_print(u1, 10, false, k1, STACK_1); printf("\n");
    field_elt_str(&dst_str, u1, 10, true, k1, STACK_1);
    if (strcmp(dst_str, "8065891156419130626512280800099061435264494856603834915288544685563") !=0)
    {
        test = false;
        printf("ERROR k1 \n");
    }
    free(dst_str);
    
    /* Test k2 */
    printf("TEST k2:\n");
    
    field_elt_set_str(u2, "1177547453828919063580511675504088878340114900327", 10, false, k2, STACK_1);
    printf("mon(u2):"); field_elt_print(u2, 10, false, k2, STACK_1); printf("\n");
    field_elt_str(&dst_str, u2, 10, true, k2, STACK_1);
    if (strcmp(dst_str, "1177547453828919063580511675504088878340114900327") !=0)
    {
        test = false;
        printf("ERROR k2 \n");
    }
    free(dst_str);

    field_elt_free(&u, k);
    field_elt_free(&u1, k);
    field_elt_free(&u2, k);
    number_free(&p);
    number_free(&p1);
    number_free(&p2);
    field_free(k);
    field_free(k1);
    field_free(k2);
    
    free_mphell();

    /* Result */
    
    if (test)
    {
        printf("SUCCESS \n");
        return 0;
    }
    else
    {
        printf("ERROR \n");
        return -1;
    }
}
