/* 
  MPHELL-4.0 
  Author(s): The MPHELL team

 (C) Copyright 2015-2018 - Institut Fourier / Univ. Grenoble Alpes (France)

 This file is part of the MPHELL Library.
 MPHELL is free software: you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation, version 3 of the License.
 
 MPHELL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with MPHELL.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
  * \file  mphell_test_field_elt_sqrt.c
  * \brief Self check tests for the function: void field_elt_sqrt (fe_ptr dst, fe_srcptr src)
  */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>
#include <string.h>

#include "mphell.h"

int main()
{
    init_mphell(128, RANDOM_AES128, DEVURANDOM);
    bool test = true;
    int i;
    int nb_iteration = 100;
    
    field k;
    field_alloc(k, FP, bits_to_nblock(192), NULL);
    number p;
    number_init(&p, bits_to_nblock(192));
    number_set_str(p, "fffffffffffffffffffffffffffffffeffffffffffffffff", 16);
    
    field k1;
    field_alloc(k1, FP, bits_to_nblock(256), NULL);
    number p1;
    number_init(&p1, bits_to_nblock(256));
    number_set_str(p1, "57896044618658097711785492504343953926634992332820282019728792003956564819949", 10);
    
    field_create(k, "", STACK_1, 1, p);
    field_create(k1, "", STACK_1, 1, p1);
    
    field_elt u;
    field_elt dst;
    field_elt res;
    
    field_elt_alloc(&u, k);
    field_elt_init(u, k);
    field_elt_alloc(&dst, k);
    field_elt_init(dst, k);
    field_elt_alloc(&res, k);
    field_elt_init(res, k);
    
    field_elt u1;
    field_elt dst1;
    field_elt res1;
    
    field_elt_alloc(&u1, k1);
    field_elt_init(u1, k1);
    field_elt_alloc(&dst1, k1);
    field_elt_init(dst1, k1);
    field_elt_alloc(&res1, k1);
    field_elt_init(res1, k1);
    
    char * dst_str;
    char * res_str;
    
    const char *array_src[105];
    
    array_src[0]="ef6af86783713ebace157160c0c37e0e67f4138423597643";
    array_src[1]="95352424ab5a405e2a05e00d45bf45247a10caf4be06336e";
    array_src[2]="70c4c2ef87f06e60aa26953ea05126536aa61dc9498a2ba5";
    array_src[3]="77a5151ce7fcda06499e0d176191bb2040fddd6037668e58";
    array_src[4]="d0519580b8a14bf47b419072f6b1c77107cf367bab8f20e";
    array_src[5]="48a0665fc178b14bdde7c3aff17b4078b0d50445b09d9b4";
    array_src[6]="a314d6381899248aaa2cc56c2a3912810fe30726d90f3297";
    array_src[7]="301725346b9b4675c30d2d28a814c88560735b3aa8153dd4";
    array_src[8]="56d402c73d77a406357532a42fb49bf9004ad89cc58b4a6d";
    array_src[9]="c6ea4086e83397e9bf44480357ab2fa053fd2aabbc5af559";
    array_src[10]="e249989f96876935e588bc1f28fc35d1560f32d825336ea6";
    array_src[11]="1ce8e7b84dd8aedd2760056a0cac2c80dbc0abb5b3a4b7f2";
    array_src[12]="ebb78a0b8eac33842a86276497d358cf83a7b6e1ffb8475a";
    array_src[13]="11500448583687a7b5c7cfc96450f6c3bf01f358690f673b";
    array_src[14]="5de52dbb274c862dc2ac678bfe945f6dd2fd00485ea2e3ed";
    array_src[15]="4fa14a89e2fae073058d91adf681248edf27733aa8dec2d9";
    array_src[16]="34acae35b139d62fc46938abe11fa4b8d71014fe8420ab83";
    array_src[17]="e193cee4c9fc8ebd4741187580aa8b55876fbb7d548b2944";
    array_src[18]="f6a71763a919e1ebfc78415c39bca43664a84dd73d2dd389";
    array_src[19]="be7fa02ff23358a81cdb7daaa57f8ee4d577da970d51e492";
    array_src[20]="3eb3274d124a74b5e3de156faf828b836e33d5392cd0a16f";
    array_src[21]="73064751e742ae18130b1e23e20b41b48578c4868d11ed6b";
    array_src[22]="64ba65df20491bda4a551afafc134765a9c2b4b4be781b04";
    array_src[23]="a7bc7799bee3345f2d8f9ffadf99f4d139e939c4d82c1318";
    array_src[24]="d95bbc469875dd90a52f98415401b4892533427f8d3f827f";
    array_src[25]="bae00542992ebfd0ede491e3ced9f85bacfdd78d04f66c5b";
    array_src[26]="515e353021dfa0fa4cf91ec1079285265f5795d2c97453c5";
    array_src[27]="99ed10be84dc63a5279cfab2f4ece67d26c140ed41ab2135";
    array_src[28]="321bffcb9ab7dc776b66b7f230abdb32d9c0fc40e340171e";
    array_src[29]="edcd42a695483d1cfb05a7aa0748a0a809825f69ba747cf0";
    array_src[30]="1e180234e0b77e73ba2e8e769763f3bbed54dac43d986633";
    array_src[31]="dec2e40fd86501349cd21f6c86f4198c7ec984a2aba77452";
    array_src[32]="1db1c354107a919c2b85cc62e04a7e92391788f0dbef366d";
    array_src[33]="4033ce92b208a75c02dba6a25444fd587d09de53b465aaca";
    array_src[34]="fe633a6ec02774c8076ea0bad26f5d6e9611bc3ed59a203d";
    array_src[35]="9fec619f102b60ae1156d0333a400833f18ba0dadd6c5a8";
    array_src[36]="eeeeafc969e343b00cabd69afaf82b9e389be52230586b0a";
    array_src[37]="1a3fca77920aa09e1e197fc41d773dd95596cd40e7557f3b";
    array_src[38]="60922a6327c62aac091a87d3a709bcef5637886873b86b14";
    array_src[39]="b7f22ee5b070160208980b8d0784383153bfe84b49eb528a";
    array_src[40]="ca99985a9a5ded2c67eaddebdffe34c215849520ca1b461c";
    array_src[41]="ae00ad473b3b1446e1f7bc1f4b8277a65909727a197ac470";
    array_src[42]="783bab22f9408cd933086e9481ea5e0afe31f1431496fc02";
    array_src[43]="809334d6fc77cec97b9185dfef38dbc4af9c4a6ae0f62658";
    array_src[44]="3c662890b556ebdeddb32235501fca88aec5ede4bd6d7842";
    array_src[45]="6cbb1ee57c96c25e8574419c7fe155d8c1ea9a3569b343ac";
    array_src[46]="41a40b010c80723653f6b2ebbe5b23a633f5338da763cb11";
    array_src[47]="62d15dde3a57ed08c1949b596498ca5d4dcf0da6681db3f3";
    array_src[48]="542e7af8eba3f6ecd57648c5e42380dad5af660cb073ea5b";
    array_src[49]="6b445d4f109f70b54fbd5132a19cba239a26de82ab206882";
    array_src[50]="4345f554299c2386653909bce63b67442564df7658e2d2e7";
    array_src[51]="7bf35bbda27879396a8944bff59faa25fd31846d26538011";
    array_src[52]="81a0d19a796a10f24d57232e584d8ab1f162a6df4cfbb11e";
    array_src[53]="34a7d88fbdf1d7468a3aa682abbed8a78fbb1c95da8534c2";
    array_src[54]="c275a8f8c7b61c871877aa552dfe843cc0be355c848d792";
    array_src[55]="6370a3ab03ada2ad770cd974b6d0e33ee044c6c02554673d";
    array_src[56]="7ce09afd7cda2196a98f4a65797b29fbac6344695c7761c6";
    array_src[57]="55506befc4c569fa0d18eabb833b82f5f3c3735316208027";
    array_src[58]="1d16254fcb1c9228e8daeb4bcf62c2138d2235d3aeea8cff";
    array_src[59]="255b75d272d21cfc4d622e53a658bf9120f5fe4cc71d64aa";
    array_src[60]="3f5c8377830aadc47069179ea9271b3e1ce420c9dd1f1bd8";
    array_src[61]="b7339e03fb3b58f88840bd578a84bed6a7a8de5b62d5711e";
    array_src[62]="768479826da48521c74699c4a98eb8813a77ddc443193835";
    array_src[63]="42b218c55ba10d39be0f4fe8f40dcc3bb632b7a090c6a898";
    array_src[64]="bf51a743c3f4aec6bca104a39f8e58e7c16914760bdf28be";
    array_src[65]="55a3a1ba97d437ee2cb7889252797ca79befeaf804ec4809";
    array_src[66]="75b791c0d9e7127a995333f563ffcc8033be749042b5c395";
    array_src[67]="559a5867c35c653469305810957bd5ed40b97543372c5e";
    array_src[68]="edde206898f2f6a3bc621e27d5ef9bf38a329088bae5740b";
    array_src[69]="a821ab20e17957f05dcefaa03c32ae5bd5f09118c145bb9e";
    array_src[70]="c12a2ed575cad62e05ba5e3c86dbdea1296f5796c7539ee9";
    array_src[71]="cae199ae866ef04e114cad11f3a9642de0c03479a072e6c0";
    array_src[72]="d0038beb9c7575f379d84e9ca7111ecbba8d7f50569ba925";
    array_src[73]="3c1c46eee5e20dc070f88f7e3d5dc4cf52858a566c275ada";
    array_src[74]="b5243da88691f455f4a45b51329a21300bed0af7c57dc50e";
    array_src[75]="fd9a60ed1245b7342132136837dd51616ab55bb9668d3cc3";
    array_src[76]="ddde919d40edbbf4006c936ba43ab20f7ed84f1539b2207";
    array_src[77]="6d2ebc23a5d98e5da31163c758d4ec27daed14d16b116b2d";
    array_src[78]="f213ddf310f70e54452580cb81fc0186ac8efab6b9b6a4e2";
    array_src[79]="d3ef6ce2feb1cbc599064da26e6aaabcbbbc16a7b26dd039";
    array_src[80]="77237325f9e065a3243d2471d89903e1167ff1e494ede7a";
    array_src[81]="88cffbe8807f4252efe6c6a91b70df79fae548d5985355f";
    array_src[82]="e1b4b8cd86e87f2489d0b8ceef5bdb74ab267d53b36f9654";
    array_src[83]="98ff1ac2ee2b9a764afe797a90eb47b9b93f440f182b1ea9";
    array_src[84]="734f7bd2d97c5f6939da1a2fd53899e84df486fc6a298b02";
    array_src[85]="b9f6f63fa2fb79e9cdda8f6ec8d215590cc5b737bd2464c3";
    array_src[86]="c876630e2c9fbf495e9f5168e4ed13a1aae638aae75008b3";
    array_src[87]="73b4e511300e89725ea142ce097e853bb86f0b5c5d24faab";
    array_src[88]="3f38f55ff5c30406e4f912c1cd18e17a92551f9125c89677";
    array_src[89]="703b8adba6d5c4cbb37e54ee9bc408cfef0f5e0497668e6";
    array_src[90]="9da77a36ab4835018eacd3d991bae3f6d2c9483f54c2d3d8";
    array_src[91]="c36415bc791adddb5fc63e1050044255fb3abdb11a3ea51d";
    array_src[92]="545035555eaca5ad5d3104ecfa26428fed4a98c223da1223";
    array_src[93]="bcfcf766bb7e2f446c903fae19edc46cf8494aefcdf2efe8";
    array_src[94]="2f027d486424320550eccc1f243abf227445dde85e45672a";
    array_src[95]="1f3ee4dd9fd40d48fffc7c969e87ed63efa2d3a75e8ec2c5";
    array_src[96]="a6f41a4d31264fd224d20155bd2500cf256643d22f721b97";
    array_src[97]="36cef91832d40415854720755ba57f1b2450ed85d2e9981";
    array_src[98]="21a59039988a58198a4b064c9c1637a2883d9f77e647b47b";
    array_src[99]="2021c7ef0582bc1e9a814d6208262b90f45a3bd119847305";
    array_src[100]="33867069469998205893085474163433710214579746818797039615066168543612165131936";
    array_src[101]="43792135742243762944414318601743822201500376757996171789115581904247436775067";
    array_src[102]="21213460922582769294235371792141657882701490114915148777567191488994866924926";
    array_src[103]="6897842117056194841498743239076357275558747652688080130691870550173181867166";
    array_src[104]="47543895860033990522930744857046259532165417508774548225375652946912154416571";
    
    printf("Enter TEST:\n");
    
    for(i=0; i<nb_iteration; i++)
    {
        /* Set element from string */

        field_elt_set_str(u, array_src[i], 16, false, k, STACK_1);
        field_elt_set_str(res, array_src[i], 16, false, k, STACK_1);
        
        /* Computation */
        
        field_elt_sqrt(dst, u, k, STACK_1);
        field_elt_sqr(dst, dst, k, STACK_1);
        
        field_elt_str(&dst_str, dst, 2, true, k, STACK_1);
        field_elt_str(&res_str, res, 2, true, k, STACK_1);
        
        /* Compare string */
        
        if(strncmp(res_str, dst_str, strlen(res_str))!=0)
        {
            test=false;
            printf("dst: %s \n", dst_str);
            printf("res: %s \n", res_str);
        }
        free(dst_str);
        free(res_str);
    }
    for(i=nb_iteration; i<nb_iteration+5; i++)
    {
        /* Set element from string */

        field_elt_set_str(u1, array_src[i], 10, false, k1, STACK_1);
        field_elt_set_str(res1, array_src[i], 10, false, k1, STACK_1);
        
        /* Computation */
        
        field_elt_sqrt(dst1, u1, k1, STACK_1);
        field_elt_sqr(dst1, dst1, k1, STACK_1);
        
        field_elt_str(&dst_str, dst1, 2, true, k1, STACK_1);
        field_elt_str(&res_str, res1, 2, true, k1, STACK_1);
        
        /* Compare string */
        
        if(strncmp(res_str, dst_str, strlen(res_str))!=0)
        {
            test=false;
            printf("dst: %s \n", dst_str);
            printf("res: %s \n", res_str);
        }
        free(dst_str);
        free(res_str);
    }
    
    field_elt_free(&u, k);
    field_elt_free(&dst, k);
    field_elt_free(&res, k);
    number_free(&p);
    field_free(k);
    
    field_elt_free(&u1, k1);
    field_elt_free(&dst1, k1);
    field_elt_free(&res1, k1);
    number_free(&p1);
    field_free(k1);
    
    free_mphell();

    /* Result */
    
    if (test)
    {
        printf("SUCCESS \n");
        return 0;
    }
    else
    {
        printf("ERROR \n");
        return -1;
    }
}
