/* 
  MPHELL-4.0 
  Author(s): The MPHELL team

 (C) Copyright 2015-2018 - Institut Fourier / Univ. Grenoble Alpes (France)

 This file is part of the MPHELL Library.
 MPHELL is free software: you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation, version 3 of the License.
 
 MPHELL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with MPHELL.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
  * \file  mphell_test_field_elt_unity_nth_root.c
  * \brief Self check tests for the function: void field_elt_unity_nth_root (fe_ptr dst, const block n, const fp_param param);
  */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>
#include <string.h>

#include "mphell.h"

int main()
{
    init_mphell(128, RANDOM_AES128, DEVURANDOM);
    bool test = true;
    
    field k;
    field k1;
    field k2;
    field_alloc(k, FP, bits_to_nblock(384), NULL);
    field_alloc(k1, FP, bits_to_nblock(224), NULL);
    field_alloc(k2, FP, bits_to_nblock(160), NULL);
    
    number p;
    number p1;
    number p2;
    number_init(&p, bits_to_nblock(384));
    number_init(&p1, bits_to_nblock(224));
    number_init(&p2, bits_to_nblock(160));
    number_set_str(p, "fffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffeffffffff0000000000000000ffffffff", 16);
    number_set_str(p1, "ffffffffffffffffffffffffffffffff000000000000000000000001", 16);
    number_set_str(p2, "E95E4A5F737059DC60DFC7AD95B3D8139515620F", 16);
    field_create(k, "", STACK_1, 1, p);
    field_create(k1, "", STACK_1, 1, p1);
    field_create(k2, "", STACK_1, 1, p2);
    
    field_elt u;
    field_elt dst;
    field_elt u1;
    field_elt dst1;
    field_elt u2;
    field_elt dst2;
    
    field_elt_alloc(&u, k);
    field_elt_init(u, k);
    field_elt_alloc(&dst, k);
    field_elt_init(dst, k);
    field_elt_alloc(&u1, k1);
    field_elt_init(u1, k1);
    field_elt_alloc(&dst1, k1);
    field_elt_init(dst1, k1);
    field_elt_alloc(&u2, k2);
    field_elt_init(u2, k2);
    field_elt_alloc(&dst2, k2);
    field_elt_init(dst2, k2);
    
    printf("Enter TEST:\n");
    
    /* Cube root of unity */
    
    /* k */

    field_elt_unity_nth_root(dst, (block)3, k, STACK_1);
    printf("Cube root of unity: "); field_elt_print(dst, 10, true, k, STACK_1); printf("\n");
    field_elt_pow_ui(u, dst, (block)3, k, STACK_1);
    bool res = field_elt_isone(u, k);
    if(res)
    {
        printf("OK !\n");
    }
    else
    {
        printf("ERROR !\n");
        test = false;
    }
    
    /* k1 */
    
    field_elt_unity_nth_root(dst1, (block)3, k1, STACK_1);
    printf("Cube root of unity: "); field_elt_print(dst1, 10, true, k1, STACK_1); printf("\n");
    field_elt_pow_ui(u1, dst1, (block)3, k1, STACK_1);
    res = field_elt_isone(u1, k1);
    if(res)
    {
        printf("OK !\n");
    }
    else
    {
        printf("ERROR !\n");
        test = false;
    }
    
    /* k2 */
    
    field_elt_unity_nth_root(dst2, (block)3, k2, STACK_1);
    printf("Cube root of unity: "); field_elt_print(dst2, 10, true, k2, STACK_1); printf("\n");
    field_elt_pow_ui(u2, dst2, (block)3, k2, STACK_1);
    res = field_elt_isone(u2, k2);
    if(res)
    {
        printf("OK !\n");
    }
    else
    {
        printf("ERROR !\n");
        test = false;
    }
    
    /* n-th root of unity */
    
    block n;
    
    for (n=3; n<100; n++)
    {
        /* k */
        
        field_elt_unity_nth_root(dst, n, k, STACK_1);
        field_elt_pow_ui(u, dst, n, k, STACK_1);
        bool res = field_elt_isone(u, k);
        if(!res)
        {
            printf("ERROR !\n");
            test = false;
        }
        
        /* k1 */
        
        field_elt_unity_nth_root(dst1, n, k1, STACK_1);
        field_elt_pow_ui(u1, dst1, n, k1, STACK_1);
        res = field_elt_isone(u1, k1);
        if(!res)
        {
            printf("ERROR !\n");
            test = false;
        }
        
        /* k2 */
        
        field_elt_unity_nth_root(dst2, n, k2, STACK_1);
        field_elt_pow_ui(u2, dst2, n, k2, STACK_1);
        res = field_elt_isone(u2, k2);
        if(!res)
        {
            printf("ERROR !\n");
            test = false;
        }
    }
   
    field_elt_free(&u, k);
    field_elt_free(&dst, k);
    field_elt_free(&u1, k1);
    field_elt_free(&dst1, k1);
    field_elt_free(&u2, k2);
    field_elt_free(&dst2, k2);
    number_free(&p);
    number_free(&p1);
    number_free(&p2);
    field_free(k);
    field_free(k1);
    field_free(k2);
    
    free_mphell();

    /* Result */
    
    if (test)
    {
        printf("SUCCESS \n");
        return 0;
    }
    else
    {
        printf("ERROR \n");
        return -1;
    }
}
