/* 
  MPHELL-4.0 
  Author(s): The MPHELL team

 (C) Copyright 2015-2018 - Institut Fourier / Univ. Grenoble Alpes (France)

 This file is part of the MPHELL Library.
 MPHELL is free software: you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation, version 3 of the License.
 
 MPHELL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with MPHELL.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
  * \file  mphell_test_field_find_nonpower_ui.c
  * \brief Self check tests for the function: void field_find_nonpower_ui (fe_ptr dst, const block n, field k)
  */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>
#include <string.h>

#include "mphell.h"

int main()
{
    init_mphell(128, RANDOM_AES128, DEVURANDOM);
    bool test = true;
    int i;
    int nb_iteration = 100;
    
    field k;
    field_alloc(k, FP, bits_to_nblock(192), NULL);
    number p;
    number_init(&p, bits_to_nblock(192));
    number_set_str(p, "fffffffffffffffffffffffffffffffeffffffffffffffff", 16);
    
    field_create(k, "", STACK_1, 1, p);
    
    field_elt x;
    field_elt_alloc(&x, k);
    field_elt_init(x, k);
    bool res;
    bool exist = false;
    
    block n;
    
    printf("Enter TEST:\n");
    
    for(i=0; i<nb_iteration; i++)
    {
        do
        {
            #if BLOCK_SIZE == 64
            n=rand64();
            #else
            n=rand32();
            #endif
            exist = field_find_nonpower_ui(x, n, k, STACK_1);
        }while(!exist);
        
        /* Computation */
        
        res=field_elt_ispower_ui(x, n, k, STACK_1);
        
        /* Compare string */
        
        if(res)
        {
            test=false;
            printf("x: "); field_elt_print(x, 16, true, k, STACK_1); printf("\n");
            #if BLOCK_SIZE == 64
            printf("n: %lu, \n", n);
            #else
            printf("n: %u, \n", n);
            #endif
        }
    }
    
    field_elt_free(&x, k);
    number_free(&p);
    field_free(k);
    
    free_mphell();

    /* Result */
    
    if (test)
    {
        printf("SUCCESS \n");
        return 0;
    }
    else
    {
        printf("ERROR \n");
        return -1;
    }
}
