/* 
  MPHELL-4.0 
  Author(s): The MPHELL team

 (C) Copyright 2015-2018 - Institut Fourier / Univ. Grenoble Alpes (France)

 This file is part of the MPHELL Library.
 MPHELL is free software: you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation, version 3 of the License.
 
 MPHELL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with MPHELL.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
  * \file  mphell_test_jacobi_quartic_compute_disc.c
  * \brief Self check tests for the function: void jacobi_quartic_compute_disc(ec_curve E, STACK_1);
  */


#include "mphell_test_jacobi_quartic.h"

bool 
test_curve_jacobi_quartic_discriminant(data_t *data)
{
    field_elt disc;
    field_elt_alloc(&disc, data->k);
    field_elt_init(disc, data->k);
    
    printf("Compute discriminant on %s: ", data->E->id_curve);
    
    if (strcmp(data->E->id_curve, "jacobi_quartic_p256")==0)
    {
        const char * str1 = "59622079090555552269371999730331452644114248109542557536021978494938118225257";
        field_elt_set_str(disc, str1, 10, false, data->k, STACK_1); 
        if (field_elt_cmp(data->E->disc, disc, data->k)!=0)
        {
            printf("Error discriminant on %s\n", data->E->id_curve);
            printf("disc   : "); field_elt_print(disc, 16, true, data->k, STACK_1);
            printf("E->disc: "); field_elt_print(data->E->disc, 16, true, data->k, STACK_1);
            field_elt_free(&disc, data->k);
            return false;
        }
    }
    else if (strcmp(data->E->id_curve, "jq256_3")==0)
    {
        const char * str1 = "57119917406246640407356964443314898173222971162321310867489136206481195276989";
        field_elt_set_str(disc, str1, 10, false, data->k, STACK_1); 
        if (field_elt_cmp(data->E->disc, disc, data->k)!=0)
        {
            printf("Error discriminant on %s\n", data->E->id_curve);
            printf("disc   : "); field_elt_print(disc, 16, true, data->k, STACK_1);
            printf("E->disc: "); field_elt_print(data->E->disc, 16, true, data->k, STACK_1);
            field_elt_free(&disc, data->k);
            return false;
        }
    }
    else
    {
        printf("Error discriminant, id_curve does not match a known discriminant %s\n", data->E->id_curve);
        field_elt_free(&disc, data->k);
        return false;
    }
    
    field_elt_free(&disc, data->k);
    printf("OK\n");
    return true;
}

bool
test_curve_jacobi_quartic(const char *id_curve, const uint16_t nbits, data_t *data)
{
    bool res;
    test_curve_jacobi_quartic_alloc(nbits, 0, data);
    test_curve_jacobi_quartic_read(id_curve, data);
    test_curve_jacobi_quartic_init(data);
    res=test_curve_jacobi_quartic_discriminant(data);
    test_curve_jacobi_quartic_free(data);
    return res;
}

int main()
{
    data_t data;
    init_mphell(128, RANDOM_AES128, DEVURANDOM);
    if (!test_curve_jacobi_quartic("jacobi_quartic_p256", 256, &data))
    {
        free_mphell();
        return -1;
    }
    else if (!test_curve_jacobi_quartic("jq256_3", 256, &data))
    {
        free_mphell();
        return -1;
    }
    free_mphell();
    return 0;
}
