/* 
  MPHELL-4.0 
  Author(s): The MPHELL team

 (C) Copyright 2015-2018 - Institut Fourier / Univ. Grenoble Alpes (France)

 This file is part of the MPHELL Library.
 MPHELL is free software: you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation, version 3 of the License.
 
 MPHELL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with MPHELL.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
  * \file  mphell_test_jacobi_quartic_conversion_curve.c
  * \brief Self check tests for the functions: jacobi_quartic_to_weierstrass,  weierstrass_to_jacobi_quartic in Extended Homogeneous Projective coordinates
  */

#include "mphell.h"


bool 
test_curve_jacobi_quartic_conversion(field k, const char *id_curve, const uint16_t nbits, 
        const uint32_t num_point)
{
    printf("Validity tests on %d curves %s : ", num_point , id_curve);
    uint8_t n = bits_to_nblock(nbits);
    uint32_t i = 0;
    uint8_t j = 0;
    bool test=true;
    ec_curve E_jq,E_W,E_T,E_T2;
    field_elt teta;    
    field_elt c,d,e,f; 
    field_elt_alloc(&teta, k);    
    field_elt_init(teta, k);    
    field_elt_alloc(&c, k);   
    field_elt_init(c, k);   
    field_elt_alloc(&d, k);
    field_elt_init(d, k);
    field_elt_alloc(&e, k);   
    field_elt_init(e, k);   
    field_elt_alloc(&f, k); 
    field_elt_init(f, k); 
    field_elt_set_ui(c, 0, false, k, STACK_1);
    field_elt_set_ui(e, 0, false, k, STACK_1);
    number tmp;
    number_init(&tmp, n);
    uint8_t check[num_point][5];
    FILE *filep1, *filep2;
    char* test_file1 = malloc(sizeof(char) * 200);
    char* test_file2 = malloc(sizeof(char) * 200);
    char *str = (char*)malloc(sizeof(char) * 200);
    char *str1 = (char*)malloc(sizeof(char) * 200);
    char line[160];
    for(i = 0; i < num_point; i++)
    {
        for(j = 0; j < 2; j++)
        {
            check[i][j] = (uint8_t)0;
        }
    }
    sprintf(test_file1, DATADIR "/vector_test_elliptic_curves/random_curve_jacobi_quartic_%s_conversion_test.txt", id_curve); 
    filep1 = fopen(test_file1,"r"); /* 1st one for Jacobi Quartic */
    if(filep1 == NULL)
    {
        printf(" \n Chemin:    %s \n", test_file1);
        fprintf(stderr, "Error : no data file found for curve conversion (Jacobi Quartic) \n");        
        exit(EXIT_FAILURE);
    }    
    sprintf(test_file2, DATADIR "/vector_test_elliptic_curves/random_curve_jq_weierstrass%s_conversion_test.txt", id_curve); 
    filep2 = fopen(test_file2,"r"); /* 2nd one for Weierstrass */
    if(filep2 == NULL)
    {
        printf(" \n Chemin:    %s \n", test_file2);
        fprintf(stderr, "Error : no curve data file found for curve conversion (Weierstrass) \n");        
        exit(EXIT_FAILURE);
    }
    for(i = 0; i < num_point; i++)
    {
        ec_alloc(E_jq, k);                  /* We allocate space for an Edwards curve we will read */
        ec_init(E_jq, k);
        ec_alloc(E_W, k);                   /* We allocate space for a Weierstrass curve we will read */
        ec_init(E_W, k);
        ec_alloc(E_T, k);                   /* We allocate space for a curve we will compute */
        ec_init(E_T, k);
        ec_alloc(E_T2, k);                  /* We allocate space for a 2nd curve we will compute */
        ec_init(E_T2, k);
        /* First we read the two curves */
        /* We read Weierstrass curve first */
        while(fgets(line, sizeof(line), filep2) != NULL)
        { 
            E_W->k=k;
            if(strncmp(line, "a = ", 4)== 0)
            {
                strcpy(str, line + 4);
                number_set_str(tmp, str, 16);
                field_elt_set_number(E_W->a, false, E_W->k, STACK_1, 1, tmp);
            }
            if(strncmp(line, "b = ", 4)== 0)
            {
                strcpy(str, line + 4);
                number_set_str(tmp, str, 16);
                field_elt_set_number(E_W->b, false, E_W->k, STACK_1, 1, tmp);
            }
            if(strncmp(line, "xG = ", 5)== 0)
            {
                strcpy(str, line + 5);
            }
            if(strncmp(line, "yG = ", 5)== 0)
            {
                strcpy(str1, line + 5);
                weierstrass_point_set_aff_str(E_W->G, str, str1, false, 16, E_W->k, STACK_1);
            }
            if(strncmp(line, "h = ", 4) == 0)
            {
                strcpy(str, line + 4);
                number_set_str(E_W->h, str, 16);
                break;
            }
            if(strncmp(line, "n = ", 4) == 0)
            {
                strcpy(str, line + 4);
                number_set_str(E_W->n, str, 16);
                break;
            }
        }
        E_W->type = WEIERSTRASS;
        E_W->f = PROJECTIVE;
        strcpy(E_W->id_curve, id_curve);
        ec_compute_disc(E_W, STACK_1);
        if(i%2==0) 
        {
            weierstrass_to_jacobi_quartic(E_T, E_W, false, teta, STACK_1);
        }
        else 
        {
            /* We read the teta we will use for the mapping */
           while(fgets(line, sizeof(line), filep1) != NULL)
            {         
                if(strncmp(line, "teta = ", 7)== 0)
                {   
                    strcpy(str, line + 7);
                    number_set_str(tmp, str, 16);
                    field_elt_set_number(teta, false, k, STACK_1, 1, tmp);
                    break;
                }
            } 
            weierstrass_to_jacobi_quartic(E_T, E_W, true, teta, STACK_1);
        }
        /* Now we have to test the computed value */
        if(i%2==0)
        {
            /* We read teta */
            while((fgets(line, sizeof(line), filep1) != NULL))
            {                 
                if(strncmp(line, "teta = ", 7)== 0)
                {
                    strcpy(str, line + 7);
                    number_set_str(tmp, str, 16);
                    field_elt_set_number(c, false, k, STACK_1, 1, tmp);
                }
                if(field_elt_cmp(teta, c, k)==0)
                {
                    break;
                }
            }
        }
        /* In the last case there is nothing to do we are at the right place */
        /* We first read the values of a Jacobi Quartic curve for the reverse test */
        while(fgets(line, sizeof(line), filep1) != NULL)
        { 
            E_jq->k=k;
            if(strncmp(line, "a = ", 4)== 0)
            {
                strcpy(str, line + 4);
                number_set_str(tmp, str, 16);
                field_elt_set_number(E_jq->a, false, E_jq->k, STACK_1, 1, tmp);
            }
            if(strncmp(line, "b = ", 4)== 0)
            {
                strcpy(str, line + 4);
                number_set_str(tmp, str, 16);
                field_elt_set_number(E_jq->b, false, E_jq->k, STACK_1, 1, tmp);
            }
            if(strncmp(line, "xG = ", 5)== 0)
            {
                strcpy(str, line + 5);
                field_elt_set_str(c,str,16,false,k, STACK_1);
            }
            if(strncmp(line, "yG = ", 5)== 0)
            {
                strcpy(str, line + 5);
                field_elt_set_str(d,str,16,false,k, STACK_1);
            }
            if(strncmp(line, "zG = ", 5)== 0)
            {
                strcpy(str, line + 5);
                field_elt_set_str(e,str,16,false,k, STACK_1);
            }
            if(strncmp(line, "tG = ", 5)== 0)
            {
                strcpy(str1, line + 5);
                field_elt_set_str(f,str1,16,false,k, STACK_1);
                ec_point_set(E_jq->G, c,d,e,f, k);
        
            }
            if(strncmp(line, "h = ", 4) == 0)
            {
                strcpy(str, line + 4);
                number_set_str(E_jq->h, str, 16);
                break;
            }
            if(strncmp(line, "n = ", 4) == 0)
            {
                strcpy(str, line + 4);
                number_set_str(E_jq->n, str, 16);
                break;
            }
        }
        E_jq->type = JACOBI_QUARTIC;
        E_jq->f = PROJECTIVE;
        strcpy(E_jq->id_curve, id_curve);
        ec_compute_disc(E_jq, STACK_1); 
        /* Now we compare the values */
        if(field_elt_cmp(E_T->a, E_jq->a, k)==0 && field_elt_cmp(E_T->b, E_jq->b, k)==0 && jacobi_quartic_point_are_equal(E_T->G, E_jq->G, E_jq, STACK_1)) 
        {
            check[i][0]=true;
        }
        else
        {
            printf("\nProbleme dans le sens Weierstrass -> Jacobi_Quartic \n");
            printf("\nOn est dans la boucle i: %d \n",i);  
            printf("\n On est parti de la courbe:\n");
            ec_curve_print(E_W,16, STACK_1);
            printf("\n Pour la courbe espérée:\n");
            ec_curve_print(E_jq,16, STACK_1);            
            printf("\na calculé \n");
            field_elt_print(E_T->a,16,true, k, STACK_1);
            printf("\nb calculé \n");
            field_elt_print(E_T->b,16,true, k, STACK_1);
            printf("\na cherché \n");
            field_elt_print(E_jq->a,16,true, k, STACK_1);
            printf("\nb cherché \n");
            field_elt_print(E_jq->b,16,true, k, STACK_1);
            printf("\nteta calculé \n");
            field_elt_print(teta,16,true, k, STACK_1);
            printf("\npoint caclulé \n");
            ec_point_print(E_T->G,16,true, k, STACK_1);
            printf("\npoint espéré \n");
            ec_point_print(E_jq->G,16,true, k, STACK_1);             
            check[i][0]=false;
        }   
        /* We Now test the revers Edwards -> Weierstrass */
        
        jacobi_quartic_to_weierstrass(E_T2, E_jq, STACK_1);
        
        if(field_elt_cmp(E_T2->a, E_W->a, k)==0 && field_elt_cmp(E_T2->b, E_W->b, k)==0 && weierstrass_point_are_equal(E_T2->G, E_W->G, E_W, STACK_1)) 
        {
            check[i][1]=true;
        }
        else
        {
            printf("\nProbleme dans le sens Weierstrass <- Jacobi_Quartic \n");
            printf("\nOn est dans la boucle i: %d \n",i);
            printf("\na calculé \n");
            field_elt_print(E_T2->a,16,true, k, STACK_1);
            printf("\nb calculé \n");
            field_elt_print(E_T2->b,16,true, k, STACK_1);
            printf("\na cherché \n");
            field_elt_print(E_W->a,16,true, k, STACK_1);
            printf("\nb cherché \n");
            field_elt_print(E_W->b,16,true, k, STACK_1);
            printf("\npoint caclulé \n");
            ec_point_print(E_T2->G,16,true, k, STACK_1);
            printf("\npoint espéré \n");
            ec_point_print(E_W->G,16,true, k, STACK_1); 
            printf("\n On est parti de la courbe:\n");
            ec_curve_print(E_jq,16, STACK_1);
            printf("\n Pour la courbe espérée:\n");
            ec_curve_print(E_W,16, STACK_1);           
            check[i][1]=false;
        }
        
        /* Now we read in each file to be at the next curve for the next loop */ 
        while(fgets(line, sizeof(line), filep2) != NULL)
        { 
            if(strncmp(line, "New curve", 9)== 0)
            {
                break;
            }
        }
        
        while(fgets(line, sizeof(line), filep1) != NULL)
        { 
            if(strncmp(line, "New curve", 9)== 0)
            {
                break;
            }
        }
        
        ec_free(E_jq); 
        ec_free(E_W); 
        ec_free(E_T); 
        ec_free(E_T2); 
          
    }
    fclose(filep1);
    fclose(filep2);
    number_free(&tmp);
    free(str);
    free(str1);
    free(test_file1);
    free(test_file2);    
    field_elt_free(&teta, k);
    field_elt_free(&c, k); 
    field_elt_free(&e, k);
    field_elt_free(&d, k); 
    field_elt_free(&f, k); 
    for(i = 0; i < num_point; i++)
    {
        for(j = 0; j < 2; j++)
        {
            if(check[i][j] == false)
            {
                printf("ERROR i %d j %d\n",i,j);
                test=false;
            }
        }
    }
    return test;
}

bool
test_curve_jacobi_quartic(const char *id_curve, const uint16_t nbits, 
        const uint32_t num_point)
{
    bool res;    
    field k;
    FILE *filep1 = NULL;
    char line[160];
    char* test_file1 = (char*)malloc(sizeof(char) * 200);
    number tmp;
    number_init(&tmp, bits_to_nblock(nbits));
    field_alloc(k, FP, bits_to_nblock(nbits), NULL);
    
    /* We first read the field we work on */
    sprintf(test_file1, DATADIR "/vector_test_elliptic_curves/random_curve_jacobi_quartic_%s_conversion_test.txt", id_curve); 
    filep1 = fopen(test_file1,"r"); /* 1st one for Jacobi Quartic */
    if(filep1 == NULL)
    {
        printf(" \n Chemin:    %s \n", test_file1);
        fprintf(stderr, "Error : no data file found for curve conversion (Jacobi Quartic) \n");        
        exit(EXIT_FAILURE);
    }

    while(fgets(line, sizeof(line), filep1) != NULL)
    { 
        if(strncmp(line, "p = ", 4)== 0)
        {
            strcpy(test_file1, line + 4);
            number_set_str(tmp, test_file1, 16);
            field_create(k, "", STACK_1, 1, tmp);
            break;
        }
    }

    fclose(filep1);

    /* We free the variables not anymore necessary */
    free(test_file1);
    number_free(&tmp);
    res=test_curve_jacobi_quartic_conversion(k, id_curve, nbits, num_point);
    
    /* We now free the memory */
    field_free(k);
    return res;
}

int main()
{
    init_mphell(128, RANDOM_AES128, DEVURANDOM);
    int nb_points=100;
    if (!test_curve_jacobi_quartic("P256", 256, nb_points))
    {
        free_mphell();
        return -1;
    }
    free_mphell();
    return 0;
}
