/* 
  MPHELL-4.0 
  Author(s): The MPHELL team

 (C) Copyright 2015-2018 - Institut Fourier / Univ. Grenoble Alpes (France)

 This file is part of the MPHELL Library.
 MPHELL is free software: you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation, version 3 of the License.
 
 MPHELL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with MPHELL.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
  * \file  mphell_test_jacobi_quartic_point_random.c
  * \brief Self check tests for the function: void jacobi_quartic_point_random (ec_point P, ec_curve_srcptr E);
  */

#include "mphell_test_jacobi_quartic.h"

bool 
test_curve_jacobi_quartic_random(data_t *data)
{
    printf("Test random: ");
    int i;
    for (i=0; i<data->nb_points; i++)
    {
        jacobi_quartic_point_random (data->P1[i], data->E, STACK_1);
        if(!jacobi_quartic_belongs(data->P1[i], data->E, STACK_1))
        {
            printf("Error, P3[%d] does not belong to E\n", i);
            return false;
        }
    }
    int j;
    int nb_collision = 0;
    for (j=0; j<data->nb_points; j++)
    {
        for (i=0; i<data->nb_points; i++)
        {
            if (i != j)
            {
                if (ec_point_are_equal(data->P1[i], data->P1[j], data->E, STACK_1))
                {
                    nb_collision++;
                }
            }
        }
    }
    if(nb_collision > 10)
    {
        printf("Error, to many collision: %d\n", nb_collision);
        return false;
    }
    printf("number of collision: %d\n", nb_collision);
    
    return true;
}

bool
test_curve_jacobi_quartic(const char *id_curve, const uint16_t nbits, 
        const uint32_t num_point, data_t *data)
{
    bool res;
    test_curve_jacobi_quartic_alloc(nbits, num_point, data);
    test_curve_jacobi_quartic_read(id_curve, data);
    test_curve_jacobi_quartic_init(data);
    res=test_curve_jacobi_quartic_random(data);
    test_curve_jacobi_quartic_free(data);
    return res;
}

int main()
{
    data_t data;
    init_mphell(128, RANDOM_AES128, DEVURANDOM);
    int nb_points=100;
    if (!test_curve_jacobi_quartic("jacobi_quartic_p256", 256, nb_points, &data))
    {
        free_mphell();
        return -1;
    }
    else if (!test_curve_jacobi_quartic("jq256_3", 256, nb_points, &data))
    {
        free_mphell();
        return -1;
    }
    free_mphell();
    return 0;
}
