/* 
  MPHELL-4.0 
  Author(s): The MPHELL team

 (C) Copyright 2015-2018 - Institut Fourier / Univ. Grenoble Alpes (France)

 This file is part of the MPHELL Library.
 MPHELL is free software: you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation, version 3 of the License.
 
 MPHELL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with MPHELL.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
  * \file  mphell_test_number_arithmetic.c
  * \brief Self check tests for the arithmetic functions
  */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>
#include <string.h>

#include "mphell.h"

int main()
{
    init_mphell(128, RANDOM_AES128, DEVURANDOM);
    bool test = true;
    
    printf("Enter TEST:\n");
    
    /* Allocate number elements */
    number x, y, z, mod;
    
    number_init(&x, bits_to_nblock(256));
    number_init(&y, bits_to_nblock(256));
    number_init(&z, 3*bits_to_nblock(256));
    number_init(&mod, bits_to_nblock(256));
    
    number_set_str(x, "ff3456789abcdef123456789abcdef12", 16);
    number_set_str(y, "feabcd1232659874ab12cd56ef4598a5", 16);
    number_set_str(mod, "123456ff32659874ab12cd56ef4598a5", 16);
    
    char * str;
    
    /* Test number_add_ui */
    
    number_add_ui(z, x, 123456789);
    printf("z = "); number_print(z, 16); printf("\n"); 
    number_str(&str, z, 16);
    if(strcasecmp(str, "ff3456789abcdef123456789b329bc27")!=0)
    {
        test = false;
        printf("ERROR number_add_ui \n");
    }
    free(str);
    
    /* Test number_add */
    number_add(z, x, y);
    printf("z = "); number_print(z, 16); printf("\n"); 
    number_str(&str, z, 16);
    if(strcasecmp(str, "1fde0238acd227765ce5834e09b1387b7")!=0)
    {
        test = false;
        printf("ERROR number_add \n");
    }
    free(str);
    
    /* Test number_inc */
    
    number_inc(z, x);
    printf("z = "); number_print(z, 16); printf("\n"); 
    number_str(&str, z, 16);
    if(strcasecmp(str, "ff3456789abcdef123456789abcdef13")!=0)
    {
        test = false;
        printf("ERROR number_inc \n");
    }
    free(str);
    
    /* Test number_sub_ui */
    number_sub_ui(z, x, 123456);
    printf("z = "); number_print(z, 16); printf("\n"); 
    number_str(&str, z, 16);
    if(strcasecmp(str, "ff3456789abcdef123456789abcc0cd2")!=0)
    {
        test = false;
        printf("ERROR number_sub_ui \n");
    }
    free(str);
    
    /* Test number_sub */
    
    number_sub(z, x, y);
    printf("z = "); number_print(z, 16); printf("\n"); 
    number_str(&str, z, 16);
    if(strcasecmp(str, "8889666857467c78329a32bc88566d")!=0)
    {
        test = false;
        printf("ERROR number_sub \n");
    }
    free(str);
    
    /* Test number_dec */
    
    number_dec(z, x);
    printf("z = "); number_print(z, 16); printf("\n"); 
    number_str(&str, z, 16);
    if(strcasecmp(str, "ff3456789abcdef123456789abcdef11")!=0)
    {
        test = false;
        printf("ERROR number_dec \n");
    }
    free(str);
    
    /* Test number_mul_ui */
    
    number_mul_ui(z, x, 123456);
    printf("z = "); number_print(z, 16); printf("\n"); 
    number_str(&str, z, 16);
    if(strcasecmp(str, "1e0c057e4b17e4af9c0b17e4b17e4af9ba880")!=0)
    {
        test = false;
        printf("ERROR number_mul_ui \n");
    }
    free(str);
    
    /* Test number_addmul_ui */
    
    number_addmul_ui(z, x, 123456);
    printf("z = "); number_print(z, 16); printf("\n"); 
    number_str(&str, z, 16);
    if(strcasecmp(str, "3c180afc962fc95f38162fc962fc95f375100")!=0)
    {
        test = false;
        printf("ERROR number_addmul_ui \n");
    }
    free(str);
    
    /* Test number_mul */
    
    number_mul(z, x, y);
    printf("z = "); number_print(z, 16); printf("\n"); 
    number_str(&str, z, 16);
    if(strcasecmp(str, "fde132307940ea455947a10357632f75e847188ab70ed9e2d599c47d3f87c69a")!=0)
    {
        test = false;
        printf("ERROR number_mul \n");
    }
    free(str);
    
    /* Test number_mul_mod */
    
    number_mul_mod(z, x, y, mod, STACK_1);
    printf("z = "); number_print(z, 16); printf("\n"); 
    number_str(&str, z, 16);
    if(strcasecmp(str, "11e9d415cd77315ce53f61fef69ca16f")!=0)
    {
        test = false;
        printf("ERROR number_mul_mod \n");
    }
    free(str);
    
    /* Test number_addmul */
    
    number_addmul(z, x, y);
    printf("z = "); number_print(z, 16); printf("\n"); 
    number_str(&str, z, 16);
    if(strcasecmp(str, "fde132307940ea455947a10357632f75fa30eca084860b3fbad9267c36246809")!=0)
    {
        test = false;
        printf("ERROR number_addmul \n");
    }
    free(str);
    
    /* Test number_sqr */
    
    number_sqr(z, x);
    printf("z = "); number_print(z, 16); printf("\n"); 
    number_str(&str, z, 16);
    if(strcasecmp(str, "fe694ef7827861a74a9d65cc5996093feb0417480de0568ed7cb204ece169d44")!=0)
    {
        test = false;
        printf("ERROR number_sqr \n");
    }
    free(str);
    
    /* Test number_divmod_ui */
    
    block r;
    number_divmod_ui(z, &r, x, 123456);
    printf("z = "); number_print(z, 16); printf("\n"); 
    number_str(&str, z, 16);
    if(strcasecmp(str, "8779583cd6527a54364a447fc251")!=0)
    {
        test = false;
        printf("ERROR number_divmod_ui \n");
    }
    if(r != 55506)
    {
        test = false;
        printf("ERROR number_divmod_ui \n");
    }
    free(str);
    
    /* Test number_divmod */
    
    number_divmod(z, mod, x, y);
    printf("z = "); number_print(z, 16); printf("\n"); 
    printf("r = "); number_print(mod, 16); printf("\n"); 
    number_str(&str, z, 16);
    if(strcasecmp(str, "1")!=0)
    {
        test = false;
        printf("ERROR number_divmod \n");
    }
    free(str);
    number_str(&str, mod, 16);
    if(strcasecmp(str, "8889666857467c78329a32bc88566d")!=0)
    {
        test = false;
        printf("ERROR number_divmod \n");
    }
    free(str);
    
    /* Test number_div_ui */
    
    number_div_ui(z, x, 123456);
    printf("z = "); number_print(z, 16); printf("\n"); 
    number_str(&str, z, 16);
    if(strcasecmp(str, "8779583cd6527a54364a447fc251")!=0)
    {
        test = false;
        printf("ERROR number_div_ui \n");
    }
    free(str);
    
    /* Test number_div */
    
    number_div(z, x, y);
    printf("z = "); number_print(z, 16); printf("\n"); 
    number_str(&str, z, 16);
    if(strcasecmp(str, "1")!=0)
    {
        test = false;
        printf("ERROR number_div \n");
    }
    free(str);
    
    /* Test number_mod_ui */
    
    number_mod_ui(&r, x, 123456UL);
    if(r != 55506)
    {
        test = false;
        printf("ERROR number_mod_ui \n");
    }
    
    /* Test number_mod */
    
    number_mod(z, x, y);
    printf("z = "); number_print(z, 16); printf("\n"); 
    number_str(&str, z, 16);
    if(strcasecmp(str, "8889666857467c78329a32bc88566d")!=0)
    {
        test = false;
        printf("ERROR number_mod \n");
    }
    free(str);
    
    /* Test number_invmod */
    
    number_invmod(z, x, y);
    printf("z = "); number_print(z, 16); printf("\n"); 
    number_str(&str, z, 16);
    if(strcasecmp(str, "8889666857467c78329a32bc88566d")!=0)
    {
        test = false;
        printf("ERROR number_invmod \n");
    }
    free(str);
    
    /* Test number_gcd_ui */
    
    number_gcd_ui(&r, x, 123456);
    if(r != 6)
    {
        test = false;
        printf("ERROR number_gcd_ui \n");
    }
    
    /* Test number_gcd */
    
    number_gcd(z, x, y);
    printf("z = "); number_print(z, 16); printf("\n"); 
    number_str(&str, z, 16);
    if(strcasecmp(str, "3")!=0)
    {
        test = false;
        printf("ERROR number_gcd \n");
    }
    free(str);
    
    /* Test number_pow */
    
    number_set_ui(y, 3);
    number_pow(z, x, y, STACK_1);
    printf("z   = "); number_print(z, 16); printf("\n"); 
    number_str(&str, z, 16);
    if(strcasecmp(str, "fd9ee8fbd0e79ff8eb757e3af5e3f701956a6c78debbe0c7b876bcf4918e24cdf935dce8dbd6957364ea807df5dd8ac8")!=0)
    {
        test = false;
        printf("ERROR number_pow :%s \n",str);
    }
    free(str);
    
    /* Test number_pown */
    
    number_set_str(y, "feabcd1232659874ab12cd56ef4598a5", 16);
    number_powm(z, x, y, mod, STACK_1);
    printf("z = "); number_print(z, 16); printf("\n"); 
    number_str(&str, z, 16);
    if(strcasecmp(str, "4e58f30b453b4096815a8918896b2d")!=0)
    {
        test = false;
        printf("ERROR number_pown \n");
    }
    free(str);
    
    /* Test number_pown_ui */
    
    number_powm_ui(z, x, 12356, mod, STACK_1);
    printf("z = "); number_print(z, 16); printf("\n"); 
    number_str(&str, z, 16);
    if(strcasecmp(str, "31efd890e255f0d45df4e0b8303d48")!=0)
    {
        test = false;
        printf("ERROR number_pown_ui \n");
    }
    free(str);
    
        
    number_free(&x);
    number_free(&y);
    number_free(&z);
    number_free(&mod);
    
    free_mphell();

    /* Result */
    
    if (test)
    {
        printf("SUCCESS \n");
        return 0;
    }
    else
    {
        printf("ERROR \n");
        return -1;
    }
}
