/* 
  MPHELL-4.0 
  Author(s): The MPHELL team

 (C) Copyright 2015-2018 - Institut Fourier / Univ. Grenoble Alpes (France)

 This file is part of the MPHELL Library.
 MPHELL is free software: you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation, version 3 of the License.
 
 MPHELL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with MPHELL.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
  * \file  mphell_test_number_cmp.c
  * \brief Self check tests for the comparison functions
  */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>
#include <string.h>

#include "mphell.h"

int main()
{
    init_mphell(128, RANDOM_AES128, DEVURANDOM);
    bool test = true;
    
    printf("Enter TEST:\n");
    
    /* Allocate number elements */
    number x;
    number y;
    number z;
    number u;
    number v;
    
    number_init(&x, bits_to_nblock(256));
    number_init(&y, bits_to_nblock(256));
    number_init(&z, bits_to_nblock(256));
    number_init(&u, bits_to_nblock(256));
    number_init(&v, bits_to_nblock(256));
    
    number_set_str(x, "123456789123456789123456789", 10);
    number_set_str(y, "123456789123456789123456788", 10);
    number_copy(z, x);
    number_set_ui(u, 123456);
    number_set_ui(v, 0);
    
    /* Test number_iszero */

    if(number_iszero(x))
    {
        test = false;
        printf("ERROR number_iszero \n");
    }
    if(!number_iszero(v))
    {
        test = false;
        printf("ERROR number_iszero \n");
    }
    
    if(test)
    {
        printf("TEST number_iszero PASS \n");
    }
    
    /* Test number_isequal_ui */

    if(number_isequal_ui(x, 123456))
    {
        test = false;
        printf("ERROR number_isequal_ui \n");
    }
    if(!number_isequal_ui(u, 123456))
    {
        test = false;
        printf("ERROR number_isequal_ui \n");
    }
    
    if(test)
    {
        printf("TEST number_isequal_ui PASS \n");
    }
    
    /* Test number_isequal */

    if(number_isequal(x, y))
    {
        test = false;
        printf("ERROR number_isequal \n");
    }
    if(!number_isequal(x, z))
    {
        test = false;
        printf("ERROR number_isequal \n");
    }
    
    if(test)
    {
        printf("TEST number_isequal PASS \n");
    }
    
    /* Test number_isdiff_ui */

    if(!number_isdiff_ui(x, 123456))
    {
        test = false;
        printf("ERROR number_isdiff_ui \n");
    }
    if(number_isdiff_ui(u, 123456))
    {
        test = false;
        printf("ERROR number_isdiff_ui \n");
    }
    
    if(test)
    {
        printf("TEST number_isdiff_ui PASS \n");
    }
    
    /* Test number_isdiff */

    if(!number_isdiff(x, y))
    {
        test = false;
        printf("ERROR number_isdiff \n");
    }
    if(number_isdiff(x, z))
    {
        test = false;
        printf("ERROR number_isdiff \n");
    }
    
    if(test)
    {
        printf("TEST number_isdiff PASS \n");
    }
    
    /* Test number_islower_ui */

    if(!number_islower_ui(u, 123457))
    {
        test = false;
        printf("ERROR number_islower_ui \n");
    }
    if(number_islower_ui(u, 123455))
    {
        test = false;
        printf("ERROR number_islower_ui \n");
    }
    
    if(test)
    {
        printf("TEST number_islower_ui PASS \n");
    }
    
    /* Test number_islower */

    if(!number_islower(y, x))
    {
        test = false;
        printf("ERROR number_islower \n");
    }
    if(number_islower(x, y))
    {
        test = false;
        printf("ERROR number_islower \n");
    }
    
    if(test)
    {
        printf("TEST number_islower PASS \n");
    }
    
    /* Test number_isgreater_ui */

    if(!number_isgreater_ui(u, 123455))
    {
        test = false;
        printf("ERROR number_isgreater_ui \n");
    }
    if(number_isgreater_ui(u, 123457))
    {
        test = false;
        printf("ERROR number_isgreater_ui \n");
    }
    
    if(test)
    {
        printf("TEST number_isgreater_ui PASS \n");
    }
    
    /* Test number_isgreater */

    if(!number_isgreater(x, y))
    {
        test = false;
        printf("ERROR number_isgreater \n");
    }
    if(number_isgreater(y, x))
    {
        test = false;
        printf("ERROR number_isgreater \n");
    }
    
    if(test)
    {
        printf("TEST number_isgreater PASS \n");
    }
    
    /* Test number_islowereq_ui */

    if(!number_islowereq_ui(u, 123457))
    {
        test = false;
        printf("ERROR number_islowereq_ui \n");
    }
    if(number_islowereq_ui(u, 123455))
    {
        test = false;
        printf("ERROR number_islowereq_ui \n");
    }
    
    if(test)
    {
        printf("TEST number_islowereq_ui PASS \n");
    }
    
    /* Test number_islowereq */

    if(!number_islowereq(y, x))
    {
        test = false;
        printf("ERROR number_islowereq \n");
    }
    if(number_islowereq(x, y))
    {
        test = false;
        printf("ERROR number_islowereq \n");
    }
    
    if(test)
    {
        printf("TEST number_islowereq PASS \n");
    }
    
    /* Test number_isgreatereq_ui */

    if(!number_isgreatereq_ui(u, 123455))
    {
        test = false;
        printf("ERROR number_isgreatereq_ui \n");
    }
    if(number_isgreatereq_ui(u, 123457))
    {
        test = false;
        printf("ERROR number_isgreatereq_ui \n");
    }
    
    if(test)
    {
        printf("TEST number_isgreatereq_ui PASS \n");
    }
    
    /* Test number_isgreatereq */

    if(!number_isgreatereq(x, y))
    {
        test = false;
        printf("ERROR number_isgreatereq \n");
    }
    if(number_isgreatereq(y, x))
    {
        test = false;
        printf("ERROR number_isgreatereq \n");
    }
    
    if(test)
    {
        printf("TEST number_isgreatereq PASS \n");
    }
    
    /* Test number_cmp_ui */

    if(number_cmp_ui(u, 1234567)!=-1)
    {
        test = false;
        printf("ERROR number_cmp_ui \n");
    }
    if(number_cmp_ui(u, 12345)!=1)
    {
        test = false;
        printf("ERROR number_cmp_ui \n");
    }
    if(number_cmp_ui(u, 123456)!=0)
    {
        test = false;
        printf("ERROR number_cmp_ui \n");
    }
    
    if(test)
    {
        printf("TEST number_cmp_ui PASS \n");
    }
    
    /* Test number_cmp */

    if(number_cmp(y, x)!=-1)
    {
        test = false;
        printf("ERROR number_cmp \n");
    }
    if(number_cmp(x, y)!=1)
    {
        test = false;
        printf("ERROR number_cmp \n");
    }
    if(number_cmp(x, z)!=0)
    {
        test = false;
        printf("ERROR number_cmp_ui \n");
    }
    
    if(test)
    {
        printf("TEST number_cmp PASS \n");
    }
    
    /* Test number_iseven */

    if(number_iseven(x))
    {
        test = false;
        printf("ERROR number_iseven \n");
    }
    if(!number_iseven(y))
    {
        test = false;
        printf("ERROR number_iseven \n");
    }
    
    if(test)
    {
        printf("TEST number_iseven PASS \n");
    }
    
    /* Test number_isodd */

    if(!number_isodd(x))
    {
        test = false;
        printf("ERROR number_isodd \n");
    }
    if(number_isodd(y))
    {
        test = false;
        printf("ERROR number_isodd \n");
    }
    
    if(test)
    {
        printf("TEST number_isodd PASS \n");
    }
    
    
    number_free(&x);
    number_free(&y);
    number_free(&z);
    number_free(&u);
    number_free(&v);
    
    free_mphell();

    /* Result */
    
    if (test)
    {
        printf("SUCCESS \n");
        return 0;
    }
    else
    {
        printf("ERROR \n");
        return -1;
    }
}
