/* 
  MPHELL-4.0 
  Author(s): The MPHELL team

 (C) Copyright 2015-2018 - Institut Fourier / Univ. Grenoble Alpes (France)

 This file is part of the MPHELL Library.
 MPHELL is free software: you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation, version 3 of the License.
 
 MPHELL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with MPHELL.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
  * \file  mphell_test_number_io.c
  * \brief Self check tests for the input, output functions
  */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>
#include <string.h>

#include "mphell.h"

int main()
{
    init_mphell(128, RANDOM_AES128, DEVURANDOM);
    bool test = true;
    
    printf("Enter TEST:\n");
    
    /* Allocate number elements */
    number x;
    number y;
    block b;
    
    number_init(&x, bits_to_nblock(256));
    number_init(&y, bits_to_nblock(256));
    
    char * str;
    
    /* Test number_set_ui */
    number_set_ui(x, 8);
    number_str(&str, x, 2);
    if(strcasecmp(str, "1000")!=0)
    {
        test = false;
        printf("1: ERROR number_set_ui, str = %s \n", str);
    }
    free(str);
    number_str(&str, x, 16);
    if(strcasecmp(str, "8")!=0)
    {
        test = false;
        printf("2: ERROR number_set_ui, str = %s \n", str);
    }
    free(str);
    
    number_str(&str, x, 10);
    if(strcasecmp(str, "8")!=0)
    {
        test = false;
        printf("3: ERROR number_set_ui, str = %s \n", str);
    }
    free(str);

    number_set_ui(x, 0);
    number_str(&str, x, 2);
    if(strcasecmp(str, "0")!=0)
    {
        test = false;
        printf("4: ERROR number_set_ui, str = %s \n", str);
    }
    free(str);

    number_set_str(y,"0",2);
    if((number_cmp(x,y)!=0))
    {
        test = false;
        printf("ERROR number_set_str zero \n");
    }
    number_set_str(y,"0",10);
    if((number_cmp(x,y)!=0))
    {
        test = false;
        printf("ERROR number_set_str zero \n");
    }
    number_set_str(y,"0",16);
    if((number_cmp(x,y)!=0))
    {
        test = false;
        printf("ERROR number_set_str zero \n");
    }

    number_set_ui(x, 14);
    number_str(&str, x, 2);
    if(strcasecmp(str, "1110")!=0)
    {
        test = false;
        printf("5: ERROR number_set_ui, str = %s \n", str);
    }
    free(str);
    
    number_str(&str, x, 16);
    if(strcasecmp(str, "e")!=0)
    {
        test = false;
        printf("6: ERROR number_set_ui, str = %s \n", str);
    }
    free(str);
    
    number_str(&str, x, 10);
    if(strcasecmp(str, "14")!=0)
    {
        test = false;
        printf("7: ERROR number_set_ui, str = %s \n", str);
    }
    free(str);
    
    number_set_ui(x, 30200UL);
    number_str(&str, x, 2);
    if(strcasecmp(str, "111010111111000")!=0)
    {
        test = false;
        printf("8: ERROR number_set_ui, str = %s \n", str);
    }
    free(str);
    
    number_str(&str, x, 16);
    if(strcasecmp(str, "75f8")!=0)
    {
        test = false;
        printf("9: ERROR number_set_ui, str = %s \n", str);
    }
    free(str);
    
    number_str(&str, x, 10);
    if(strcasecmp(str, "30200")!=0)
    {
        test = false;
        printf("10: ERROR number_set_ui, str = %s \n", str);
    }
    free(str);
    
    number_set_ui(x, 4294967295UL);
    number_str(&str, x, 2);
    if(strcasecmp(str, "11111111111111111111111111111111")!=0)
    {
        test = false;
        printf("11: ERROR number_set_ui, str = %s \n", str);
    }
    free(str);
    
    number_str(&str, x, 16);
    if(strcasecmp(str, "ffffffff")!=0)
    {
        test = false;
        printf("12: ERROR number_set_ui, str = %s \n", str);
    }
    free(str);
    
    number_str(&str, x, 10);
    if(strcasecmp(str, "4294967295")!=0)
    {
        test = false;
        printf("13: ERROR number_set_ui, str = %s \n", str);
    }
    free(str);
    
    if(test)
    {
        printf("TEST number_set_ui PASS \n");
    }
    
    /* Test number_set */
    number_set(x, 1, 4, 1UL, 2UL, 3UL, 5UL);
    number_str(&str, x, 2);
#if BLOCK_SIZE == 32
    if(strcasecmp(str, "101000000000000000000000000000000110000000000000000000000000000001000000000000000000000000000000001")!=0)
#elif BLOCK_SIZE == 64
    if(strcasecmp(str, "101000000000000000000000000000000000000000000000000000000000000001100000000000000000000000000000000000000000000000000000000000000100000000000000000000000000000000000000000000000000000000000000001")!=0)
#endif
    {
        test = false;
        printf("1: ERROR number_set, str = %s \n", str);
    }
    free(str);
    
    number_str(&str, x, 16);
#if BLOCK_SIZE == 32
    if(strcasecmp(str, "5000000030000000200000001")!=0)
#elif BLOCK_SIZE == 64
    if(strcasecmp(str, "5000000000000000300000000000000020000000000000001")!=0)
#endif
    {
        test = false;
        printf("2: ERROR number_set, str = %s \n", str);
    }
    free(str);
    
    number_str(&str, x, 10);
#if BLOCK_SIZE == 32
    if(strcasecmp(str, "396140812626661920197438341121")!=0)
#elif BLOCK_SIZE == 64
    if(strcasecmp(str, "31385508676933403820199794216801147470938794532762896302081")!=0)
#endif
    {
        test = false;
        printf("3: ERROR number_set, str = %s \n", str);
    }
    free(str);
    
    number_set(x, -1, 4, 1UL, 2UL, 3UL, 5UL);
    number_str(&str, x, 2);
#if BLOCK_SIZE == 32
    if(strcasecmp(str, "-101000000000000000000000000000000110000000000000000000000000000001000000000000000000000000000000001")!=0)
#elif BLOCK_SIZE == 64
    if(strcasecmp(str, "-101000000000000000000000000000000000000000000000000000000000000001100000000000000000000000000000000000000000000000000000000000000100000000000000000000000000000000000000000000000000000000000000001")!=0)
#endif
    {
        test = false;
        printf("4: ERROR number_set, str = %s \n", str);
    }
    free(str);

    number_str(&str, x, 16);
#if BLOCK_SIZE == 32
    if(strcasecmp(str, "-5000000030000000200000001")!=0)
#elif BLOCK_SIZE == 64
    if(strcasecmp(str, "-5000000000000000300000000000000020000000000000001")!=0)
#endif
    {
        test = false;
        printf("5: ERROR number_set, str = %s \n", str);
    }
    free(str);
    
    number_str(&str, x, 10);
#if BLOCK_SIZE == 32
    if(strcasecmp(str, "-396140812626661920197438341121")!=0)
#elif BLOCK_SIZE == 64
    if(strcasecmp(str, "-31385508676933403820199794216801147470938794532762896302081")!=0)
#endif
    {
        test = false;
        printf("6: ERROR number_set, str = %s \n", str);
    }
    free(str);
    
    number_set(x, 1, 4, 30200UL, 2UL, 4294967295UL, 5UL);
    number_str(&str, x, 2);
#if BLOCK_SIZE == 32
    if(strcasecmp(str, "101111111111111111111111111111111110000000000000000000000000000001000000000000000000111010111111000")!=0)
#elif BLOCK_SIZE == 64
    if(strcasecmp(str, "101000000000000000000000000000000001111111111111111111111111111111100000000000000000000000000000000000000000000000000000000000000100000000000000000000000000000000000000000000000000111010111111000")!=0)
#endif
    {
        test = false;
        printf("7: ERROR number_set, str = %s \n", str);
    }
    free(str);
    
    number_str(&str, x, 16);
#if BLOCK_SIZE == 32
    if(strcasecmp(str, "5ffffffff00000002000075f8")!=0)
#elif BLOCK_SIZE == 64
    if(strcasecmp(str, "500000000ffffffff000000000000000200000000000075f8")!=0)
#endif
    {
        test = false;
        printf("8: ERROR number_set, str = %s \n", str);
    }
    free(str);
    
    number_str(&str, x, 10);
#if BLOCK_SIZE == 32
    if(strcasecmp(str, "475368975067139281496144115192")!=0)
#elif BLOCK_SIZE == 64
    if(strcasecmp(str, "31385508678394905456169567667321078449801579122691756029432")!=0)
#endif
    {
        test = false;
        printf("9: ERROR number_set, str = %s \n", str);
    }
    free(str);
    
    if(test)
    {
        printf("TEST number_set PASS \n");
    }
    
    /* Test number_get_ui */
    number_get_ui (&b, x);
    if(b != 30200)
    {
        test = false;
        printf("ERROR number_get_ui\n");
    }
    
    if(test)
    {
        printf("TEST number_get_ui PASS \n");
    }

    /* Test number_random */
    
    number_random(y, bits_to_nblock(256));
    number_random(x, bits_to_nblock(256));
    
    if(number_cmp(x,y)==0)
    {
        test = false;
        printf("ERROR number_random \n");
    }
    if(test)
    {
        printf("TEST number_random PASS \n");
    }
    
    /* Test number_random1 */
    
    number_random(y, bits_to_nblock(192));
    number_random1(x, y, STACK_1);
    if(number_isgreater(x, y))
    {
        test = false;
        printf("ERROR number_random1 \n");
    }
    if(test)
    {
        printf("TEST number_random1 PASS \n");
    }
    
    /* Test number_log2 */
    
    number_free(&y);
    number_init(&y, 8); 
    
    number_set(x, 1, 4, 30200UL, 2UL, 4294967295UL, 5UL);
    number_set(y, 1, 8, 30200UL, 2UL, 4294967295UL, 5UL, 12121UL, 8888UL, 789UL, 9565656UL);
    
    int nbits = number_log2(y);
#if BLOCK_SIZE == 32
    if(nbits != 248)
#elif BLOCK_SIZE == 64
    if(nbits != 472)
#endif
    {
        test = false;
        printf("ERROR number_log2, nbits = %d \n", nbits);
    }
    
    nbits = number_log2(x);
#if BLOCK_SIZE == 32
    if(nbits != 99)
#elif BLOCK_SIZE == 64
    if(nbits != 195)
#endif
    {
        test = false;
        printf("ERROR number_log2, nbits = %d \n", nbits);
    }
    
    if(test)
    {
        printf("TEST number_log2 PASS \n");
    }
    
    /* Test number_set_str */
    
    number_set_str(x, "1000000011110000111100000000", 2);
    number_str(&str, x, 2);
    if(strcasecmp(str, "1000000011110000111100000000")!=0)
    {
        test = false;
        printf("ERROR number_set_str \n");
    }
    free(str);
    
    number_set_str(x, "abcdef123456789abcdef123456789", 16);
    number_str(&str, x, 16);
    if(strcasecmp(str, "abcdef123456789abcdef123456789")!=0)
    {
        test = false;
        printf("ERROR number_set_str \n");
    }
    free(str);
    
    number_set_str(x, "33", 10);
    number_str(&str, x, 16);
    if(strcasecmp(str, "21")!=0)
    {
        test = false;
        printf("ERROR number_set_str \n");
    }
    free(str);
    
    number_str(&str, x, 2);
    if(strcasecmp(str, "100001")!=0)
    {
        test = false;
        printf("ERROR number_set_str \n");
    }
    free(str);
    
    number_str(&str, x, 10);
    if(strcasecmp(str, "33")!=0)
    {
        test = false;
        printf("ERROR number_set_str \n");
    }
    free(str);
    
    number_set_str(x, "123456789123456789123456789", 10);
    number_str(&str, x, 16);
    if(strcasecmp(str, "661efdf2e3b19f7c045f15")!=0)
    {
        test = false;
        printf("ERROR number_set_str \n");
        printf("str = %s\n", str);
    }
    free(str);
    
    number_str(&str, x, 2);
    if(strcasecmp(str, "110011000011110111111011111001011100011101100011001111101111100000001000101111100010101")!=0)
    {
        test = false;
        printf("ERROR number_set_str \n");
        printf("str = %s\n", str);
    }
    free(str);
    
    number_str(&str, x, 10);
    if(strcasecmp(str, "123456789123456789123456789")!=0)
    {
        test = false;
        printf("ERROR number_set_str \n");
        printf("str = %s\n", str);
    }
    free(str);
    
    number_set_str(x, "be6dc5f3bca8726edc8d23dfc89f5922e12826336befbd97ef9ad755aed3ed6f", 16);
    number_str(&str, x, 2);
    if(strcasecmp(str, "1011111001101101110001011111001110111100101010000111001001101110110111001000110100100011110111111100100010011111010110010010001011100001001010000010011000110011011010111110111110111101100101111110111110011010110101110101010110101110110100111110110101101111")!=0)
    {
        test = false;
        printf("ERROR number_set_str \n");
        printf("str = %s\n", str);
    }
    free(str);
    
    number_str(&str, x, 10);
    if(strcasecmp(str, "86133393776046821453450145628028159937852042268439921914884420878249602837871")!=0)
    {
        test = false;
        printf("ERROR number_set_str \n");
        printf("str = %s\n", str);
    }
    free(str);
    
    number_str(&str, x, 16);
    if(strcasecmp(str, "be6dc5f3bca8726edc8d23dfc89f5922e12826336befbd97ef9ad755aed3ed6f")!=0)
    {
        test = false;
        printf("ERROR number_set_str \n");
        printf("str = %s\n", str);
    }
    free(str);
    
    if(test)
    {
        printf("TEST number_set_str PASS \n");
    }
    
    number_free(&x);
    number_free(&y);
    
    free_mphell();

    /* Result */
    
    if (test)
    {
        printf("SUCCESS \n");
        return 0;
    }
    else
    {
        printf("ERROR \n");
        return -1;
    }
}
