/* 
  MPHELL-4.0 
  Author(s): The MPHELL team

 (C) Copyright 2015-2018 - Institut Fourier / Univ. Grenoble Alpes (France)

 This file is part of the MPHELL Library.
 MPHELL is free software: you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation, version 3 of the License.
 
 MPHELL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with MPHELL.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
  * \file  mphell_test_number_logic.c
  * \brief Self check tests for the logical functions
  */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>
#include <string.h>

#include "mphell.h"

int main()
{
    init_mphell(128, RANDOM_AES128, DEVURANDOM);
    bool test = true;
    
    printf("Enter TEST:\n");
    
    /* Allocate number elements */
    number x, y, z;
    
    number_init(&x, bits_to_nblock(256));
    number_init(&y, bits_to_nblock(256));
    number_init(&z, bits_to_nblock(256));
    
    number_set_str(x, "123456789abcdef123456789abcdef12", 16);
    number_set_str(y, "feabcd1232659874ab12cd56ef4598a5", 16);
    
    char * str;
    
    block zu;
    
    /* Test number_and_ui */
    
    zu = number_and_ui(x, 123456789, STACK_1);
    if(zu != 55168272)
    {
        test = false;
        printf("ERROR number_and_ui \n");
    }
    
    /* Test number_and */
    
    number_and(z, x, y);
    number_str(&str, z, 16);
    if(strcasecmp(str, "122044101224987023004500ab458800")!=0)
    {
        test = false;
        printf("ERROR number_and \n");
    }
    free(str);
    
    /* Test number_or_ui */
    
    number_or_ui(z, x, 123456789, STACK_1);
    number_str(&str, z, 16);
    if(strcasecmp(str, "123456789abcdef123456789afdfef17")!=0)
    {
        test = false;
        printf("ERROR number_or_ui \n");
    }
    free(str);
    
    /* Test number_or */
    
    number_or(z, x, y);
    number_str(&str, z, 16);
    if(strcasecmp(str, "febfdf7abafddef5ab57efdfefcdffb7")!=0)
    {
        test = false;
        printf("ERROR number_or \n");
    }
    free(str);
    
    /* Test number_xor_ui */
    
    number_xor_ui(z, x, 123456789, STACK_1);
    number_str(&str, z, 16);
    if(strcasecmp(str, "123456789abcdef123456789ac962207")!=0)
    {
        test = false;
        printf("ERROR number_xor_ui \n");
    }
    free(str);
    
    /* Test number_xor */
    
    number_xor(z, x, y);
    number_str(&str, z, 16);
    if(strcasecmp(str, "ec9f9b6aa8d946858857aadf448877b7")!=0)
    {
        test = false;
        printf("ERROR number_xor \n");
    }
    free(str);
    
    /* Test number_lshift */
    
    number_lshift(z, x, 1);
    number_str(&str, z, 16);
    if(strcasecmp(str, "2468acf13579bde2468acf13579bde24")!=0)
    {
        test = false;
        printf("ERROR number_lshift \n");
    }
    free(str);
    
    number_lshift(z, x, 65);
    number_str(&str, z, 16);
    if(strcasecmp(str, "2468acf13579bde2468acf13579bde240000000000000000")!=0)
    {
        test = false;
        printf("ERROR number_lshift \n");
    }
    free(str);
    
    number_set_str(y, "c97f022ffc89a67cd8c05294d4340b2a764cd8f9debba97863920e0f7f69f50", 16);
    number_lshift(z, y, 2);
    number_str(&str, z, 10);
    if(strcasecmp(str, "22784821809917227344264557541978921797239768048024325217199343671480027872576")!=0)
    {
        test = false;
        printf("ERROR number_lshift \n");
    }
    free(str);
    
    /* Test number_rshift */
    
    number_rshift(z, x, 1);
    number_str(&str, z, 16);
    if(strcasecmp(str, "91a2b3c4d5e6f7891a2b3c4d5e6f789")!=0)
    {
        test = false;
        printf("ERROR number_rshift \n");
    }
    free(str);
    
    number_rshift(z, x, 64);
    number_str(&str, z, 16);
    if(strcasecmp(str, "123456789abcdef1")!=0)
    {
        test = false;
        printf("ERROR number_rshift \n");
    }
    free(str);
    
    number_rshift(z, x, 65);
    number_str(&str, z, 16);
    if(strcasecmp(str, "91a2b3c4d5e6f78")!=0)
    {
        test = false;
        printf("ERROR number_rshift \n");
    }
    free(str);
        
    number_free(&x);
    number_free(&y);
    number_free(&z);
    
    free_mphell();

    /* Result */
    
    if (test)
    {
        printf("SUCCESS \n");
        return 0;
    }
    else
    {
        printf("ERROR \n");
        return -1;
    }
}
