/* 
  MPHELL-4.0 
  Author(s): The MPHELL team

 (C) Copyright 2015-2018 - Institut Fourier / Univ. Grenoble Alpes (France)

 This file is part of the MPHELL Library.
 MPHELL is free software: you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation, version 3 of the License.
 
 MPHELL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with MPHELL.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
  * \file  mphell_test_random_files_generation.c
  * \brief Generate files fill with random bits to be tested by the NIST Statistical Test Suite for Random and Pseudorandom Number Generators for Cryptographic Applications (Special Publication 800-22)
  */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>
#include <string.h>

#include "mphell.h"

int NUMBER_SIZE = 256; /* Size in bits for the random numbers */

void print_with_zero(char * nb, FILE * f)
{
    int i;
    int size = NUMBER_SIZE - strlen(nb);
    char zero[size+1];
    for(i=0; i<size; i++)
    {
        zero[i]='0';
    }
    zero[size]='\0';
    fprintf(f, "%s", zero);
    fprintf(f, "%s", nb);
    fprintf(f, "\n");
}

int main()
{
    /* Number of random number generated */
    int nb_rand = 100000*4;
    FILE *fp = NULL;
    char * str;
    int i;
    number rand;
    
    /* ---------- DEVURANDOM ---------- */
    
    printf("Entropy source = DEVURANDOM \n");
    
    /* SHA1 */
    
    printf("Hash DRBG SHA1 \n");
    
    init_mphell(128, RANDOM_SHA1, DEVURANDOM);
    
    fp = fopen(DATADIR "/random/DEVURANDOM_SHA1.txt", "w");
    if(fp==0)
    {
        printf("path = %s \n", DATADIR "/random/DEVURANDOM_SHA1.txt");
        printf("ERROR\n");
        return -1;
    }
    number_init(&rand, bits_to_nblock(256)); /* 4 x 64 = 256 bits per number */
    
    for(i=0; i<nb_rand; i++)
    {
        number_random(rand, bits_to_nblock(256));
        number_str(&str, rand, 2);
        print_with_zero(str, fp);
        free(str);
    }
    
    number_free(&rand);
    fclose(fp);
    free_mphell();
    
    /* AES128 */
    
    printf("CTR DRBG AES128 \n");
    
    init_mphell(128, RANDOM_AES128, DEVURANDOM);
    
    fp = fopen(DATADIR "/random/DEVURANDOM_AES128.txt", "w");
    number_init(&rand, bits_to_nblock(256));
    
    for(i=0; i<nb_rand; i++)
    {
        number_random(rand, bits_to_nblock(256));
        number_str(&str, rand, 2);
        print_with_zero(str, fp);
        free(str);
    }
    
    number_free(&rand);
    fclose(fp);
    free_mphell();
    
    /* AES192 */
    
    printf("CTR DRBG AES192 \n");
    
    init_mphell(192, RANDOM_AES192, DEVURANDOM);
    
    fp = fopen(DATADIR "/random/DEVURANDOM_AES192.txt", "w");
    number_init(&rand, bits_to_nblock(256));
    
    for(i=0; i<nb_rand; i++)
    {
        number_random(rand, bits_to_nblock(256));
        number_str(&str, rand, 2);
        print_with_zero(str, fp);
        free(str);
    }
    
    number_free(&rand);
    fclose(fp);
    free_mphell();
    
     /* AES256 */
     
     printf("CTR DRBG AES256 \n");
    
    init_mphell(256, RANDOM_AES256, DEVURANDOM);
    
    fp = fopen(DATADIR "/random/DEVURANDOM_AES256.txt", "w");
    number_init(&rand, bits_to_nblock(256));
    
    for(i=0; i<nb_rand; i++)
    {
        number_random(rand, bits_to_nblock(256));
        number_str(&str, rand, 2);
        print_with_zero(str, fp);
        free(str);
    }
    
    number_free(&rand);
    fclose(fp);
    free_mphell();
    
    
    /* ---------- DEVRANDOM ---------- */
    
    printf("Entropy source = DEVRANDOM \n");
    
    /* SHA1 */
    
    printf("Hash DRBG SHA1 \n");
    
    init_mphell(128, RANDOM_SHA1, DEVRANDOM);
    
    fp = fopen(DATADIR "/random/DEVRANDOM_SHA1.txt", "w");
    number_init(&rand, bits_to_nblock(256));
    
    for(i=0; i<nb_rand; i++)
    {
        number_random(rand, bits_to_nblock(256));
        number_str(&str, rand, 2);
        print_with_zero(str, fp);
        free(str);
    }
    
    number_free(&rand);
    fclose(fp);
    free_mphell();
    
    /* AES128 */
    
    printf("CTR DRBG AES128 \n");
    
    init_mphell(128, RANDOM_AES128, DEVRANDOM);
    
    fp = fopen(DATADIR "/random/DEVRANDOM_AES128.txt", "w");
    number_init(&rand, bits_to_nblock(256));
    
    for(i=0; i<nb_rand; i++)
    {
        number_random(rand, bits_to_nblock(256));
        number_str(&str, rand, 2);
        print_with_zero(str, fp);
        free(str);
    }
    
    number_free(&rand);
    fclose(fp);
    free_mphell();
    
    /* AES192 */
    
    printf("CTR DRBG AES192 \n");
    
    init_mphell(192, RANDOM_AES192, DEVRANDOM);
    
    fp = fopen(DATADIR "/random/DEVRANDOM_AES192.txt", "w");
    number_init(&rand, bits_to_nblock(256));
    
    for(i=0; i<nb_rand; i++)
    {
        number_random(rand, bits_to_nblock(256));
        number_str(&str, rand, 2);
        print_with_zero(str, fp);
        free(str);
    }
    
    number_free(&rand);
    fclose(fp);
    free_mphell();
    
     /* AES256 */
     
    printf("CTR DRBG AES256 \n");
    
    init_mphell(256, RANDOM_AES256, DEVRANDOM);
    
    fp = fopen(DATADIR "/random/DEVRANDOM_AES256.txt", "w");
    number_init(&rand, bits_to_nblock(256));
    
    for(i=0; i<nb_rand; i++)
    {
        number_random(rand, bits_to_nblock(256));
        number_str(&str, rand, 2);
        print_with_zero(str, fp);
        free(str);
    }
    
    number_free(&rand);
    fclose(fp);
    free_mphell();

#if MPHELL_USE_RDRAND == 1

    /* RANDOM_INTEL */
    
    printf("Entropy source = ENTROPY_UNSET, with RDRAND\n");
    
    init_mphell(256, RANDOM_INTEL, ENTROPY_UNSET);
    
    fp = fopen(DATADIR "/random/RANDOM_INTEL.txt", "w");
    number_init(&rand, bits_to_nblock(256));
    
    for(i=0; i<nb_rand; i++)
    {
        number_random(rand, bits_to_nblock(256));
        number_str(&str, rand, 2);
        print_with_zero(str, fp);
        free(str);
    }
    
    number_free(&rand);
    fclose(fp);
    free_mphell();

#endif /* MPHELL_USE_RDRAND */


#if MPHELL_USE_RDSEED == 1

    /* ---------- RDSEED ---------- */
    
    printf("Entropy source = RDSEED \n");
    
    /* SHA1 */
    
    printf("Hash DRBG SHA1 \n");
    
    init_mphell(128, RANDOM_SHA1, INTEL);
    
    fp = fopen(DATADIR "/random/INTEL_SHA1.txt", "w");
    number_init(&rand, bits_to_nblock(256));
    
    for(i=0; i<nb_rand; i++)
    {
        number_random(rand, bits_to_nblock(256));
        number_str(&str, rand, 2);
        print_with_zero(str, fp);
        free(str);
    }
    
    number_free(&rand);
    fclose(fp);
    free_mphell();
    
    /* AES128 */
    
    printf("CTR DRBG AES128 \n");
    
    init_mphell(128, RANDOM_AES128, INTEL);
    
    fp = fopen(DATADIR "/random/INTEL_AES128.txt", "w");
    number_init(&rand, bits_to_nblock(256));
    
    for(i=0; i<nb_rand; i++)
    {
        number_random(rand, bits_to_nblock(256));
        number_str(&str, rand, 2);
        print_with_zero(str, fp);
        free(str);
    }
    
    number_free(&rand);
    fclose(fp);
    free_mphell();
    
    /* AES192 */
    
    printf("CTR DRBG AES192 \n");
    
    init_mphell(192, RANDOM_AES192, INTEL);
    
    fp = fopen(DATADIR "/random/INTEL_AES192.txt", "w");
    number_init(&rand, bits_to_nblock(256));
    
    for(i=0; i<nb_rand; i++)
    {
        number_random(rand, bits_to_nblock(256));
        number_str(&str, rand, 2);
        print_with_zero(str, fp);
        free(str);
    }
    
    number_free(&rand);
    fclose(fp);
    free_mphell();
    
     /* AES256 */
     
    printf("CTR DRBG AES256 \n");
    
    init_mphell(256, RANDOM_AES256, INTEL);
    
    fp = fopen(DATADIR "/random/INTEL_AES256.txt", "w");
    number_init(&rand, bits_to_nblock(256));
    
    for(i=0; i<nb_rand; i++)
    {
        number_random(rand, bits_to_nblock(256));
        number_str(&str, rand, 2);
        print_with_zero(str, fp);
        free(str);
    }
    
    number_free(&rand);
    fclose(fp);
    free_mphell();

#endif /* MPHELL_USE_RDSEED */
    
    return 0;
}
