/* 
  MPHELL-4.0 
  Author(s): The MPHELL team

 (C) Copyright 2015-2018 - Institut Fourier / Univ. Grenoble Alpes (France)

 This file is part of the MPHELL Library.
 MPHELL is free software: you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation, version 3 of the License.
 
 MPHELL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with MPHELL.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
  * \file  test_sha1.c
  * \brief Self check tests for SHA1 module.
  */


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "mphell-sha1.h"
#include "mphell-util.h"

int main()
{
    char *files[2] = {DATADIR "/vector_test_sha1/SHA1ShortMsg.rsp", 
                      DATADIR "/vector_test_sha1/SHA1LongMsg.rsp"};
    FILE *fp;
    uint8_t hashval[20];
    uint8_t hashref[20];
    char data[12800];
    char marker[20];
    int i;
    int inlenhex, inlenbits;
    int MaxMessageSizeInBits;
    int MinMessageSizeInBits;
    int diff;
    int success = 0;
    for(i = 0;i < 2; i++)
    {
        int cmp = 0;
        if(i == 0)
        {
            MinMessageSizeInBits = 0;
            MaxMessageSizeInBits = 512;
            diff = 8;
        }
        else
        {
            MinMessageSizeInBits = 1304;
            MaxMessageSizeInBits = 51200;
            diff = 792;
        }
        fp = fopen(files[i], "r");
        if(fp == NULL)
        {
            fprintf(stderr, "Opening file failed %d %s \n", i, files[i]);
            abort();
        }
        
        printf("test %s\n", files[i]);       
   
        for (inlenbits = MinMessageSizeInBits; inlenbits <= MaxMessageSizeInBits; 
                inlenbits+=diff )
        {
            inlenhex = inlenbits/4;

            sprintf( marker, "Len = %u", inlenbits );
            if (!FindMarker(fp, marker) )
            {
                printf("ERROR : no test vector found (%u bits)\n", inlenbits );
                break;
            }

            if (!ReadHex(fp, data, inlenhex, "Msg = ") ) 
            {
                printf("ERROR : unable to read 'Msg' (%u bits)\n", inlenbits );
                break;
            }
 
            sha1(hashval, (uint8_t*)data, inlenbits);

            if (!ReadHex(fp, (char*)hashref, 40, "MD = ") )
            {
                printf("ERROR : no hash value found ( %u bits)\n", inlenbits);
                break;
            }
            int j;
            for(j = 0; j < 20; j++)
            {
                if(hashval[j] != hashref[j])
                {
                    cmp = 1;
                    success++;
                    printf("ERROR : no correct hash value computed ( %u bits)\n",
                            inlenbits);
                    break;
                }
            }
        }
        fclose(fp);
        if(! cmp)
        {
            printf("SUCCESS : %s\n", files[i]);
        }
    }
    
    if (success == 0)
    {
        return 0;
    }
    else
    {
        return -1;
    }
}
