/* 
  MPHELL-4.0 
  Author(s): The MPHELL team

 (C) Copyright 2015-2018 - Institut Fourier / Univ. Grenoble Alpes (France)

 This file is part of the MPHELL Library.
 MPHELL is free software: you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation, version 3 of the License.
 
 MPHELL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with MPHELL.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
  * \file  mphell_test_weierstrass_compute_disc.c
  * \brief Self check tests for the function: void weierstrass_compute_disc(ec_curve E, STACK_1);
  */


#include "mphell_test_weierstrass.h"

bool 
test_curve_weierstrass_discriminant(data_t *data)
{
    field_elt disc;
    field_elt_alloc(&disc, data->k);
    field_elt_init(disc, data->k);
    
    printf("Compute discriminant on %s: ", data->E->id_curve);
    
    if (strcmp(data->E->id_curve, "nist_p192")==0)
    {
        const char * str1 = "e157e4810cc57a9b734d68ae2d9b93a7ab3d49fccc0a6e87";
        field_elt_set_str (disc, str1, 16, false, data->k, STACK_1);
        if (field_elt_cmp(data->E->disc, disc, data->k)!=0)
        {
            printf("Error discriminant on %s\n", data->E->id_curve);
            field_elt_print(data->E->disc, 16, false, data->k, STACK_1);
            field_elt_free(&disc, data->k);
            return false;
        }
    }
    else if (strcmp(data->E->id_curve,"nist_p224")==0)
    {
        const char * str1 = "4d3c8c8d9e38bb3d2dccbb285d301f4a4b7ed990197a514cd4ba3a17";
        field_elt_set_str (disc, str1, 16, false, data->k, STACK_1);
        if (field_elt_cmp(data->E->disc, disc, data->k)!=0)
        {
            printf("Error discriminant on %s\n", data->E->id_curve);
            field_elt_free(&disc, data->k);
            return false;
        }
    }
    else if (strcmp(data->E->id_curve,"nist_p256")==0)
    {
        const char * str1 = "680d8cb6fbc0a4201dd499d851c1ae23e501d15636a856b19e4ce86d8da606e5";
        field_elt_set_str (disc, str1, 16, false, data->k, STACK_1);
        if (field_elt_cmp(data->E->disc, disc, data->k)!=0)
        {
            printf("Error discriminant on %s\n", data->E->id_curve);
            field_elt_free(&disc, data->k);
            return false;
        }
    }
    else if (strcmp(data->E->id_curve,"nist_p384")==0)
    {
        const char * str1 = "f8adec8866891eab3ca72394d2e0f5eabd13d5c5bb3ebd1ea9968f366864a74288beff97ff84f4cd829a140bbaa1ee31";
        field_elt_set_str (disc, str1, 16, false, data->k, STACK_1);
        if (field_elt_cmp(data->E->disc, disc, data->k)!=0)
        {
            printf("Error discriminant on %s\n", data->E->id_curve);
            field_elt_free(&disc, data->k);
            return false;
        }
    }
    else if (strcmp(data->E->id_curve,"nist_p521")==0)
    {
        const char * str1 = "c87820384c786b11cac777effb95f96ba8b30f333dfd5c31a8333f3be983f91f0f0353f93df83c8ee9188e8b070ee304fc907f42722bd20a5f916f670a886fa152";
        field_elt_set_str (disc, str1, 16, false, data->k, STACK_1);
        if (field_elt_cmp(data->E->disc, disc, data->k)!=0)
        {
            printf("Error discriminant on %s\n", data->E->id_curve);
            field_elt_free(&disc, data->k);
            return false;
        }
    }
    else if (strcmp(data->E->id_curve,"brainpool_p160r1")==0)
    {
        const char * str1 = "94137da58982bbff125fd971343e2da5a90c52b4";
        field_elt_set_str (disc, str1, 16, false, data->k, STACK_1);
        if (field_elt_cmp(data->E->disc, disc, data->k)!=0)
        {
            printf("Error discriminant on %s\n", data->E->id_curve);
            field_elt_free(&disc, data->k);
            return false;
        }
    }
    else if (strcmp(data->E->id_curve,"brainpool_p192r1")==0)
    {
        const char * str1 = "26830f6e4720ce206d6468c5a5aa9473483c9e7a7dcbbb73";
        field_elt_set_str (disc, str1, 16, false, data->k, STACK_1);
        if (field_elt_cmp(data->E->disc, disc, data->k)!=0)
        {
            printf("Error discriminant on %s\n", data->E->id_curve);
            field_elt_free(&disc, data->k);
            return false;
        }
    }
    else if (strcmp(data->E->id_curve,"brainpool_p224r1")==0)
    {
        const char * str1 = "86e31cb3eb437b982c877684cd47d5b02533a29dec7ee06c1aceb805";
        field_elt_set_str (disc, str1, 16, false, data->k, STACK_1);
        if (field_elt_cmp(data->E->disc, disc, data->k)!=0)
        {
            printf("Error discriminant on %s\n", data->E->id_curve);
            field_elt_free(&disc, data->k);
            return false;
        }
    }
    else if (strcmp(data->E->id_curve,"brainpool_p256r1")==0)
    {
        const char * str1 = "213e362f47d0ceefd90ad1d9286fcf3add58d2caef97efae26e56a5dc0287be2";
        field_elt_set_str (disc, str1, 16, false, data->k, STACK_1);
        if (field_elt_cmp(data->E->disc, disc, data->k)!=0)
        {
            printf("Error discriminant on %s\n", data->E->id_curve);
            field_elt_free(&disc, data->k);
            return false;
        }
    }
    else if (strcmp(data->E->id_curve,"brainpool_p320r1")==0)
    {
        const char * str1 = "a2ce4277f2ecdb7306e139e6820bcb4de3e0ac7513825f2dddf3338bf8a8c745e75247ed5c25ce46";
        field_elt_set_str (disc, str1, 16, false, data->k, STACK_1);
        if (field_elt_cmp(data->E->disc, disc, data->k)!=0)
        {
            printf("Error discriminant on %s\n", data->E->id_curve);
            field_elt_free(&disc, data->k);
            return false;
        }
    }
    else if (strcmp(data->E->id_curve,"brainpool_p384r1")==0)
    {
        const char * str1 = "660bc884e30b7153a943590f04963c65505df1997c8412e32280770a31524377ef206eec62b3d3d67ae83bc96ef0d257";
        field_elt_set_str (disc, str1, 16, false, data->k, STACK_1);
        if (field_elt_cmp(data->E->disc, disc, data->k)!=0)
        {
            printf("Error discriminant on %s\n", data->E->id_curve);
            field_elt_free(&disc, data->k);
            return false;
        }
    }
    else if (strcmp(data->E->id_curve,"brainpool_p512r1")==0)
    {
        const char * str1 = "87a64b641a4dc8875fea5fc31f284e2535fb6b1a7219087bd017a27ee45529b4715522e51bab786fd7a63a7fc3135796ca6aa323e45a21cef96cbcf5a240c597";
        field_elt_set_str (disc, str1, 16, false, data->k, STACK_1);
        if (field_elt_cmp(data->E->disc, disc, data->k)!=0)
        {
            printf("Error discriminant on %s\n", data->E->id_curve);
            field_elt_free(&disc, data->k);
            return false;
        }
    }
    else if (strcmp(data->E->id_curve,"frp256v1")==0)
    {
        const char * str1 = "5179c60d95be7c94541fa93a65bb4960ad272774abd51fb61afec857748ef464";
        field_elt_set_str (disc, str1, 16, false, data->k, STACK_1);
        if (field_elt_cmp(data->E->disc, disc, data->k)!=0)
        {
            printf("Error discriminant on %s\n", data->E->id_curve);
            field_elt_free(&disc, data->k);
            return false;
        }
    }
    else
    {
        printf("Error discriminant, id_curve does not match a known discriminant %s\n", data->E->id_curve);
        field_elt_free(&disc, data->k);
        return false;
    }
    
    field_elt_free(&disc, data->k);
    printf("OK\n");
    return true;
}

bool
test_curve_weierstrass(const ec_known_curve id_curve, const uint16_t nbits, data_t *data)
{
    bool res;
    test_curve_weierstrass_alloc(nbits, 0, data);
    ec_use_curve(data->E, data->k, id_curve, PROJECTIVE, STACK_1);
    test_curve_weierstrass_init(data);
    res=test_curve_weierstrass_discriminant(data);
    test_curve_weierstrass_free(data);
    return res;
}


int main()
{
    data_t data;
    init_mphell(256, RANDOM_AES256, DEVURANDOM);
    if (!test_curve_weierstrass(NIST_192, 192, &data))
    {
        free_mphell();
        return -1;
    }
    if (!test_curve_weierstrass(NIST_224, 224, &data))
    {
        free_mphell();
        return -1;
    }
    if (!test_curve_weierstrass(NIST_256, 256, &data))
    {
        free_mphell();
        return -1;
    }
    if (!test_curve_weierstrass(NIST_384, 384, &data))
    {
        free_mphell();
        return -1;
    }
    if (!test_curve_weierstrass(NIST_521, 521, &data))
    {
        free_mphell();
        return -1;
    }
    if (!test_curve_weierstrass(BRAINPOOL_160, 160, &data))
    {
        free_mphell();
        return -1;
    }
    if (!test_curve_weierstrass(BRAINPOOL_192, 192, &data))
    {
        free_mphell();
        return -1;
    }
    if (!test_curve_weierstrass(BRAINPOOL_224, 224, &data))
    {
        free_mphell();
        return -1;
    }
    if (!test_curve_weierstrass(BRAINPOOL_256, 256, &data))
    {
        free_mphell();
        return -1;
    }
    if (!test_curve_weierstrass(BRAINPOOL_320, 320, &data))
    {
        free_mphell();
        return -1;
    }
    if (!test_curve_weierstrass(BRAINPOOL_384, 384, &data))
    {
        free_mphell();
        return -1;
    }
    if (!test_curve_weierstrass(BRAINPOOL_512, 512, &data))
    {
        free_mphell();
        return -1;
    }
    if (!test_curve_weierstrass(FR_256, 256, &data))
    {
        free_mphell();
        return -1;
    }
    free_mphell();
    return 0;
}
