/* 
  MPHELL-4.0 
  Author(s): The MPHELL team

 (C) Copyright 2015-2018 - Institut Fourier / Univ. Grenoble Alpes (France)

 This file is part of the MPHELL Library.
 MPHELL is free software: you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation, version 3 of the License.
 
 MPHELL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with MPHELL.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
  * \file  mphell_test_weierstrass_point_of_order_2.c
  * \brief Self check tests for the functions: weierstrass_point_of_order_2 in projective coordinates
  */

#include "mphell_test_weierstrass.h"


bool
test_curve_weierstrass_point_of_order_2(const char *id_curve, const uint16_t nbits, 
        const uint32_t num_point, data_t *data)
{
    bool res=true;
    uint8_t check[num_point][2];
    int i,j;  
    FILE *filep1, *filep2;
    uint8_t n = bits_to_nblock(nbits);
    
    char* test_file1 = malloc(sizeof(char) * 100);
    char* test_file2 = malloc(sizeof(char) * 100);
    char *str = (char*)malloc(sizeof(char) * 200);
    char *str1 = (char*)malloc(sizeof(char) * 200);
    number tmp;
    number_init(&tmp, n);
    char line[160];
    ec_curve *tabE1, *tabE2;
    tabE1 = (ec_curve*)malloc(sizeof(ec_curve) * num_point);
    tabE2 = (ec_curve*)malloc(sizeof(ec_curve) * num_point);
    for(i = 0; i < num_point; i++)
    {
        ec_alloc(tabE1[i], data->k);
        ec_init(tabE1[i], data->k);
        ec_alloc(tabE2[i], data->k);     
        ec_init(tabE2[i], data->k);   
    }

    ec_point P,Q;
    ec_point_alloc(P, data->k);
    ec_point_alloc(Q, data->k);
    ec_point_init(P, data->k);
    ec_point_init(Q, data->k);
    
    sprintf(test_file1, DATADIR "/vector_test_elliptic_curves/weierstrass_%s_param_two_torsion_rank_2_curve.txt", id_curve); 
    filep1 = fopen(test_file1,"r"); /* 1st one for 2-torsion of rank 2 */
    if(filep1 == NULL)
    {
        printf(" \n Chemin:    %s \n", test_file1);
        fprintf(stderr, "Error : no point data file found for (3) point(s) of order 2 (Weierstrass) \n");        
        exit(EXIT_FAILURE);
    }

    for(i=0;i<num_point; i++)
    {
        /* We first read the curve                  */
        /* We have to change of curve at each loop  */
        while(fgets(line, sizeof(line), filep1) != NULL)
        { 
                       
            if(strncmp(line, "a = ", 4)== 0)
            {
                strcpy(str, line + 4);
                number_set_str(tmp, str, 16);
                field_elt_set_number(tabE1[i]->a, false, data->k, STACK_1, 1, tmp);
                
            }
            if(strncmp(line, "b = ", 4)== 0)
            {
                strcpy(str, line + 4);
                number_set_str(tmp, str, 16);
                field_elt_set_number(tabE1[i]->b, false, data->k, STACK_1, 1, tmp);
                
            }
            if(strncmp(line, "xG = ", 5)== 0)
            {
                strcpy(str, line + 5);
            }
            if(strncmp(line, "yG = ", 5)== 0)
            {
                strcpy(str1, line + 5);
                ec_point_set_aff_str(tabE1[i]->G, str, str1, false, 16, WEIERSTRASS, data->k, STACK_1);
            }
            if(strncmp(line, "n = ", 4) == 0)
            {
                strcpy(str, line + 4);
                number_set_str(tabE1[i]->n, str, 16);
                break;
            }
        }
        tabE1[i]->k = data->k;
        tabE1[i]->type = WEIERSTRASS;
        tabE1[i]->f = PROJECTIVE;
        strcpy(tabE1[i]->id_curve, id_curve);
        ec_compute_disc(tabE1[i], STACK_1);
        /*We then read the points that we sould obtain */
        test_curve_weierstrass_read_point(filep1, data->P1[i], data);   /* We read the 1st point of order 2 */
        test_curve_weierstrass_read_point(filep1, data->P2[i], data);   /* We read the 2nd point of order 2 */
        test_curve_weierstrass_read_point(filep1, data->P3[i], data);   /* We read the 3rd point of order 2 */
        weierstrass_point_of_order_2 (P, tabE1[i], STACK_1);            /* We compute a point of order 2    */
        ec_point_dbl(Q, P, tabE1[i], STACK_1);
        
        if(i%2==0)
        {
            if(weierstrass_point_is_neutral(Q, tabE1[i], STACK_1) && !weierstrass_point_is_neutral(P, tabE1[i], STACK_1) )
            {
                check[i][0]=true;
            }
            else
            {
                printf("\n We should have done the computations in a quadratic extension %d \n",i); 
                printf("\ntest of point of order 2 failed\n \n");
                check[i][0]=false;
                printf("1: P: \n"); ec_point_print(P, 16, true, data->k, STACK_1); printf("\n");
                printf("1: Q: \n"); ec_point_print(Q, 16, true, data->k, STACK_1); printf("\n");
                printf("\ntest 1 failed \n");
            }
        }
        else
        {
            /* If we have equality with one of the points then we have won */
            if (weierstrass_point_are_equal(P, data->P1[i], tabE1[i], STACK_1) || weierstrass_point_are_equal(P, data->P2[i], tabE1[i], STACK_1) || weierstrass_point_are_equal(P, data->P3[i], tabE1[i], STACK_1))
            {
                check[i][0]=true;
            }        
            else
            {
                printf("\n We should have done the computations in a quadratic extension %d \n",i); 
                check[i][0]=false;
                printf("test 1 of identification/comparison  failed \n \n");
            }
        }
    }

    sprintf(test_file2, DATADIR "/vector_test_elliptic_curves/weierstrass_%s_param_two_torsion_rank_1_curve.txt", id_curve); 
    filep2 = fopen(test_file2,"r"); 
    if(filep2 == NULL)
    {
        printf(" \n Chemin:    %s \n", test_file2);
        fprintf(stderr, "Error : no point data file found for (1) point of order 2 (Weierstrass) \n");        
        exit(EXIT_FAILURE);
    }
    /* Second loop identical to the first except that the 2 torsion is of rank 1 */
    for(i=0;i<num_point; i++)
    {
        /* We first read the curve */
        /* We have change of curve at each loop*/
        while(fgets(line, sizeof(line), filep2) != NULL)
        { 
                       
            if(strncmp(line, "a = ", 4)== 0)
            {
                strcpy(str, line + 4);
                number_set_str(tmp, str, 16);
                field_elt_set_number(tabE2[i]->a, false, data->k, STACK_1, 1, tmp);
                
            }
            if(strncmp(line, "b = ", 4)== 0)
            {
                strcpy(str, line + 4);
                number_set_str(tmp, str, 16);
                field_elt_set_number(tabE2[i]->b, false, data->k, STACK_1, 1, tmp);
            }
            if(strncmp(line, "xG = ", 5)== 0)
            {
                strcpy(str, line + 5);
            }
            if(strncmp(line, "yG = ", 5)== 0)
            {
                strcpy(str1, line + 5);
                ec_point_set_aff_str(tabE2[i]->G, str, str1, false, 16, WEIERSTRASS, data->k, STACK_1);
            }
            if(strncmp(line, "h = ", 4) == 0)
            {
                strcpy(str, line + 4);
                number_set_str(tabE2[i]->h, str, 16);
                break;
            }
            if(strncmp(line, "n = ", 4) == 0)
            {
                strcpy(str, line + 4);
                number_set_str(tabE2[i]->n, str, 16);
                break;
            }
        }
        tabE2[i]->k = data->k;
        tabE2[i]->type = WEIERSTRASS;
        tabE2[i]->f = PROJECTIVE;
        strcpy(tabE2[i]->id_curve, id_curve);
        ec_compute_disc(tabE2[i], STACK_1);
        /* We then read the point that we sould obtain */
        test_curve_weierstrass_read_point(filep2, data->P1[i], data);   /* We read the unique point of order 2  */
        weierstrass_point_of_order_2(P, tabE2[i], STACK_1);             /* We compute a point of order 2        */
        ec_point_dbl(Q, P, tabE2[i], STACK_1);
        if(i%2==0)
        {
            if(weierstrass_point_is_neutral(Q, tabE2[i], STACK_1) && !weierstrass_point_is_neutral(P, tabE2[i], STACK_1) )
            {
                check[i][1]=true;
            }
            else
            {
                printf("\n We should have done the computations in FP %d \n",i);
                printf("\ntest of point of order 2 failed\n");
                check[i][1]=false;
                printf("1: P: \n"); ec_point_print(P, 16, true, data->k, STACK_1); printf("\n");
                printf("1: Q: \n"); ec_point_print(Q, 16, true, data->k, STACK_1); printf("\n");
                printf("\ntest 2 failed \n");
                printf("\n the the unique point of order 2 \n");
                ec_point_print(data->P1[i], 16, true, data->k, STACK_1);
            }
        }
        else
        {
            /* If we have equality with the point then the test succeed */
            if (weierstrass_point_are_equal(P, data->P1[i],tabE2[i], STACK_1) )
            {
                check[i][1]=true;
            }        
            else
            {
                printf("\n We should have done the computations in FP %d \n",i);
                check[i][1]=false;
                ec_point_print(P, 16, true, data->k, STACK_1); 
                printf("test 2 of identification/comparison failed\n");
                ec_point_print(data->P1[i], 16, true, data->k, STACK_1); 
            }
        }
    }
    
    for(i = 0; i < num_point; i++)
    {
        ec_free(tabE1[i]);
        ec_free(tabE2[i]);
    }
    
    free(tabE1);
    free(tabE2);
    fclose(filep1);
    fclose(filep2);
    ec_point_free(P, data->k);
    ec_point_free(Q, data->k);
    free(test_file1);
    free(test_file2);
    free(str);
    free(str1);
    number_free(&tmp);
    for(i=0;i<num_point; i++)
    {
        for(j=0;j<2;j++)
        {
            if(check[i][j]==false)
                {
                    printf("\nFailing with i %d, j %d\n",i,j);
                    res = false;
                }
        }
    }
    return res;
}

bool
test_curve_weierstrass(const char *id_curve, const uint16_t nbits, 
        const uint32_t num_point, data_t *data)
{
    bool res;
    char* id_curve1 = malloc(sizeof(char) * 100);

    test_curve_weierstrass_alloc(nbits, num_point, data);       /* We can re use this function          */
    sprintf(id_curve1, "weierstrass_%s_param_two_torsion_rank_2_curve", id_curve);     
    test_curve_weierstrass_read(id_curve1, PROJECTIVE, data);   /* We read the curve we will work with  */
    test_curve_weierstrass_init(data);
    res=test_curve_weierstrass_point_of_order_2(id_curve, nbits, num_point, data);
    test_curve_weierstrass_free(data);                          /* We can re use this function          */

    free(id_curve1);
    return res;
}

int main()
{
    data_t data;
    init_mphell(128, RANDOM_AES128, DEVURANDOM);
    int nb_points=100;
    if (!test_curve_weierstrass("P384", 384, nb_points, &data))
    {
        printf("Error\n");
        free_mphell();
        return -1;
    }
    free_mphell();
    printf("Success\n");
    return 0;
}


