/* 
  MPHELL-4.0 
  Author(s): The MPHELL team

 (C) Copyright 2015-2018 - Institut Fourier / Univ. Grenoble Alpes (France)

 This file is part of the MPHELL Library.
 MPHELL is free software: you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation, version 3 of the License.
 
 MPHELL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with MPHELL.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
  * \file  mphell_test_weierstrass_point_random.c
  * \brief Self check tests for the function: void weierstrass_point_random (ec_point P, ec_curve_srcptr E);
  */

#include "mphell_test_weierstrass.h"

bool 
test_curve_weierstrass_random(data_t *data)
{
    printf("Test random: ");
    int i;
    for (i=0; i<100; i++)
    {
        weierstrass_point_random (data->P1[i], data->E, STACK_1);
        if(!weierstrass_belongs(data->P1[i], data->E, STACK_1))
        {
            printf("Error, P3[%d] does not belong to E\n", i);
            return false;
        }
    }
    int j;
    int nb_collision = 0;
    for (j=0; j<100; j++)
    {
        for (i=0; i<100; i++)
        {
            if (i != j)
            {
                if (ec_point_are_equal(data->P1[i], data->P1[j], data->E, STACK_1))
                {
                    nb_collision++;
                }
            }
        }
    }
    if(nb_collision > 1)
    {
        printf("Error, to many collision: %d\n", nb_collision);
        return false;
    }
    printf("number of collision: %d\n", nb_collision);
    
    return true;
}

bool
test_curve_weierstrass(const ec_known_curve id_curve, const uint16_t nbits, 
        const uint32_t num_point, data_t *data)
{
    bool res;
    test_curve_weierstrass_alloc(nbits, num_point, data);
    ec_use_curve(data->E, data->k, id_curve, PROJECTIVE, STACK_1);
    test_curve_weierstrass_init(data);
    res=test_curve_weierstrass_random(data);
    test_curve_weierstrass_free(data);
    return res;
}

int main()
{
    data_t data;
    init_mphell(256, RANDOM_AES256, DEVURANDOM);
    int nb_points=100;
    if (!test_curve_weierstrass(NIST_192, 192, nb_points, &data))
    {
        free_mphell();
        return -1;
    }
    if (!test_curve_weierstrass(NIST_224, 224, nb_points, &data))
    {
        free_mphell();
        return -1;
    }
    if (!test_curve_weierstrass(NIST_256, 256, nb_points, &data))
    {
        free_mphell();
        return -1;
    }
    if (!test_curve_weierstrass(NIST_384, 384, nb_points, &data))
    {
        free_mphell();
        return -1;
    }
    if (!test_curve_weierstrass(NIST_521, 521, nb_points, &data))
    {
        free_mphell();
        return -1;
    }
    if (!test_curve_weierstrass(BRAINPOOL_160, 160, nb_points, &data))
    {
        free_mphell();
        return -1;
    }
    if (!test_curve_weierstrass(BRAINPOOL_192, 192, nb_points, &data))
    {
        free_mphell();
        return -1;
    }
    if (!test_curve_weierstrass(BRAINPOOL_224, 224, nb_points, &data))
    {
        free_mphell();
        return -1;
    }
    if (!test_curve_weierstrass(BRAINPOOL_256, 256, nb_points, &data))
    {
        free_mphell();
        return -1;
    }
    if (!test_curve_weierstrass(BRAINPOOL_320, 320, nb_points, &data))
    {
        free_mphell();
        return -1;
    }
    if (!test_curve_weierstrass(BRAINPOOL_384, 384, nb_points, &data))
    {
        free_mphell();
        return -1;
    }
    if (!test_curve_weierstrass(BRAINPOOL_512, 512, nb_points, &data))
    {
        free_mphell();
        return -1;
    }
    if (!test_curve_weierstrass(FR_256, 256, nb_points, &data))
    {
        free_mphell();
        return -1;
    }
    free_mphell();
    return 0;
}

