/* 
  MPHELL-4.0 
  Author(s): The MPHELL team

 (C) Copyright 2015-2018 - Institut Fourier / Univ. Grenoble Alpes (France)

 This file is part of the MPHELL Library.
 MPHELL is free software: you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation, version 3 of the License.
 
 MPHELL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with MPHELL.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
  * \file  mphell_test_weierstrass_point_of_order_2.c
  * \brief Self check tests for the functions: weierstrass_points_of_order_2 in projective coordinates
  */

#include "mphell_test_weierstrass.h"


bool
test_curve_weierstrass_points_of_order_2(const char *id_curve, const uint16_t nbits, 
        const uint32_t num_point, data_t *data)
{
    bool res=true;
    uint8_t check[num_point];
    int i;  
    FILE *filep1 ;
    uint8_t n = bits_to_nblock(nbits);
    
    char* test_file1 = malloc(sizeof(char) * 200);
    char *str = (char*)malloc(sizeof(char) * 200);
    char *str1 = (char*)malloc(sizeof(char) * 200);
    number tmp;
    number_init(&tmp, n);
    char line[160];
    ec_curve *tabE1;
    tabE1 = (ec_curve*)malloc(sizeof(ec_curve) * num_point);
    ec_point S,P,Q,R;
    ec_point_alloc(S, data->k); 
    ec_point_alloc(P, data->k); 
    ec_point_alloc(Q, data->k); 
    ec_point_alloc(R, data->k);
    ec_point_init(S, data->k); 
    ec_point_init(P, data->k); 
    ec_point_init(Q, data->k); 
    ec_point_init(R, data->k); 
    
    sprintf(test_file1, DATADIR "/vector_test_elliptic_curves/W_%s_several_2_torsion_rank_2_curve.txt", id_curve); 
    filep1 = fopen(test_file1,"r"); /* 1st one for 2-torsion of rank 2 */
    if(filep1 == NULL)
    {
        printf(" \n Chemin:    %s \n", test_file1);
        fprintf(stderr, "Error : no point data file found for (3) point(s) of order 2 (Weierstrass) \n");
        exit(EXIT_FAILURE);
    }

    for(i=0;i<num_point; i++)
    {
        ec_alloc(tabE1[i], data->k);
        ec_init(tabE1[i], data->k);
        /* We first read the curve */
        /* We have to change of curve at each loop*/
        while(fgets(line, sizeof(line), filep1) != NULL)
        { 
                       
            if(strncmp(line, "a = ", 4)== 0)
            {
                strcpy(str, line + 4);
                number_set_str(tmp, str, 16);
                field_elt_set_number(tabE1[i]->a, false, data->k, STACK_1, 1, tmp);
            }
            if(strncmp(line, "b = ", 4)== 0)
            {
                strcpy(str, line + 4);
                number_set_str(tmp, str, 16);
                field_elt_set_number(tabE1[i]->b, false, data->k, STACK_1, 1, tmp);
            }
            if(strncmp(line, "xG = ", 5)== 0)
            {
                strcpy(str, line + 5);
            }
            if(strncmp(line, "yG = ", 5)== 0)
            {
                strcpy(str1, line + 5);
                ec_point_set_aff_str(tabE1[i]->G, str, str1, false, 16, WEIERSTRASS, data->k, STACK_1);
            }
            if(strncmp(line, "h = ", 4) == 0)
            {
                strcpy(str, line + 4);
                number_set_str(tabE1[i]->h, str, 16);
            }            
            if(strncmp(line, "n = ", 4) == 0)
            {
                strcpy(str, line + 4);
                number_set_str(tabE1[i]->n, str, 16);
            }            
            /*We then read the points that we sould obtain */
            if(strncmp(line, "xG1 = ", 6)== 0)
            {
                strcpy(str, line + 6);
            }
            if(strncmp(line, "yG1 = ", 6)== 0)
            {
                strcpy(str1, line + 6);
                ec_point_set_aff_str(data->P1[i], str, str1, false, 16, WEIERSTRASS, data->k, STACK_1);
            }
            if(strncmp(line, "xG2 = ", 6)== 0)
            {
                strcpy(str, line + 6);
            }
            if(strncmp(line, "yG2 = ", 6)== 0)
            {
                strcpy(str1, line + 6);
                ec_point_set_aff_str(data->P2[i], str, str1, false, 16, WEIERSTRASS, data->k, STACK_1);
            }
            if(strncmp(line, "xG3 = ", 6)== 0)
            {
                strcpy(str, line + 6);
            }
            if(strncmp(line, "yG3 = ", 6)== 0)
            {
                strcpy(str1, line + 6);
                ec_point_set_aff_str(data->P3[i], str, str1, false, 16, WEIERSTRASS, data->k, STACK_1);
            }
            if(strncmp(line, "two torsion of rank 2", 21) == 0)
            {
                break;
            }    
        }
        tabE1[i]->k = data->k;
        tabE1[i]->type = WEIERSTRASS;
        tabE1[i]->f = PROJECTIVE;
        strcpy(tabE1[i]->id_curve, id_curve);
        weierstrass_compute_disc(tabE1[i], STACK_1);
        
        weierstrass_points_of_order_2(P,Q,R, tabE1[i], STACK_1);  /* We compute points of order 2*/
                
        ec_point_dbl(S, P, tabE1[i], STACK_1);
        if(weierstrass_point_is_neutral(S,tabE1[i], STACK_1) && !weierstrass_point_is_neutral(P,tabE1[i], STACK_1) )
        {
                ec_point_add(S,Q,Q,tabE1[i], STACK_1);
                if(weierstrass_point_is_neutral(S,tabE1[i], STACK_1) && !weierstrass_point_is_neutral(Q,tabE1[i], STACK_1) )
                {
                    ec_point_add(S,R,R,tabE1[i], STACK_1);
                    if(weierstrass_point_is_neutral(S,tabE1[i], STACK_1) && !weierstrass_point_is_neutral(Q,tabE1[i], STACK_1))
                    {
                        check[i]=true;
                    }
                    else
                    {
                        check[i]=false;    
                    }
                }
                else
                {
                    check[i]=false;    
                }
        }
        else
        {
            check[i]=false;
        }
        if(check[i]==false)
        {
            printf("\ntest of point of order 2 failed\n \n");
            check[i]=false;
            printf("\nPoints we wanted :\n");
            ec_point_print(data->P1[i],16,true, data->k, STACK_1);
            printf("\ndata->P2[i] \n");
            ec_point_print(data->P2[i],16,true, data->k, STACK_1);
            printf("\ndata->P3[i] \n");
            ec_point_print(data->P3[i],16,true, data->k, STACK_1); 
            
            printf("\nPoints we computed :\n");
            ec_point_print(P,16,true, data->k, STACK_1);
            printf("\nQ \n");
            ec_point_print(Q,16,true, data->k, STACK_1);
            printf("\nR \n");
            ec_point_print(R,16,true, data->k, STACK_1); 
            printf("\ntest 1 failed \n");
            printf("\nCurve where we worked\n");
            ec_curve_print(tabE1[i], 16, STACK_1);
        }
        ec_free(tabE1[i]);
    }
    free(tabE1);
    fclose(filep1);
    ec_point_free(S, data->k);
    ec_point_free(P, data->k);
    ec_point_free(Q, data->k);
    ec_point_free(R, data->k);
    free(test_file1);
    free(str);
    free(str1);
    number_free(&tmp);
    for(i=0;i<num_point; i++)
    {
        if(check[i]==false)
        {
            printf("\nFailing with i %d\n",i);
            res = false;
        }
    }
    return res;
}

bool
test_curve_weierstrass(const char *id_curve, const uint16_t nbits, 
        const uint32_t num_point, data_t *data)
{
    bool res;
    char* test_file1 = malloc(sizeof(char) * 200);

    test_curve_weierstrass_alloc(nbits, num_point, data);       /* We can re use this function          */
    if(strcmp(id_curve,"P25519")==0)
    {
        sprintf(test_file1, "ed_25519"); 
    }   
    test_curve_weierstrass_read(test_file1, PROJECTIVE, data);  /* We read the curve we will work with  */
    test_curve_weierstrass_init(data);
    res=test_curve_weierstrass_points_of_order_2(id_curve, nbits, num_point, data);
    test_curve_weierstrass_free(data);                          /* We can re use this function          */

    free(test_file1);
    return res;
}

int main()
{
    data_t data;
    init_mphell(128, RANDOM_AES128, DEVURANDOM);
    int nb_points=100;
    if (!test_curve_weierstrass("P25519", 256, nb_points, &data))
    {
        free_mphell();
        return -1;
    }
    free_mphell();
    return 0;
}


