/* 
  MPHELL-4.0 
  Author(s): The MPHELL team

 (C) Copyright 2015-2018 - Institut Fourier / Univ. Grenoble Alpes (France)

 This file is part of the MPHELL Library.
 MPHELL is free software: you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation, version 3 of the License.
 
 MPHELL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with MPHELL.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
  * \file  mphell_test_weierstrass_seed_verification.c
  * \brief Test of the function bool weierstrass_verify_random_generation(ec_curve E, const char * seed)
  */

#include "mphell_test_weierstrass.h"


bool
test_curve_weierstrass_seed_verification(data_t *data, const char * seed)
{
    printf("Verification of curve %s \n", data->E->id_curve);
    bool res = weierstrass_verify_random_generation(data->E, seed, STACK_1);
    if (res == true)
    {
        printf("SUCCESS\n");
    }
    else
    {
        printf("ERROR\n");
    }
    return res;
    
}

bool
test_curve_weierstrass(const ec_known_curve id_curve, const uint16_t nbits, 
        const char * seed, data_t *data)
{
    bool res;
    test_curve_weierstrass_alloc(nbits, 0, data);
    ec_use_curve(data->E, data->k, id_curve, PROJECTIVE, STACK_1);
    test_curve_weierstrass_init(data);
    res = test_curve_weierstrass_seed_verification(data, seed);
    test_curve_weierstrass_free(data);
    return res;
}

int main()
{
    data_t data;
    init_mphell(128, RANDOM_AES128, DEVURANDOM);
    if (!test_curve_weierstrass(NIST_192, 192, "3045AE6FC8422F64ED579528D38120EAE12196D5", &data))
    {
        free_mphell();
        return -1;
    }
    if (!test_curve_weierstrass(NIST_224, 224, "BD71344799D5C7FCDC45B59FA3B9AB8F6A948BC5", &data))
    {
        free_mphell();
        return -1;
    }
    if (!test_curve_weierstrass(NIST_256, 256, "C49D360886E704936A6678E1139D26B7819F7E90", &data))
    {
        free_mphell();
        return -1;
    }
    if (!test_curve_weierstrass(NIST_384, 384, "A335926AA319A27A1D00896A6773A4827ACDAC73", &data))
    {
        free_mphell();
        return -1;
    }
    if (!test_curve_weierstrass(NIST_521, 521, "D09E8800291CB85396CC6717393284AAA0DA64BA", &data))
    {
        free_mphell();
        return -1;
    }
    /* We cannot test Brainpool curves, they use an other random process */
    /* We cannot test ANSSI frp256v1, it is not generated from a seed    */
    free_mphell();
    return 0;
}
