/* 
  MPHELL-4.0 
  Author(s): The MPHELL team

 (C) Copyright 2015-2018 - Institut Fourier / Univ. Grenoble Alpes (France)

 This file is part of the MPHELL Library.
 MPHELL is free software: you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation, version 3 of the License.
 
 MPHELL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with MPHELL.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
  * \file  mphell_tuto_ec_random.c
  * \brief void ec_random(ec_curve E, const char *id_curve, char * seed_res, field k, const ec_type type, const ec_formula f)
  */

#include "mphell/mphell.h"
#include <stdio.h>
#include <string.h>

int main()
{
    init_mphell(128, RANDOM_AES128, DEVURANDOM);
    
    int i, j;
    bool res1;
    
    number p, n;
    field k;
    ec_curve E;
    
    number_init(&p, bits_to_nblock(512));
    number_init(&n, bits_to_nblock(512));
    field_alloc(k, FP, bits_to_nblock(512), NULL);
    
    number_set_str(p, "6703903964971298549787012499102923063739682910296196688861780721860882015036773488400937149083451713845015929093243025426876941405973284973216824503042159", 10); 
    
    field_create(k, "k_512", STACK_1, 1, p);
    
    ec_alloc(E, k);
    ec_init(E, k);
    
    char seed[41];

    char bad_seed[41]="ffffffffffffffffffffffffffffffffffffffff";

    /* Create random Weietrass curve and test them */
    ec_random(E, "randomCurveWeiertrass", seed, k, WEIERSTRASS, PROJECTIVE, STACK_1);

    res1 = ec_verify_random_generation(E, seed, STACK_1);

    if(res1==true)
    {
        printf("E: \n"); ec_curve_print(E, 16, STACK_1); printf("\n");
        printf("Weietrass: E was generated from seed:%s\n", seed);
    }
    else
    {
        printf("Problem of creation of the random weierstrass curve \n");
    }
    
    number_free(&p);
    number_free(&n);
    field_free(k);
    ec_free(E);
    free_mphell();
    
}
