/* 
  MPHELL-4.0 
  Author(s): The MPHELL team

 (C) Copyright 2015-2018 - Institut Fourier / Univ. Grenoble Alpes (France)

 This file is part of the MPHELL Library.
 MPHELL is free software: you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation, version 3 of the License.
 
 MPHELL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with MPHELL.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
  * \file  mphell_tuto_random.c
  * \brief Tutorial for random number generation
  */

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <unistd.h>
#include <string.h>

#include "mphell/mphell.h"
#include "mphell/mphell-util.h"

int main()
{
    /* AES128 */
    
    printf("CTR DRBG AES 128 test \n");
    
    /* Initialise MPHELL with 128 bits of security strength for the entropy, RANDOM_AES128 as DRBG and DEVURANDOM as entropy source */
    init_mphell(128, RANDOM_AES128, DEVURANDOM);
    
    mphell_status ret;
    int8_t state_handle;
    
    random_state_t *random_working_state = (random_state_t*)malloc(sizeof(random_state_t));
    
    uint8_t returned_bytes[512/8];
    char returned_bytes_hex[512/4];
    
    uint8_t additionnal_input_reseed_byte[128];
    uint8_t additionnal_input1_byte[128];
    uint8_t additionnal_input2_byte[128];
    uint8_t personalization_string_byte[128];
    
    /* Convert to bytes strings */
    
    hex_string2bytes(personalization_string_byte, "7ec2b597c845ae8bbb63609a80c2ab4f", 128/8);
    hex_string2bytes(additionnal_input_reseed_byte, "ae2ce6d1dba56775db17b8c6d9379f14", 128/8);
    hex_string2bytes(additionnal_input1_byte, "57866375238ea56e97dd6af8c5010618", 128/8);
    
    /* Instantiate the DRBG with reseed_interval to 2^20 = 1048576, and DEVURANDOM as entropy source */
    
    random_instantiate(ret, random_working_state, 128, personalization_string_byte, 128/8, RANDOM_AES128, DEVURANDOM, 1048576);
    if(ret->flag != MPHELL_SUCCESS)
    {
        printf("ERROR random_instantiate, state_handle = %d\n", random_working_state->state_handle);
        printf("ret->info: %s\n", ret->info);
        return -1;
    }
    
    /* Reseed the DRBG (optional) */
    
    random_reseed(ret, random_working_state, 128, DEVURANDOM, additionnal_input_reseed_byte, 128/8);
    
    if(ret->flag != MPHELL_SUCCESS)
    {
        printf("ERROR random_reseed, state_handle = %d\n", random_working_state->state_handle);
        printf("ret->info: %s\n", ret->info);
        return -1;
    }
    
    /* Generate random number */
    
    random_generate(ret, returned_bytes, random_working_state, 512/8, 128, additionnal_input1_byte, 128/8);
    
    if(ret->flag != MPHELL_SUCCESS)
    {
        printf("ERROR random_generate 1st call, state_handle = %d\n", random_working_state->state_handle);
        printf("ret->info: %s\n", ret->info);
        return -1;
    }
    
    bytes_string2hex(returned_bytes_hex, returned_bytes, 512/8);
    printf("returned_bytes_hex:    %s \n", returned_bytes_hex);
    
    
    /* Uninstantiate the DRBG */
    random_uninstantiate(ret, random_working_state);
    if(ret->flag != MPHELL_SUCCESS)
    {
        printf("ERROR random_uninstantiate, state_handle = %d\n", random_working_state->state_handle);
        printf("ret->info: %s\n", ret->info);
        return -1;
    }
    
    free(random_working_state);
    free_mphell();
}
