/* 
  MPHELL-4.0 
  Author(s): The MPHELL team

 (C) Copyright 2015-2018 - Institut Fourier / Univ. Grenoble Alpes (France)

 This file is part of the MPHELL Library.
 MPHELL is free software: you can redistribute it and/or modify
 it under the terms of the GNU Lesser General Public License as published by
 the Free Software Foundation, version 3 of the License.
 
 MPHELL is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU Lesser General Public License for more details.

 You should have received a copy of the GNU Lesser General Public License
 along with MPHELL.  If not, see <http://www.gnu.org/licenses/>.
*/

/**
  * \file  mphell_test_weierstrass_arithmetic_fp2.c
  * \brief Self check test for the Weierstrass elliptic curve over FP2 field
  */

#include "mphell.h"
#include <stdio.h>
#include <string.h>

int main()
{
    init_mphell(128, RANDOM_AES128, DEVURANDOM);
    
    int nb_iteration = 100;
    int i;
    bool test = true;
    
    number p, n;
    field k, k2;
    ec_curve E;
    
    ec_point P1;  /* src1   */
    ec_point P2;  /* src2   */
    ec_point P3;  /* add    */
    ec_point P4;  /* dbl    */
    ec_point P5;  /* sub    */
    ec_point P6;  /* mul    */
    ec_point res; /* Result */
    
    number_init(&p, bits_to_nblock(384));
    number_init(&n, bits_to_nblock(384));
    field_alloc(k, FP, bits_to_nblock(384), NULL);
    
    number_set_str(p, "fffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffffeffffffff0000000000000000ffffffff", 16);
    field_create(k, "k_384", STACK_1, 1, p);
    
    /* Create finite field GF(p^2) of characteristic p and order p^2 */
    field_elt non_residue;
    field_elt_alloc(&non_residue, k);
    field_elt_init(non_residue, k);
    /* Fix the irreducible polynomial P for the test: P = X^2 - 55152c4e9691e647b9555cf63eba66803697a5cb418a5806b62828e520c4773c83a21f20d3f57a286fb54980aed99427 */
    field_elt_set_str(non_residue, "55152c4e9691e647b9555cf63eba66803697a5cb418a5806b62828e520c4773c83a21f20d3f57a286fb54980aed99427", 16, false, k, STACK_1);
    field_alloc(k2, FP2, bits_to_nblock(384), k);
    field_create(k2, "", STACK_1, 2, k, non_residue);
    
    /* Create elliptic curve over FP2 */
    
    field_elt a, b;
    field_elt_alloc(&a, k2);
    field_elt_init(a, k2);
    field_elt_alloc(&b, k2);
    field_elt_init(b, k2);
    
    field_elt_set_str(a, "8a6ec8ab9f90ac4401732bb295af00d99debab4dfc11e73bd6b77f5a1b2f11b9a23d446b34f96bc1b4c4eda073be29df,fd5bf56d4c395957b51f9525846c9e8a54b596b8a394e96979f052c11d51510c37f15fb25e396ea0df7a3aa1da56448a", 16, false, k2, STACK_1);
    field_elt_set_str(b, "92fd9d13901b6a9ba9b18db5211645f5df20d6eccfeb9d04a296cfc12bc28db255536fd8b4d2a0bff20bdee96043c575,3defee1d6388f23ca7eace5f438d348d9fa4fe7372998a5055a76635c39b761d8185c6f64f7c06404d848d5125007eb7", 16, false, k2, STACK_1);
    
    ec_alloc(E, k2);
    ec_init(E, k2);
    ec_create(E, "Weierstrass_FP2", k2, a, b, NULL, NULL, NULL, WEIERSTRASS, PROJECTIVE, STACK_1);
    
    field_elt_free(&a, k2);
    field_elt_free(&b, k2);
    
    ec_point_alloc(P1, k2);
    ec_point_alloc(P2, k2);
    ec_point_alloc(P3, k2);
    ec_point_alloc(P4, k2);
    ec_point_alloc(P5, k2);
    ec_point_alloc(P6, k2);
    ec_point_alloc(res, k2);
    ec_point_init(P1, k2);
    ec_point_init(P2, k2);
    ec_point_init(P3, k2);
    ec_point_init(P4, k2);
    ec_point_init(P5, k2);
    ec_point_init(P6, k2);
    ec_point_init(res, k2);
    
    const char *array_src11[100];
    const char *array_src12[100];
    
    const char *array_src21[100];
    const char *array_src22[100];
    
    const char *array_src31[100];
    const char *array_src32[100];
    
    const char *array_src41[100];
    const char *array_src42[100];
    
    const char *array_src51[100];
    const char *array_src52[100];
    
    const char *array_src61[100];
    const char *array_src62[100];
    
    const char *array_n[100];
    
    array_src11[0]="29159421379712059859292863008043092449709449532120814445012227572267451462465148175287228003742049580572635874910857,9470261556631047177210681681368037314594226031723845678525461464772715214305631209709808380281507743076407744116330";
    array_src12[0]="11908382132288771853481296742762833319944772649918771606273395336282536588385174207751600878120646559678772870989910,18189843608995376154443282565348533326294684944693978973971704636369102626009222886678750604767328078210569171868498";
    array_src21[0]="15863226127479154484283477952469408490536365881162592176996561349989061816526486039089010583370387231154079114556521,31521359260976666195374928632784804748303914540419473399988602814282526897037073520759250466013964962043773017452870";
    array_src22[0]="22811847324339611169250151016306099954477728408070375137475947020706145260103450941774163166079420451389898045198627,3827420698043313920676867542811840018001539176289899507978848500168736742831701754439868789204779903496864275542071";
    array_src31[0]="2079508815169885504648466927841359777922482858790232227973167881144770232077945164481448584361472037796517052604234,7644266803876315997741067000554345325095615604494159715298217859237552847633846554930675230522214722469623771554573";
    array_src32[0]="1078622655947362916771101117188246835742731660668400292560219312517529069008406065857504428276627474627560565064504,17711175545179494682954329260232364874878116219495464588066281841820342519439742112470458272127024964193216724389514";
    array_src41[0]="17228911077078793154940192275017605697348380069908740459449887647682129445104722358876355739055739630337842601912179,9248633306558686739556538920991005987707861390607373788699563508755267467707423851803737897207692579429987661462429";
    array_src42[0]="560312227415721416788334603666719453855659375291559375009330015837740636937530678295646159595001770019408766211343,11183400403082284725625725522647940808418609256064607635770204376546934096222640291989980562957323971066837162746810";
    array_src51[0]="13959828796496060644778268007456217325834163750168008789599769747215041736648063030155109032489210031213580367354188,13872039867108016414658915225469964117133228413869124337398820107723133713350514262392767965751643192876116501657520";
    array_src52[0]="9353776862898405646197726111111073216498645704170792538596025227655811431047009129032922755475179460615484004352999,32464452707339593225517590858544165051726578990760953232257585344984938171850972871728569639202025469769173216274261";
    array_src61[0]="12917298836079879115907394101137555954701026576627910010330217679813440741699683294562657844581116326917239513656363,30275377118126747315645232218705673670126670477162892545801278758051137747203800382154542470146362124514608433772270";
    array_src62[0]="9635746291643394607897197305340240267589875550241274236492621197660864722607165436537820329950335085816399362888125,29724309961687524433737741663504986761457311231749922135458952982626831549826392990627888759397351145108904154352486";
    array_n[0]="35119977218682706985150441182705888305272635258121486680087311235722431540247986055722011177090537545957807450438330";
    array_src11[1]="13295560087016160233403212546174413276309689189632682061342765875994794494083941716904716430953428102918786283706548,35234950025613691487431185950093336829352860115254984335531120138975268454419552748132238303075988244597545651195216";
    array_src12[1]="12173568659909205974843917838670076396963766193498144524895391337509165497642080740736990921501644306122322524721512,16207846366793615331933055914720341402821654249003737932344188884323152060841263788129403792985034262415272621631442";
    array_src21[1]="8915937192828087881745524627088917370326452779996150423039355252710351175234074189032994677888054052972242597155643,30707127236509255169394569842933156162664042881390261506593552521078975284787700560251311361073097128459321490690316";
    array_src22[1]="14828263517972620911829399128146943760321307441368805947153414188913606254346345793404669269977948884228961269858626,13240640917331001367558557454130423338304482354136345600177083361375304024786416253924431488648818416309049063196868";
    array_src31[1]="22032101437325296768324569601809752330462894015660151931671668788180120318107281262206408338427766572528236996455885,22103712737888346907330893639969009791579415133474042641732568829251916274323444781474973961046908924719069314361189";
    array_src32[1]="36705637514943956997988648010273990658011267752240196210679521968641800611838754295123726976503495239771144696764523,24697431614842875498190633969903646727386010225045971198846618776003297124798435995163533119371104432502949680090172";
    array_src41[1]="23033288181945037787495396794804222288275424670169618003754785102603742589014923861548894914304391231760514454076361,16979197408747789220832341884539430930779041901954762216495933071606912160097133590500290186408418299753480988089820";
    array_src42[1]="24607061786356302870391360012167138737595853416730243874839927790071586143288402415560602906116385434152440715729659,364631789475809573737438579509622351387619274534505935246634898611209825362637882770746056079178471904698230918648";
    array_src51[1]="95397802379873898220309972642191063105236625919207807093519143564425743933330824597403680015973869068675262169406,27190703051068113194500678189886832163895797285879621364968794706771166047057712543861779380809045914885554877933142";
    array_src52[1]="12403436278887041870986318549250601257446898921049186601358547414495876457596031948002980624168737135800084055390584,8301348020356107639042584537571346178099721975830366749694349765589394607557198483209898256959267126140372469339119";
    array_src61[1]="29194229045440231120255354812213372445161892235167285044144719620521226595949156322644865650924944406407686143385418,548689277429785626254402191045326505918594628453103400613715155298595117380810805439836177495950279943346908025273";
    array_src62[1]="7932637607424692637785294718539232782923982802563204127389807699157148227096228607683919521425182108857160619546227,33268024595291535910120538976060098041662909204339401740999028134687907037843672072637381943780412977757315313228230";
    array_n[1]="16551020303862037433096153595410708866178538858571763149066794020228414291752814110004191003255307984766897225134037";
    array_src11[2]="32374976845614867524416976408985374144412598723887451283817522056749493491840637770961272408216696474001634128062434,10713626090478447484835363491310713764194605636764589793245233187680262392575419338258384962553154720776133927380004";
    array_src12[2]="27561084550882841750818592863953276534811984479039777491708396012384551262331603207744511382417760292942316537439427,38952084223857342468016405033156800282999917764516003224182572093854824902421235068271395320369474330313191968275186";
    array_src21[2]="6312869652267463523508795971468529070422401890090210124427005998932047487774199760861544892097371453296909614236973,31941225105513011357973201556705442904938368059938778845217306398578481181395603278025031099530788228980048402570646";
    array_src22[2]="25018428950275757402087526946556477369878981974993249883333993682823689162516790503601516207500999794922967697033770,20309127679151087524035753155524344785902569457179483919360647786690283264498275844175133118845453619072314382511139";
    array_src31[2]="18029130069814556307501132156770929587469473588993218562867737945869759789048687514710361234941452467636880338882614,28926962082747462570062338503984682852314816307160408556406020904499536761371149361201473116039883611748589270641711";
    array_src32[2]="1273460907365469070567180718868822937430580513701015180660873799506949950953962455860460841665923386487095671410069,3701488877887941697935916583277842622040657356272520476946377301112173534390387718990382236982699373319045487862388";
    array_src41[2]="38830953699244993622279594932729329726788099381381953905335516266187641828722574424045104589191380227820776645999063,31940589404933587085620410737094155132637022171925233170001762304570854176987834773588702329331635453284580666783660";
    array_src42[2]="18127154520061042653812094060550964446843777163920588298518847315448412591143257196698484785406036012701910629995793,4073626918492051230381940912753902743283389610729255076014984333723618443479000423466744110702902552760700733767069";
    array_src51[2]="20673525857917152956039482707063301465103173288593336695964162182548941775925818650779785573623121942205359318373921,15912460426366435732601067973214352836218568060718717047815737512838360971492005719976906822514035725962949530022245";
    array_src52[2]="23980476218736626462398541032413647892915132431416384933804249278499667619030215080232323779769718071937930596902146,4966755857025064200699047324587549346577367860409595887782706613619537657670526434522737465649939115293560020822144";
    array_src61[2]="31839564792889496699888746704611699064405382718550047337658055341534336891453658550284424540305532752974285996722866,27659660953448572031960560143521967222426225057109843042565344471602780786250930544840836153779874079687756495168969";
    array_src62[2]="27866627431296841374869627956608780065399381033548873615110850504795633401495340604777048228159533491114043656553793,905346842704041205076929436860290485689199240353932153763409926979880433275359643415422655048068769420016453059105";
    array_n[2]="27486004244005337910822963746409074617165222910528181608832019447929343734271677748494611318484363409864289691309864";
    array_src11[3]="18413575919581155942643531652506966363408460502562887176093966003584047188395673125261362764266892243339475857246406,2320403505637911179480622244933948945206520929938029712146505456926685626480237127634976751026521241658242455969894";
    array_src12[3]="18978537770733271452941890915036472275798588179405861125298347304140910376078624247736642571662752678163810336290512,12605245811850904228205262164541525977893759087232015117821306930533569403758395279508501778859112259813153044219474";
    array_src21[3]="35418008265690719848368636623151521329048042210391542338551822184406392017477267467595986783779805842487651814002813,34313185170065774945072273344426804938331948287581121314098102011533418582893687845377361940555497854525191655593065";
    array_src22[3]="33934007542641842920503412068272199126068962008569361090238905948743498240137240481182249217875633998691171883092595,4877608409990517532465046996830156331678876158620853712765477163328895388436032176834756515716456041463857876894458";
    array_src31[3]="9816514196821767717857467094055445231879758000861799172048473994272015880716719249942940808955486552012064288580681,26592283366219860431924560118832853998172902319006310555553283125501204754797714081856714809962100601438844046303945";
    array_src32[3]="17000492585432223483176839997050915671883235838947613183136278542875000126227875107784429001492461910198811834841866,30180368653524189521748253288289589307737326949970182338561984981897071305701118196264431994890690852635298816544544";
    array_src41[3]="29684793323971923573883406261980748506421868423733963311321629562387397048098667359491182971660661860769461472641905,33408144065254714938930562122358717429322826159050226465129475763694644726661628383623935121448417235589616436701860";
    array_src42[3]="37895766224682340075920971077756248333283436366386337877257616491229259653042575001697403269685453966007202359444569,21706334602904658712041842915230027881412901626858286403211189712086882611095143696757945287861085717615915347216468";
    array_src51[3]="11858184312782482411241412618073795069975762427585656537906818203216122274436950131920175300042933928204234669269592,32293857692216175266383866723722792341697527252174767200500575813105571423625603211604521619417487898468455914871018";
    array_src52[3]="35170553311369302124003427577053578764421898264574103906269299176019207753236604991064426840068574557624580838914405,29336007243635298092172214013121245578783151459207255560664229259941437342752572243550594798111930919309170162886805";
    array_src61[3]="25296471524960545975960391975478290492433851766161777433008042640598722011948450821790154482876548458766105048593277,34201030618023331134465742653566498033926249461091394431930871560487315598904676492978525441367889536097632997448352";
    array_src62[3]="20978109489759225946464615649549702157712348557280651755088950989296332788397536805281561976677820578968891442793278,13179684041988320007604780596311842975977956094082924721375002875558347801070988176413828818431187947706546233955170";
    array_n[3]="23177802194546185089947640619739468869719754203369655179691650530708939220026901000053956812295714468663261439255818";
    array_src11[4]="33372918843502514634390424059604298777607298974394717583754621403013639895715264455029434807972313367557986648715862,5431429864435853355235761124565856493459062638911138068867004976954736373816298505916784034626314185597236636184427";
    array_src12[4]="12419745292758040249044082052497421380343872018903711812422412119029352615910288109263968468936088090153092867862786,9854540786730907334231197959704308700690480674644544771813477721240590871815540907518794043021848842076092636386091";
    array_src21[4]="32903301300346255608761389187886153252405911268780611988917591518169530104806185946844149570396241366008808580862958,21846572551528830624814388745292454660509779647448673041638188280945731664108681911504840131445388406049278286228064";
    array_src22[4]="5283640258376633420982259370194797445934547581750716147679669364755592663998580291414101263034107423427082721958835,16622672874685286373072616509163900841692935044648536405753250693559839374360020567700038138232055374166383200034919";
    array_src31[4]="15011491322744519587519997016701116795476933017314880569896275842117466602725723426467677849350009486879323418663295,3126827240158010622394265946460929291566386121295285390727238260398886337612339103726308705162080071583330789819020";
    array_src32[4]="13737843897294353127074661478343138518613409676261494461977814871098872581796711467347622726537044457239293911512676,37133528321114632109091898807955365320789413023239683269819953809708401337398684063189223847148813225257547730316086";
    array_src41[4]="514373740527406330266406878152056179281157831838622977702711024567460635832946014526655769682558850095900339384454,8534296296231788150028680595001701715250519535059171223121774820186008661792938371333355084259489236231938644544891";
    array_src42[4]="14346146616650373963671643185191144765291352691960187957069660292814405778344022367431098813932269486487002502585876,37147252655499646751107540296515987745054953585537807988108160480858841805726080864332622577270858338739318333446660";
    array_src51[4]="1462324251041950172595808706514329195094865732550810320216727691862570882174287749351900080428266057644932001943066,16235989970257047925189955127363924587992634196319648353333108740281890319797871259590768293145255453714086510336198";
    array_src52[4]="39051086886977375529019048968318863383569084142236270630453048161707179625945362733523025765956241972897994510691260,37884956948054928122830662373998319561382930220987883960580728780588942568031612555675482922035233824821219139434690";
    array_src61[4]="15282600982193668108343815277363345045536834597565249562365110782688847953770076982140878591566377290974859600733982,30047309724451151654560631880432600717040568661852517369007248627193780080918456184877395915324718430915076223797466";
    array_src62[4]="15395345710403756656259388054244460073555966145760688893156526618701685563250856081237038923231349488705942109395404,28622339631183557213764718203673800349874620046572433359541047856798868632895166015551342921120925768209216700333574";
    array_n[4]="21945705767786180191759904186690553659540169278897750085284245751367401351537592808813789762438624660795042059445174";
    array_src11[5]="35066992085515417325972869224990569864681160829693589005795144349717093409064166475784806354580243477791112244262172,23747563499739731567515372223808268404749437791661157160976302627769601223500254950456985899730938972495696164086579";
    array_src12[5]="2253841878627082811158221075422255030716817486327113102468194255552142797512573430333335482069410722708037587936958,23639057340139368529015210751047582999779169668632577020629320304225466443504473099923990067551539324448042880787730";
    array_src21[5]="32467874806938385738795477633268544351180311710733891503144848691193683032387977681430930012836005863539904772223781,22832962944687343965273080374506170472460394819392382324149213408366806268174961366997803852869565005198874325762585";
    array_src22[5]="29045382622111688839151840278219225127727519965523349298174643361488788396850187715301643818085581640974076513848900,22754414538935073097200563077300249248467801149090574657957891184587924696981588857852715244878335854603200316437715";
    array_src31[5]="25396200163645227019824876156524458107872835961494913873121812243289695560531061485608248081174217873461963514809419,19759259141194748155086971810939418055540547679525442202773246085565820393935052823370815023633484869321443115125840";
    array_src32[5]="3396808111087201811710357321119630243269504202019403238015945409687066463746359698453061773007636811947378813051417,20842170602801924133870420538962246702120329716788155488515338531352477352681453649682731822470047699967052248463165";
    array_src41[5]="36621083644415688376091507832217732854945461069849972635240022194878505302183385806724147719446350251184205342187071,17686695302181175311703002335118705929223135992728899281348558291109352496861958289564125827187138910531427897471332";
    array_src42[5]="4964684679115713224201618824393230429579938863163210238806173113649653693321988931502095970133280041761217684464720,9298797339066996024573369796198273909554249112280526319625267046932295819877855429611997270395522778799072307648378";
    array_src51[5]="12261124798762066228655254335324465067385839337051976941022231591739832616979364233839468623288044299553258375952405,26906206102981704695550198315075362481012471559362172019318605348188726197101690047919779416460835862009102580571608";
    array_src52[5]="7587950999742480079858435677202424511430642815822008946407348855092430555795609215281162602609379318172687875388579,5471170323985160587070273279938957311446258375796101683941168936806713392112921049545239155408052524128220286557714";
    array_src61[5]="8470577537263882576745870709066997869692181040460050963647390834377741179589037590985564326972096400899424457453831,25405132860240082530342980970860647111913536792334538153097854780960429749239951094559610195527747179241264780618356";
    array_src62[5]="18801299155346175515126783690674818336401633994602844976534958697246784893251048093625890804559928064697005771196947,33338438552755537652256920284753735301185975163761572596770788152638794747283347201690930758145954383847166380357086";
    array_n[5]="20149790618545187744903729306132822654866790668190627904135817050677380023387115369929272721040287000211803466135594";
    array_src11[6]="37476204072124130319428193353270065698307870043412581706461396719832653316731790610218296822520284291229894285579179,8529853758945794658977286810757269357072799731061803822839069718223083055439959166502986607316947639797009110837284";
    array_src12[6]="6208191438280424272499693870810540556846439495680415222465522117014742190669175860729486255003763241157202429149672,27955958932755446902303920141308785835336266173663537979406631107833556250143438110044552357240211335639159046913791";
    array_src21[6]="34572461081511779079354247726885737862702031098716571215826257303188149859056953969383354520088827208376723836992545,29479494567927153740806657378746679232097772548920300368452015228998667062255455691727280866237046012024846544499825";
    array_src22[6]="15886714750532512502862499768235641246957512593812652177414698955414253885910542288314226732592532590671790997779313,13408543153184809460094665872395956936771659614902001046537779681630796985231493069560518584730179730435269157643445";
    array_src31[6]="34250905207820912697653595051607462064526799492584358297934447235825012396143873356705028549221406622079414915835656,5072337484191229319892359135622258779686756976424332779190706892838935480698824920424853794202326606356358755528753";
    array_src32[6]="30165317314202182270968028428640293298804418693437995153543149563751091833220001804846758381286081374471411396080604,12589104268325941170830305483589435112030258685570300276611706191571370275475354453791042800462901473182226782214603";
    array_src41[6]="13653158100135988748211984287464951096159394058335543048874749190398009592321318402903814816012690214360025373325677,3771740352986423521168294125943035745302485725279267384634972857838308541793800811330571390989768983480078362263079";
    array_src42[6]="14748546638318649330739687974412459125611950402132606430985416613368885556939877325331695077308639968860558414682448,18840648936273878736519572958148613738935869665880213754452378642686340953501630718118205466294006022195008992005169";
    array_src51[6]="14429783072561720248292899949234465744848641512829653498199848868941267663592575576506299702333110054762027713559486,37342863622890549510895662613211220465322622315720289514650601944168496594848075182519565214357885864039372734207169";
    array_src52[6]="27297880906659914851299392346817487797849822985563907989488666303158973437427019815883689054363469635159251600297174,13441131161091208712517429106747084469267056442219863827421081866118340520685866938918624127202394617159501225494443";
    array_src61[6]="2205955079306308569134693616539661776924312349309416340513439765390214111210109613012240137277187076618704375721960,31918326737988016508405854965342906769789114396204272064826488753438535729462949604950454001250625288690196165294535";
    array_src62[6]="11614957067480901645224190091544873444263331139587443895000604462050119171140446092960193081537225471055269215033105,14202491321214440956263915765120157513220349334073889084190697884407735800083714385034570261390615143020073935953752";
    array_n[6]="11158788037729933418009710706840907569708377617878170181319862636859396294572654348007760695560737335369834735611151";
    array_src11[7]="14707263463205491560247134362083344190912990144132203446440464710650307749619056601588656114715606527066030150078475,31451503610283686026478328016553101625195140662805971850486033241740943712275309902432422225617379673759193617347968";
    array_src12[7]="5188509360778165393137028406757085583782990077404217004726409593413743949772624029876751755359433199249142056001119,21561530218749132333846219180457744831634776007779091747647061600761889279380197023185404206860611846956799168905521";
    array_src21[7]="29527124512502228581007165589051663748295252185502627636398598923568030516338876761687416235164559068293066986993377,9742268782965788401575323289745970089127569601012580730560472714220599657053653492312155871803745973034848946228632";
    array_src22[7]="15221336467073090482733296029046253857509922931280433027999887124127604587308311095185581714447426788949275310229549,10981971668400743438655901825267159842179964681857369322702706292354757608235301855675741972018882207630967182985143";
    array_src31[7]="26721615573035450005817961868295886261938951723019154383031687980592692827700017856350152403369990075146445466786835,38652224326250750120605770016210306903483981264071313181208959515694149099749031704931516800745965588098010234411877";
    array_src32[7]="3853149848633643789288497807632709854500817509946860298805013581802169560636428185188941235949553907673140147556352,30134983707503609901377018996658930944590892319389681752258422267399146229460959059740419747299413423678577166443309";
    array_src41[7]="24714121893251238151420706451353442242937216913826795374880583726649448168942884922915212874974209038935203238152104,38420196719693521817995437202862572508019259332705718122175279496895071046578872479352832589889338724963376802877107";
    array_src42[7]="17448004873324126866171321869046656070422156530414113434916632663568394570544228263081888626647904095461790885037449,9271559710225788708937216682120462987354034529293614964279833320856361442399738800628440879414111449081334463223953";
    array_src51[7]="32407791544382860764045987998198797170476003759348252903821016029992951421184715570231937158107282556823547447349052,1272103555241797564468745715594267249563416143704113806125776598070433456704467261548238406665443400899530258583998";
    array_src52[7]="3617301581267386712184005117796899117244233242539380886602770158796271034090616023326956670805475818905492848409958,16626473554205605421431149485078961922825642422326654952617284306531594594777367451220751119210802468392943428508532";
    array_src61[7]="31520807944561027783594248733114771474719099254655874373676593023561757383449318428962412622784714517668204236635792,35090174235036291647956360057362376739526484253238339900066819329865041781115573086061724884429599036475604837344878";
    array_src62[7]="20233090857980519012698608267728835432717324101444317040166328244212012306175490104058491715156515917200497702461685,17382483006104449660389966319073480130896067676456085630989568000852306204884615011323843048242293105263836451915222";
    array_n[7]="38646529624948138959861888702586791054399250725907931987869709991500837744781812575219365425108179965922457132128516";
    array_src11[8]="17313432041240139873296464077262321532252363915395118560905115949567821013328002257995554834691271305148132586505594,13623652682214757334139732872017988735751326980553225451634163339056250736214321816105365284840181144963649176836553";
    array_src12[8]="19113480823343607099958459227091201762775618235388169158051444027793799307196621751842830348296866095250449046656815,32389969785832513155569262821166877691658814374837994969750889931134140277561781019059217253090319741443148729181974";
    array_src21[8]="37550490406088322197391906171254959631370288846629424938931872838519746100467352046073994218115076615320659845638658,28200669585264172212084703335866030524508979378080675725351362293406439383649569709933090742140938754115966363595401";
    array_src22[8]="6824471407246177002100161149992073374647974201069007317313691078082402619317710777360331397785288695343038811884873,4508175857832496122168266872811132207575349330083563424804134794279538029553880686779260989325807755404672593263112";
    array_src31[8]="31882754772553658597240682256476303748706732347446827386612736978819571521071581580373167004427063722881969063840823,2481133430779852391545375480391576513347548588518241991815097793982548942937912981811955976454530739543707768472262";
    array_src32[8]="15525130503682131546736699029176148466665680763623501771405068855195137642855142336247017838450574919610124180660236,30956710480265740835589183786130632353124824040623827904560379061416728811693836947590236303931149786884339116838983";
    array_src41[8]="6580806958502798086725818277294416987714967482950618487371867175297487678819109204901381159519554361574183398034478,38611136786371146341488567736381189410675084388961944720168561050443512350828236621383080202340060941349416900650619";
    array_src42[8]="36406136294937927013402221066142681390369043841705945291480153349730705891278169278071170906556031786575247585191109,12563614868129918850078909158766222909603530742573793612017996250842983467787598761103867783401073012169008248816240";
    array_src51[8]="11277218719908142297656045472347124627251527125289111387690971262017882104924235246902129099196529484838848905115139,14426578287851181801144406934293827407296528270328192007600078902985420993812215165459437755157272616942809363036041";
    array_src52[8]="30892164624555215730913394884762764223888080420471425393928885909224636975198411622140630097017651791367549009356294,22012716164188218305975003565319153324493080258121656759738121192554353223076413753635766757350593254621490260306254";
    array_src61[8]="12009002676351199652892755211827960214848914438603659113912541620781304538271254662225906634296752428524325602264376,35544764518019729984985226230909252505286905369627018179481284356463146542128215365957384612915683166854548723000305";
    array_src62[8]="39374570632909037603461912370420921530993451169948544306129904531474617611976001216328188443114534778442969486265699,29968449446805650018997034776083265444639733142420749272769656755934515991742940364445707268261381488472749464688768";
    array_n[8]="25647351517209701792123711653339421415418557108580115565977022960206371631704798643563973878057725717365412600414976";
    array_src11[9]="29120809631071332288872779668937894175481478529681669490625461440920308619863947573694406610410046789232097343991524,11118466307463021171700309113981095764627961619662307277527828981469751600530363656929758848884826573720734947014874";
    array_src12[9]="25561131590350747635635772176843033039013278920489344087815760750471981350526185565215975587623623371235505044083100,16382036087864061401030383459342664097962546649977063880137794316494700882411292289489367160400303684623727236019273";
    array_src21[9]="13488013125577553294052703864449648861342077339009369254328768531291661095165143600293078158302831225058074168073321,26472139397326894590982547033059349680240040593407866307072494107646185946810353600270898018651050654962771745315688";
    array_src22[9]="11436395383725691902781833986748858220833018598023551481153017393308651628064143100734118318220426512615039440508306,27039368591585612560406919927300602443445677639464421936672807601214847458557318690284232825195789301193888958671220";
    array_src31[9]="19165542031769424620927992524783616563942350410644459837199416694738944635835328732263892630371590914962686039096758,5653765331114933438306842524193951570119438964974799172883735143940664770491932340175037468164086519014091809755976";
    array_src32[9]="4722132131790371978576483819804023092625665627539446076732049062908260149238391188024429808269711742013789255940969,19376754730301459227229400530880245929570821180184567430652852693211509175671118990878770977634893420385720365006071";
    array_src41[9]="7535982230814116391974571017017558640685870449068918200329892267217954027865738429524401983206170985182883396973361,10987157596222065632633756106742332188224964877309187946720792069992033792622820557633571580649533722032310670019115";
    array_src42[9]="28960272363583766843028924148516297750703995077884765781946936121762500003182642439879558324715389448001239069750561,36054674048005539759534474267177541314131914279501813313888013265565667275029236757536423021928542428785018180644452";
    array_src51[9]="18762498103811710240498861312601391725316839776316402129327999212494903977123319018025540798185312472664656965027175,12893328585827985639238201423152023844253196654151636128361390601717658416440438336808423628648470147590806561886996";
    array_src52[9]="27137901380117572354308355062064176783867306648442902850370105033194373514023071728318662239921792311385140790830611,6624730060662842257796762956386831127497886098288816108531781911280181433040962646976255677195755117577459491397920";
    array_src61[9]="5740276453689069023542490448336666240556876272730150036345093909081862962029145827280677785673773475696958745485795,3822058285290542536871719631815500883566255336529565360875317608066702484202648775774812551559283700689650284848522";
    array_src62[9]="12925426509929281298668433089934491673505638979853644389521954039271933302889282002930611574352949646618670590887918,11073466984390968083379976557208370647490499496074105783002988497844973955721587843669627694619176550247373524189831";
    array_n[9]="39307653432991010301512820343969396224121582348939053969162312597292424874075351883148460281939819921318451263387042";
    array_src11[10]="33406326082598086178215787245845160648526196827238886079075445810667500011975350141112112389220372139232039380728599,10973559594839830104836151306210144792577081691028050584324809424937082914416496768252328448244230851537904596801108";
    array_src12[10]="28492440800383324248734287202073689512448186975090214069134823855607963937895787552756997677963714582397708454585459,16299171799392317055094660633282712724713632352262181433005798358774173030171570795620876262711976898269165102776552";
    array_src21[10]="29769899855394522480473178631690488985841454984735382646624391954566691537431825060447240211283708565214065902636889,347732131044701709637960681029160519791187639442067641974886323522533878755329225286042760227063839965859801842489";
    array_src22[10]="23582215643152172054911085104515838953838275409508059203655301682938686247767256048716059786315503920922668382146724,8502781926742994062047032244119654287859796609154909007558385866676762887432552086056189906959248974399235650486795";
    array_src31[10]="20101325221231981235651472568197378470376122482588760488124113543691760226571944330965198652741228242309922625641621,36040585106012646890177393420057666871952277782082352607458849280285992210810169924683519214720191705986412203242107";
    array_src32[10]="25429141692403803474623790548168795695374783858547335922793152604480000985932622834672857122872659760981840447797328,23245727674919908483844145095581510109438543102694175385020216911997772135141494498918235446697429667901811764432507";
    array_src41[10]="36478956798281945835425707991508574435267182576102278097002189210154373371901501728066067128356398310793721346996800,32556998075135465514640295922657093243627086000104686911951326798836959269137249322560154373943691373098069294175950";
    array_src42[10]="3535868081957745292064523038454819352359148008093907313824140315094088325269854058828135245942504353733966815651523,30635461542379612117381572599346140045205173943804435609016270843939696884812365256651228067511091486568606215046788";
    array_src51[10]="25171818782256032750169351523215702961255744011414053270533538163909327700364017562203664883009939269845011474525288,15978624491333714117407621408763124911271657704881367286915027069665106156078866523560649113515080251536909420726731";
    array_src52[10]="27399821594799654813974940844237040253530627904465733858347769366566983547733441835544880077309751202336973099773182,33597690690900710860265031356263046920968337807955302731008301289353717751460805103738605249608519718723274387105287";
    array_src61[10]="6535410530893983049692255267195825027800240655526925201020012661163591622827816257562673231592395192678099021016454,22771722245167325905282075620700043970486114950191178897606511948379526531994084335587408285522689973393942410864601";
    array_src62[10]="25565286045137641666611454528790944429796189727355122012030480977412620485290620837438274624597773714984026234427936,35474837842642452352004059976495269283291489143279240770164055282007734313848134359354823266281035331939925964570810";
    array_n[10]="11858410041887206912216021417690750780065921162425819753913852246308913325037711239644536770848904281131836602640167";
    array_src11[11]="5942791847699302298495041907834313562807916927134227782121544332037225650624968420305541510485326774975193951649507,38786006013864971192014558644366772572981880954500802122129790801459879798868517712787099396974881201481996165956620";
    array_src12[11]="35157675439514599736370876419409512184013004595353383020502410044082650533905680090856038276695728556355622726778236,2710239190258448989630037447808576458812801935882526182392672617102980058240829579089120003400128173643760416430139";
    array_src21[11]="37509612928753161224655602135663203898813901100352426022628825356117782362179697323790207546652562643485341028464823,33946574433261677226558847353492722460217221252580625021248084527811565660075837253567544579579370074456187240626198";
    array_src22[11]="1444496661992336059137897604586684020922084839521551835830617558074478438923111780232716857403566920669291225810531,27672687310281234632574096614237924834437134677273763640026485380850573223017504770270155484540679099612499221746853";
    array_src31[11]="19138564942795854622842906697854398641987867573325719024684218598059498329297940052693774622756307317775134181049948,13198549198447631728091899444994740967232426785137930066146418280443122086257491342192401469736122575874173805745872";
    array_src32[11]="37916748488426182937437264671038692877611493279959207226596600739526082852831032670307612471777335528349506696170556,38643615105801477404628115488397728561839618331498452237265873724401434486274500281960473623464852153433801110122566";
    array_src41[11]="20111684319307098517801363181250697468140773941906889809999809033277428876437053818156450266343610702133995281681107,14149173553568119234407829152222360231288854862500782753383419033574950982549261866997965416161795915040436157001736";
    array_src42[11]="3521322470296731900929109272170639919915627315483827165006901692366682700618126556107134650606848232376591685735212,10004795599236843357893158467404252773078167347519745197419875993407894641594941073328997272100260696404252157548663";
    array_src51[11]="13456444389037478618816043647043808244095966371991674926768404072762384468775097461185677272286757309690587408506776,23784451895098227000267165030197860643037125509788035054997270953581453849796575445035139958601660192384800495521184";
    array_src52[11]="5524782576116718230614219265665955949624257160594065020494548861800343583761587805584564122865583896099303495798799,31559394981566712778051378280325066840191000550085329515160619033200626196292491193019974890945401976887279264245562";
    array_src61[11]="16914949477773362248197250492302278597554441884897395066338557619036379960517933994834326939593354955003766859885685,9675803950611662439289146611395770074684456414987255338158760117011514861137335794411948144543336984941503022872455";
    array_src62[11]="34388355304074500481410068761844435671756425244844168369021921976695247455368681068732646915771484416502815219412067,29289008581229790730165486018556852794045799198738164238923751215225958856050138810422443509803862685580001511228191";
    array_n[11]="28392771349001035018961277414103855269505989356657177122927593356626299587300905168662702637170889589239893427718991";
    array_src11[12]="37827346412119601811728522842856561939096338513152918755657157156009468953076136822577233785655311838977817460608740,34670061504929548996064922152667002059476576981473726407895306500541678217614561300217971675856115516753524048831636";
    array_src12[12]="35787271063199135403963684450599771181748885759670295954881732447136118819489535768443451925176574452027150265551551,31895488008453579487883914164846830325172049731805687066857738154653598901233403110361358096792779316974884400075059";
    array_src21[12]="31519287327560268917634515343824462549752913153660264014610194274616599411126812156041562861277873132289288186747386,23634122924849069751903321338469847190711012995186922778938894714012850221995037212301641644232406863992289084843302";
    array_src22[12]="7956274507656572027682664713907120884107019169695077171402370743351994584143929204097372812821350703475863052768551,6599954439659216670576710324268905516741335896268677539414049542812562598928780172337518736171118010351467021439196";
    array_src31[12]="8516369600744884438090041402561495922521805016885872380685372686217308260863509622340142775090013885913343827354067,30267808332796801685391591906698096996591056135021927574187849964015476906687849036240034970820799666160204845484297";
    array_src32[12]="9123840114284269685758465523483544502972452146383228124865402145949093964460247989530196658318251983767398257818616,3028049493422927140849098532624574442046174179857069625149075409581153181296206302777495769305086168145468878455479";
    array_src41[12]="28337749762053740806912759126701431048532605679638700711804789408027341084039361771818219009657902100570585806213327,28579579186198723847863170633409397670047771622221325752903826581871792629468375091580049365643602027681089569640257";
    array_src42[12]="1211055256181658933103690163759463922232647835865387202265655375816346633712429549415532551145666870243621447078855,26157842593880759092371046327563146151541720959596867394726558173334179140124631716445332670115284626568476277575638";
    array_src51[12]="38511178768701753514474308019369486833632457786599818461916905540145226904533051634006734498554023406879494932354274,27480807920611095065330217053093817797535430946128209131099348271614707118065516211812322007798984325200110011484347";
    array_src52[12]="30097135029590716675453006265122777823124718487577473789731759911803522783413259603416015807950950275439344337988015,10513128191660934032273595248824115250993800299620624178983508855679688536069957660561481877460723882104624912086733";
    array_src61[12]="36254462680191084114200969305913586085928619853357846950563793685737858498701245110062452849845448922363485829405504,21402208549371324129030412521761914300805055045513462541750029133236698491592100261527177115102433061208734643636688";
    array_src62[12]="17537390959475506386884030067885139278723534998240072818228030335291231973974012191824013165027290666492151262338760,666186692859440984200985036057469418798004328321113339044148648481290114158629178195824452451887757145785818369512";
    array_n[12]="10430162077073761497514498408500887284490955941043904454576587343722739956069239604946500995145501136746106459726630";
    array_src11[13]="17066802043227314283051142293559938337380623206766523433788552750379654988349291773970025880875654460484257254222965,20059485343020620663399291273223701993794850714878388983857241500882460811884938137337522206695517253540580684359412";
    array_src12[13]="14343268146581035749617511989285098030719897518979954101532514620001724562449283110408062374177898567293715018929195,8795079585494011742900138675668463031825752459646059927982572138683616566982104832295409358298242023021960019236434";
    array_src21[13]="25405625812491108794084351686121425876128392046876262420548020282506618400867613686301167794305607369872376516838965,26201944836051128356452698427129834053502121534213393556614343636292496510217916787752008978288009913013732360674472";
    array_src22[13]="11834951516107990688130652011860493576882331795906003058308830122257961157582175560841331903406180178057370682824345,29746204336427490839066793995601157828133727510586898542515879291634168411752432483416734099361544468838673197341827";
    array_src31[13]="33414217144294274033308210247232173550927337062399140252022220071174902790623665890681628944188643251734212395255929,24412816275262631486349934361729607434938973370088249802498558494730834758708168679269763036153312571639437503490933";
    array_src32[13]="27039384038804550075905356687283908545857482799465718891635997512071238435684059865586464963139353845037530376938965,21915150934208038867324397831648361632355420012416773510601883202548015700726864514235598536577573614476236324637409";
    array_src41[13]="39253942503991432646514689809003022103749181274100372400310563149857491919373201386554247235995036805736796673039191,36011035259145655646245712816611290551681370881104668776789051851941398368188419716780156357561329106769038605950673";
    array_src42[13]="25622624319054982615592049509766290742224573170018408153963665717724632971007155833782256773144121238040925730838615,12931034006938597724583305385154665159057706151883313617017853083683429350876671676514717417614489522599020078440480";
    array_src51[13]="32184629988630260601588428881463226532536946986517503379613591888227713731989980268071432253412837416221773983558100,23762454166966607408400360640229337305771260801994658011728102672833651454395197350700126204857324383641764454280516";
    array_src52[13]="38956937760496708821389995093909520144319022868063046288603939684116140034693694797463986267698044950273193007943688,31459112460819689053957733789577663466745166480030002187243272227722749719799973735920182939612081609319676801637779";
    array_src61[13]="38086742692946512144086850947245844271055608427081565404142445861867505099309298862531086182430390448148172996642443,12449496711843687427941820834153424068595491487442553253020200764713368122002508971314851455299334762179118694202378";
    array_src62[13]="5349566343777217072533083754594579624832883416808556540983853097240362191138483849758614974798437355329294230934450,10348925717232233732137046673963520581820131631519404046883746481496968322253397794380681998786211681839808443561848";
    array_n[13]="9798360332197523239150538996087932784511932857931288304041423537012305881815869603664848259539793374244672638987240";
    array_src11[14]="12736531561209335230106546759708263438201387133077000084356243193237232250021942666935049995914874244003160405802492,16458503947968786003622162424386431220106016206053088071708894104014848842118700110706607867980658405832919954536805";
    array_src12[14]="8906983640748843192020354014362263439250890413236427220367966615064856206429862921293892420668639332513327250328569,36101749685543140754350290441861339587635982104764694639204856168101399771151351641587502479653723338117548223058898";
    array_src21[14]="29043351937573695243954960225964090296572703475475794794034976508877064986359080268214434263452148418864287761373161,12525665370127183335039225439124701452629462974042370599454228182678356877903656149452062219920013698698025579773023";
    array_src22[14]="36788383365700481605072965524074992192254638817112150674949369556029340085164867260633988014330870862256920214584779,21334455710803209296628916217582146525431770311417890302645413033534811331873148550923918941329268219270939052555223";
    array_src31[14]="31286369126027663542044231634656443637754421783377248251847039916592485908411226201550738683271310388189385740688594,3578009491109127339561060822069169254557287388058161252160455758918400888532713397677994276397637509888637863703131";
    array_src32[14]="29091190431069819110987934331135443900919930696623620933992946251755146803666803515195059076348979814664231684009052,15935403812407777172005854125435718714770157568963643871256159699500259133366336524352844379977544146913559168736364";
    array_src41[14]="11397951496129215441981127502307851679646723137746745639078952053449195944761790751136893344687974489324539421896154,32982549448824438330414937385183989236155078857785385891972960204373921176550250735960020099368448007262770082619423";
    array_src42[14]="8872610869812608036325584324220990883812298951143884824481712696318919089317636622820198029298885358773097020551221,16859523762316837865809621980106150395926440214113836854289280995889456991281952056668167660130141585486247852961436";
    array_src51[14]="10311180864399920632112323931123104027942755609699060052019723085523748737534525783725784891965565383850917481140104,36436037874579672217497683512031213815008033532339047881344049578094987611693606406825285397318257065055979396838062";
    array_src52[14]="25712879514198557652090225910778944702132849140453548204524617625673699105156672902567570331518041914850382528718841,38076605740865891965632237834883955755837985803303492731270824991520126401733001623444569953406483045177477596608661";
    array_src61[14]="1108134905085851549720575218960267249790798762717974776693294571935757441463220537946262256698257846947224481049029,26673391145849701611834587967464230777441495509390184397466269684048659030611585378443468314321479289839720205071348";
    array_src62[14]="36002523334877041117382649333972150863807174251561831023860785189551551650503137986653945888844127694847106773215042,37336936207881524581094968715794724349480776211675011862980541664601344048923278931827521222250321363230232142875321";
    array_n[14]="31129715039866249947446281601466165692339349162135911901492543004342648189944093426760548322758765479073688770280631";
    array_src11[15]="16737591273421607270189796115042199621320517654897229079972021865594479008056431208311917314798966402939357947910029,18502267970675823419697297714044038835860672378166885522020701915381195428206845544283955765500155174410410504518704";
    array_src12[15]="28375859944667416535818021150861077394140961240501965971841727996824585496774479121547944897089498077591245978073436,18248086222931620385489495562765497054665385798428828736801196376350043348259631045235614659020209170214494187466732";
    array_src21[15]="26198576705393223620757469549820837273380333423121165621693631661997384069441670008987646629023371863040298279976030,6162846895773079389669527140269027447528410872703149006961123428553010982093191181287681116110100776017574205894216";
    array_src22[15]="28588950727949603330832927379669985574058082728730362914525800872095155723571513299759000455306016353730374255316563,27762963091742928187060923854766170449347518960973395763544690189718165843073862258995743500151128779065741021950927";
    array_src31[15]="38667849454907347289393707600269658584372117754923491837786677368968393453701423663273827435478789639982545075963556,22564770216411451513838472476620165096022343155644820080118716499498833845338648163357458505082428078724183313712481";
    array_src32[15]="27922129926080420668508456127801577081995958602748408600510114243114366536681871729420360980874478118236212533756792,12730278800253862311546329144822065682057401549110102265793384662006974226644068152378636420520779803186755179943760";
    array_src41[15]="21003481011861068948236089861122921906431344920292652120173118811724101360679360339068965236462875741573168770289363,33188005642732152860184691558830782213031266065910668649698558433530708411390317071426759643495800775127459826548604";
    array_src42[15]="10574604022220202007182446192139022739612385345236743236094630306448004430018082273964356741604679083110104191866525,2176653519557553615147052983999070139917605763810329616295319212580690279870505873996069564399575969968153881709572";
    array_src51[15]="38583684701750881830682049217379706485742365457676775564064680181570885560447856936304963259585105490091432663455078,36866861524462114393199476132259655328830432026708614689325589147581004997524943486030188843086814220919764377989037";
    array_src52[15]="29425145913649377529672926817469626087269454728978917637126011326422033038751920634928040370764547178187433544217582,18849222370284067561832549412203547486580228400975197004482790657770552914048086037461370048419694418412350668944965";
    array_src61[15]="27664838695944905191085446753554787694202135548158056592452730557215921503245743679005348535765226772926724391513254,33295801985136977312145032472977872359598993417754778412361218040955056100380894972163726560867227981974621691570178";
    array_src62[15]="31293378625160171499881579724998278220300758644337762831826814709006050984191061958757390683809487894768581489964001,6337273673027069094592169616795196528550988613273934307589629132554186995836353942472272087846851351616666636705166";
    array_n[15]="22196328417589981013425333858820262037487309146926157292913734606329402060864293374606904123683040442895777200356260";
    array_src11[16]="30170795753152051190220986455999134591787439972625222569333143863855403706950747111099493550385936643508610989550988,17941421468502679839719378275847372818714656082240413577287065716675933225790813852117244571970200188088888925296892";
    array_src12[16]="36860723075284030902844698838782133591759580873664615494819324980267333913903030168529796573406862791496118787235932,11514528134386516666154394070330409308928472422070338696964087835163559426750759526650418004796942197922540100859568";
    array_src21[16]="17477684733741975986215538872389990059525898257938863738383937201910916266814330213558896918889768479879305229385314,22640368028200033270091007993301691286034456024981143824221999259468341729213100385054482765092269832864447163745504";
    array_src22[16]="877163774804857578795589402041915875293951147001744732293944646246987279502209456362220026084241474121983485625053,25794697257521722220739523539012831603727708166533628153626708135276266848750962754763261030032194740224604312780427";
    array_src31[16]="31106653525327488564089379971145626779248928184761823228206103014356967607874779220357397422703778605870485928084016,35508184177895375850362924490232658110452454004991949312079164650026985347601130038944784289032019043570195202252403";
    array_src32[16]="5413556107404825507481976047085195902541075400416907400565894414871175993722461495009911765597787721461330166595526,24548969255156391133233927793820756814669459728885905667192496075373655923231749726448188641900624012154369679134238";
    array_src41[16]="16549649629506120347763553243710026118633799867362271819743709138109583363430159469252765065555004685301666952647804,1203920755670706425890949188476396375764981691929610606817679263019273085974771770712606326960410318608069665391895";
    array_src42[16]="19410146408170629629540454947051020701446326266141468282447860360504654484319667918437995903019955957523534556613413,24453711489691997606742846974301537748539776844640684310880756738339222267627288499909406342370207685973482555052879";
    array_src51[16]="32811793728481305477953772109867767507621072726100907288900406445644627803774278440912898217287809994046584491985559,11580579445389386907266551675995847245465324824124112264885750079594931790333886736553882887300306011583098869052047";
    array_src52[16]="9951849741819172784036652988515646256950694609225918011904186918070538615957392110894599617260759679409327526657997,14999561892498057912909049073967888310861407336130589752776130517536405154173185323392961921894169711269320597661599";
    array_src61[16]="16514499053119337513389362092237247394032919498854644438122487833554547012913100627736883419095441910686904724352891,27808167260985442568663238437371338118212698232903079845779834992945898517446528898461253768430588921270362483659771";
    array_src62[16]="13504307833107810185881557086787980056441225323631535309391059124555134443784111443626311767134628880051627579969076,30670356066623879283404219105719326382504446290215022133580064099021699901866213315877090609861863578729073185098433";
    array_n[16]="36173916017905921157653348684706043074926289447417306860495598912743262945508065203476987337729699588675906772779275";
    array_src11[17]="27837195667631707430859737803229189432926244598361416532520619121886313521819162503566570124746203145625387831353815,24169208720056739559370423838692061568304942971677061923487658897579607379075213129046862367975213205457214609691932";
    array_src12[17]="15587426813399749217740859373883745092888316124337348618433711866468980488088985926516219669524070877173620251402349,951191144791561480955109164949434951229704869890359267981910663800105205267507435150876475211608170825587389066378";
    array_src21[17]="13078127523982663772566406766057202943681796476126598725621276109543738722049500042746093343772555403766002927606640,1864432512907714979365532183004000861032436258179016494735036393096236472299965363509125228628153121280330562170843";
    array_src22[17]="4467411670632769775072543756091497527738335136912561997235390744211429027492648693227643748543558842143735213221133,25713490251967600625773920594736492216695428527221371552290590970654531968926585843537242262560223210141149098390395";
    array_src31[17]="14748222449945488504697081604372957012687995091432228858859213715340974270098238896606181592345781952526729398132343,33869128059382966483265318529127636264025417630855970703298024805853410581600617903014041640404443246232807942083200";
    array_src32[17]="12169252721246957190408955266648061698576036115461342549741513482916265749500547349581719351953453729566621273728258,32967848066268744894791317481101764551351949398744807098499894755661446597683043833222969635483208723375155338350608";
    array_src41[17]="4521688849946755438742234745891622741701777296617451266903380345463355255403217532635946619072208879509158133836763,7696566629418544904234284471604273757541478416482225698744275974164912485258767523954587072794891521705251264086447";
    array_src42[17]="39213533550015039865457299488878346057395789188479514864054793807334264790060810985916308969735596720458923546754522,23787972407447966142639727126203496852327620569030409558636455138826569593214343992219925684363762350716036558638587";
    array_src51[17]="10924376248044763286933097001297594124507323373732584508373888412732030288448235705487034790361167508222834938358943,573898546133545186633713867082445696728816773661145858240474278255965153754429130413774023886421505607728961857079";
    array_src52[17]="13881454307140154887881969610552277207439477176830445637628820045426592494183465375385229018644530554443836651425126,28841118936206782042900273756899263039731358999030428081361738519083159403151222248059498210362936204541936048619178";
    array_src61[17]="5459946257024729987737963646322331688458344770979530951540052269664898624730166465043917757725936446878441457360936,24876622946000560207529569301855522375491853505687439437497784617834193782689741795218018308333031349884045881992915";
    array_src62[17]="35328020891780203786701948971449654951573364837081333460331786182957167638556979458789641132099666436568554755613788,32557861970559281904127118300342927036169638423789352164254418105107621161104886979782187998933253437226151433229806";
    array_n[17]="4485177929182324777530015028746524807572689206977849520557070931564335261537256690390206881842151987749103709378188";
    array_src11[18]="7630136725911378792038429488275029805531428299583408334967540866704042834035207121187883132599155116472624279676315,21466277209673331684081747592471299205045203139353838598892495951435921375330898481901505700870088204614544273636502";
    array_src12[18]="30818761899677344639649201154998749855059983726266388003803592245384553075719099875788361445048490653133672238722945,36156687388907210773591214538515290777884848260838576867196979547682796589452584677359869930438184323031272556896637";
    array_src21[18]="35111846549039392722684683767011515148228388055418624368975028054539646332687057984452104666068223178068964212433687,37422855959290454118613689200208211114425424108862160834381587912678975393343556443305079373009989561619328827011316";
    array_src22[18]="27119653020546877317225061111198166001851534191809310588859322073093742943094221435545178146370551213806333195350967,17583001644308570603871922378378660625323964777667670356972362256392832890569482276692922880075988551913117926604527";
    array_src31[18]="12219653660163169869242555483175711698763816630361021765259687522022358521995779826004883741829850488855271370631107,38874442464540115292133586287645052661613834512299531916923429925294024885289563823368611525444022424832603258398495";
    array_src32[18]="9171990300507991646795112610702220165582724306304704766871421060955586900892576987082962331880103338572494176443169,36794078996867494482819756621998604207757035041929211427413908046264212194713328954711645394751887560394373332966937";
    array_src41[18]="14264909045349922486425481055558032215093603169289028649208946919697881087360035987692840295836059042297295453540911,32993510635450472133017725546564798783254624982440580428668911178188218410823455222877872342259056993144622018039612";
    array_src42[18]="33398625231968370056083375546007030340478895980781363065741762743084186908328309913607822223342972772522222243003555,21445574197494196469208952220275579232319019565777042646572313559082491354688609878083166356527756094927249777052756";
    array_src51[18]="9454841352986138000599844087442079402882773887676566835896009297232526287780246608260497284953666298104769511882608,3385877493929670968070417892540561549001797076211939792430914195093826500546415602442640469701703587784556585285593";
    array_src52[18]="38444805389877990371892109675315241425728617939230149335440909693216225235488339839733595161665607340950326417876845,9192627883191252303064400524379983109073538360390208238732901135439910868340996337497342865567023750728739570774226";
    array_src61[18]="2145031022973924693438783259486390085901571033846502314099786582561307022308310803121077560104159121126012591231006,30289948844399666361733162993473886673709571963637677231094663815660407801898305988348563909411887599609021549104570";
    array_src62[18]="13115583068057681614022203192943974198413445089580675405814572969812192885786357380141778859545444050848656253401951,26669555538291108771497381160059317599556957844484259650777054175635870226168244413971288741862477432449116026709649";
    array_n[18]="13280046394288740043543768879309462237468318723053175216660702744144124245189560854328379363392793499296868641848415";
    array_src11[19]="32349413481367941201362930208741853575262950314701482662253815572897918292097857069570943807040600925200853158288748,22653854203211052028717467096208890595115122586524295039411519904494192050367096257965913716820518358977595623134376";
    array_src12[19]="12590077662902068248154366358021341821619298690615171779441969574038281919827768635782261032992968311510798107620443,20966981071371668501551030090962824698038239375023670469803910906429713394863991569237818593325069935566455669255312";
    array_src21[19]="21930091088355837708773633918355961687178935345498542515039658561970038319706393305334730448698338783030070109278855,18662967340290314874952241147596911858817504077916483339441858444320620281935601604436684896478332030209804414344838";
    array_src22[19]="19138007183350382795576674277291408443637969942228572838403092644071745609042581592199985846192409502454610842174864,17809520194585912502962547870487762416900605921945433144848962193441262710083507925166256595084614563470509432399668";
    array_src31[19]="20973123798303528413565673531260174921687131493862252886089247963089503938994707430041704129800417530629588362734149,26551059969935167496397426299570106383306548187189759897581921064311126957489372032073514893407133507384455506604613";
    array_src32[19]="3599552319579417713228013565149162778448624137370482307314392618360836772033951274199027451536398709265845486177706,25974380763645520705861617949841362869852519858520011230767481320496012339003109716501490631905524168437789478091349";
    array_src41[19]="32183608913735433552883507704095060459982615485123409404448694217132333683957196828393264701072986071689430579361809,13002132149918722613765738577180386959789920061899160923324357405938197839796374936150515519790185314709000278640932";
    array_src42[19]="28171072175754344511723302521928256618193571403220745221390278357116749603041628559663321065211939408989174319909276,11881593911470564283104921805538023389205545405022145729509968564282599540573637129236715980953472850169933511609900";
    array_src51[19]="8423546439114074174005472620048456436281241027194833187081785360300554707081846617127143688338299757821092997779338,9664489513873486284357518101756001656740446905214094979107864433762523823642819248108507860086653225113536043984079";
    array_src52[19]="7899291565586099683494696920349018752322726679547627026614603773103556087410722074233711886741173464223333785752736,5544239001761174255458021345183563287723664622750586892929181825249071682434675649615116404084353222950881618007986";
    array_src61[19]="6048353199305721833972621709747144013454324498701839448922642722657311001570850687438569553113138192845184288084672,37901095881580765491953641988473331174521179982179915482839636861559509654663033021013903676162518651698491327118750";
    array_src62[19]="30973602977751479940316994275777152203695416841417459132634031311460289468948631513794393531240118336364132953735935,6755695907931098911110915669320222351858058351380557158292504962970742968372762916015261266579290943442110194612549";
    array_n[19]="14517046442462623057776605815157854809183879165605606318895803686677048230386638546712602865020293964945848963484324";
    array_src11[20]="20502437996553359939535100526292767988466345186641418279374444364316466529531619654956747969078303907432615974237648,22797337459483072693236177541552566645888255450764212243636356357309529855297462511199102971517174512839050722669379";
    array_src12[20]="13785027035415717205803244211117720506786750364540362856712330960700323245164603743656471045225002647149689621657918,27814506337509393549540115626460734639353108121081222784179662655256104922597940543552371766187497825396155270111724";
    array_src21[20]="8952903504721663233637578887534288099650894325988509011395485128016773769284586701352599251675697765497730737563351,19477169083404050999883336391832741440467581135290864645490705042701871302095669264776253734120918047996911586608821";
    array_src22[20]="2054812975102267032272462642172433534496882070568351962202803033418968643519141102012922338173309240309506566631285,34970251949853183515245082354383799738830092274271836024634565672535820057878782010597326022993491571886667791162571";
    array_src31[20]="17831924291745556166157520466169573811326267704624267389959352822761105186693931478136768330832743189315708398331786,22867450070827557585440926130252882373930928700339831359978087021000688985956593651082731977556320979010064158855823";
    array_src32[20]="4466905218124517507921780381979190122941975883373133416249595495771966893019189102804768637809598549599452713289623,2051494280959182096362591528642585265927886353222333126171060867176108538467651514600909398336709057537644310952616";
    array_src41[20]="24962541392633034223489346972748023482175932992432815577066209147102552515885669880552449898473868070735768725618248,18012787242338077062923773499315006078117698723977455004460800991791673142767363415770647174924234594760905472003293";
    array_src42[20]="32562091574112635181840742637315634292248630125334528564568480428981746965215444775428741863805139539273670826752148,4622747095788132658493126245921092286583452550202755278190731370870180615254910507058398775121162957186531463793314";
    array_src51[20]="23478241190451782711608172055405530753631155017859334429205119854164758103399160008954634929417181114466854534875521,30249001932501304948373723871819466039464267241009918826129840456545073957365652544658081810973544338730961353998531";
    array_src52[20]="28781285082247394164900489271433487049886166978273908212198278890211492223933732597555162883759384830137496411698991,14427496430323487618585180798555555972622298079002832876926598475537678913698416407194642142669742510053825690364263";
    array_src61[20]="3877834465694935484009307511803999029500769525078072991714990617522520216732696915791551340430188873967568112138859,19055029231232333650348265545707559523477981730117158942927530213232226191328109066988446238080072023417486548367881";
    array_src62[20]="34730952196151274120209650836957328397794545555623808857043912097941632579970875521232486612289867405836236603883264,14936141754263051779825555493010926555139068950440076090337514455968291459693296082495924479896208137278574112238659";
    array_n[20]="5246595735722872565408766183020408230470109499456973887285733491647012528629901612438087237231825965646291497639776";
    array_src11[21]="32551965175715643352516613597965527398818884144371360608794005382001690102365316125987984075451985189040235523323839,24091297717350644008293162194165965771455556013537575787770460219073263725237130515078643197039010036992455211430349";
    array_src12[21]="17664789057014708719020150093960448487163781191061399009639374658687552994500533721497754176920625185787404776144611,38846007991991208548196168159188218891885814422909822904307631824747377951458444722393055726179214555067953291754966";
    array_src21[21]="33440822105331923500894249779303502599873799809260694044741736697468042800203859032442096861051876202345014748674380,24285211544661241980689875197184607394859510897316324560797821775505882477503443820039533321014421023370903093178775";
    array_src22[21]="30102630530964840251006515359243911906768356494797896028697371948695286594246614688184952904571200447209068389745807,32790663412931904381844530985861179029188310935731439211488129122733363854108895103844799228067785321551420701872210";
    array_src31[21]="11314080227701414303475265391166822958854880403265259310445314955759101346057887086485203580396003260874921426487604,31096415891091706169673511659512488360285322666624792437737261605166096299855851840271693780165712495123046749390672";
    array_src32[21]="15645532160487526351201598085578222920940456949762917668598354343657063222511391947202515751235156218861585136880848,15719376180568529423779361214469385894682741471079172259258305332930396464918589235111923642887687233876184965272001";
    array_src41[21]="7133898451593800404775657224211896705698569641206998799627328798549370098512405604635610891263676185585506826972290,16615643558479738158938805051725603543110773684631093111466946084137492944264155880500862914697375299828101604126190";
    array_src42[21]="35969978694941742712158734732263763558748320577341846057121873440876836092332092338120182143043923626917733973792887,21443533943527660242559070260947680762995980353166720306793512576833543020632551160624178612955657521790517665175581";
    array_src51[21]="16214185198239627562682715788388517315676851951127576096272799451140739743224890449020221006759847897598112483035525,4910200556291679564743545228703508958793948784843429018953690223388611634997970560479674123205317309763177836855234";
    array_src52[21]="14497933432744053107208608478634774928124497525607905323566380166747687982539744751766041853147422674059041404912765,5264642441201478589304337657981632657493101603318057530145950210795036242406599061181251741209495323262540073352044";
    array_src61[21]="4041514257989974222383260223747227362432630260160513542521359304194355267453100931930887270983323095447532242390160,17092459793717172754224439770753801530299109358658188521032708031362379798426216901773340599931618162955068724938248";
    array_src62[21]="25484858168463075837931129466319074425212785281476758942310539155337350414271814374044681385213501168151234201846401,37397279714967750201851256120849202474002558938955503924992094256156651145239934899154646103714960196534169259911189";
    array_n[21]="18659753123239563783488599388565243707999099313960157231047652535789952640741970684335750766795180804154704209323265";
    array_src11[22]="11553757808231671520771131679852809329933187144584314624278944589723953102446413805973327687372867430479037944640445,28908946320934718637526552603261030983117454277583505191962881620344012110516304918496990183517785634916854209295980";
    array_src12[22]="3334228296350343914971368668308515444174064621186785385983805496805705231166295005102111826056198407394946989725846,29881269373751853782853511825454320365415870013217094117190359488260583713537727413428643624504162417083914374716476";
    array_src21[22]="11006668835905930824825685037819602693267288964920854976605412835450692069642413994023933444148954050840233323367087,7897706760475094899242088238593014828038732421319408559119369864780133267922314482872536634001069629414212233308889";
    array_src22[22]="27252851070161615871899915992796467964643856672914691692548054559277306425246151826062507852068604175774857249970722,24060525982462250049516754543817199393153980043119136317853299367205603961011668354283039248951876412183964498349640";
    array_src31[22]="6825619452544392174040009855287482346733689776952555767555210563238281643990651227251844897888952115213040988528784,26822179709737112925496177622302685902417025854510696058018104831860340059548556473792377979895768712330895407447918";
    array_src32[22]="19183875106470927113632591670149108809402652652780506488032344157878372593719311711490571144785329535442371011164492,20920555552939197346497856882951782041211295381117820518992344490976134513049461410050016125276429638308332806808193";
    array_src41[22]="4729826856095960600978081805494155749391966271803117943038590820693224773681669095643089116374456228325896595235024,4405531728415137796022385566633287770312851021797062133958130727413167447079964335399558433881405440550296479509101";
    array_src42[22]="30976750561580002387704124800398424481031557230664716273397016663824948801311738106135047329139886771383452115743497,37565151108920031630717201180718783612590143907918720886926343772077313385343738325505386926185937966618101057836282";
    array_src51[22]="27284670785323608346274743171685888039373171232182817801038137390977294804114399270950840976024425620809953192888139,38436907023864141238013798006195440723011168579791770742171042773362951443372909528474557081716132666393539617996737";
    array_src52[22]="31116987662566525552382240265923268865363930665991381244560971128300542134129478377562329688244430651374926293686726,39389358670311004187591155358199205795565222511592953627230494953184275733524866859645377034033698124691689661374801";
    array_src61[22]="32025385224643843520665291539476228682841972682904129768020007340749010101325299038357134397208230383682294664895451,24024341844902781748213070508428813942954174695572545768432574788331929444461007453916275965777777953379906257365645";
    array_src62[22]="1855035534664767665285978089456114139129203117530665956918758886125547214190561115008783280324858203613370133445104,37106231455364799513129978058590373337023794456046075466162700637113005073147050943466738230998716701651756788638213";
    array_n[22]="8764248830221713843661788447649893337366094851444406981459389095732478589877734323388755785353651494416351861902957";
    array_src11[23]="9070238632011297938970980264901907019958310346355860952400012485116627514967072385071661637656547449248004210461847,7541931254525188615896327617427392438057109475946170594944614713461183552018805017083275994214647972338214949315990";
    array_src12[23]="12916184951284397783811927309449178348022125321712605811065761612695960040972773809503432126894141946398547359317628,19667066479567662053284725677654593838952488299686949094238945946426731294931282502293114860618321876974603410109184";
    array_src21[23]="37676479089075515703496010039105130536687310166443039937123477603268006088701933851537883222626542598492817264096328,3415995011932288850192653562643217929388353823856305259123533971024419544589234587227973207962707448976385562494731";
    array_src22[23]="34627425763954281763004502522777657164085869757427818761700376658915699208431591949478041758043682954409034386102371,22300234833993305123750666826526098815918974159716812875454965105755715181297397911535559180735316772581235786065934";
    array_src31[23]="33169411919042769099237749101849751996957475743173120001887701902455837440359716990637363701280188100268307784352630,4879013592913875088402735910395611742077962145456510567058314652296044781562101445020655477359498854550957905226681";
    array_src32[23]="21774784605573968382648513277130106728336623537582023338041331159449765368336811720663235783845584012216893809111641,28897819907184230688249889354908632033813503237296507389900582254213227523384638587769639559873290577262815377862903";
    array_src41[23]="4891413028434187419283775778592543472254380886203504246875472088692994974350152980709961703890683603957770655008927,37919311625935275508178512138424627976307150764315703779414624747804362229487793239063471991798520314018233039831248";
    array_src42[23]="25706697013662483084439802214690710419390806394949550160910078505066847844427624160049005545148133130783770199376606,72770456840894967455369478966744897094347622546729190446432366417000927999528174553983291479237341664323955319107";
    array_src51[23]="20933856304664966566619883280805212287786133217730493452308003141282815917681986031374114853828704682261748373426536,14983947722548766243154769118194242319813142282015352360809740427221095087480028220702085554575663241213671951378926";
    array_src52[23]="7346665979568531201088276369694070644709402037778877286862329659081431284025183452011313388569489273245036826087526,28852357710478838445461170812380233070181706180046954664008214543857460140310128550744655575273508189868785973619918";
    array_src61[23]="22688655211252653917471708806458577618224286980590823524309082512516443886001918346363127942470687808127795538958832,33366528697504375641930436239466843803192649308717186104367952338302478313398467240906063200326481429900543491302857";
    array_src62[23]="34450989632522556809854603681709527924952889466022357727527823227058534032800634923264639064481070656870687605498041,23925875098584150720978005884529861554994524410100990406915120303476379269723362177606067328618281068333159492655734";
    array_n[23]="3897094756779105589195554261733447418617468482036534696867224625387993919611575385468285180394343367383035282366136";
    array_src11[24]="15868600282140295612058128426639629698149237497064657520938492339165825470220355377310729050564664738858719867470226,13213673768019288966287503507645406543331243393578633203036487374421822166841210529543641124077124351337610153462586";
    array_src12[24]="33423153738047472317447655054161586880894629036674505908070545445786105443394228243830243111217657494340159607167711,15189455802109374908298330865835198601163465953905419072726015877853415513630407694166707589004179082525150068756845";
    array_src21[24]="22048357823759723757856850942372465708913497159877681302153752973402818162754375982703582086280010659686521162353587,21816015077444967347437966986912772954974653373642523882062238346713581117571771450853013122198906984201560400343351";
    array_src22[24]="31843798618121463856892846001655788261114050489393555349652354580781816217890451670639941382611665349508886751231920,6499792307361398499902117279133592873771081369117232824517441138579937985358853158164284443226849494461327590456526";
    array_src31[24]="35769728420886625617161941368894375445079596271939953219874445690792925348925832516794385848754333478999349621994789,33876454783946351933808380640577084241228187852904125688511607020335878183368441601416431000770462776125160914533151";
    array_src32[24]="13297315320693393077612222633227905186064272716854761449020091088238804285770742880054155992161911493609442611668755,20097915937363554412075823482349883646691038469333014740619957236510637067898156589447856523335022249062896029624045";
    array_src41[24]="21931906980612012655364853386650603745695318297384959334637375355264554324247615592750090468501871044922747115451588,26339046593372349661081115186136679881746601885672391564006424960148851153386163039774835563584080206931087077207611";
    array_src42[24]="35501437256911989980911171080962464344061104989980212627606344881457107619387278881753773835928168790136538295718157,27373862467902490017923492312378601296416444565069672392734388403838639264226371711917257514801241992880116056538868";
    array_src51[24]="27016284419725436130480785092712612040092840422936022210343568746175754828573332067943145875197846270660403860860137,5711174531470871433271171748897922302046186248491025067555331738813525864855696040093483208366667845614260592997338";
    array_src52[24]="19906024087744971058060047800214915368393778445114597071111184299917709710937897235047336026226470085445024205466415,31853247483924907394476361592232875252706973874711003984285623855255035110341893302852231479377218640787016603913135";
    array_src61[24]="17207216511068844161968825853667627220402910885180851660228024813293991844627320139020723863639018537789260362664900,37735976194295660535628150752901717228225155546692494748810426057165390375123420026740433951853697572830363355922685";
    array_src62[24]="32719178581455406383078966709597045611667593601039777297802182331329119930141329119793351537950662404852014996702765,35803419442490035056232760111059179328244801071757538228063324998580830261358088016782093560742518712675285201078678";
    array_n[24]="13845002051299646852580118787690908999156514474019513350204271680877636942874877737387942684865324513172256170799099";
    array_src11[25]="19526802317582527450033987925232136581172297715380322555180823542690467258319042103655886631345220156176136331498244,29478554830608988109573509217937856228129092217727465439871610744293523344390705452353037055295899491571978952864741";
    array_src12[25]="19026816226898428120753424272522394701424056463786996645992878779910942918479070135601500833404370240746334426874347,19988759877432667116920320957001056556561067174621528774776657315814236022383272194006667682610376970863777288174208";
    array_src21[25]="35501218830830251255868361080053511420909881833770960372738871114154886919576318288412587681060363451171410035380207,36744753981302295490337368187472977182627935505801998986188357573789264189023246866242531254314725094303771643699128";
    array_src22[25]="9960885125008705085414401598716513697409803705946764229292777592010750929974392526497183923248147872901731688331426,31051769362414525734887489855638577149280745984312200473419455837440665044016130171840661111400481013663863776392054";
    array_src31[25]="21195382642111443114588393600731227584830889498933773624380888671463389708186045386255505853607436557333616487478075,6406299507003799707013019387957289929762971184795332874497834810807148911476618568954021750000109577079906834970767";
    array_src32[25]="19070849287199745701223274367433004061375070765614059263767718529936769977678242884767008533230298992550622733759591,17999662782889706195570560779080120887170584488514834571665552538498573547236111659820653319617594162370107274883256";
    array_src41[25]="38536745405408997935635567059892875992901861601419325308363358489310160132516410106815005468814961577408664766883151,19176187272821766215442187691074223696143132974530910724171948633709129145612604001099957994092150248374829174936563";
    array_src42[25]="37402130357458185853060059256958675526921125026518978718044930382011587737002313074094505537154135396244608222439926,12519235287710392931245062029331706813195184251054677846211219210701884395384622282712289343546109586703705240624711";
    array_src51[25]="1416567227163571540039464600515152024496396403450360783525191542233041504915484334951373081074343668349363468306309,5741548060694914835190516202241443618455351729389420988466916453388088109306901203061610537632962472957541206962789";
    array_src52[25]="32961098843792568100068298421208952046649477312233692352275994801886751672049123754952264646680249085764648629462068,23492387922878866268550786293083350971738898841583296526189936385004663078842654912809743138753547536573867607177875";
    array_src61[25]="2634702672631763108642292000860223043395002298842189876156192501266965801220746575442653902897141493868771015246903,590975403587706318431095182887469271888493887174020624488024121779029032199380644769336376776106707108906655989919";
    array_src62[25]="2219867989200830876233490563907913668342841524480112433697164490783301480381584678002172279588534013480276241519488,16211802434283974455407037000304238943748356400403372790496253150490408353539247935430368135044372649911238124330721";
    array_n[25]="8427878676581644357776289890265794207504524679171930914690348089724673290546781525001615481047437972810760437317588";
    array_src11[26]="11154388683454177888246610945377743175773127467922544271484703374983803013656976736263141321523026681490923717438046,15995090001001260298374201487854406132182886457683677144917628591182780322476647334372725600310757428418177251977556";
    array_src12[26]="28990389549672151094462770673020678474054222426343121246421798361617608330933453483616566952149130293988300811107438,38170196766027775118125722901302763042094791971567524929763893436122003865440423412861104738600743559958602901523053";
    array_src21[26]="6471341045079601987644326432997687321351435580467022707063462063034967149839792591157895851387151900010791175754550,39309898328116020951945418931516001699463444563294230926838544426258202249016319376733489562057108389642724070006344";
    array_src22[26]="7249124176577800915839450573365129589457353963987245474166335803428515413817907056637512381109297686598027190110728,6403321856757600567759638066091667597931977359991591464627945985925418376457540921347850948017432630617432786694713";
    array_src31[26]="2638346254787157126285626523393517010320125212708340785285778344991942697172854046210029018257336438814320904322819,15537427054167895020533071491628874977140297229359704374466062098987226817029907362141723476727756542278324406730721";
    array_src32[26]="2520723420434921093879959998451699632132373196270421086576981292975902248447920971128757118843359223905015016077046,23290080144014656306288855506629232404387090995346167731794442910196926957315168584102854849702257997123648860637814";
    array_src41[26]="21248223495910034013386023217000749053393244830572756358030966221137337010952544860104212535995990156300837924370708,4976613732082123008566846608922562483614855266936891538702383414025291603487459906474469399982959520278766433909179";
    array_src42[26]="7106208140566572885100088469734430736540152928209450180789654475851685330325010759148290695326705018527138898543528,3110352869457608974860271039122857390596801576680156731757873302363444660703758604614906155622534777152300997688401";
    array_src51[26]="16204084217339216075266011266305329664463867188127959126348289211414530744497011579020999967611311308588124119323276,23254317217192109083716717498507529068330679038672306718513997032983413713735999725444905691450013246506127691554847";
    array_src52[26]="384892909079462676335576532212276042145838567691659791456478180740954654808049939988513506031261165122907424204103,2633385072570597409075301861756771254846339835277834665910037160694735558989323243701017596617318550963594838208620";
    array_src61[26]="19678235955730486351765752181801220604246091039566939211889937914843975357045068559119407990010358405282330266230084,14979867414981393652154357784754601038830634831591057697023885523253570736514479629059730819940623815055640750444101";
    array_src62[26]="10673490693947695151680771282706703767041210078358949725878011023349947880656447106744147617446333709774666785942176,15752252509974391793113889602564671852076303907819254361472898147433999585128271499807988358533419525963982107652413";
    array_n[26]="5383773869091688464524403794105468481246560238978160411806755139491988405111740608103454991562442011582102583627455";
    array_src11[27]="3457332153905541658787540344173652844139921785125371176523320970284880972281268604106559952369147800989566988815493,2995685504337191143706855308669637316859247619067465700771387300614903442478012870102537813427706298748430446759876";
    array_src12[27]="35436753877320258789542121486347128703024795976247217501020523102074996302183989806956760913409061012440861383920899,38659720155862170179700204400374576651177755845419526278634769675045123818972476885655133543866324358304076281121589";
    array_src21[27]="14942163584854682585991231144779624878421191464484245885271120600396608546478751623608378091845929552684203807543670,8233867290984686239958751130463259691030713365986028249420303641548338679032501450654873874102278103881120082270422";
    array_src22[27]="19200852250200298466366351171045549680052193808636073085055504829839483796473804837715864090096562485804170844288436,1002093833877810263980210722217785170162110202121626646674452648282145826254323470558762742068896397670631483934315";
    array_src31[27]="38939205298743411022288570157574103143786491615270185658449136150253403139195651721194049548463652990884196937031756,32087921062550076298750615375323465700052576249684025010567171881971747276508025816127597353620146401424413800898523";
    array_src32[27]="38053030159792811973015827685702332705581565351445667393511602804506675703741374797079877581473084296577541904634051,9520970355163581093979566232272961464383750698383415696146258700084898644069125831715274613410252424863934985429341";
    array_src41[27]="25045168401475928283573279146614175840189054497218096090884622845460639958250535896910979546248978535665135492348686,25915897802903592478627681450310596755856715456916129092346947680990831288242930516032817136667725678110048203826653";
    array_src42[27]="11479244718491099965923727938406459949533180141798721927997943881093754513303977917865369029033034871510757320069731,12277733173951645272537662555094572497952579081618275619860747053869315507695484870583116702295735016967232786450813";
    array_src51[27]="10369404406650529770547607080867372326514468658590715405006496551171728312753936378791341485107747172360273607072038,6694545832164938033024942987826411189415304122436622667666266900904620689874621717352108917017724955131302449021511";
    array_src52[27]="32033940286109137749350657162380971660238527074028546976636404993959232723515008280910122864634383466280622312471327,13885222767928394713188154679220081239762984127700556247688032444108273573456270862874748960873672683935184802172711";
    array_src61[27]="28221744491271454562805177070912038966711020492582882871423441271302594364642220212197520127032357747817717636921873,9464341395678158039242132978675774019533227163216520824489421948320069631221862764003745940675276586844312005187734";
    array_src62[27]="7311212964828608757956419123259325475375725187416349383367418116738956257735127292061724783376814889990112583319820,34088974080558989759542437021112497979466133114309907033027637076586102318702953513382242544204166623861433366124511";
    array_n[27]="19849249555347737784356325717034717859715961072309565978604396512697472910576988286534519494077670223042079161613961";
    array_src11[28]="19471808383828739164154650227687919960624598278405101096961531733677719444847830686864209870434924722829489500104675,28593272641223285729280849892487959993524320462020063485677669013716415989204557993685234352920191616592789675301807";
    array_src12[28]="1045190204604261187221628413527150608559151395731430342529669652947914386292190630312974352186873828338466416397146,4774258340099020447786287054108702765503283542193013410423741334483700170497651078451986859203192993487533470219605";
    array_src21[28]="23482705196023475331860529111337219329831811854171454842135383869573361357510062956067077402457899950679902643174359,3203098808578820827395982581143490404146688080325640808220932485755262381215249203175951737436893272134703449479044";
    array_src22[28]="10980681804610261917978341764012121661292975305247702691834139756387478104473484307468433534649373070649095902159576,8189777957007883573417358989993926876232046688843879229228115189168845104745196139209599974697984571275706187014346";
    array_src31[28]="85590852094068181243316178075668806038686533717353354098526209404494914984803896527336306122477325466955425612409,26383014503234997702008222035976681636185014631053985933149734494809129827774552510536506263585422620692299126404974";
    array_src32[28]="16788274778053168195985407279346957779796990774972968879394239380881791197114414061541866349796124268921111856060692,14048919253533351914663006039973829798997828516983313289229380080347115760995940816132961897549593174992816892275244";
    array_src41[28]="19893121981294908819515861126482697662871426957513475103592319114843608567997472624021546541489099131402702626109146,15003078805974453983148503321041815542708216615506484000360067246433376213245021687609854182662912259927107697912377";
    array_src42[28]="22306448436169492011278635629466250223212320767148525185394101760463093060688959386019675481071957884351621288531324,26477452528960427746390561805420993245963340340994988520680436611689768033097211462484940852567422139387774620603257";
    array_src51[28]="22452171834706027465101747976362799204552272249103483732799299853084377697473997566905244275027733391404411202147494,13446426072644247560714677259368130002897001167292449225681496954772672917798961214257513640565401028408641640804103";
    array_src52[28]="22231006557820454264209794023177986834749099082862853780706601262483228731372240383053361442387833929607471269735654,25237026138456231735478330359627347049351858666748012919305427539632852998685624283070716397170729889575023617067523";
    array_src61[28]="16038661076039584793827181037618577306425940998298031190479121373576798391697189656924593982302643046113702440655371,6117417019937387620640972213208603658973759156176189515307981843754197512780198754796211948461645727728607732267339";
    array_src62[28]="5954783951202716790908606063522462734110432470391270505219978604666710408158146620961678225730351092071686543257146,3135905293693775858425835575816582533874907905183335019372751660202471827337114248999149715759176323384137130833748";
    array_n[28]="11837453482012328973728340272273209749915395687944442836284798324089929067083163843086509550657582817964835290473434";
    array_src11[29]="38874295723772604503438986269667703383011977222305733730285345389301238427283483098617525673879939522537441700999657,1288164569534434572705896637928043677284811013929257247502243801685907802353693728115379882986791628628779534821434";
    array_src12[29]="39274631038711458333186065920122857678126364374102955231236653405855363321030442138821981126825120875251490456611156,26332153065787962036737347268564004109127969382224183069902685959411402899812801492407697555378311479097691233210161";
    array_src21[29]="11135205458094206389502276133753327730689066203432445435159022333728064221427184346081137667834476136352836326435535,37851356013347286578215683645588028670293530097841801018333014161931600638940917884100452778376687005248313241687872";
    array_src22[29]="5641481384667146878085781031380550810854200695419045797712193328993012386587474910468208986778465646824959855245464,7163985251124635758530498058158163579269179422721803486605617228077114908882665216504187348363358536240981884508725";
    array_src31[29]="9472013976398452109882516120869452301930660398526707117644528454369160142879935320834748741504660245428148291488867,31206549169460060076367300893946294611025599216931273883961074502329107014478429230120999287369092862557845951753716";
    array_src32[29]="7341160328560571913155122614078055368531187223850513774972538817070777983339501062805856671015592075808796601539947,34978968473327141212310556216208811457553629783541808308042510597249040506749181082378943606932701263804460278055702";
    array_src41[29]="17554943523667683652706222614114687293203506371327379773987291944711591035692380632123367307830037445729623576491540,21493059301028296111869029691988037587080903715404897903500447837041602492647991665589119399972877219622734274947933";
    array_src42[29]="35085461154284045887974083527808611629450058713860050660969264320873733232295510511824952605389013735067613205863166,37486203265525263116091239032796183564652286763960609894522717488998441148697095495850953438318298556967371189232688";
    array_src51[29]="8421299149680158316662260701703948998072015818909723853202912891852628411800823494273705200092940078280401130588812,27144352298006143649723698063404978127200152708399967901013619031422854831108296406910406199030525325455567569130302";
    array_src52[29]="6401875313457921982006896622554266901064043915595481255385065572829095757902380577889217990956452502611315252175957,32575567199642796194330445134351596381911254914734003274294430462580898612515599204713602124231150178989524764273772";
    array_src61[29]="10140298697159081611919262662894330966056186863495113044489669093272899368813762528273995954027411991222115692082096,1051958986549702482344338258560014297603108871862864690742518411206173126647299061452549836497639046049334477824857";
    array_src62[29]="1087553637755286399560149426040522406316892828410577798136591131205882381563185360926572615042259680826546626872811,31126886155590150986701335858211347165060588577969164301333640070086702013963602496322885237862633710206541042239200";
    array_n[29]="34370291468051440623009944493450192373953966407648626918647796958521589385057111164516591903740738769825455728476877";
    array_src11[30]="610176782385006865870972142113189979714385817339226020522040061997196013984926239806323267598076695216890289987486,17584152718294110017109842769489374517707352076031259244431307705561272846515002437565329929807522760418438318685115";
    array_src12[30]="5732896026835772664226862701093092785179982073476036013426517064552546528524244089949391844710699882786994863420518,28901164836619437070827299116501253738671282461667577429333853256355030009806570591720898532110150528665087632954283";
    array_src21[30]="1124143202253283349717377264706823975639067851052262345028702748795975986654185567502047216941117030521996025612025,14407733236657642380746425131895759473462117736482480742931361456962736995480169927913744362674200014616078423563790";
    array_src22[30]="28689875162520464753309029586280933745568770399085373657600492093217992485781494498537418615652366747730872196431536,20111867134415717657935380970492372503508178250847246842310546043288139631897524452299236051297365381344688222281903";
    array_src31[30]="1264910511959826255006106753790891676636426124119964313580406387149042788155207824358694043898970990816108561279626,7422446013580309640311698521718507800983163678822478107061738319198025500047195232160465208325888098439962708388542";
    array_src32[30]="28196215674979047626501622797969990091119020478180259907277887532484728371512487376121248330015145401429843778923333,13614475498527249648310952014023903614965520776320161854611143567491243790273900710341364473134834752798089311947696";
    array_src41[30]="12627311451483924966561362160714555247595367279604143600360354639744415188655451586563693517031596078335681371128082,4290917951996900441488159412749920079970117933921809856253245288215856184916128221788152390138532400365894826881789";
    array_src42[30]="34314164558189806329664762782892259774178338381466881524330587271813717674482241841068283515158095442560061402092676,16911517368074472783457492605544615869218297239307262958461527398029517099915773887427046458137537434333809904805742";
    array_src51[30]="13836194703474467091107631618448566032863533462311260465132414224421338582041867187776775486420210663119700122313640,30158560758113108861100226271557909224100524881942231634156212275408719804389891150308337042443723831625404746705173";
    array_src52[30]="17798269223117399072715275618788433935579730132678143123396723124560756962151367056644939802500111908575119693059323,31011952186559717301230599131870533437606028723301848523012177946398885962338201300919632192628523312932469837199756";
    array_src61[30]="32526883983691209944530357692282859184424511188168125201974988034068222273879325676461407217282300141738938636371943,22947301851100147091980758428062978660332348281254513919270604926893920173287110246789824403702728347568476460815661";
    array_src62[30]="12133598645730796821891911075101039107700159967013307072674205134209236350685840275861835793588456673145929524401455,4529334231713748179855029656035305081177458407506311455324264879276302873141775402608230450406856255506585048062786";
    array_n[30]="23027009470485923780528614672279411230146940623321770611221194373460472014105222208033517555099361778148877744917768";
    array_src11[31]="6780313465221258171893903479514868409174954751834672743689127855283194002848103190811302626602303566984278588024904,24753300867030411574076868942450317836883410908863644085229772111627895535323590583747715035649290765125299273364615";
    array_src12[31]="17832689619399074455120367900436361478335534800634678458466575187438900809975509656717570577798294694349105456213804,2285138872811809832723698118475687040188389192014314890751884259630579307962373204087618459015830525059931738167241";
    array_src21[31]="33990324303279509105806995594571038133600973233898283559078288381887312831531092724391397545421819754779037266249074,24324228036492694816590009309150539877785905660791696887307220510987141540125473118900435883716412332290646939309290";
    array_src22[31]="4675593347136472230040545598997260959356330278403717479847846092786005501163709467964100130310869515309577970674407,15100862421874575344848438437972375774638741783098195801578429639717086197201311869355763013412668721479194915048275";
    array_src31[31]="16645359215819651515746060857776726019439144460929417602376182756291362603646254162050758541104072977544201853976700,36169165448168567843392221065471421569661654597014851183680134723075505523399185862151995903116520795525966520125640";
    array_src32[31]="33142020110469309334166522817557619069338705702556584873547861596339656948219060771874495325973922989812733704676997,37563042922124964605926982308083724932003063213764816752076280445294138516595324593971856352867868478507122501052788";
    array_src41[31]="33428778190388463100807118913001235486334783473389345192473864332785780181815013237322742876212596931300599575812173,36079469030525630374587883711724373909023689445692861807378025519395732795418722439066205495016459013584784058794575";
    array_src42[31]="20715581418053259347618054659733472802399243182115030914298618803184925829144180599607454572111138312676505152623118,672427865656110158258657343123055141732843639154158019964731115767992442162363780382781888132506551913124213821170";
    array_src51[31]="4950267295273883387860258484368363137765736770262100762720624967354656825506955384575972803744368671893837288531359,35060275332524842196353746407284451297715800515614948381209265444691111022560846748550144825404099288393399378548303";
    array_src52[31]="36197828311522639157005812442425352656550001277783298748663859827393418993051679587353972560502878026493674450748246,39372126173381964650797484562637724745706342290148754145245572613116079497709719063087296348159509856139163621297197";
    array_src61[31]="8892723743498890746416312324871105854576190949482010221306279059881832915272043608493360782171557375852099289027001,31072709421302199549941618123943119766242243981041378610812500950693939063548400070281008916960768004392804760685582";
    array_src62[31]="1696530974366595169185102298953568570279020839794207258358732012364453321674297352715548462267415166354573211637006,38957656148252053517030682141857000473319176559943177485003925402329319200602476225214704596309623794662781248903210";
    array_n[31]="24553257701519167807431979173140325850307001807821887212205889909332217474769255797914645942838708512474994432155637";
    array_src11[32]="12278700242026171507314496744953070059947195492451911796616840192296234385298536072152183176666478515322466557954424,5840858103069078315774416322388023208442702565191140911437480180048030166354655180425378338263793731847755282752306";
    array_src12[32]="25600994798962004912754038903540282261349096401105414671441793629779184468498116967627248617554028303007276936258362,13371314072177417665897884562320364689144791519613281794707864240142063167601770426939645931184827027954768062056442";
    array_src21[32]="25404933554170025736510020215590934207871607110615017218294778938224964197568274998327471627520680998576775544471073,26916558971300376246372338789297565362009743395281818880120982321466950044066056234511542816128110181926147498946117";
    array_src22[32]="28485578224519044484912340739592901371362964797028141011837493793166634812656277633232752019002761586840625002635420,5403309638239774618539886480743854765543250654165137164165004467650525063997565204621825181182663381620486437102976";
    array_src31[32]="15257581100848717947808987608927051938041621278496908706377531417310587970734747125584552191778114011927113902107936,8884535790730649147714122392720974699418877671315897613552028337462458246853742155687542585312751001019441464887343";
    array_src32[32]="23470367449196280646799033451859176922387463407259381816915614759800026764522459574054069941086960744860935703161427,35404663348332935019914119715834555169249775600938206770186876031299713406441583872562059420057629319167343492880554";
    array_src41[32]="25299120160079736695254938071277726782898387344033771041788046498203479293062133917295327715261915319824318924268233,21084710080972904426551670960768318008463086391647991957526923315961188119985637668697012286849751849833962584048009";
    array_src42[32]="36280549872353842945456953126544080689086609920210564159872598447241607684527041247429361369576988543038459776776869,4601202036318104913812577636041093651485787829822575472730348072618834090586077033724997404073132561633954828916241";
    array_src51[32]="3303007094029101641334279402218065799908249632306675400342188024277093067334967066526453278400499327384830518182528,27313389372504106713864639346013357367992193269853209492008228397246831190897739567614141148891217056933693538748749";
    array_src52[32]="6547819526353707859727892511419211594211005894987981177115501641812704333642440579901857114215820378610924763131258,15320892101126173923626400256482497038251044814767315242850081970065116116651591693636042700057411171346876156621209";
    array_src61[32]="8221435339089379630467994530430958312158511487256271627076085160520108558924516885993003161366477177790291331388618,29122788607736370057565598939974340586601848184373154063720816961015557987491306769960226464918755984760587373821114";
    array_src62[32]="26696216625610855767246643174183876251575523806583325891727892293253037934530330528274649107477166032299537202604042,39325127815053875409405169081859028022393340639226435658409419603450971121307922441541155828003177884618864295822458";
    array_n[32]="3177610580473280146005366321955135434139223853878465687037424663663282473740756978570772852306814863880660366523043";
    array_src11[33]="38314182489754366154721896710026776764609244252978330822893851800404339154028740695923530316682115347010511697754155,38433206856794846761262243070515703501690271252333703596723596273485283560263175464784997635363014910388526634731842";
    array_src12[33]="7737288700121558291247048765122423643440569804088016279792044049207263259579820802395377241215847641340519650040128,12401523954224795666068333404749909659774950844012156096025813537502251056554323122580876459294999353122724068230257";
    array_src21[33]="13882641109174871268579523160561239337830003017684842495103131929588018355374025439715748972023729839361030517436883,5133703502755501791208855602459131500250207136969271990246854541295046344314170246910013732686347584385146199727238";
    array_src22[33]="28790386487409404154174314615479974855417217542584958709155848794235744470060798355533010763063510981064194406773855,36471815582076434785345455297573954467772728027331278804211986090079018251132558985875665752687827342384229682801229";
    array_src31[33]="22565223306045680120408187033348519789813213137638894852714629994504547971131305037535149257582798391160259718512927,36366486426826430763659294497102070908250560848140862235488415382911121093941734106629638510547560971451061298568352";
    array_src32[33]="22281610065174826935583857220395050532657084651134314032194174065326120875125400620160969024708507852642170872113132,28390591946597745241700839268134805628878908984675643475909989320848252534918477202486055018681058283812946195302765";
    array_src41[33]="33559723241132556222386519161207076699883909887073013772383144900671572616796325843173488635277603737429397068164637,30096922660195383569686674600899364037815570206774165010501967997781522229792499449770343259307081050626787263917703";
    array_src42[33]="15259060068977200190184504249185370315525931784259833650987178726356577551187739318310710999103930927639753701569666,17816413725484117039904779432364267717295902094830628856310960322182502551008022160936339935364212119145417247709256";
    array_src51[33]="23289309071840520381839814367141816143906810273211996893472591831953087010268609540311422732505571919242731387550395,38647843108529956527719778940363394477803496961531620386940496380273842038259874547229040478083478036175999363284657";
    array_src52[33]="33830973141709664228649563731059947109558282729442937661112159043756395394281021719765797789306374672486370937867192,17594139019106871056464941055086409950869137455670180462607980824495806052688480631944214085639069269973252917385873";
    array_src61[33]="31934214658169159125846148204587117627511312431555699210627277927790709002540597391671316307112177088491777673708792,34572122658979514713584337823227241132923453653845349314005407621859254694770814598323484811470263509180254589623178";
    array_src62[33]="37380793087170420279966104361912240766792176512918059341028549172578724063216110113729543375273528343439681421869122,9935802310473318469900646318344028125786426872509409082396546787415802483652539189395137706569157075653350061071841";
    array_n[33]="35948198853587188509188643404242339214687830294319344673642741024316010335781865847441102907577141446998357662559070";
    array_src11[34]="25343140114390297586382819366951741433879576114081137812138294998227395095168566433634252283662323970010934078531830,3914473049974280986095494273755304806106754701526360111616437169620796580519516646637481214139880237646499218408022";
    array_src12[34]="10613405199633932093265747445614898289530121537584173926219752523254524063461768620144508388814198804531951803299011,31604246640207887166410200825267112249309485973699179937694329656683189458445762365340200474903572349349881860011737";
    array_src21[34]="35389064142971587106746045629773397425553100987566100609571849528955308898477157710836117436176285982650923845675124,17113699903612245387399726510509790161858123998142503283929415264649649452250772687451238026806765216659548073338776";
    array_src22[34]="3314492473008015829033870400329580715765114099793023215753570728351827748873526466064434720102803243156273375331203,17978095884173934568891437206283465935396110513989947468851496674722154146804998332015316970979854058381572950034114";
    array_src31[34]="14546829514802815326786909722680821804131415752685724699472868993947486400627821929265973737177133801429964493565017,16764407736151423278338531632005766179887566253236228446747164534675153010835837875753949304723601798670700106411581";
    array_src32[34]="18116563342395932102700390698510600267948432908075916194414249448748066932590660985977642533528418589352612922612383,13120848307398356246143281644213926592310704184382444372862330060299189202390322037435182835545703750170481339141583";
    array_src41[34]="15017662254692679814983777320309042829730197012209224763164283581821892651533332540509578401850845209801821669569001,16621423760968184961544131331918260641255179310961792548767505625072306562195465598088412941292870891389584599848875";
    array_src42[34]="22623166137871530953632658738717409245017827721437610023888599847712478556368180662952192800107165725061727193167745,11979122445250383088218523192172700924230934640342558329198109128980672628165918713237408545487952872486838273660864";
    array_src51[34]="30000323658629028834035114462685160365508510809265058684108056580960352149970632836267934148954356528825669897302162,28244806326182099579000186092925985267089504336309261654816127956944524947927959676919839411995167351444966291415835";
    array_src52[34]="26727053855970330611898054017736956971727207131483722441374005739257967906710817716282590887578304885081012801909782,7276778618011882806844240279496214485439668706268938993195676505463916722033355031924049959884526188386558754761623";
    array_src61[34]="16023088819521269312036802598655014654437875603180947904716316253805345811102759025786685487889083913724737979985392,6611096151251175588105300238662938322579422912224220385466348348908623389658833638279959520284343929815889182751444";
    array_src62[34]="10405082322428637633785852847352970599322853889069175166779536117417700515408011140794131888715608069780310373509252,31956853404887525426327375522419371420720608553910360308572288979428093311469663547033734608842116897400544187032452";
    array_n[34]="3471115660051619459889370036761463874648843290064411841255357886657641862989438918759416120698967904702684302675012";
    array_src11[35]="2001761811353630540093709955483148808501439622208884150518746733374499370598918635626230445071749552088809403381064,4409859062415003213369306921660120565510787093385743466271978123703484407213791663468668448019470687670417997831201";
    array_src12[35]="3289990904253002481959770843321133558415862453564033152215838547046259601364372365398831992233017899423322463016094,19841733976096310486603172668850119241067695484795784167654121046326472731475729263076832488325844064741615318552121";
    array_src21[35]="2832928907322989377004096496640835239775202962946185073718758103556262014445344245120758661332793773956262025294688,26265101327880214701263272893766245270470462689518327964989028587265173828684237837751533833716350648000994970721490";
    array_src22[35]="29243208500142283613129610034792275739384526286964786530204951648145438614571410642206856087185664119944389933990854,28935516011488164559517934040648176415229396410338697472978567284017150929761210026813365621940789623375940095519745";
    array_src31[35]="37431748108869533321841443219536944371314285265891332739123799109564715286038918502330552428942070042638750393210297,34489750111076409503822258806608761965123486792858044762500619506110535955603636045000533920283673818010535135692920";
    array_src32[35]="24850332491503419213647112103639255285771179964495660666443460422779658865831097470381604510093183957183323510633345,11972394739163023931646383431652196253229348813115210473796909496688975207334402145044605247082331847084845385762573";
    array_src41[35]="6767350837836317092349009109794845187197188648021671412966006299388721162885931751707117628953578541627190052771704,4679187828518277203669127920542322117940871292944019595216283660004993442445671302927685944727819920381900613805185";
    array_src42[35]="38553855383604663625865246940002514404722345322151529408394673383346116875027494638969520565965972026992343968535736,28814538009394812766871473391521888562614801616830089828542235059042943138562373102399879303951287409611518947598968";
    array_src51[35]="39362002436284976316698896156397044456872330477199618267196608081635367106930975458678385590114766088032124494445996,14401555304059075046386271346014427706960063351071998298087065132408999922021018852627544292723484251099312575731330";
    array_src52[35]="31223281447400430120050174475313644105194967054683157884143862593879713659495281089961902585730721252569217047005505,25367721583649356546579115942415430371388173532852976362734834557072514086975934957574392783813841166591207627869878";
    array_src61[35]="14313288656865735805440391186523286956973725086982910997116915247620750748985919779284736469601770369008136793262336,32269267686114785928945500093839208488916240313134767801486339617367293131430799245435569109624854483853845836561204";
    array_src62[35]="10351822884774359762024171601267391002583198223340065527225807834928346835810016370443100848179135611645204840917254,16614776273288380950720160346178005073864654095780390111891400265521152846035803419206592517010691045032320321746369";
    array_n[35]="10482064314772925108211305552254532360794119343683808342074099441508793925755130060451232171124674153274308673100971";
    array_src11[36]="19535541072935803363161836486537640270897249996031072183411939639249833459938113930875070687944601604530685228704144,38421515794850379349070140917795093902705713183156799935987640371914187106852670846982787614221689734718862228928619";
    array_src12[36]="25788983601840694086553563941353815956867691016110026931291915100434947285013477008550462972085522759235807831599335,30142297413095915537600771944764339784342182119501350574969742807271990866668957006112989565468024895083189102956595";
    array_src21[36]="9646176909563721254736007304079847397871209220022161622005619209977956652337097174938363463041096998242024677906450,3729380928601275609986964705508148093602187703577096851420685874701394448139155935466261332344818354544201018610093";
    array_src22[36]="8790944050194010917092839420730306243297688777791322548744531047644183059935105235776838337133954132352683231552703,13122031907871650802289243239138598602069244434245412037292576118832685553136841119343687871464851176218322886852964";
    array_src31[36]="18156782920655622496720296443001294253739044908371262139034871067436537813549895071312908924489143338482026928909742,26609678061898137017811645733493859452960184070335872940759721561778332452582553254937543969612075543805440851414678";
    array_src32[36]="4058700455192346765779547169627748561597813143499708666198392477426883702100181127607816461735171014346251138310152,23682273887251624661505479383873365772853788281056363333625885387769309100480978094839015263978888297810080386675285";
    array_src41[36]="26830567527062313670164880827267543526024542458340805598233046846262584983457021703195322999232627510651545543099698,20539276315468230466882007461391567795378222170469471295066450914297200543763958969472686281993680809936615880839412";
    array_src42[36]="7280058628407474110997981049587473683385306463046809631769585519311664104797844174456348621241130917091748259102675,1589926352113989787753441656560420105449398853369005582410071248225367154799443798858325157159005414895620850290342";
    array_src51[36]="18696781681589156103388967462315278451499835733840240214409993389011626704483329951562924885528211694460203943885946,18599320636698756736143897905177291435972792232882405983412880672665592700706376170677858540435155224732061396188972";
    array_src52[36]="18465926890595941655902556398667720730962536340294999546438267325696445607561880838400541823458673483528658103116595,1255288614481561573395796350467951683019336722811903440320177672063925715646555118048162101388222092050965622089829";
    array_src61[36]="24299449993982433026830388544842078123344715208861270286788266749551382042604139384856363578719210941000766656806766,38922782350028658564272066721102352545990414746146615953751016419970014848968376182937239243607993643538568491320312";
    array_src62[36]="29549052505764599870368598770220359093062269171946214062562894987217422194179037881830175385315448318807484481573059,37723536154203284157794642951485035175693718897393793903483582695791869085139421285639614142264201098995756884567345";
    array_n[36]="30343993020246437830569781298778928587626598717609504724821538908153413172623011761044085355204144301872164728527934";
    array_src11[37]="11207583466985332832532446596748002104482227352145935761431665936273645542200088994621780142917349484538823963947880,12069338006506063465065490705136787461944554042441988671804985866773749700163502306509070110358638109542786930980454";
    array_src12[37]="27408701267668867916453770681104803796661106077388115268455782693840255937131373133947739440682496847368891574071972,38365621629645586359159742437240989365494447616668587300752173899810809071073116351319320467207967918739491785695617";
    array_src21[37]="30544797701949300715332388083141355964549958261251911613709283069277019992260103285685705533960015394584095667865010,21100480619355660412886631777292658487337024213798489207108196616789474497198742685109425272792737190674631893807833";
    array_src22[37]="29646210902777940139841168710736581997612239857201701575054649071169327373576432350927424985169520408748969692110089,37432456737756722129841435092686424718127416596633070461112466155462248028178242432389711819771651623123055681852111";
    array_src31[37]="36654652810058466241087799856250752505770240626926457213082770422077275185493503520532147160393972375380993389884542,1496388902905789294085391166228714853280547620558654375395949153472780751137470198979412115565741332689761475220114";
    array_src32[37]="3630330572146187481447652204442262544303735796692387989057652200898652998526097646272354061721105891567443512110827,38721946918837819212389367404940578234338481068412498686212573106593552732253256852097407085935509318703764460695550";
    array_src41[37]="21384137114211905391534585759538281101602864565998058755012642144115193565931992074224242380578087289606441152403761,5140118660532626874530276849711031148170215175462743963307750273534892312207716685038771338206404049630373246696290";
    array_src42[37]="12772322030091069549878156844062673863095684136056125753692103551408795111496165821663985408003963840280414837008681,247947640578966256064230150091016988085621154149755528324796415839267717018394619078024057104131981832649503438644";
    array_src51[37]="2710744600740493951250505495619011485755584297272045279290389287875365946878333154911054506651560180058452654471152,29699010734307135649504240155497049482954761877234380485871284425494339486190971010445553341187921533497521644483058";
    array_src52[37]="22146436388975111487077074774420010721899117632366151570762227751169045727079898418890197258743868206107983527526005,16484478611852417825583646014163385746328822444140396890134462513030999715326926260853425069921942841555795310438090";
    array_src61[37]="27061578327977959707709901106615256349606419592638747132822044297995870474601293646041312511283081594021367158582389,33133909277284131526175638529337679290050885232933985058182636182000355533026504874499793295919430061388762970475708";
    array_src62[37]="23354552998469103890529257197288924865660102199282036115944066011050965415858870512845340604390223554209430797876170,11584892340349481055102863743262233734067254420156250515719974287401018297487125070937676442373180669280926922408655";
    array_n[37]="32256620687153314139391638107125404792007253851736289173818710738995509455485325994209736612648339463958390984144848";
    array_src11[38]="14540599257569260491106232249200199990191704283936023121681368037330531080367306533192624446124199285263852371044824,9834975680093681752923404150947059407423727387344231525871824486338312906868216977872279865967169735757241735434095";
    array_src12[38]="25065661248127973378278042077497606983746469519926429951741705963316066072597639390025887406740907902917033394162155,8407971646081152868643358060415873855880408373286080296004115015824367205603759431109478061041614943528813453633840";
    array_src21[38]="13013079813054029746336819349207651899660915051727369248134245190882951748653634652111620107470237779595988627248832,6929708871377879954506265816183970543920712088332119785873565371810777017896096229621640821932479183945791147484410";
    array_src22[38]="15825297756073561527757508714619752160210693022170539611028349490561982086585759315711593035880262880098537525905291,27689185514032113122648485825059422588950936406888605149119918315890790241671508068014707964315847677437234155076276";
    array_src31[38]="14564212795079466896590364592866537242251383036127059284972552084006548913318866437535380973639016461024283774956169,35717987455972449957760947387810805136140565735564297802153655618745249534069526342764023028159504891491540478258168";
    array_src32[38]="22449952455623515874699930648772129939170134174176550723757521309254111068928733348409363715093950720814121776000696,32391246874279403004599337579314222473394370602284426865978191778001509736908540661731017834248653593147506044945273";
    array_src41[38]="3403972384484149804192235509794824534861748579734291171788822785339115757903847952408308833001684481340630856200494,12886770461087810034745058522950260566835183555370099853968129305714946012968072111324266519704268607684134560509697";
    array_src42[38]="34384926937048578851586308036547324908358374184176802118528284628907444724574798680577860480817093191610285609937103,21093174841360458730965803823878555374901080709825719308842624319174324158388622014522281931029317559814007175706947";
    array_src51[38]="38934299051531822000637166873857167708020216162895924613082302525983367118479756281251654721634357743733532588115311,28896787357844225816873075144600022503705810667728816200802648773126261583210037643097647458170881004754855418514056";
    array_src52[38]="29660545561913534640431586946520014686873198685306880223791518487670396015314739234226550076339757340548958170789435,12698536830346010895889698266249297129611176852762771320652245973517715890070964710776715302065047168317015599851667";
    array_src61[38]="282819633806489156130599554335429351296107346798060386279973271873752301615118896951241210217496125936243873222845,25584624125945956182728652543529086515770942302409667352073059516024967256692648784683925454179589526171443454293362";
    array_src62[38]="2728922661115160758720453571904992135541617137440560139863659347188360731924914144942255693895533243109297046129660,10771353391602025856799934144635733608492774551674754905054041977520368317371936129097158780115613536522193354263102";
    array_n[38]="23687270271666144286408893502666056206738515070731486730892561760859938097879189049876166872288025769801410453480393";
    array_src11[39]="16505064730496221607998501347943657811421198975693231815635701173861487607078095557703551526032115267422190727758840,17128157123268689130613644205021010968131023728618937819122264228084636226263584374153787591075292959941239174312742";
    array_src12[39]="17945374489627659852875903123793866536520526200023269959655191473626720672773656982917961658266469905910796435356143,10530267574256809644204561277036597670636094879927912175923263167921952412452431436302616224668296658471868541118550";
    array_src21[39]="7276739782469081719101999896067859989714794670195912811189511320641727048551770463688728403560116507919110374491214,21909614677892278168328089803428406949923859085397915468871802598911286083201164106684871884900791966116429887486814";
    array_src22[39]="29552924551630070444228524693931571160115524852192572859185078497269837563522125948195236539936352472029265235522417,7992123469516177873125738630221467023207007707289498167607537346540717765159055900356539998760047682476242827354389";
    array_src31[39]="38171903932950757365546173198390817865422063936039481757795287955630152324642279264434058995126404774814555624652887,19014157750292363020759960083777709581774799895289272060321828428855442522926426287718851984866298004772498613193900";
    array_src32[39]="23222961747064674330029026311159785737606357245383282029562933799379793559609332453789724308836638698009180886363606,2585198670229843726534894321753037998999131484814638509287437129376661424289995775556770696768892568123092536912655";
    array_src41[39]="38648247688604522778048352184160144561124764560092073141485816550429964682403487137850960522430666325775531165415796,8490375462157306529801768577378668277914623222804221582101337055355483771860376871017421223420188566682314478470627";
    array_src42[39]="38329276001276393946581412287792820606774144271906208776844889548208227200724953030362676050493474636555301008139673,24851094337265484472456439490423916031022810056516505477196027431878023538070482282690286836051984385609791702734410";
    array_src51[39]="21514077899275677693466060924570378264104483361575066843982142144433384508191094110053114869859940911184647891654922,29503807025454411700926684127370315616517558966435479915933809359322833581771001182933722030754763374953166364627739";
    array_src52[39]="10921812262875832132769362489184863060510195630770159015537479298361685573360183845327597381441335928867864308693142,156643024403962298824307534890219265568098868280584616025275615168580256741941393838386483768082892016453938745052";
    array_src61[39]="23064926924438387558702141013000980290425678179188724810828782774076181757229111440428368977207709873117879082763046,38352103973473143770037593088019558922843660817816308345665317845373305891649945547727299619300079416126032562866179";
    array_src62[39]="33421076009014684495330195640600323612091400006702453122577025346717579056632235520751234799937705673002885575201156,6782493174665724473977690266781448103599531984168306338293210263153332876652648553128497211433045963754856350142869";
    array_n[39]="30083302306104175814054757464269703126341245367211157112622983629058101513460398657180750388975981173655495352658281";
    array_src11[40]="5422747158256897413149105713120508901982484119892312729755223102330208105939798222491254466079849087832174639360590,4092574765352649663557958758584173601078689851861953417818009661952185079929383784773217722192647198852955184831905";
    array_src12[40]="20538659861879470191078170912700953920283268971488334228644569584844756690694820312737921911751192381543315897164089,22413186619702841500596441981616751039004930567556226029796885875501034753425431657712256869182926300451756373721882";
    array_src21[40]="19531368417016268470103492296879953188614541318888212297128714557670696707689896197297953344450626453467083181979202,8352723563339123426338686282617414715245042347142117248008171341937346001235225793898504918372636712057766697531366";
    array_src22[40]="1035226743737124431232553740752768040821616909908309012051489405347361443491574852897722166486319855185417676823724,27463385628377909836340130922963119454324499930493530490269623501894640669572723968136945133648469369897675120418857";
    array_src31[40]="35169710464823344216353545834904628763925100219343214047479707618967822142093210471907748512770756182996887057881736,13155020404101440005336053640857240208076090033622986061218828990990314153066744765809129916410035353965028933643523";
    array_src32[40]="12934100108410363122410076088548047961355278279434692796261491043081848250145686175522823756269943373824407299540285,27435023372099422016767116937707551230783499830152122843159350736807293602955381513186890966406994546942450612745314";
    array_src41[40]="29656346284002010256133575572819911087759832556634725308906145634072936109102753787686066162722751433731716456330979,19136832492860741419846665353546126290235675390825690424273027246959276862794651964388858789947585958428664824570921";
    array_src42[40]="10614909564073333298110154462238068701630104919513821845857295280619903834255278728688251060889572197037598986777630,6002708139025506569384813818704741065757128675317688976408464976379310014437220666016641493278953888549590430179931";
    array_src51[40]="6727909425965591103326895211686111639008859188994459647452944546034766111386641079512583697496410979359192982560453,19077036276630538649021408771171237287276095585649923678053088860561901652069989733395474289610842300682666079961734";
    array_src52[40]="22934628560192669884255961054329732469563001757860398287925657194888449113585985087544577083245726431343089726066701,28163185644357029624621657766370156461279546040446232899843054831809096753417627631088393810625108970587517863501242";
    array_src61[40]="2491331605086041084163706573471576416505628548018529469031745667287395709947393390900392927022349955348930727637135,32162619964967804660059627310181717431333697866317564477637094875919442815881519287211873782604567754981944810516472";
    array_src62[40]="354632330856330513774921764149934942415349858468172002165552638545122098374799785260050044191017312136456877351569,37564302696874183722369716482714334363196533964421908101605445162149446515995173855522198594796799699275044672068414";
    array_n[40]="22682716516948223565976172872226800379491275556671805217276180333635457478589196462033048299843661500453728566675238";
    array_src11[41]="35494681579918322193737051703964882092222346285518069682355922669504596229286393313876186850123940700982400335376710,17434453766343402628761164365125901144164590658720778473136767436261773434509931985425873805587804881933534441218442";
    array_src12[41]="15181894079594349479166207540602793884615833708866828332036909289716162610049927582115557839961805805912106522439798,11371193065365264597359734670342783515583844207818623703667074470361821731809299051118028278391102133726088554631860";
    array_src21[41]="12094079023854140066554529515561921314513283731915947257165366287944328301276493022290384953803900088433892047765932,17963223738014226788483718911243110916050425545872466580682302415034179464855640054540918751023777295338175136266579";
    array_src22[41]="9761390847110229732278089880170620081002897822079433573204327598747144205504704225728193065659706507633284316280444,8921227545808924241908661750894732113970308847932114924150793590370629190295185048648542185182104164332302419393027";
    array_src31[41]="36274159280886341635009019776607155862583173087928405888540983298581471106375344637284916220641268248531337970467291,2784817058446015772458234875926582249833040365921376017698671455435844687523748616812613824116326551724795889842632";
    array_src32[41]="2085365989088241114693480211773673423685129914388469421398577454821338316071142452868449182875794668613848850389179,20297898731592027978713646996311438943766487291146487995760606633288698607941262739003086858815236714441986205919919";
    array_src41[41]="1311137859441796704050652413425243702589663115689591492137673133478550679870733108361789955888289945259477711619280,10556270729156879738738709549062886817384182750478162935801955271852667064198190777888106725765928344118051285099342";
    array_src42[41]="27505453284834007730294485360955157194378574617609220327197622868565126496324676246315990306796441060782365102729623,13289173822916066125703637875882471122580095259917863417501482212010448462704491744478895780142011845890413673165175";
    array_src51[41]="35562305295241856148859327228811816011714226181651528029116992251318871978539541766385638515364722302776296866230286,13635103633516617441623094033936073119966749743752489781972026214993931194407820404807122450862672404503723926574937";
    array_src52[41]="4238259563617243086294035006238441034106148898988096210705458842146208404578537590585696831497026020840399168130754,18361434145345142320572941463885926036862473681690368915315863238732294360125726777054448748943069036561169278647451";
    array_src61[41]="32542957356969940314324977890187980035871416154558498573754221638627617290387217297349225931642114217449980479793982,18893591251576129453595956399538499488253797555550380664801109306826721963640519909289511090157334415239620084284741";
    array_src62[41]="32620509874018372580690037673177678575364562388974423068519116203010933074514510709554650090218911802492390825138477,4583965960187137485632072778822711137423187899519973787581732240215685477344047516471668189545030110928308603605961";
    array_n[41]="21228864125939141852866419884701360707237819989184279819054462518652329295660475406892265533584204719305755460117598";
    array_src11[42]="17795946929729474822170392312223063909086728558979792340893410176720751133199080904789875835232415258923663291483728,30151125750869197801197947517899856478353572114079671095657370931423856054466331112505481814706095794151136818658835";
    array_src12[42]="23713214411276563289309001070232695829625498363539542871262950597774384865125273074407867978763549724465757201586765,5485108308914393678161565757215091876528712394315633738970974282267978503805313293162169929880973403422196137971406";
    array_src21[42]="14754880534327650336610595223060755033720723444970699431024007432500488472901122000783528142311611217128767279027794,35595630705563371516594766091685060999555606766319857249428278664548106794674727138362990073283051177878908097252922";
    array_src22[42]="29378548850261483433840164737766355423435073836785613652075870969631142899740464872774534772962826793643284641961889,14368826066825919646209736335687731266360331404604758387519557613320411880643053647339394450295468502735420359625349";
    array_src31[42]="25257414576961030059207276391528052749684019920149616878406917545599972002753732281562206448022792776399577081980383,9183269837058145553135421303165447763841033770494011557411875593740422110859954020664012943595986562875137039895676";
    array_src32[42]="1135319215476447099579018590168391854571565651650762583055157605638346410521207014005852706632075229401324438680576,13968838180105352774559523319047310856638450830827864323336123838486417648080002483587518267862131998169286266432694";
    array_src41[42]="15762188885563250049082941708954015078989094615594125650749468024478928695936992454936603283070172727410338639813069,37697154765138714794423054019947269693838405158608218578874972416169773997813576567700425175058154881452403529851633";
    array_src42[42]="6679169648995632258598955784716686963527486996455651544552175024833050218226223238245479135561229284254291320890074,10780829329780160700653713435332903059330945166156991404668462549448360431463784838560342490175435643742317636358224";
    array_src51[42]="29798010653820391723714342374002533279910907836385870857107681793860021920078553080414126186409567538175309222918382,7473195946250759387843668752601257974163370451789486534667131570712247192039077701629681092729860580837190122583051";
    array_src52[42]="912995507740692623925622711677177107674970381154327242139487206423252581782360266630996493911028936253561241675915,3132582096859284774920788719109561822498985199977663070987026872077570368751655219904976147135056295871782207488286";
    array_src61[42]="13876107696011437983408931261960534749607710453468847187172700142450494007794956459474200073129034011579558280839747,33423349841160764867114567210956999734873962066872821282630042522282345561853808902836293950705905356894472117629267";
    array_src62[42]="2161491369577809604050906168182279762881042065833972529390101565781467779615830843947601484096091077968241542192410,4394894359617045883775134675976884491820291369128352730188920543063752407741773521321195876830066042613588498105516";
    array_n[42]="21583121590087160800190584596285877443111352408243886056576742963067441400731401149270846022329359983958045744236798";
    array_src11[43]="10450267647247620202169016553222099984054545278624951484441094974193741056071054943484759783256514098330535841915486,3754279310030080707323060328374977822610089666371633427567543431275345253617433691286219445037087951313626772091338";
    array_src12[43]="31329693795655535600593136583974215448612347281251808880145071603038213202603438516412267066117481006016362512317949,39026330782118899328202345989285715300170560511892326208085963254779549975307060924316768942216249932156127277223306";
    array_src21[43]="38924120300401007037797196048085285620368157335508310525415358382251752588656317898162208214181925444322262895401971,27153423990104835293985360573037212705395033400693888127330861121503143785217377950795811202222358387035218693922431";
    array_src22[43]="28112699074253427923398813363336609313718494729402332181821604689810157097672470962470520505348011307083212256978356,11393483714290646348329518586487420752660186230274685571024262386653169752792360971505881724478913826822025248812750";
    array_src31[43]="23628598706463052986902867670829926345210605956117700626035734576306741781354711202562881915309120611427416529539358,18739706198356578610436861997086357747954171071886321168346668734386775936162539938719642649193557938293644621666255";
    array_src32[43]="24658433133344205250271734277315104179351315270576672208201044670738870968903573308813873091642289761467441072851085,21050821446187053898572285506698032385109553320534730475768681488192549656730700190833772722665915560158387056850739";
    array_src41[43]="16422298267698324071307793312781134632213786304846724498491280081940808037041336917898537271813934432599956032207989,13808947955420239678499922063303845165533709833954959012248366915937460488627199086116599968738914308454141866722248";
    array_src42[43]="6126075562673921456251582367187928185093976609834436862087725816463280986583764570931981410929256377509274541674444,1196548782246442839650476605509368063746888237081364259481916786719965716699854448374199271229423646756850927234311";
    array_src51[43]="22132583468860062298714734470827411359958733058074371332909099265215618467014056302456922775553152429175035317782755,30258521836498051124181174264514441574156801184021291813657850471933740441035319902790931413524660096114519130560360";
    array_src52[43]="29283620716783755468739269050169005452699768643588641574038578858400076969165497416686090835881156530500543687821756,17792722869224439687258547764979562568373481358827622854008709493854718949648926130768644513376374240773623438248728";
    array_src61[43]="32627747797376103616147414382255382294644119072997518121897752546611275770600464774446550422322902150897636346448481,9810441970593357737569168762201895930656760093435811703180792412736345342827463309552543583006807855505466860237901";
    array_src62[43]="6659580552022873820699976751760837885705670541464904306685985883059425769841109839924922353933395320982594036824607,23088578150900638904109931814238713515882499952062192637719577142709664470116193995066749607796335518314593021104780";
    array_n[43]="167577311232714857210830126580071787341566520493168741457051024405143849252468304207567396129382097097504345058047";
    array_src11[44]="11390931873074450402101655507370071401037036776624116425212684134484162759287206233292662103797118610395807475194295,1950376684605456571617072574187344066852451491467977858071196203513842618894112635123402510293741680045481714054732";
    array_src12[44]="1324767598475198900210550348130982400072129767460220147710689816145808021443076617512133005639385314526615786634150,33486895156748435674280948175090538021675553664405794820238647542071205628950615380335553625066679032651077784157120";
    array_src21[44]="31126432524446237069395954467093948793525859693546528135308865754652877339171176418041994475965356371334094865663408,37251496633206404461912994975773572916616985434231383344413413025815308861592598671887880120895357472705795988578987";
    array_src22[44]="29763769798549599900600838580851202114659884057765007318621592700168940795466494032677027570057154522689698292882311,23561149819632256548654173398551598701288623054202636753293997080202051412774828113085012000745350511386557664316766";
    array_src31[44]="14698760657597135000244229071014989856611613794755743495022246088183283739687892969240676844946267205386076394025898,37751638377821192546872139529290526947362175463972536073144689500545934471314857261134962691777386897976522156443487";
    array_src32[44]="35576536618452580723706060180369297129058530836548667320350473546486301666671305497298917788692525867407363892778574,5435814500721705166850641461448655105961895849141771092842201884082113738420038313846786693314574732011404548060468";
    array_src41[44]="508477027922189177470419281557299840851987460114999209845604354579031162373482477758726008092979574237776660843609,35441487874774070005931969471424774034782708245563000816791633135096557874292827856372132895725687212041112236972092";
    array_src42[44]="20966939278721563951810618962803566774956773744014278084097149116550652652888198053391854035745271429164707688271168,29584873863618202634243261799134431770119270401892113145777949033179224617466509472534800588988443004501590646513389";
    array_src51[44]="14293935482351848336323857693147017286496067341119573705588630565344867963546227655855468577964775278433493742842188,21910984124291632177931015241517085696280509440871875834261609855706596652075917731492570936769442487171318697903925";
    array_src52[44]="18726186258173513010212476446150177640343563874824035725861409104070564640405439784180958606852519453457733954547848,37422207558465542993732181704882737417481719214015551013919891858278620258802660706789318073957965773726431589065170";
    array_src61[44]="10990163379717838130138612910153662872055392291914319585780908361361350343312961148954987416691129319887065348545334,1505857517539666180534303759622438554862084845049566285186801881030297346319204755193592319944561613742053500279066";
    array_src62[44]="34175957174135272900981174775027146795345089562960141008651229463571489070860914075024162583507584841199569864233193,6061091573585277220141768222500382029428241258374276977561147544662582371444599429400962307751197761323252037671981";
    array_n[44]="25809955101225956283409179027523508604316280848983035127613156195791566692978686039108960649010976948365826039757811";
    array_src11[45]="33861636575847927288191548093279137044584172016916041048979344429783517163441828582003499729458662580711009612446005,2795236554531663108215102425181399628381697953398271659376105383768012110408935207359096267570329454765795397909299";
    array_src12[45]="493006161848874960267709028400349074795357976022050289675712181537538093852825748929436831326984824308537239289274,23090128160953850735754026435738416961887176785477793936498784364820083960595104353162622193663001039117357072888728";
    array_src21[45]="32927430223726125909389694116196516559457112790554319652531177904779767949623509159850395581131190705764925001660481,34459662523638463965268133982966272313984947900015111093542155458262634563406876613251383202744438032751046151640673";
    array_src22[45]="19803338426728791371216099786798810341837640091226207634847787780378041016568865495927813887308040476565595290182684,31301545075172764975606034530836387349968966539624275281001559942724744108579840928720620374013260315260382083760491";
    array_src31[45]="37972779663327114141819083477550257059650291598089499638849007076671577559050962821346785366803415591501775175384822,23505373964162996778567425582990014293351744151473557394151606751982325550748980121100325937297207838666312068207210";
    array_src32[45]="6452188516126627120537784084802064743768526925212931691815031639664521701690772965234156373646403757554193538895831,7561142406233458844500680135754103842845248802305808334072786259636877804020062347397299774097284717098414128760140";
    array_src41[45]="37659401619288409417877366091362161817880947259103474001461908005850390114404992296127172163460291252133836406933557,22446151976545595467400816879658503436134699681116650641149422453058961309162309799472768449936489786216874875326347";
    array_src42[45]="12013627800674750830522419687569548250342019774060402026668334670155847304108529752944610155401084472403971630425609,19901545078833050938597927525425738038848145058822552807938403570091741449959653650998042005821123575326390129571652";
    array_src51[45]="3006944148274524212349126042177841528579347754378969479234167642561457652950538113058593645481902953828289689017112,11036110391739576958460357134472168002802220596919462259127975861443389774450294474651814104052779877380598213832286";
    array_src52[45]="35083629220539438127736352724737981912682411629184134648514704025473862089175678007145566442777250772606846882974231,27466400653402470870173391689735911909570388007429374375366917544603183738989908308078208857577488043945081549241940";
    array_src61[45]="12943756739315789539770576055537008325896648570598137665062597452117468471752597643022185928881238336410305426910668,18563875638241237132298142645450648543898790163236746209111861989978737777190944428195258429980664075171074379539904";
    array_src62[45]="4796518156190096298597282224151664631888555059053394997419283757830293971405978830811555646801717068396294157915233,26987584392662187978960056899482666748865795516728149305639137430062062112855448850342099008361632167375677243187020";
    array_n[45]="26397435543301043704032522470143140774475567729739877946416804076139752465511575662267012084010650319959617966419413";
    array_src11[46]="523969000189215945157126827546952766965432203280163183842439458812587874924369827925036854146319812259942178725395,11850479164367681688179521597703472270653272011460499930444440877752378368145094013590733379387939118499840747414938";
    array_src12[46]="864882762789921135358873692603884423847313568381465603480708548473527041176943461542957372359575126457529321133003,12671812618485069578938928938448463778501961160183578990707480063674286652713706831145417263091089311548016277125232";
    array_src21[46]="15700949306384883819229351431105768750260128126643637952852591626346516366036537458770590223932527616083392385404280,2150865910486875166320286511427408859317079769767689352411919251352900902104984300219084796997809165039262421972317";
    array_src22[46]="12406789074581863912783049022356649890299557630219991353607972481484148678580356211933296132287883641608237336187987,33476541509216819492370344410145781826881500410595650793620414326986470817643840212543576474721827409806203066336553";
    array_src31[46]="20629737768768413164073611683504457049921961473743979632092400347438599160734881765042012219601016026222279178865207,10602377501169768173292335390095032937379201911155125238670829466695951390352653487050244634990329624343483698439722";
    array_src32[46]="17930163218869297441387246708476058169334911321419151373215359089437893226348295477788713336261380019510765126186796,15946323157918072673089306760876861885368889259640377207418697011555325956552694142135642756305705155904298065254364";
    array_src41[46]="18793337851797177759337658520659265633033113388224326023539511326638237931097079694105776280605264949461442056880318,37244002795987502420307409943556462860692354575845853586921292105102597332476886680668460789800610406794789990397871";
    array_src42[46]="11678732433353005229518989966078420999319029569454882168307043522214193066134377223735613352202689567690714047367437,8302045679737217817206475730438687859995704492247432872157633662391973661749544857187995500029794668257323633100189";
    array_src51[46]="26280268493311093775812483691516305553786907106824805015528041193093666346693610260946939055767704226651012265573870,20114999267651802844970302132549150580510909568201137608220574530966318677791840968286029037626773594657217906511961";
    array_src52[46]="38892855917660975311276316974853063072039952076357468703074951966962360483550712840977063806580721543591048191129000,31599314414195746062554220740322242995425691441850263581640895472344928899504280361044027959641118848911141873956779";
    array_src61[46]="30999154250124545478692269584385264455264896766522552569379710376237491541746517570522435509368025825363303446328821,38151511552910668426030885515379207960379935326246333150807568705849495124558552189193320208428277130765905550376034";
    array_src62[46]="30579920741335885213606805381960823563186750563597233734041821696096537206023787390802058520191107947964937478990530,25643892729744228364504589202262350756097773518563659676126047314396767259587739018176534978531007684066707216616471";
    array_n[46]="31806499958939719293295161088717329281151898330832624869248955010158136219602997445796045679434992603794135154899600";
    array_src11[47]="33026752501848610043820497204394353761989830393629329394439888638268567507839531543842113959236338925828138372741504,23803411978240999868346146852112458324242626007168448409435283214926380729921537239809328719515394717437170403329890";
    array_src12[47]="2300698061138178428573241609295586514548406732012469490586884257009280516530618718551887920413961774326569648964291,38240722543938117371354140848916217268469345131652740433697445475841207697766853948061924477106595067161788591035113";
    array_src21[47]="39191434527373340973276964841095953372830535784727408599336679098315445556530263698979593934308889781976088297523030,14013355304247091467456857609826008739297893467316569565932500184765199146362076452635260984689254165670802752286613";
    array_src22[47]="15154114598820561488455969020627438221847271105149358898863197552678775172280111031643336299214310597423228839615986,7160338720479270885402425159882121912531280941448866967981495259871764259348472792291812215490920367485986848097066";
    array_src31[47]="26129419430973876288217592143534263704414197843558806916192146740374881229019615111629979157080920401958037462343568,36197525756672648510133442665656125204340165529838606117201136870349180158001917765370567615188715310094614710207782";
    array_src32[47]="4847120368694677342710645492288990961235057806318216570453723204847844995619237681345837349135217112013021110499635,17071117028337986270552715187547417882592530446941981179826896215407331265169327414778153035169112206398661657101286";
    array_src41[47]="38365259256514244052500605004485183908874682193568236374412104041257841034054644779461857650206634027526733122657940,33961177904980403988146520988420260557266667739191244012616552006983011488291331108817111043821184139202005575545735";
    array_src42[47]="23214242494017712573003802740160865119598918084250198898656054423512125014866006042217191212677467731813972262041949,17750129792829954005414004424691956585750829090108734777276804934190502324999290967554604985385836523266881252979072";
    array_src51[47]="21901731988664989464031982601601292467151804531099088454656711030127868241864652721341510087451181058075132695615659,806032865086391545107340119103089765208070833680072786778450184760620606657836334926084585055341483088241340135962";
    array_src52[47]="20036113919156740511524328424044786649413753832452697748075435210836245592478812841724662764970919115703248397998853,36755454570585445306481983399944162097854210277131430835935198034312922774174455468188023635025839511716716107532594";
    array_src61[47]="7380148896664836843232743519882904686785090147366951092273620538761673960233979328378746985809939959652107973533696,29273295073202043954941418684037653232101675020618088486087522904660653568488047656601246431637712153187022790305025";
    array_src62[47]="37366562217644650920999011482126698646258903566803969658133636827256170168072359346726489055161627695836568550201585,14847172356398543140547387306685464496665870865933698056054802654915479737509809101756181862061632314928796875775747";
    array_n[47]="23242695930230991608531095276519159469261104682109038192166190275583677789536952824474721363912501951708705955034248";
    array_src11[48]="35483337859252120468748765047992801630277004902166282369974078582203502522696074743730931572808781520317870524125174,20587765823267230054376507326580531404827652237878757498248298994080369285508413127462337803320318544206041669437349";
    array_src12[48]="5111756189069888033038082431449798241940053086951782535320069838462510808061149461980135185898389843418983669046190,31866925419470726084063961921342664997605946729410729750299883878342116532579488804677306539179423252377798111314946";
    array_src21[48]="7883243162297798744332339013622827137989117887692394715830539765277810025515292624176955848251598428269164145662587,11841957062146494131037939990998625507861842737720847505800304123824740097726856398498711090224122493239290528912260";
    array_src22[48]="22867525834742679692314755177251439700790303132123121451564624373678185344349615295944436207239028141219874095341105,7925124198887103194712246160895199118261695501243316748907892540551172184163601423511445729540284548766903809704177";
    array_src31[48]="8102609709854478267473648894603474367710150113542754441839336959419524694393330017905793237494183264652722816239819,16581796471796855078562398262656380267879669785440483165154894264090042228618402357729491874990789240900028391514767";
    array_src32[48]="39162435697399943257597768834965808456345967968194120263846775070321019201202359361408238817852945354409664184794068,33885812745726437552053367071334259803694737562021528470596958514231916213738450012282526940011788348058390584134085";
    array_src41[48]="30510990889091855159742931632991478835832709292405022878573135473796306808561603279751205558271490576858569601150058,8983461873968079402939804200374409860185086679484541867222360095182500646816715209087236999571556434682216195720478";
    array_src42[48]="33028515983571175122257278777404598424172100055180213288824727199547378355030704357865144204497415047639070332578219,17286972762464796723271038475773110746889643802909293506710284768076723696718200410234325714420947410430496576958653";
    array_src51[48]="5203871542482764879461493045936192399809541528306834520264941843932423682363275473245425564229907375598810496058660,2070002767333302126689169969697582261346156267907815445575973251961540254887907414055703767982635520018250661095571";
    array_src52[48]="783729618740012710352929325196527225671188332222042244642141246471903682852933935094746063859497502659407476456021,6836278394915633152935156416915486806630459084826025288679323468914211102849478792920749774646690995327274730760783";
    array_src61[48]="38417876983089225616706046525442577400600289390287474312577172148719641443285472167277757152727892169888443803967236,12649318193073593886394351274624072841084744458605939522966545125327112937565717826159921863179902929305979310248476";
    array_src62[48]="3992050530100099549327380999458170333467540737710730021493046436278484217043106933861851840825620607462239607302659,31226565949880257833902066602529785292216951629732636222685232937103512148400435533810175921960585821100731455126242";
    array_n[48]="13869627048843864177362397273088558936918252036282845745946253957164289496397564554790741931069657077233830547021955";
    array_src11[49]="16237026676758522453457256753402168296324205571748761182243708909969465336335298133603948819689267724584440915498413,15217268392943940519517128702223149505488158847581432047468813817214263858829605283937180786365908493534393091032796";
    array_src12[49]="16788281329143502718412475433085088098084376314053690321443568382619427496721965691338694113513311718284368533729851,14148183694036762264633862437554831460232759802354773330049171034150506087795229769351965595955923915834170277026100";
    array_src21[49]="35215372134523044016470529559484059486306641472627908563930912810334422488736968817402356984275987364258899572465112,19621120989112223187418290029209266559819331768413567124951117834811286694512859898191238295495277692069235175301693";
    array_src22[49]="2629861075786311487005520298569781463813408847365542290027029668573571084393712021904242083625734680882869032840106,6437586815852137530352969407187195608653213632719463095368456780930138659338104096683577403707034788165855513415876";
    array_src31[49]="15745520658084070713902379692915705926041981120694220076016218455966117623029222124859107124021162621604190300261534,15897465420795953966132419875122558277657329769435222790580673512669522488315541011583238638426740299268629035314750";
    array_src32[49]="13774261546710988204829360605839104870937657374564725755031688356143244566391496210857459284089931093122340181051204,30027746203301370644317875637186765406381644096151797469910946939540861697794397245298469657422527456867704337550007";
    array_src41[49]="32925991081057869735619585718148218933348728902632498018416333872362410983383347479968386944483324998719693995538492,26994761077144267592728403852931306991331003268909040714709745281585040072727095520747816560670888927033308140314243";
    array_src42[49]="19884152531528421197682504536354390666158643805986792966660678113687523591143216485819729141736445044930489891296822,37928039378739552970931757787780817096753259406142428575448297546932856677607986000137435504926985441512827987001014";
    array_src51[49]="33391546409706863318895365448378931543023620042754430550013076885675429298787547439582712585543821838811381429051203,27677096228844949372909058518649140516717233037670660425415292324099727427489983078690869712657565225472060518898381";
    array_src52[49]="28614460062236572553459080996629382527597718015102400297666197049978814010956637962785043092975794391152115101921417,5509662757103592445229128848058139613291271694862755628090352243292373329663758943460928602405643557110734556643343";
    array_src61[49]="5782813489942681810835494224759009820681209885478691703253230093918871931773077798026586508651309867596619323431002,6929255210180601912078771908979134480153737095684574345778086526062103852034031505745003376982783556618942636598575";
    array_src62[49]="30399498579826860995182426592055158200759651761135848319279341770079952126145374166890624006571102575532615292901318,3936659478326492513559802799649290296938365033604342426275071385338157429238714084168411526930087011444684171037259";
    array_n[49]="25082420043953823556972449255198693685150534565367999500182500267946682544927309772690028383493901770577140951904023";
    array_src11[50]="38602561717057176216337215184376977859307629659513859297128921058929906365914650393225001503885593901439455911844002,15139675108725214968135931294289955959165492271228978301279369821129179280205464137180008055094734912918618845656160";
    array_src12[50]="9336844211808056689010118725430717878832486718293174599661062651513643970828245057271183233467140995459163057500212,11112248710920832117663747148132830425400252588440548236638156610298995998056365645496953660288936485850598675431574";
    array_src21[50]="38020171653324486574734373137331020303058353573759415337523677457891163908362888549373097028418523425792257766028471,2407370964758090577108759956031757718834495332495214142628292302421687582777202146603226042531562161359299436692067";
    array_src22[50]="33853392710453872364676044177988268142810712049847351519449119535489143586978899824167221804723880171531395794630303,27232688469491190482398252071386317811099316962268725252572629072174176877594938345487154600225979315951629443408188";
    array_src31[50]="847698895442796546544973348201200861815094470155184195326515659960983797225262374242961931752711900162949865029792,32509142171130239261329401600454524684797865808504455799660472450607342176480473029332619569282829091870923382729365";
    array_src32[50]="38920055750556974645924630749349604758509981980789702128873738029505401785733087050148164600882714112977022112104042,30853662239206886136606423456428652378499257862326029512532522904943806938729053417931501931439324431816428376427731";
    array_src41[50]="23123702052172600897161981268465220348518656411023241911840652898687306135661468872837224117886170496236732188376908,31662609819386815408890332830350647838678356873292643439487768001834565709717966790832557369451229690666090313678417";
    array_src42[50]="10394517738619114556920681595125424851407448679136240945170483446976114852956639452726287414256907757860218864964640,11868356851792673798361414093711748823977940745788485669324207353224530358868585417705777177991616442582670606948217";
    array_src51[50]="15467935731605377257989416590553519040779216083105907328301162183506390838115626410745998094318582702304565127975410,3670550720405670299346729481244421929646711437772855557980413099852861535574435267280529195620826103404226641031009";
    array_src52[50]="38921942557775865985797804024501949710909740696208986464670716170992404483250378551750525669121487709546565802399748,66443642593672311280464981405755610948566105010738585917273883529126984285863046416654248272952936954944590472234";
    array_src61[50]="5140200231077493797363258986079308141481324664324137386000279274123172949888490102564836323302972541574079299418430,17388415873788799300091440737740673501662381089980935662181812944992212122787770401375992015408887423509574666625574";
    array_src62[50]="34295752434555835449321713405038766744196214873849568643644709379635811135120900430590236956571682870368776104041295,296060333956964457833072927984315671531200556341838162439995322313512963923077675905474137474047218294512856189566";
    array_n[50]="4907505958490246491582575058054377963133825439491246901982867730504763360066408885226302410604777638658335364675268";
    array_src11[51]="33564857794699488240539829502606049347186372630008566321774326777156770343692415189653820190761195112999914888894434,6426200642390944110963745550899787001947514543457759934999154025601059885710325144247417715786159743151641161224821";
    array_src12[51]="6327351070614948256320514468922858937977078679688455538828115133085255512968333116548111513172293455581553013423158,17847744422651824255209012614914238353580101066370231343354820397256824143938351418141617217055406268429037318759256";
    array_src21[51]="6919737860065806753926055406969962741910169063566368207826452265968695756399075555336622478357412904001901929105432,28852434663368689959704565057058422544189966937597277088518542306838179687890452938267392448567290602731404118926668";
    array_src22[51]="5013400142392988052667494212617842436811599795982751552884183072451502051351097167534361518207450041181998679731062,27090182243376652806893381544869267723947229640605959427919418369583727951473625333570922417568906468226916982705391";
    array_src31[51]="15936749369021705970402837157181033172531613814368758689083188419330505560482748451444934280031165376774806732932248,19659492472098953159819221983133842102844362385489778287405955066469861883962558168963046871792180895507743815392510";
    array_src32[51]="30693954354920339107599265444492497733979045765543379959148422488024556520008106232883861537576866158479932397584350,16692916073586851037742424792134391268289770504953815382772131973854901523006029673055529721312639524599241997639237";
    array_src41[51]="23094015378621951798403876065518983418648272673367565390707667869338167331738654270371252964561090910522943392464428,31335040168939412747899574854877729476813226916452054946083319754076338102451363271979117086494963356111745483375631";
    array_src42[51]="25366771873759109281593117902837120477937843191488532051233839894130284020925662988977107652429324012524555768173005,9444760011202132289548271650229671358185305513674066124901362869740963125034094276131267582111930875935363797551018";
    array_src51[51]="24627226482236873706304032722081826258626497683767370684217876113819326564926827342627196556860911353581834748254629,16056866150399752597410027704036247969047961099380307689139491810256952812653283316150617529293232941108033835679135";
    array_src52[51]="18557865575144664673934837134756528753394587890562769713995204115691981058726045230937752408528634512464765606861072,18149733176639839657613672889968058470358950175715130796900689262469648662689194152492489499995116711647251765270835";
    array_src61[51]="3185856329047075034495563462218984497567165378386879323142271415826489407214956239544314694339508210639242989329172,13193839245697659738195543795321790795640406796825827474835054137119837961497484812230087717822117040477636518003059";
    array_src62[51]="15052149391728710064793189227581172238965821275869763032936044549915172163618145244453480640676982271535918190231802,7945692270352479781310649200223575296314998152188321774647637166629887421669920497039820993246816176879639848341587";
    array_n[51]="4920790826689165873749062175052048053387886396360752146886079128120169759058067516129089503705334430195983104599291";
    array_src11[52]="27229238079312610333015290726195434640971280022143921971516305698673655654866729539491587902151469266335053153466934,22877968098506425698910400738597719545206966925138624151023454937659506549330484618977969524157784466330692943789861";
    array_src12[52]="618892676920791954819360125127929592003936544272556657796524662245905183681795201714681427532163729548448877832246,448240317388608105548549814974512184161761623130070903182408420799033493316831665710322831211628098346040340961126";
    array_src21[52]="27457830665777651523108310132776864310108178585929346780678943990174489107268384239959374974905549110713478282356290,14815364850348926762440843120931373356712642553554406966127673496932520664161493165371201969808715144638219059427686";
    array_src22[52]="24671179674997316957876452868791497100115703377352024653541181522624451935704023530803823318394738041767579800466820,3886505630011256411554726233982136559610967962303309219371437538393510943168338762976491062077627281232112514047716";
    array_src31[52]="27975832054068314662072373252739632034172867676952694613366382856888576589602889032121124512041177158142091687183453,35460236114457108747018263429822640401759187431706122078254680257544215557051718939149889166530327586942725105776473";
    array_src32[52]="1185077644083254279478233771987312385274792538064600926419048015313689923846795446453571434812739979161492108934487,7932335815675704783452176624417292940104110449132271523225952198765676089859102690452527151668227826925412776948298";
    array_src41[52]="37103553163865560893333357167815419525554840210590207549933672222227026489458249227122350994161177126046539255977822,13192331175894846997360641807594238896320128744798420899492849125586988146463824633342168205125649454252952885987989";
    array_src42[52]="32275262450837886891442752446551896392102595167034720090878768395313893722222874964927964519065944954234555613964613,6209124928787116344729050117626167230342085844731742536897541674061875923199819457536102524630698616607723665985183";
    array_src51[52]="37227822259838806165057183998877374239126904944380979072320161672380964295780723610981652258455749834186988909972908,1764112475000129969587920379623955034504847894117264090881528077779553931726567599303626328071931086319458306308893";
    array_src52[52]="14072492657480495267732169875891142356271679032025344382237068061399217785540673277622915593760091625232596457872242,38353764436811188455040576824101816054862015292381006701920057831826260356986004914521563608127196493409012122701638";
    array_src61[52]="25623673075384937554264511053912077562865646528908811250850278595325090483255822389270353768502664054022883801372913,12232746826760614136940779203808713035853437553968085001877160589262275638101484756016834102368228186761175391205671";
    array_src62[52]="5834918370938989408992497708587308523096987181292536840041653739191977671422194690767519943994291446297903105532340,8113059401259176453749141343349367565562763365810891562402406810100715025336302234086958041236177750426843588892534";
    array_n[52]="15859179129833791165674457227886028097025894515620242408434693629835611158358903411005784090582090684793081909550525";
    array_src11[53]="12718813070407293992820421220262587977279067030162343720170474038786556793158633096412175699513413011799037268828763,24575557838832830811287354016770344460221375739625612453966737726655799069913187877991865117128341066871106608171181";
    array_src12[53]="37345102151374108672297256192873432606103043693119796787146617806134821315229804487193389659601323419741695291811346,10733357439283131121006408815250050880725445096115599679011248627277894043658606618554105151439945671603638848892516";
    array_src21[53]="2736530677013293045917823338557915919734287435775122694534048720751691697224127542073242239440914188478323879611025,17647520217007313061358868061520271202601474155566316494980291134450929936415935850253066017483322725783944626364619";
    array_src22[53]="4743936809743561401053205051511836868758390669790669808549211322763257298789363277567738483604558857714083131530544,30443807382033673494201041202671062022582093074273687397827689194944684062301124879191057141593567350845153156623009";
    array_src31[53]="2845890181258103579948398310466494158726547131811699102468081062630775287249089021451135792456259606139431910831907,15038614793960542239443240598023185038087162991592337972377224455861666242251007333699585208434131585518647730370395";
    array_src32[53]="12248979258888703116205349346822487199367071927659107309116974269801405499635607592293072144507174214553031469926234,6919771757632615830289739251587383309685936305812935288615440982095965820000529257004814585068978572655716731987317";
    array_src41[53]="4033622592676434904713558584882247479144191098251901262729874567578480217571206957708276250859407402905503209188029,24925330130474666319552819263428115125467309919044239352845795189137259051704470790370987691297761729795838973470654";
    array_src42[53]="14503595612895955813626922895580506498248355543981064452092090712627917300818115970163849375642020998304098549309841,25237544959187171402687501137682188724890598213407388196568401790929090740903792327763485421559014821784209815602233";
    array_src51[53]="33626888886778646317491680949747507235231555667188010867104436293361959409751578731454597119807955608696546575389302,35387853062925636381340534289232016812830505506727577661645539604658481185242303804834799811156937322165990180453330";
    array_src52[53]="7671678429795982188262321667400994004976730107563693220039084994140317823749084137981489219266000248712967951734866,10153295507671866700242345602559395370714390250358129436209442234280987162623947414220401608955827620585492351188501";
    array_src61[53]="33180785363122479780991758732291684495440340517516124476484649296795138524048306493253867832825696786666700481717459,35130861093273592456467351492006319255597530448353887177042306315719870611119081623744171398425631529637663239340509";
    array_src62[53]="27585087804542873554269183168211975747639983935588427214218808986789553061935627820446060026498576726697759906077436,4743025400477564802830689079781285907960462370839738309242273326135173442145620625862270903626334079914950683002362";
    array_n[53]="31420351280689756582860830005223311903312055949594301748769613355250737632363362057986605491149615950578586961235341";
    array_src11[54]="3823781905297271269420940866860287498535490741697397563745509707063361598284874477643457719540109225543145277370415,29814603039923530936668967282105075212332281766693711759149760037078645505904121145110575069412287144605568320100590";
    array_src12[54]="19962054057639601672108968471312688857134811450440163593270459765883237089519089500617370029859311868542190551579669,12771076354261301736665638816592501884760786141183627771443792261891801536760479171602355918767533162554732459622221";
    array_src21[54]="3216690069632391425855601998237783217653296732476098798310701863840404493739748120543959265377048702765372649659717,24234666920225889860970054740805619115021868070483060277043828364485367101359181281065032311628346023715517792599128";
    array_src22[54]="10644719436159194737130869097490849340556292757885605112504261363354525047629436239698877851113658340052708132690675,3124415800621898349236140642132183335588052215537569533752460813393463651920171002141375507592559405327117748420198";
    array_src31[54]="28231225816104687177836615014024758430163366776818915306093835808836475265826052331804808406395534508382766705432543,3074890280067388573938853279468098062135043361091884319086631165607789769003098270027002184112608304443516736786488";
    array_src32[54]="22975138256633426650677868762871498896990088777444752602559355922792746144742021254030725298489229378654555407109388,2922004733071720609511646823413022326091766179902897600091666289734503210869852417821929991995039027329814124741727";
    array_src41[54]="5469276285535411682377583716869545301729847039842336543883819583483334211376833848019035324773083550740537757906480,22247600427001441037960130494587708958845059370167184214955082931543064511732654941976387683475130505225342777612301";
    array_src42[54]="35427697745947411501733238404795122559160943022852855036647006540848369632515401102253829655523919270429187650997984,33995275801593209511867159426937623604118449797545134330464326171991392543439577677625339583277267775966841084027442";
    array_src51[54]="24395377552516812647294758815904689421699179865688368109861824922987900116420211073287151239488065888045604290541109,14230565703384595023940245479246164616189452446287125016003477160666687766902191043312475868383493113864578312949853";
    array_src52[54]="35448982855666099677987718289603327077634160834207329456379208959542610792007504111447257792492589534800128003831971,24966047401017723852557024067406530841174819882988476940514616238571359283783517077649850844000285776720887902558702";
    array_src61[54]="1461580061745386707495299007305195148915928444362937204726970593934534023697935153975885843976320787521086953630707,12221539725311517028836807972683197966312042102750935061634461393439191798924940136442070642870883853488846572784342";
    array_src62[54]="16792467302361804439688479128682482638728129801781667863760692109580081804835894811142116343428961690612727596289239,549981581185054009567286916059659762516765459550807095256203893066697252267968160794994459418526220548742705594489";
    array_n[54]="12991286473628674910767903245663435284243473298547773037499269637052013613168227681677628935854525336795345536024878";
    array_src11[55]="33549920259037916197247830853203739317634000368146276074129741383401805371639018086481181458829966101179707794480585,35844334201236926724268582231925551740257900873901810691917949426752115280639694773678347516413370656243827201038856";
    array_src12[55]="20174766690845924396700189385894326070963528469208294747548528518944483271362707554513770731429239471590747742103868,10549804719988740674499891359064209698920845854348253831517097523086020378059792539719389650962072192105641309999164";
    array_src21[55]="15426632844343102790406468883908259704600100152339978545119517451191811614865082904875537146250438605415246068769010,16640741137491431579252226749418306258597756499328925194820633849727217410210145815403297134084973793277515305822386";
    array_src22[55]="37175397904004783061858022977119722905999790925274672789154996499483177615856313788925020317457956216023604551253524,22425586411902145010441282999133431980143010068274571242996955544170316456234629069186668999558680488364187477590830";
    array_src31[55]="4242068040205633663492827363522933933521804112799056761376321352735483664386568717110619836521365066468723490017388,21885132685518026679918891054971971312267280089826831234726768476862242291293796507575591876297990794175063508595915";
    array_src32[55]="36470062272659853464447541360125479600825005364381044966149246719128589947980097270895832051467441289116235416056126,18949846564674639842928426543436905264658033048409428481042193500433903591011201207249565554199496541936939193782673";
    array_src41[55]="30677092015102623638067408421718351955387911665653388418676069407545448523568496267083515819086798325110381697589732,39341983942531340486432475094500144739980001532297133922607483719692787428933047961587406738312157592312886922864128";
    array_src42[55]="14949328424411044295865929759117713058074558665901698702142774464216189514028129783452587402795119762430953067977592,37477341211673288622761695206056514491769836542426721710287917409653012522867191715118887788681743077979803446383495";
    array_src51[55]="6642176827984527637406793725229284096069764304587686372848358164262118393150236201717979604732805602489124418740951,30157899054439125232975569822776858387411991586534237443697840364673151867236544715430132737364216122782910707215510";
    array_src52[55]="3520885057767464273247501612103893399631742137716438661921770617018435255883265259697777262120618377243319460235345,18575399770101356517349864081284272100156116996369993170870669356676650912356629601329577030666308808596808189083230";
    array_src61[55]="6450177354373939616730667301184780434300583212745347511663058021023149899899912092321420869699603740381373228292712,17934581906725454986930004411673209497570329038498359665141464295992575683313961793022515444048452731124967240232176";
    array_src62[55]="21304299490735505401442652146902200701034374132875544117008198685209625299140950764636687895170832651809084845116073,36433668394129659057175934567198931614157099670552509683511109596428929270770806885034540013060128892783338709708496";
    array_n[55]="10471872595952819107231087929126628422194404138410561867215174684616511939218309443495287213697553965376945258326029";
    array_src11[56]="8703459228312377181009767251216552761256433712848061532911535820130349813226051625036802552797566491266984991655689,36874944125245690963331997591561241690495017140522172331493907085518155226833217956605883507998449517386985261461471";
    array_src12[56]="19400719949431681336119547494739026720899233641414269368750240972797778027886859932555302058766900646523563494689463,25726237168935061839095447490533889756037607223766627184764058829031387691381182662427753575541055782546553985226671";
    array_src21[56]="21592758914511783456229086588885430329813354528200290103269640570566254521195405595777584501362662455973729915206234,21363125513474852860905167847142441846470237490087576719149805594042245715430435021378244076649314028320673636692569";
    array_src22[56]="17037705925461338087395011225115514712620900134045507705902457985595105700757995596429840945219209927398866423646291,6729927422718819911428774519835009906471073797089198527330734637095753075767747293601855156543749049277771565575791";
    array_src31[56]="26120418374605962003346117170747448934440637899599806899395944368355095185793352038827241260431175378412107388389021,25373000838308395672134432068276617245631606941141738956589816189067687812498049508467500053158779673814522299350625";
    array_src32[56]="32373340747500984699119101365939903912209334458906524076350327998853334114281830927380063991137262411195301140527787,8166372899445213111653237013017199451905013464855456587012527728892237650473301546792231636877790736232151017427219";
    array_src41[56]="30101882797834385764555161504158412797186151189264203581033664960192563688356302698722252369026098431963388948943180,1966220962753078665747339376316665145599651652404899003822116343090113141980937482427324223661631907605233092164957";
    array_src42[56]="31295430042002501681549577994811000979685537078879960698006760093824906805140337142005404679898509834646694449386620,11939245365570041619644940477441401401695611077904291781534291714495373508495189242018648947774935419175835235400914";
    array_src51[56]="27834773050011924918785555414752968990040183619184471572994677256640265167495283797097162399940329536833049964683325,4206381294017175450296077555930604951618311118341564864903414859975778498104828381511300847507327821040616320268540";
    array_src52[56]="28755071190387289910054476951220165895707803010574403632833134270527831286759249526054450475029434960620565207006809,2273439660725220514139347571113044408496058551745207984981263749460191418481851727832346009159064572372945528058127";
    array_src61[56]="26083438589956407307674747198966729697713794121697048665533457606327354935536826200897622521522356977004129256557694,25382408054439630835436361710455454445733771635691891885828684153914718577918698589344453619578672017255188839327213";
    array_src62[56]="3463997193715413489918721922438744867247271185908499207738540310351596525277773183552998874448594865184827234245876,27508859166363763627955006875395028032337682405506622554702063922668293923967098978667219310538034289032734823943403";
    array_n[56]="4025113987734095918936198679100427272620265231615151028518470862006535647642566140551785724437928740210906360336726";
    array_src11[57]="16487299681301075980382179845018807200910620425058729711549549431370944586934377855437455208028950559525587753847667,25292588168879639496707419048963600239011104914484921437724401965578545367558868650217322775741574205838798709979947";
    array_src12[57]="25380676087945336553543218510626052726175428128064956479662431905902996021154966582877668281349837358826134228241257,20089369559534241890564283784577283193245019496704613076701133948477407985918730212426759296843197161140481928028003";
    array_src21[57]="18501520676854610540933286105553805539365908734416232431082979089670954504122020256103865236837910679550718286084563,12547720269603494291120723423685937666427583364120269285767461170030214443925838882445346726021727478340723704627715";
    array_src22[57]="15268247966402294619077992671219303513608870761238685078775368328042058088855526260821908145557030288097420420282805,15035083605505406714029375128498180466208393892632300119487616408871086103294841116437495521020330275840603496858403";
    array_src31[57]="2498770866804172081983615818927904979712311037065332456647090840532606678629565989870215884185891982269627006720818,37835110884493874460214275401441588284012625321105688846076774652800295673406577073268614931598118254161990509823660";
    array_src32[57]="12615170416983593087460243174815775799618785202671506945856541481984034072712795630514207179269339377097857506722815,32228271283991336699455605225140751451613677193158829846692174721169104354870368323117810236295742451952163634393289";
    array_src41[57]="12081925742254199598408469366370328349111353016184511553114311722745497335826003570714425848972743253765897172131927,23714536451107444059128062329854847142822076666571012362318972422662960370821843769016482296249087468534682742863169";
    array_src42[57]="17593913194724994326253721239698546093234276756097614663164918255246875902457867478759260547649450670392597338826549,23857800772201671892097029843871839035171553818183236501591665406319176226459084028480478523754042566412894701048302";
    array_src51[57]="17273133880276228617971715942015047065482498790838497319891030505531322425272329493448404534572830511572939307768760,39079583665740138627178559921424753750176794673520447764377197416985931350283595460983555168027046356821017953782486";
    array_src52[57]="28885812427599332675882341952159556614811400580962920241827562380286367832924931737440909702156379175860597829934688,19811417288714093592963014681572840618590693876518391598125208810290672266972717611160954420682077021564026521330811";
    array_src61[57]="26981997336952310212348431602990920576469962275742851165111725918205583790987834623631024159721873677517853245994385,17685380838493099789262882552641978693487342759832471578455882246971085591165899903691036570050995910160299676404163";
    array_src62[57]="7586654641748747308674816887010129662146240310660123059283426293743705000962789376879223294695471466561460332940254,2928539468180536368619114129634476211548351558150246845047338265464741447388686741809890002582402358464172855335513";
    array_n[57]="30436995743526953069240648198606525184398332835460744805224588113129760666625982580458147397153748195922241038557926";
    array_src11[58]="11233915435554431833060032298707650991309199667856102259979003832750893784436921193318324366615185346904714989662883,29311210388390137217606220724631804162201938412147227541908745950727308607169358097131629034657950630071591457801209";
    array_src12[58]="33837619400600072000652854823713361993427703191971645115546436493431660481466095938279681126039418377489329175615228,20615084865151083422599754459994149541974985480769449378701237467901236257911874830682673676460011636139494482362044";
    array_src21[58]="21130566165198342432176810336481100885762978990511826284454130609992754430049826473437856294373572511203838318959579,11438623265578322048908657596074769659884034849309218670006507462885202256288542931879843445953440710399175583787617";
    array_src22[58]="37023721046889315290462931893104503544769930096847996300146856142381950734641573486034182624457089239192216872762994,28252805485589422961501844177882260716676994900241194292101724867852749171584861299788249587293044341658886587465230";
    array_src31[58]="9861615829895739251448378631860932253035148972114791052550351952145462380714926316613173341287484948954915619104954,13888297984743200390940829296147473604767602110269820380500271570947962983740263601038150303160596889279617704707836";
    array_src32[58]="26890450917597401163027013401758554679083429922574518669868125301852627060517968947228285868903517822738692562861226,6944184135101040902454905445285502293472362328210744111132268769391278481198319562825818633385047067101247003007715";
    array_src41[58]="17388894819733555949187008578208765118109056617998766036807099313742001362551750233060991825705602855416904541734917,765539743839068002228676202835471760418497943908054398977582580277184342732903496391573167228583641210418962001682";
    array_src42[58]="21867764478722667844915330314356197381284399388701447251704717866209607404159043266172123412276981972476859678908562,3163081278988631981487476368389943266876689321831141377914376643536212195610154844967155086702935507851457548406505";
    array_src51[58]="8130875678382727776234531782780404279604225270686530754872120075387360069489356052657633952171809480903852613742126,14011370511944631754509397293365253352895400791610514996172509194137609597917634328947732085457743706667462462593750";
    array_src52[58]="29712433979159682032115559032011672234873686736734956015633692042218270077289310799557978243053075065791441315566217,9660608917901346369432210610745638965079805771053270599696527937031128943607249296788120146642575685134702188814872";
    array_src61[58]="19516866984914755201288245077488980598734478859203527434085436111327388433235943075054989890044468426905515613219374,7619982840410953476081523619955256046344824404976746336314795327164601692233766122773667619015667012333710879645779";
    array_src62[58]="3289161825998675593959125187912596793570830190383459581195753478609140880629828926126260876789909827700078594075486,176477101824091722172433005605306772002638000244882164506745959589378105135717578571470444813240513483990994413246";
    array_n[58]="17006274457617228936764621319588145978208146674406425885632347634831017320545390424228725257796788004235443941476978";
    array_src11[59]="30086974910330045550846888414461893235954827593801449452673872223243291285739903630701653433503099740644891251024373,20067541336262631452020616013264156588986595619073214296999110303843562406012196933518346480243387991085429405466343";
    array_src12[59]="23407677954554096680568213834338527999201700518124686329160682936484385128043735204929839615898315445921629826432411,10736207935438552394759464006170922332854215454452731631549356058014425032969263502853185332507061115102684795155054";
    array_src21[59]="8384933630912992467468072521585584548785642102289171839399310466994179934977009726824802630192079461539886893540718,8031804041401466046665646624507245240910413778468158014159992839186112216089400051619345904848559751188603932103612";
    array_src22[59]="21009762051340176619652249858728686475018648302485793218725029708468188507403168401076769017802806328308783836638467,2807820844414542543197408835690812874591776015875113331923608314834000342281198709211501293533399804052219248034495";
    array_src31[59]="14498569639721000013730775290695942387793123863660618174926528059214944580939778113028806938688179569429764900504645,32005270451009005261624356643247992558621645773016503648479116745401169975370697637016037339332729816234575810162938";
    array_src32[59]="36906582836771475881252872240853551185706240020703949759086815097192710600220294768866539950202242126227189231011600,7660709819974418292411869999677988070980437235853548199462774286564439202905833701485892575185547091552582419285921";
    array_src41[59]="26863216217116977753985475228779529636430564400022279826544616705646822143736751482665664113749639397250492518996544,3867513831695217948921598632640091037647810333259433685860477206756257980721247149827704053658673656439742887712725";
    array_src42[59]="9522355465731236842137616061087733919513559181632505730814880610949861324418980632646695441471262724980130468068866,21617099049727139868645493656055982649449769174664387709554266674539183019423002606440095310028517004229186951800932";
    array_src51[59]="13338613514526484707706108015254316400644286161420707079642363009749540535158840953581259705016234229861726630126782,35983742683976589790592961059818180605480654704869560761056907898852678421438132912802920836208716776545305723547883";
    array_src52[59]="6632148974103873492276787981060748888974826017631874302016702736280850184394303195199470777212287902577541655660495,531089424713324205102129714407690459384759046961840325656709069382895567520627037484522368779894912007364153349915";
    array_src61[59]="36999207964121467700695209398082893815787897976348869703539707224066532162769904931790021581296016732687600930196792,35479887486511441847731170977072957475462682319457813894943824490578945740098770643718702558292291837651211272232497";
    array_src62[59]="25075380531480486748038963929096109656872150326400516500479204215558609737680262548843936610184503422294254306360552,36671569226953908956645400872238263327080876294391658937027096913068643705336426276250790942239718891135842850376025";
    array_n[59]="4976364282042330776246839331730656639061136816872671437893886527349247527356197954151905241183261407372603000362058";
    array_src11[60]="18264809473466541623687692319960924517797478496554024682896744300530472145884890802345302307620713450957870472344761,30080928195949962470699197326887775614170151122223827876668937226151418564363727214750872947699679775062380269710084";
    array_src12[60]="19945528075197462603359334402641276778098672083818533919982419844692338959376364227046032249823574554210766019694710,21760448916036695937363801862396778419307713954709282232357975403159649934137540868927701641821865314730800288753166";
    array_src21[60]="1220178363438756715375908196246282731630883677374405417905684116979600343828594075739562466044666836802683735541967,28872722864856795972718746528719880148049816736888032039902266081306926020819917528380634097093806191016469086919929";
    array_src22[60]="18398294987332558361077149155034006464647321686377844898396617102111412347023212466553631626102855478869030289663994,35959229135639450303395961133056228499261897410968205916841050822172723094522611435752312472195114151249339974706779";
    array_src31[60]="32378531215637013590030691636697813347168042379261973656863721562804762538758490238513232586824145468438259835822860,8121399589753940876938019708901366362651342492368265790013581661319809401089192118397703653447102674294618626030879";
    array_src32[60]="29751864576352102743226678520575185629446371794113124500147404814148995102094627126512811006338430484731247644265016,30813584572687923471559452960822134093982174021252782094696689809822069709577710881756307091289893450831929474476862";
    array_src41[60]="17826592097977265588055829891089079499810709903162832966399953598550998850997690130756845309366756669578602346426107,2773993672539941461181923693931298223604045250806428046798607707999506447106833103693751846004732442847623794729723";
    array_src42[60]="25489294090065866868989463317379014804178716742893118782404873079022012406631226833292447737286880747192234047371609,25271414695622910868339324655003228397396952061548918137652897765529763111788238905400916445123879012029777941745648";
    array_src51[60]="14762043895245404384917935727832807077609362407547485705552597451434899994911391297267587388665115655271975649825377,39266706099466416567726482530830098688451505960798491832564022876170230397482848027890112449960054159457096502050357";
    array_src52[60]="19005664825190210429351046788238248423532497362153668307258721908525305617163417094648039281627426096943852323823420,9642895574434413620710712460919379637668087232617165689595374913797248863084759310955614888183585776329703980007381";
    array_src61[60]="20775099377701267717385259479156296010523507784742888946941278665058469145192298899971112421765442429996900650055270,30291621411368518785780103132840687735586819716916399264860573154041200774506166976100668527297406484312339080719111";
    array_src62[60]="22244952944291373633030445269139995997707362540586149847742301819533217297438805791274104142856542764948091201210784,22886717113189229246169359753462561158529691600994071807943325030540958461053610191943775422022113744549962453949203";
    array_n[60]="5022941314958190262351259574395815244703817085441815731743418876284261105124816420764580399388302990139122059352039";
    array_src11[61]="15552214835112596949652702109433663965033839714357765568012560964551307162233432927742848964290123694303796034605594,33587503017498035603165261826359009076305458356334100533713764499534078041462817051929118514683841485235591244583663";
    array_src12[61]="31576472006838017099265415875195878190681743997845157267888920587865167030317951173539650350770345105990034357406273,11111316660882256849576253088961793917536171656432047946376047266005078049504737251986512410518801992535570759935322";
    array_src21[61]="26272394466657025089231625538835660144631300862038966246906806104955045764689267781768155644914177741254717530548277,19685768045805855304572535192730671702920088172348130384779225800523543430917378776184719970185179284555734104605184";
    array_src22[61]="18978418697017770286610273995313587369397042549965243480417220523590999795956243467784081559224215867933009145648715,34021374660167237940141833456560766407714527475891675727677846614537924460723866892220388331948065687865081469293066";
    array_src31[61]="7860404233451169949967949599815536490709681307836138360027408581745353513166142692528875800363054643181073249903789,25520057941093411997636861809139535007995097209712427477722138685512814368520734582885638369747601357754660439380011";
    array_src32[61]="7049725536041831636925181019950216460336139277195830964565700075739927819021380469275692987947592941239650560846289,31209310535048747898248570814156750027865339067190537882602411002883308228648608981630816597000519644364359208833695";
    array_src41[61]="30326737245952165655777156892153852453706287759758634754364923867704927115077354072050777027096435941864009063574069,26882116322940865215044578128563836019359038613304475323362029194443986679260274719513112281232958437180949786068461";
    array_src42[61]="25858515088627333771493011430822379036127866195225671971851500943734983459011192278189177658387230137618135573174007,17822544599718637703248909912187498378001134365042809956072812555475227018158819551352184699796542052279083788497462";
    array_src51[61]="5887675705232682483479038449506285401667929524495641423189797071035444419266080686550022412745284656817887086152845,5941536293101726673891607683386453217986943026768786656906574121367232777382744169260344534895373578799858989716752";
    array_src52[61]="35279480093850567078039747864294472027550464093319050088828696182906837633825965108325096286467345544880110676950358,10037849444518825634309421680589681285223515570457305634535927100037415710926192293605875244163841772620800433912158";
    array_src61[61]="2195912630699574751097670790349059894978552260508888697068153492143780560743276727540413202743864957115013724451783,37628339304042198438578515600525531644660041021633787181104142722295458504803870667115041973868241700319550566925480";
    array_src62[61]="32068314677304171715404572101479342276190747390394862227597295675404138414992143168760514325590822504484883190909885,28133492839995707087612174882608169044595218595379372365906182089554113676349620739560883476245564899589369767250458";
    array_n[61]="8940867059175232948742800910333869774845994845451685729237499404992984126393878151305391529804104768946821671261489";
    array_src11[62]="29115140897898767362372519528217771788529663985115764479562579040243785629091484552380140677667405121058386967708589,17804219235954328106685776186986531370964144270524480563292734799546725371267350024656456123484246901732435644521321";
    array_src12[62]="32039761501099497355852152380455682885129794123226215577791848773492715094100090037520133648857106758022775191501608,9578421085973342785081340409775919809114122842432633537711763832538115957409525048623374357500670963556697711670674";
    array_src21[62]="6014037746911394344490595448096877153260973157620431954793727060499920591102451840161071007170235812790850485577026,2169637615711293999854772063547838407142564698131823194215487050318937370035023950874313837667451182235136626018435";
    array_src22[62]="28467986120803312788094176412695948559209002630097376912001419129048443755121623438618555594115382544259709928742664,32787023181536985929128389434766473545626113085261566987332404208603251696876911531151865639380924195849517350694081";
    array_src31[62]="11975434833771128111848120621359060236793677560306660373341385479326039960479922779086941057332546370560038700984238,33172511727320312322317113465522779144686893913334236863776405036738727180130740165389815050904817180626289029120662";
    array_src32[62]="38613925172167123990665465064181579481933908256285208279024377229589131819654103718781149453055217209980277490327209,14216349220541456989863729058031415314282358230442258746006706078184508608048905861940339732451179751668853310803506";
    array_src41[62]="33254778901634533014725075520608278320251003972716480718251726495287287025218470578876806353718725224823624011920822,34972601890345318530460912585752065041375026432535972935742776407951462654175456132902397378502669958411362495192807";
    array_src42[62]="7527435767362941458169795944321287579318581194383543404383972820831284999133699484281654573414395120307994520601486,10734992383238017389149266434466984828814589822969359323195190483865572770603775250624988873205221344436969163642977";
    array_src51[62]="5645599760362119639840255445257628448439169332769171051702089944185371477958200021052983988155678716676391768999904,4232278159122884287205131062720570763510893994690779433988339779700780349083650251788910649787706702877039445761808";
    array_src52[62]="16280297192898502351509345143252510271630376016151490930110548932022855842756081991969209693901838044584615977647110,20799837062342994824610282564175312230201529256825102744100553457001138772134181178291017245950230542210162788448421";
    array_src61[62]="2797237132557809239089110206815225448120826458481232500158494349580355650199751098446573022472527954038585257594431,7716270378090766172630785796553460386667208311604867879322504591134149999550877339340155561171780390527612254028783";
    array_src62[62]="36165646184271906412359648453367014601543046101440046398952911775784681261388440664674371674954498716560271759820142,36689446011340175977726193391068826943309932194042203044469473359742952026223780527290106534854606320920969213833446";
    array_n[62]="31431160380989595266546099250291188717354036881675824186537949732769197437496657837499438470257732640783520165110759";
    array_src11[63]="19874632111273406196893951251899296883616098355292624233998443668705599742106210493159353863712985194300361858270452,17806266092361455090740722007852032844361552570601536828915367829856942853460655579027942241594956980814435008974258";
    array_src12[63]="33381110906711363961340084207021408657839093259665316766649796071348949049631066757585464534912838513899505807279545,7022901737964196398573610271615808701803403437555079926232760237084789954122587113935030720370073446044575220496461";
    array_src21[63]="13828547408481785658412660460234636179568115274811949406717917559752998994596082254159756115924405496822735534084615,10688009385033898928992064606203999011494157033635076043607660359482834639836694001932304115642039483378425624071765";
    array_src22[63]="6712636782468050965140784495920745182505357763108490620349414107730529000640992779985911155675755948538398491599832,3316350002087864786758632917042270019367227706668718119007522261395389277168820140594184458464689838459000518670473";
    array_src31[63]="27868068912590563102413864785376407880687110318849353243003594867797591967629121641315029018704667050339360439024508,1681793708839865839137398711253211179295121679788981929129505296217747358912516576281730879017586818530543914873924";
    array_src32[63]="38634890516209530524121863209442734396711616976134987680198576961485769037855222677140119117821143963875101103101148,23182089627574800104280360741536145409501728813439393409235378346799235236549478003378576144448644973422910334263238";
    array_src41[63]="38919292318966304116921598097525869179906850839681703117360053334625088941418112490942229919687407661698488436957957,13895531251445826996104107414620408903920796945854259333467408610495281986205252104048127549056031319254389316113939";
    array_src42[63]="15668271499389559390445407202760956058506252580733260448990030243261752492838013064790279102384917262780561455252342,30316736500093463750680955275057762139990174228648929466022926267246522201469813801565302758616402110095403542455386";
    array_src51[63]="26054364528695548225869851527223962505740932247323048612984828829016474960819290983473005380650244914814911161152953,22227786506965264367673906197033908420078013326749250390771799467915697582646235827222937915536813297734904156441746";
    array_src52[63]="15763249206040524495068168542131869006057613042521213023168443437003267923525030745926602494531764111263091741571795,37796463364482971735724661070871124728052705466708929390543605232674215414342048974230170197568443363798390186161501";
    array_src61[63]="1008988824944789311483432080740958377541412053435376760460675508088449971970243941507292778839225655216762902547284,33601076087673825644362496059649677548909134482608828342427090535797823459027449048461260252305510762687211268752063";
    array_src62[63]="2569808340303778942682331863069654769225885673449763386110030361295623267413570729826569778038737301777781577098276,33624258496431266317266524696770727305706686009470540046641519664272527399632205509905358592950239521904759831399405";
    array_n[63]="5409212769752670004766603534334875545742800553827501461827594208989782820144671151210193688264775384640241042499564";
    array_src11[64]="23678125049654942938452376727611254750427122017613921025486622323899950794286801255224716121177163746482432643904822,4244691297624709734555708343768463235230069482110908503484588310055970564575818845191412262289998912513239865828468";
    array_src12[64]="14802272059211653972768290608951528162867531246532304022540927842068803413197576113364019495634905085686187792302883,11914720921171241335441103650861892073022631319257038436764841836613019254353392460393301604065549080812437204617976";
    array_src21[64]="25232397869355807142694029826743954157090016167109914224320251186766357692555194393536245099889901795340300679138754,7440268639368710572461347474070225120927942040191554618023060382518555709141668588225486227573900857206782261272048";
    array_src22[64]="25280778181535842557789413971233398768341407706767516494176675365537661662689889496192352094567722990903294952032692,32695624144518793727676076475847997436325825612892693298118695127299682882672412979704634233889168513961055442080946";
    array_src31[64]="35096491789012470946001096270108366507517428459843603983244190429218523877609681984194954348246684758752831095134159,11668535464674168663297090222465111261173861257255161435078424526155902923792479018333759878420557251051290157579573";
    array_src32[64]="16724021034736482512682331242584137945114471348071493719536619540303370129297278681713569707652249018735261932847266,24158422328487388381710343095338750797073618736881262246607432276319555397994507946148815719891803778806014708443052";
    array_src41[64]="37908513804581360178059671031279665389493633442910479548496728271599516203808555213055600658901584638762631646485988,4774467040633513065542276912592302746171668227109341409631814938596662644823452114921540335016262437057048990565046";
    array_src42[64]="24844317823541702497769118847356545142486350059476222185377802197592284758063213470707699852260253161131054137370941,26774075173169443820253971176169888685007165132419727592031336061191907777277472798150422901201231228759584994436170";
    array_src51[64]="21846349040240812924001186533182495371036839890991529348235794767541611853214173802082085893706027523518273217355075,12286986704959616638371555720307286187565656393763873967300380580518679783575182386669489630753738557675896513823716";
    array_src52[64]="15111888998488919941521168742621899610233007820254790252154506544390097027024660046421453121119337195296119918144017,13999394899897623120742824361454567959996322700437908515081847878787950840449580762294990256232293468507446530683672";
    array_src61[64]="29323397660188949610500593534231014935692199059505816152193476490750973532535384286060915907931225065763489724866981,15847076443792059655246768970647616556846431008449160591252918572723577521610878914908066350238502916567901131433700";
    array_src62[64]="21451059707175083091194721636695823742081128703905532634401242850016589582989811910730875389255725782409135115483235,20740654822303265310230681325157508670797787014444815906395079029570329652581285634294134205731345760203844165342300";
    array_n[64]="18753510138444390455729018626045099296119744995086666788233585496236658305413705794239709405572596574365182270660708";
    array_src11[65]="24850698506386751051846956400258808988507309610606621171667066715103993399293985652822049002601102012866164205473965,33630232493147801214293914945634933877936327118070072899065407072049397670755669100231530259208149095978087153253731";
    array_src12[65]="7154637948930119116777328133680571254874727429067770465044558161609085412063983188146218445021163585005040180635068,22717081668689808829402723450302199423241232318505776212757245007838344777194665546066935553201343307930052641310191";
    array_src21[65]="26369430511930065008548772737914190248336136327723337467023815407750632069478077369439699845653396859441896480026806,37219929819151299654497485841278629174479877016299767282881348415533158894679546722158972425323160336949922936833803";
    array_src22[65]="13117568336798787943393293348829736310047723437360694822649301052761385859801407777065725377341647897941706635344921,36784148507084770685245052777753589745402908050704082417401103023082350170874130428353742741021838529284781496247572";
    array_src31[65]="25148995186090260492508914474774219478704352204300291442933270510211757435760492548653463632621151110737693500918924,5298564237734682042912927901148827744176806901501508580217278561961122239256285973764668165019679602790825059896756";
    array_src32[65]="6950052592752207581530937279067030457823287815414149016580725433574903031802026139680197625322224638384522538807546,19776390363046590370688135908128223307830236625563167413640336366207892053742075675214638294508893825953730163984012";
    array_src41[65]="39147640981392434372599389987905961967696934872837086055578645517238879369208798978928214348117365072448919019822434,10407984707022278815651538527458785845171400301172522781970731038957633630828385099449834943678509143922180160941904";
    array_src42[65]="14793170092867638348775857892086126747366229725166562384064718219035221484027036953293022608242208115178249192897537,13588530396031249840848379924221506743872155411030058134750091358051466463950605285952470811559265423853657379324300";
    array_src51[65]="16345383092261346811590378343614107807329508463867598786374875244614038734339023405980351990973732637931226115298237,36367578011966443981832505130720651189531075833908655368985249018514391728916679725565531377520413716046856866175905";
    array_src52[65]="9832030027043945468471287531202830198356589500134709798314433900759411351846159055787190529334096969140588803880613,8834999769380382482894076966170580752457660464754619326948242532664497391722499128777213040597815739007149150510007";
    array_src61[65]="18279757408809881354302673011762184256055706451672298226511933193064997480742477207961950588050834973156238303140686,6297550586247532701382830021945867539526467679013538529577972973739404528311169754183632384608099872279810833735024";
    array_src62[65]="9736998935167749382008841772751201235938468646365690353694540150098914115353789726124121255209305758212556980481950,20145158229390644617203666640746393955013345866382567387377980694884486596364949531068388393727370196673717826535079";
    array_n[65]="20635172170847296048258997274759693175716414426919365801086142776015889427458879556190515558950516509040171481340872";
    array_src11[66]="28361242345245864182906752194099454931397972634134675881721325583672584477371324053185520157979766855595548921833052,16103759750645984631525654266097361592151800374542857741356027060928733371129306597576893449628470266208165025058085";
    array_src12[66]="33537303472663195962671942114223520053795264489163555238466371930648788808488135492750927479899188235838663763852016,15697346876330060111072244315682180984621693317324522023198213176773565892613254057134027215194389658860980543654250";
    array_src21[66]="37878936297290125246617859341374451083740492926914476738176303859288652158384861046092336408983212171016770313926349,5254743603247151565274426337537415468083671026829984237338791767567822118893058577827067675932085214564225480329707";
    array_src22[66]="5586696688479847509908724324421755934540455313816543097183807146562741338663728397984733908428740097238680763014559,34658345206472175017469144494849543348945206787018454221049259822149519677456895587912019616757329056576430574128401";
    array_src31[66]="6401070941978684674555185936857946680245644346455062674529455485596419401302814923141213366455955272883971260252904,34152545880293275919607433432513098101147067497461041021688623115177730069735865611205892263054743935832576034442015";
    array_src32[66]="10981771303947783814995176235065575475211824390430485508194950811446226969810586421579267188689903792612925215317159,11993780078679543018986616515750692330584046089934376042414294944067422296098869449663531101996636959882633260088048";
    array_src41[66]="5311395150138557794586747374342502489599234651828861447452634781983559060329325577309884798121865945836341219234090,18568743075106231554580783232872114006333321081604331519445561547233983465854713022522815558112893429277459481728397";
    array_src42[66]="4201699238720174446463411447936128610440495315560627106269940701378134635280872839382369780307669749628753084558423,8931534798843463528860259995904940887420816226505160492294926882469409539862771085720869588678758111154120299616574";
    array_src51[66]="32595008215271695700362263127671227994204466169853791531418880360506117373837891587393596958854415303268501691115816,20700767281073613164796041610489988878764333326993209725446917976727415034377869499702202721751575869057984946758235";
    array_src52[66]="15330236236847535322285410485848187513650494482104601541989602085488210165391925220579963973017617347572108388402282,12221457990722877826402502513372549601904551305785498531885021503128526959176909702699099814450035196286897402403569";
    array_src61[66]="9051729586200428914412851512356354900046107903199485849112760876790944599730881242074696453269879707271963476182123,11068289152676230805466432640399819912066897670640340417391696470614126366952676568427289653046991081578501038813924";
    array_src62[66]="31924701127668063554101715211072006178029945347926951089205970406774721916155900762719056802060947280141200880707876,13602433658079811253956457795500582538837234935732377755375866278435717396421723893258346077850348649248940232558453";
    array_n[66]="25425397309640797449402539981087298357291516707120168867326055934004582566335564684774832137495381884065296327016561";
    array_src11[67]="18801177218072419988381365935730742851026465152021157848726640032652265303960524373870159813168050456163340800202613,18912736023954821427008359123729982671670298906630685694788795696185352265561252842823154500580121836924504086241505";
    array_src12[67]="4526006080290781540330752116391644539367916722192664711000158462546224972659618108989631265727520814046248148748904,9316645143253145316150296399663841200245256570237158426037497333551800710085993810479322977933188953290507165035249";
    array_src21[67]="7483509550589213977328144889616045686230265507400390671089336535052430042496284679721009622008117383496238049097453,13771568724852075401623625405241307010085066662319116556597788998454525040309044956843058100965298332589726739576626";
    array_src22[67]="27066834008568819506471997468285671109473680375916959841609139161820699889629332318193489497369043792991166419576122,412203341079457505655516437076920496420417596874420855748717515587096859764900884308637404144857858514941847825947";
    array_src31[67]="35731839772732278400257139009617188650115877943306538023990358031313071630963260941325709072318053885407430268933300,9895758564644091800173551087860410193012275627906340551328084995475633753842449942625399656737653639872705983766906";
    array_src32[67]="14647334472001064211403050551447981147462121681393608315889077226368370844099901027220639400932676037483268445561625,12429735310594693694512682077730794210612812591014869853988502180910011510913639089139152276647692360000988189653444";
    array_src41[67]="16344471148506772680985904612655774613960583038748302939725486045625354949964758845373067080906269379575888304377560,38387315817443962965143951719888155919052141501047298826045969968115818696149486483900220648244825784367653281163007";
    array_src42[67]="19588513285716529999974970040712513308767472337205883109436775324649894986827556781594607193736226226857336625499292,1009775574105642702543879950442991658558671217148798788771825779017679183912885896816327288740327354068785566013239";
    array_src51[67]="21393900135239601742119247003640962939949460043728221725737077300813025870476559129265288293267240368121904658529476,7423158036343091635674062606210010581891547054782953156681530495128966484596215402116373732722871831440532838283555";
    array_src52[67]="9053560499891120527457290623711230015983584445080804942547897408329493982576616453903401713302847731781073649765661,25901902547459986949266982068790439029830523445358153140430600434846663600718572523284548396027198400762540543203212";
    array_src61[67]="10596454024734055408684982993621890532924960219358257044787136447159515465056082407767527387120613308728349249941565,34867824638948000825225654190778840838302942853488392011512814177168511844597578198261658645102305143502231100816049";
    array_src62[67]="31164876349418855926895063664016616550108591212180382857565109872606228812150763988370149973738574959807656735243839,26673740138587099424753482111535671326745460578472592053637350680100609727713851868424339358131470882605588026683307";
    array_n[67]="22687872219582731330846902620705337282122413773917836522373703262671818350185946099798897246713478782507117092139713";
    array_src11[68]="20418263277182911353261523067482867503185588171557053765677667728993318258721164446029846909911881925749323652021205,5719422495781095050478685803662202817443003859240120674007581099678122430590705725702414241957971591330722031937242";
    array_src12[68]="2497688606077477970129918059520942743626014934286320015349140320062169108521152434522137610586058683715082754492640,31550252618817640614256763727049929664820370822217336771439989195930133985677062042640002815836515398545068910209241";
    array_src21[68]="29873937440836064492063239405304623808603858014383925381144013612225926953369529754287974499338669251899647520247132,12157602579654459862188273865879516957737750315371854949541941187724736697222784241265502769409504629220202284548464";
    array_src22[68]="29243863201464163619944316657193432130712373206727113940884192202535491887966448213348139894007825595365740262135170,32806823797061159936337280396454352266285457372588723658676458326048932363983652098966395500456220124944106774069713";
    array_src31[68]="31352802661633433682817690161556301179561188255826017042899039715060307984781603241486847637462318647101573995883312,30164860946728819147856105430635600983273234850188624693536824479907306910530524264095249022114649060072517065928775";
    array_src32[68]="33010465929489830275636217907197294467088167011666177906564474265573670507338006878062109996708411081333666938736432,33772920468531341655134018320570795657481432273126116050353202625971780139648507893488846998953520014148796283625830";
    array_src41[68]="30132119694521959522906827359288829386402732498608220156228568801471689558209900482103320857674681747201687613961768,33935553763450002750040123032394566479402658522060574432367430321017599512236049047724735782746744369253916056478928";
    array_src42[68]="21504251602992873840366703632012581689827670056262061219426746134673879359339152959071388911335728480103203845239455,34638214709478637473233217762916671468594399438024900762361693150559861477234220293910077483184421966296839544719495";
    array_src51[68]="9383839241778205401706885003157473082350257066811674148896763623131494163560162760162354545324748741605314499775223,13211328161752507000519859312292084597523481427589434030598867372303811655476255532337382624060282280164163914843937";
    array_src52[68]="37797594966673091580815047165858237079161818568720741451966649117133633801442149169550173260277678147678346106310808,34829155858036541385262547761041931069715897871814212922473949886349799303917561160803745351115536904165078002287690";
    array_src61[68]="341647122585198027696399806656844767555582361925245073819641869570046953453629044997895002273343952116102248106474,23914403811112824396291058138590403329854767672412586335559211377638400785855972253446295277049638204066135824115722";
    array_src62[68]="34945585502982307363358270858997814756974980965514753495063790839164132638546072704948938474883409635849275306858983,3843140569033517747598768266186588362400830909731323711093533722159892507148676684136929981437621147399960550292759";
    array_n[68]="34032534663350803948265027910487178719127205997278181492328417423437177226060558120887026498958507035671535882514506";
    array_src11[69]="11366709624157307976999846098847659353253650605372511710246656488202860156914053247102599785406196758626336220631363,2841092745752221128120690502491813742419496014215730735848188466531873636691825637205652447925218414998549100923062";
    array_src12[69]="25407212736588886424616485708710630394818602145948448827813708438797374095090379961440741956768653252986975378064804,32965991462043974891924746104780384150895642500669926592573179234524215755425281331063851697435237981587598038375489";
    array_src21[69]="2262254840906884909786078587803488396190171120792680893961659763001315298180748264586788895733647797156476147466304,3252182165801107562609438117332988656072898224501007135364444651273034341128656995040580024839490955376979106820339";
    array_src22[69]="25344835036754047077724802098016630717342776502674903339599594817055209494898924552559747070055250263192389622039161,34238173219477133264807432758460794740063375831137019855118281955236750765394913991017100827084425591716355290359313";
    array_src31[69]="27350486700728472026239075158605401854315306045151458217487087063526693558575231040592291770795190016453830856544339,30044846649502079361508422198568526687226823297036575807777208496105955515459771099532216727266337098981608422525686";
    array_src32[69]="2227665456449564827254761680866367386640499946613476718550688941626863228591648366575613895320587561692730969626600,10545043722515663149598588874727432958762984211140212219688875325991102907680813677985298046342072586992896378667316";
    array_src41[69]="35068493440642913088483017282834729790341397224368300304257518357449055481070816313791186594377772134972549215525204,9316041853668025117813922761215425862726628198236842769526564946132459526315056750484619834604002486047939714157292";
    array_src42[69]="25263233532455156349339750001341775557014462429867911535840329315155341497841049742412260190808444958232857623448625,11988908509058475844036412208107456628368699535442546215473542027159595796021117486814666801517761709940371487329093";
    array_src51[69]="12034850447678633524395964205785946807542199546358506954777268849742710298156963713530699553105967976999227621260810,6033859971878538496235106008519320197262859095422008465962784128766174786348135538427086131246538503564900191205236";
    array_src52[69]="30052345794618713835937386866095345684762849848792488303486856127846643019497246915848225045654664574821334892583865,1137745629161580929480765256444684603046274264754959503342960736097927201743577887744981255279623866888931703196438";
    array_src61[69]="38788454068065786621174363407544721227700416658249367211107944011579391656101411074330128597032765859726030372068220,7707912614589167140229123899006788241958875638429237530963713557145428612475258972919823828295193147664666711424844";
    array_src62[69]="14342538670816986183606298295082460589452505182581532567530430859845534171352134752632626934346235839597154488275598,32221346376267760681420869932827100398920654391673626782584077460259617363496906189314278013661955112154708771423117";
    array_n[69]="38696619992668824851197619926921631709878438999532502215564824743185312932921211480145256746055629411220699814220239";
    array_src11[70]="37395129845291274670646386129820953431227124504660648638166605529320748180039091219738066131631742634698920040127284,20749441403521255784131266340905215991431667724504429868697912957042944494412490122851158921175964362807217547352990";
    array_src12[70]="16380350124219309009796852674819911734141948477407506428843378769541864277396187599550749188103610009648957028329345,35729567866402491678285468759303959907806795582648301727741785499625815562537898912687177141623481318039227814312201";
    array_src21[70]="27605765005483372586151832632646849941607500433084345892661923715638013175123103663278868935015757284002275828103898,24679888381465823968226772697986425378253095121347428289519198455945358575821285922124073482808580098754670895660470";
    array_src22[70]="11935233908340055645215808061006720015934783907529839430802584744884129878375328795720318177079115450422424147352834,4679535075147081034075030129273095186541152409427912168567038052356481545921964214774105803657649155138615311870103";
    array_src31[70]="1042701679766083499425173588395963704248750453559670754178937525549876485386979793774527265373446713767494462137482,11277013931137102020634008517274000383751236477834867003193725740779843246018069921942168024454863124001429128549580";
    array_src32[70]="6379110315233663353322328957657775930643957196321637203258464104990297706778831635692691494011585601415325420220881,20783978229132428066352088861120161607654150878907153997785245498603174085607296242361405828270777513003116567632765";
    array_src41[70]="9686570897866272291110801366576250121495526558845836898404402275998335744435418094570689259970020788625361237436802,22123325027278482679880623118167665141202729439441312118914951666538201948316775444892008858226192034122873079420427";
    array_src42[70]="29906324558406616636903496293710661140829555563714420903159235002493887149519112208180256314818246555964163342923153,539773833538822010889986318395852929890099905558977008030786091386975491323001187892322365361184522078469955445";
    array_src51[70]="5779214822043247540432790677837639884202824594721832137590497183126651679537096639567095404260664720428969395171772,12121844615212505941641722986461391293524808090277824794144355939877690728670030860533222877695229436111452056019082";
    array_src52[70]="9240713645961573968982207060322736515233415131944752085809974395678450574107258744896680967405872092157362398815089,5025869827972981307805835129402411188619849182834386723902766879406252436255203354705858793030651111746305065705922";
    array_src61[70]="12136994426509115663521745991069303362787985245850322767700264785788330182549528769111040389375415471221021725866834,9598085233193003514833775364449766700108783304283513682076357150171417302774597567169382757235672127784448695998611";
    array_src62[70]="14137974835041211759475943560999260290738791710722583342474123844354137721803131240909017340307130431703815584141758,19984544370456975355320328076971449866220511632617430463754641960261627705894135233194236300153880339848523305985067";
    array_n[70]="12675339164524930360815170379528640275578065015565026711451694193206878217404925704936456782131945443679245874550023";
    array_src11[71]="30753081195696669700063972444670499822879101157133412059337269091162621514609658641729387646057435961055816913833714,32029585940047591137695688392834158280211032669503577561613002729716030567348185042068719860242004982594051910475094";
    array_src12[71]="15107857352415815504386030972011591210577284330179142866548812121513198737869792723242618412519973061623898169174999,34880612051245276180609900484837987110471709017593805434273889784176917527759180643828784534165903644848212681718485";
    array_src21[71]="28471152437532981739559721659486745099140438876795489344967237968802596912329841138199468251352618632798923023775740,10244077497948967815951312427915685744693701820972990359903254987409206687559235678564217851188049618221395388883797";
    array_src22[71]="2902366920679809474830166936508271644778750375092026573742052905120413377687875342242344628012714729242206654193512,36911328209978116386373129255731272000175383864327082097529282266012987792295735197891063493758218435174500878792499";
    array_src31[71]="25958825292062915529378492146152780664981332572054360659499979364807231931543426843261657884525392203027660665944735,34318448840934841333825392511688579679330989347014915903644934616517768124272689386024657642763204110720980370176585";
    array_src32[71]="14139512269846356854643715419231647891590942002701744837557787120417697970080314842671073438795134335220310653584278,30128878937347351529732276560708318350618156327216613490482861455326530505217514656876384532454379971361122380474832";
    array_src41[71]="35086207105346375378362270468397102666083210146579253581666809943696299106019618228222293070833159577780582564161042,18943420060837249148468033399546450358916745501909481884560502765395072768932634194677678304685616347733077847163468";
    array_src42[71]="10961118832970070028408170866496918827902609443483956960133360059860450672468102003727248110825081333724241149852472,8054352823116333997715221754002387101211524013359621871156248921275729102506409134727544353756678852462184530072652";
    array_src51[71]="12460743863581762720859799701859133761024331471099993036057038653923768263049780762280935269500013289560094183578168,24630152190968194102011559573210340294337307370224990689195801076544643486830311372749057809190287656624648765570236";
    array_src52[71]="29031551562854828302352853651894201875126634086198796285016317668919438323471635931057001516965296934432103046806225,28570206716780205486145783740104944863705543400183481050177401544412933656513250728332671456772293831551043037552780";
    array_src61[71]="10251278210111045682438533706071479385458744028953940787096028330101847486317844829206409967495355014093005883145233,28366193712806520955552603665851023192354085965126265768028457582133066073245379204405562053665842464173286795258430";
    array_src62[71]="10803361228144450895451474868817044971057615869549967932948858663591767930641140653449842786721655799888896348794995,37296095284146499120164715838770703999918069803710450874991919113107283537794601922338412981555424920435289650438309";
    array_n[71]="17268751134833162871727822040754166108726518935059790660913023999963395341468469195309177236244753984021554515676634";
    array_src11[72]="13497982018797096802692802312267992741105707538365151906107718705648627823442491894376717895247113479414909151555756,32010622217651510182522392803684835586362962624579250150945553293363476102971753947820828814411278898083748006944714";
    array_src12[72]="21733430510515940559649931426014565547561506186751431290436611551348079597570380909818547868774554461737042249722405,4687839529986889531244706170160633960985330827819635506387357872660591676250297268183319458950682926263727901485430";
    array_src21[72]="36878026467051221041444151449275800511700744100428174893061846192147541321235048396097609353527800743161212407401781,30118971047443831251994732793284203767893154082934392883808031628943618412713985066576368526314675674699799792821611";
    array_src22[72]="3034710759364208078376887793119170851823701080841314781159655915166447856227961238404979367459861518714155850937583,15356397798398269431174639760920761321478035512177030635103812941764619979641715355110000013432333603176711543721314";
    array_src31[72]="1778274280651503896099911524349352147412183051293065154151076412798470400002572880120814639493457740197065726879011,9292962686157429383169331025424576487346589998699320442626859062755632676182342503472626517115025542955857372518889";
    array_src32[72]="35276451111107922592091420177255183115983647923821850919255917314970797002032549834928880751799947054933980729703062,37360944724174410993766816762932748681043812614850234811331969154572031194970467438801457741694204303993372221557928";
    array_src41[72]="16976420821467969085372673999468032866305969575381313099689880374635241277102711325424561445789458955696703581483619,15044913882132128481466586982196173519207902842697676522208279703461404762585970921261983878333726897329857893597090";
    array_src42[72]="28894053156102749878578679483192677192388886420952682215016994563434839193828757061932823006145011981290396637481715,6346018248057109549985467898791871128671042649015967533409887022610187832955411148755434941132339680780639001519974";
    array_src51[72]="13414403351516111394385228942284059433847232880370198584157309848288558008948433844492004985042563834487370433881670,10601184289291801463541819073922331664533773335516467538153269556177253142025420568017504118760932572067352938768944";
    array_src52[72]="6122740536524794603867449229868177142860696462152405031189355609926405147728585380609277691620838516681825246181590,38131461588602291948725972908214705528036201627568023426470732785091956954120785647978613598170111894531668694408719";
    array_src61[72]="13166970315463122654855467085634400004599983124085290194796330265461672758127048804299286125731598449572279745820462,5040018490517427260631109658514917929030983040659492206684408048348215210705378727787200056862245213929128211388110";
    array_src62[72]="22777921847809127319588472694111009460573250809922036485780475152226664789564400509374485501078181478585418206304678,26586301686808813080218856478157541411014149563807430309205277132557769964645355762625776465221246302826672489998325";
    array_n[72]="25461950204730896004427979721818123763776628229190741550702725880825012764002340202727374736545259105132070700277074";
    array_src11[73]="34441922473331207940749921267090609138088875041817306126397312871926435950648597592815716581196742461335963597365699,24901283216370372465114282116971551504939870318277642607984758725529371229620812940399780275607504896280237388350697";
    array_src12[73]="23323825833733113725174815548670316455623165556280392703530173606203898936824661180086205898996008411218041862751214,29438921560819123585420673864091900036819832774699163446212593707153240806473833005995184809845828655599376994159536";
    array_src21[73]="1495254780141656271800375645173792172901205875597773419336037055188391192411757390827296147752517455517773942904844,24579743086148743090645703238236554394307336591936779481096217121676486439895973468697685387435174055655537426770192";
    array_src22[73]="28784204424856546078814642685185472438393036193568350941831247315256876745317709935122685356937085927775395232676659,5554521727175277306571541956221010897940787050939172604012324409181239617631440649256358693056544873583462535067494";
    array_src31[73]="16387719750448017336866011606035884057465759972309687874166914945785539601819982505811042661164214675682178549971625,32512644699569324258253380478329719772839979128353033673527489424391295254203582316507769530589888664143847782039401";
    array_src32[73]="33225786826934994390866305544763956582852283612016344927014601991022007983088497210416459110456809361362364079015398,21694057948799962982388576941035925552642247228593331016407016364904727724341752496441558201215815974096735275041103";
    array_src41[73]="16923789166409342757701958340063766507099363138847129491178695412759997531568125705273984844573801607894865927133451,12052618306252905382017877264026096251861281503938605984556916251630040050958563164683906586268400737115636759274532";
    array_src42[73]="2607796274774287866039527520570774687194697842378961895205786318808664599868623705333727624031028564863559609454377,9494209973482464154697432574864621681059806973736434227860570432423076625306279172755352710478191668824354755230595";
    array_src51[73]="34329422288253024349152486572325962312292532531963580650285134444073116830359066608894556456514710819658876732003213,10789496940354059228761469212330633213375387274345512089716479454377167800918201262463870209834178960902300553815685";
    array_src52[73]="3520748701045881917456933617817059291901401593029971396132532145261058441279343224333220351536822213704829606291289,35918602035178134086924757459354941143515113972455953965919132154611122845093699399662100684717634895838076990335067";
    array_src61[73]="28754486317603566503468512057212702318609773844411127484671535879035620498935643177971121562882412565628615190921291,30662937723363301116456857374901669222629853359901675055692010980972298048071933084776649179949097781481389814396400";
    array_src62[73]="9200684544921935195922669506325771912941224464470746257239707347042316614421774977294457645685524610165134465582892,1819813928883360105686844805305750871158999421935410817818671664201677132554691358606555668317497760791707662392102";
    array_n[73]="36054643267922587798158753249578125206480310643164386552711774497996681175064121366131322935776054589359939834043358";
    array_src11[74]="2969445067359750667338113744301128360895112367975333908104418890544020329817084933529871198124927554371787050106242,1404008885347131291511506913283962919008360470697416809160345749046224829334341705764559409360369249830158339427844";
    array_src12[74]="29074264354782347831311065435681335216170864749085827394398729739415033670567588054463174061565098622449386948667166,18680246445332872572889766653774313375549448202514557803797107454246566013428029822344248222346609983136850543369296";
    array_src21[74]="10166568734214397460819138995204967250907399575386161556628676424132194275952088625852898300202439371945104506799506,12434545770133403290509298994428131534556242972243575702871692778512765396450447145586408698643109513771566529744907";
    array_src22[74]="9147187868329821432148413561072632258367949588014999778222072267920915231721282816758562655047665911522632250972357,12284684004212855627742526338575771720129235023481867659044386759353007250864846476133919250156884648341163676658447";
    array_src31[74]="7413221904576882757966651737583594008040515156932717400513584962709907849224552093984337233628699667196250630901547,21582845736211790957765762252488839878304921253855939487706899959004347870525847271106503824043839512398564921677942";
    array_src32[74]="10247337301444889840315280612828623604422712744797314984709862346806505227455793825171196087351129522984289281193191,20304242820247590799195909318756726488852268491184935024123623183277493812659385533044693307920552634677242307984854";
    array_src41[74]="9260862939624902019404386376967860887612093540302836087942829626131642165138553999148956668593605652817122105778076,27816111389634681397595487124531467425591785972200557981007205860012170594743899925339293517748353080282554614264874";
    array_src42[74]="33170549408342673106800889859062987653469477022380790096957244050877834576081852350087088012400434657179763945816759,1787340051071367181181925095210244987827358204255632654776677651532189787533750916435649648166064538267514137439162";
    array_src51[74]="13130062860678923158997645274589897314673033525366013511721369938545462981051356398446523610955514281219095153869164,11082612424114625104090122486940549428279919261725337983076916318944929305023045663910416043161471428502012031198981";
    array_src52[74]="28340399662269101048765250687022055354467918243079326042218075634598788119092327117239044099945306804766614349308769,13813213499590141462862865461551831503912761364965985589870397128362375934683674522371668146055297048340428992440843";
    array_src61[74]="21638292152592610697159237387339191052645595212331613697039772145841273554091444212414946473708181406628564904050356,29918526833726495427795225272466760967062911410588673510178165542929463942880805753516366551338992414459566280528265";
    array_src62[74]="7258660256063752210188698389387279397949170506805720164871861814918675743648099937567340195487882438208580613253469,22361618139141694795499023185069345923136417109065462779861429740030302627752020988420601661473565855538172136765242";
    array_n[74]="15304816115771627303410131000287219292843619837799271637642691383698655900771103712483778130895535174486156214811065";
    array_src11[75]="25934339717264171893719107370296261581332233713602141956269149314507791841253908329348732171548135822648208842825475,35786451441904119108716076376340289119742159731504762987296975413835352346178624188521747966449908075678562803870494";
    array_src12[75]="20626370882554215920658550881738943263682278281257733428689847853239027831662453254316433930221730035900759421180677,35027504746971974743151623512576208572555315060287006599481535241621757508852756933089613205964819783061369738157994";
    array_src21[75]="13409129009190473619912937484534596261375036281646385246563908374830948675841283159183721414408102568850807473942021,18169090094009098958536522130413602320711992063001009364812967930817386683210991905806619339877078153505330911839962";
    array_src22[75]="8631218463420866369623259393638816104939290981293728898160006832918802947401301703409180172900734755864562206174582,16884110287511622300791689028250597561957274192552408573069997329195384709354274833265959086898007484672064882269919";
    array_src31[75]="29566505010312603574505486587680035170672807053701956656689296141706735431289944524655199823611220004594823349403401,20039758007064676618950103024271357637562148892411007750059361961974267049501323179859665780923443228602182135731832";
    array_src32[75]="8515632159107017511555420542509677701827046562088399044638310478208440364886260015721493125878492767627896030138646,14239947931934337382313871643633350330692929044311328970558158343290211100126135413587582874915924891469085599846488";
    array_src41[75]="37694522773041989755731380553180696353982211292228373001773732176214908721806737609940847949705802363639815209204637,5851454479604334357660192902039733115357023023481037597659347082825167301685033231902949402542804202835343810442310";
    array_src42[75]="27569822045512148102354989972311310737771781905421582477450164169067886822347631916101376435383896656218992831833306,38790226795546253929904987408436805383031933193343345287618968385638176707282117162536732283612094464332898412256072";
    array_src51[75]="4195213522165054147329532694694949702817685763809190020732788819676832309505503935353172062367160915103327227616178,19082451643993397455535716728380533289876096816830828496142482888401584481616393292198779594304629183752224734334370";
    array_src52[75]="29017752533338732018248113895997377230969134521857831904426691837219352915990514902927824583158931728458785530016234,31662693958833312685618045259272211380672065486593883690261916553623945634425752838084070714877631248034870663565805";
    array_src61[75]="22398861368984363720359741071690332555379409042437398239728770340995488909111209761959099266042269193538737288143593,28964040167793639414574191252714786729404613549498519205659434484829232996334592690740721162607424383206702709335186";
    array_src62[75]="18991045075788141078927159073379304679934095810726681139865788801813243178075301523483936892898938820904190627630217,2026755172764039806443577131259870975728204710280192530718170554310816927738598350099722731113778955445392333881162";
    array_n[75]="14754409882658521825526109608898205722693011754212448684151135995225814652465293153801245069190920329868373405433278";
    array_src11[76]="13103111318947836541036981530210894126790508506139867001106633922725651896587731453654306844881736504279606400906761,34677453495529955800431179343413134024503056361719830951747007646170410366686764866082337493511358342762531917022633";
    array_src12[76]="12745250403352627198963626614400359903078036347754275323201032734202611659152610953089264124830641187759515479648696,17353916726772813684180905405907833234514151323238188237358586839321564333838959788922252055047084170946903877956147";
    array_src21[76]="3060847049697909283326873372227820288671038261277602843283550081452813367616198080265858345599248338264131682804997,29065977858103248260557012933521193749298035266750126786230109887638304544447392143197270589987584248215167552668145";
    array_src22[76]="228320480262483297849980453347007208184605349202723296839319170487194312763797472316270839484285562401974713575914,20576185527327641020251338375821764361015941454090152791470342311793298827674230227189250905782649302080530553362427";
    array_src31[76]="26347203706790660221450500855350987840860857000172704272517441836013294284000123110768201601313465000938270519838472,16323456280421005615679440917992276468315519267715979123905977865431611307825418888633420861139339322736067467905261";
    array_src32[76]="36496511184946149627784892891766781793609863146234567563791201918810043600054015769428993672748032430928067027769023,30798716014054558037572728955174437920163443249524065561587076102797906714762438111596351492504385417523730213923343";
    array_src41[76]="17353244197972529124174039112455342202884078185665384410495138386869179668198939132441926118605142194723417075907715,25778365864242029204157344000945793582128487460204037852290167518695337564150106405211181582670007033913163809308120";
    array_src42[76]="36858554862341955477355342633970768233920162785171787621126986258695117624191531139365354953130081041380257046122786,462527899078515662119584414387171585420326512300930019764519884358772595262942724389990450335148117264584596920121";
    array_src51[76]="25219252693057640105136314978900201154442753741494549970064226981673652082225778602915333390016865726116693980444861,4081042584648608432154573738304596564454790379114079258558158923431187074893757143680566539901985065316660778011576";
    array_src52[76]="6579626194725051449587903836923980328290002313649776441215598778488934531446433494879692743248540243922430639754845,21009645338491944196478760325066299489434909614317648432519756266400666181592278365458338949224343053590868654401615";
    array_src61[76]="9747763623996586350270289198025473294559741063746017822603511707338377947462548290543849530997743908974976592166028,1085976577688061468596203027607878105223678687167341893118229550644427074288403212011108663853423258227719457004465";
    array_src62[76]="15648416181171190035705471699189486662592918141351594873157935204937480278275176134189025347903590591484818327248920,28340094423119550159728592362634465809588964038913692689270270542312911602366112708466955680418711797531744081585465";
    array_n[76]="8866165877333765842190452419974974607142570178062291840753223437467620730884118153074868917398722589580832348982684";
    array_src11[77]="4967526738841537210954745482856452400878839969201465767633224792042610751024669129321445074780615650616991393117927,19112802793898179749955451740414178324386847943275718746840974267113808804596340936985406691543356855078227714805416";
    array_src12[77]="2959471595599349513634636035263512563987144037824181747146247444146419507845985228946803805191857057806614395133876,22922735652717115549853538452362701218600680901119501402403333003026247711898035946205030616723833058098851458492250";
    array_src21[77]="7475664956015020520031281922870900544839627755092036359319150302406289523575137612625462250087013606707604226645786,37210478886440776110329698047210985794367326509710629626687725584191781106691748421382696053822794838985321753320302";
    array_src22[77]="11402423777078110445011361294451484646724651586087227548022502549317432050644423148083437335816133183422731761258174,30118990185248756886880575181601635192521714979803538935244095542086801594197073022026990067006675498077229584965732";
    array_src31[77]="14257330739987506808889812842468159434248761012037005337815788764203821327947545781087777444600157613428705991423914,11851875114534944912124664355051669722655615213538825932455536510006746581318493334414981769841962904390716489792104";
    array_src32[77]="9630829535307690332901120967616950790982333563307013260413684462462170542848210742261599987059827414853551070437985,9276008437792149873278910470238743322618738664416123538971331807410441056367389000466444771585292502761505481993700";
    array_src41[77]="36741201476626047428444388450155355431139770582657520267288506545966558875832841034728984670362741444926827644135691,8131890260270893250693267167885239355746416176278949507726375579069825286291166619544106985247062799902067753653006";
    array_src42[77]="14219401440392245195267680953735235258061304253046305883945564216726541651045922129506746926257879093119830194525605,3147460544026946749618591486329932612146875278938403543414474038253553594960818443876408177167922448067914159758377";
    array_src51[77]="36418215451798102645974468407345392069186023127817290673158312339581194923599624812967408044856747251052169673022058,33122880362827757238289560653022028905240019343975314905891721352951322321961370192952168125177103624510671444685873";
    array_src52[77]="24938011985476524528928914385269859156448433159025192676663846098156899108143919755793350825168779832073154647877756,30539654563618257223734365897190902303797918463052050511690200175552164023584546360587136197602506012732607401030945";
    array_src61[77]="28326487753561000598232057050581031149410698525434187772370876780175432820245453125753407163311514901107899955415181,6724984253334932342387514673022565337996050418614034238371133541527752225510227410446253726581388273649849739796070";
    array_src62[77]="5888062465816703422682096314452134548213752306822029716273524115455203159101453569431304177475332071223084508411897,11627745652433498049181826411315110599328214270564465688050520728775419784046526800083863432069952128652771491620382";
    array_n[77]="18015963013643284172875187395633785675997683878417331635430833199713081542043379741400911262569663653733210451448702";
    array_src11[78]="22536346746311044795352330435415721929277824390519802002610928707785672257495192925014712181824292694998712296373387,3448309736087345502224528237192937425233559905899496726016577154045406763915893403230772013223569786261822753488436";
    array_src12[78]="10306802625308161210569839164712307465624690316570952673383230611675402929389935180368880630802085137718916540333324,5621745070983579176320319227435703584657491891118309230891552083140364478359742039722064685088928796950602907004040";
    array_src21[78]="30879930979439634101570990725838247256849979297821808925702499018361737154527030810963515954335674664688694005370593,24829400635664513483190343735278664204684416834716301023449857598478472489818239065981714926880300410621483977291232";
    array_src22[78]="29428554287067900430351005544182065646468560961766503544442019538541516604852219437176242624275133590380786720349502,33639319380107892623221860561672258681236945960012076977465570398017157697220869580398381458727850571504759910265938";
    array_src31[78]="10589705738651979344448568309027395977269667870691117412365437628223728641507248740299123844020511806292637511810278,24677157814795417227866120627050200588398259372802487862028606648880114984198461581127666716132591489338130778612556";
    array_src32[78]="13517039445178759234264332308484649987766697222350366868220065477513478010190151784073453423162053757258491537832213,1081995261703507973167725621763772638735923377492725920316641486389167775441423029020092195687598764341542608252447";
    array_src41[78]="5167780489459590199922870076649279753169586192022396325700558867398834002997322543710079327694894918109727194840360,20344950272290434772826802041179759720737641065228894717127470103756598357725178159549612690461638033002215955349240";
    array_src42[78]="7138661058376004130391041744439771833269086384064810279301235568935268636954916411280007560968267742311520107417923,3962985666252182819972543600114315942837827780803673183235633543636120368593447257255482636291124610798869689501395";
    array_src51[78]="1436641963593207769791842880892977540214775504961655296550391221830709989611505729486597531273052155109008167852413,17085517972922952885119558743710245770299031544066737393588500154631610606112550927023086497358986498566406359512183";
    array_src52[78]="33128801846104851697184452028238464197695663847168894489468889303431662413629589207167579526393500772982809523163936,20406336807129583906085706555840105033729748901069535669767446460288579005142157301896481641863602064330208907312966";
    array_src61[78]="31540152778659208524067159099550861766468142376363796718133944675801402823676332781262184849563386629599607270739754,32293887937266537687444662648590017513393494695765157139364880375263661063157702732719234273319536561615838955568368";
    array_src62[78]="26879417799684857333115540494097783896223597469281903595295597869321922798277299313306637496501372661750654436899017,29603504043682070078594312945705018006998529287931849922325266184049258765765987191470719746122794625655254648463945";
    array_n[78]="39121493015983755441274750553889755499279116751471438177683483512148054075549364040092970752556722301926738402227811";
    array_src11[79]="37132405468402928726559881854333957191128440529726016087579745613516839365404868847620105739666494045971605960609295,28524316245042027171730716092708157584252578611366276467199702591721544917140580014268721746373878243204995753982124";
    array_src12[79]="8257682774606385861673205061341600786698232142132095318012157040226203172409679924607394729561705675585661158072769,2851006968953453899842715973919159846632909218476906715896975750999697579541177168466615483265332378074481624808304";
    array_src21[79]="32001704620208517906191888702407225225303845751007673454777735060967231555302620488384726879022655216752968604348116,29511177538200887782646835861067609440316819160311526315507603966458514552905357458190069621658337931799180419028081";
    array_src22[79]="38757028521836549520711163348988449195926494437619864967463094402511902542413074186781950927328909323129798884105173,23866075891861547309019455008025309640626348112362603643523071538369360283606221025609010633567540982058793444458202";
    array_src31[79]="19688266216387833688967453577679708909975209099911897459191589301107096888452035013602629972813679416086249016895516,18738243152671240986817755869328989771393351356579076834399790255848206735249602654686001460312034521081471775096834";
    array_src32[79]="34176260744932437228459357514329209219003832094347409888599008960222968655363767884042792982578334896541796220201054,29084392867573935741984934689519473442226347373221206513091242170645015325251635065357051940251204392784862333486402";
    array_src41[79]="2751428983664563945069525071974634227889002772161361977691335771006148594127522817271907864708101514099352654117421,21007265509019668122703963707322368373876484551601626073094010003446521502661986889577194719214122677772805025396532";
    array_src42[79]="23399788842644964870056952461092920461885753743054080408154724088623359672717302106722132594847433829790253567487414,12764611646432089945294759243627922354766107906229788158146470400791986427029013827461936541021919307907664839598115";
    array_src51[79]="29251594445563224063755734682573376687019055917056957206960837940085489793198387807782696832314902031547673456523791,971287620877789437604470585253562561800182663405862677776641024001015546278763119873113345368061371890796769162750";
    array_src52[79]="13383276303206198969463002199114547565430376740151825330996632314070134962565183946383403312254175840907228206592363,32947667442236158886885644473110281567032361993876695688481979831251543590812879405045547393037079575072937694129021";
    array_src61[79]="30608450051860721686150299329587357356673892961097316076895058301688877253330765987882168178202953319356899588862454,35124654718155443817176078013376403588053036579904177553161595511106831420513224484375100422200897998296114903838542";
    array_src62[79]="31276557909242033148748898883114040548403460699673758556978915641485038577349470918294833446017630362064086712976057,20474694910212065983198220320650059416012309478553977270371272441183924542188083320135643286652061278403414941752009";
    array_n[79]="34201933003368381007663955054205486510200270461573919098636066005033265895102998460099673525895040387435447181533659";
    array_src11[80]="358704572100530627369017376838175765226741998208248880257541283461784535657853510168125080793912030498209447178102,32495142213411408857508645448843020098784642286414024016097175651498186741928267219267888140221906891776296904357024";
    array_src12[80]="8514078451508563897935340945106214080700750593977612462779211721705938853123184632917193339471413499252916073205985,15136458365208775110617066753355302708813677172626780609213322274535592230407818156694810200918550098179747086001988";
    array_src21[80]="13311377857763926748836387647714050961248088243470771195356966973158254836596406478129343026482897903116163707315053,19258729798223952643106580542953391514934031025556063820568980501878786814424463258237831819658946760239649652999398";
    array_src22[80]="2942844983123472226633432633873652761746588781621778164590604086480569728683704439894571088156471522231333374392322,10638074167838317727610361288398805468050228016350720606631484795992073543323548410943902817566020516187240027792525";
    array_src31[80]="18600980555400885129647174888058506494130005201617537526492147259364472750757414045487688305117308741582593870626771,15192748562649710055151600371313082135239184009599996397857257149996565258455390949325090810500671517952850142210056";
    array_src32[80]="36989091639049640961968552705850203134515564358839132696070265184825248558641934605624893225787451919781333201316298,15717633610650852988541157434847247833654460327076469342757811329148485223492080228389998097911079773693442309726230";
    array_src41[80]="28013356961282756117247755890248481048000059828767628025506451739006717982947069806024961192966057993456118241947571,36932366542544922752747750857990955220635767689589685777341049246350689353351143874257410301872440130445194778064900";
    array_src42[80]="4142063700394042382351984894883973387199801869859483236637130367066606898748570426441228295450131604089073665657382,786734592637547762963135539057305924700754313784294355230873278409538909644373146285283732948007923679707144909910";
    array_src51[80]="36588953559111455413329604196566689666487746396848613261598221885199758683686944965805990285048659320290736654793145,22926651533017989278736886827531044186609411579328524921966717961188422631578590834087256782425989360537565610105799";
    array_src52[80]="24806322946763133285202453463580176334947834299018787955256801235782081731691769324021686629838069620627397164042458,34598108283582824429061920453088377557802647482372365895002075870481513593495242338188493015227639963896156158677025";
    array_src61[80]="2031239214056492459147502397825199069670039757138056231122627220518303814677333636946773446404277962249667910588950,11706877785756361246739806421422144218661726951679516976926370016155342481411769980203965746142031218183061779955505";
    array_src62[80]="37895303772998867714003467594510768515247042527769848222425219477695466333331135857434451936239315401140277009057100,8132973774317585934405151915809379342198190754663484584374905788068373528576046322942887905095707945220642123877313";
    array_n[80]="5811640215648430520868007097665143948706607996396373821642845081331051477780020850873365497281917114343866816513995";
    array_src11[81]="11517790098349040324069742922286603357284590982515675063094231652774820759878442633673567329814065571149418441038170,2027798082719273176722808051650071463350944274874836213461701497711897772444236594123317131661894026109835073108155";
    array_src12[81]="31991434809390992007055334883307849032602226892984250793852619591379000911727537488238994976579295852677064682199042,34718850220052287549960300425197282708933768499026224838205219324457880441823986372483068096282843767028605590800265";
    array_src21[81]="36878805293641447713457174653499627966595265695347036220718129866723903862140266360691275407022482095200351070636010,28163380093272623578727872342382648590918727780945713642619611132853213214633203886543002448240901383342381101880923";
    array_src22[81]="30730582554099282040099922851005577820752473025164052798268033498028576679399489783224819653629409442678812261770296,30960466040376239525263425665012839543263100724040666897136198297614762106602014572679696156105635589028772687828768";
    array_src31[81]="8930547280318398085028391682645477902106552155663277298758991515019191441658344801230828094404498881379200474142392,2969493309288699103060353672438866681143303521509967005439120908550345527835480922968229847896784049652056413511570";
    array_src32[81]="5881657559090297274377519115100627127244910433278772997170753750445607599909592349630723435622482974137330949859919,7826691457763375505005522498862486066117885909911564384308947106774186419425841058715924601545017655016878762593045";
    array_src41[81]="13818837634955723927118584641107370812409642545552291839083035804654835733658931835406825512457741398724871944176411,31110729181521290680242034317767951575621404749731929408720068438427823258590528982483791764991749970128168132132948";
    array_src42[81]="37921902798999082693553308604380749749628725256882562036029465923010424081901850068243281138139151394109879935368613,10307176531586697469765836717030753057177290947162449397257479923992903584897068469978361153967374098208904692908323";
    array_src51[81]="31101781104169451402872736505124828531417666882735181100839124562976999026476126451373731012394575295978047695216330,24608930735790004460799740313739475521371175846531693977862012851676104924894312668905761717934101722008743769856360";
    array_src52[81]="18925784194167943977267497585327095908434294497994783481702148388993370016578857645972082566877499491553314143628777,24709059862759828695692591090685179110344853730458369038407319398958707174998086316902006126440408573052245618669975";
    array_src61[81]="22332488361513947783835951858237797148623490594544811598375495026398056918320361272069696609271606710953575585574637,31312124774716257142082407624691555943646621484030134689231754423302539912296759855615343499285968782383189121769454";
    array_src62[81]="11446836200881557241068742282029061062669165565838370176428154014030365348295873399567080088704841771376325476752436,22141119157176990941216323185794878932151999202036724566181948132547739789621641140486105499807014886708279156844098";
    array_n[81]="17536713152532133709277678302652453663426246603665893999656500979569730136970719097132716025940820537105813935484470";
    array_src11[82]="39344621848739653485740618844978424794801768776183819614877613311272659840209276043059174316627761730565043633714558,32248455059257321728990926523416356503480324116997591857946470410504006320042765159428507250082572999679577064366532";
    array_src12[82]="9051432737697008384136549358863370526496339189553109377358465198244670968126818615263127958977250191329022892600286,27445456541162176302083968398897538192348908222679542073535864547948046728691733718321055059643667935379214184126954";
    array_src21[82]="9308645096467439271619963874948743626372290514753259176834413428548921238525142236939257703479421639130380393153971,27953163796067416768998970211953487583578156837784232244554376921058153031836862880369665658164398959511500300341768";
    array_src22[82]="28370517181084339173044574244166538437579742886103612904124595603842142160309195540502780757114643915523629689550100,5255092883295761920015729200123940859577747184072896551808254805347788220180308772614836625243215682181918314813898";
    array_src31[82]="27707642180720221339638734793471425364205735253322571328914433091467422790675955425140408486646580440236302609344122,32825983357359189585525622258763129568047310816394065920600537697245974301826812933999753259614764376907349418923204";
    array_src32[82]="18864781656208515013584430020763389698236083011524488558193502414862762457458206114688636960444880863698489073828884,4544940664446809527584521368728989361690226623305576592373381180192791961721998669470583182046982186777531253043725";
    array_src41[82]="8385920672206293711949287594096390875883691085036259061200687224810712175557403014837828322890665498976484166110764,9261473766866152904338539213927839874664786408304634186666072715791080621521707262418154103481779616514715014716871";
    array_src42[82]="16531220006844172380035252623079219103290544614084208711099331690380106271623728455623195878064343381661626888839117,14775577734965398597165056542139827991621509114005537546003732194951134321827143484624399922684365128612170924607881";
    array_src51[82]="32757374443196382614508685855983046873369173770317684144577331187579760550844992403124287942933741031665871494778142,32741915384099161843417108239626978082619639238281207226316396103544431191576543558326307395519676821698307987981383";
    array_src52[82]="20971505083201504419924799679054080633200188416501160603173421390444793704433257011119423162076980064550349987028836,22971859958395566037645148246707764851528079481553586051021691618667219147369025256275117147771233978714704122142052";
    array_src61[82]="9008384144837509919730077516694050072970953654292326172746161418190928778590809783769447999351815255203883146416955,16487428907301521225578258877941630537463426104737211321293508640682074290658000894253946352335016106953665815658477";
    array_src62[82]="19868900153199850464241992697552114562429879196389572474077764162689251249480919301594672131991536285276310283189568,24272572020201228266193517582099626765424704478227312557272915012409552560854491022711736226220768679591001068680168";
    array_n[82]="24533458016493529554697138640478695318572558780670014535264665766804465771418070535354238958225714545287866341797191";
    array_src11[83]="4366194403177953989388367511000010131350365623985039192941427998837781114731649500415466432610578028849339227250442,32444329523090440120108254756516105089478302097587828874734322641581023045131874425546619649008531906072724875660125";
    array_src12[83]="33773373274023082701653636497231992388598151127218631607704899582280281028274918501645922419698215058527417430201382,27415674911454966441462851164596356814396877294797015643441396804559475765741323389275664662497772243880654326298821";
    array_src21[83]="5775995329677279308460515056991939050380016891427759560502239411725142032261104880830256980792089989777801705330567,17241266208692018543232978727740204037309116168294909799999521720681044314485186271382423346232543117276980747942010";
    array_src22[83]="17538353923925611261419415755934670353396656444332561118169925273709118924834753859987830617655758761688189051728510,6367093584129733600726405552419352094465997928425290623938805456248100756123967694343907696880623599062249273053782";
    array_src31[83]="11776152164184357024533799821034695260548675396478550256795532565288229126378726220707640308534794808186179690642113,6483285316585002451411944198265134019880250924530782441726826476769501110727975963867258861092435244845420241326142";
    array_src32[83]="37874734695046390504058963590564046099998743601047064060611614981582481855722834608548355044376398049334707805867289,18586657860932427840660966296619264815780778574434465953088524036259829815218352934792584020681136973848560899106270";
    array_src41[83]="12313846940540207605968769896376909175799716900832168081714118941977864981248187498260568729676251122942271866035874,1987195291806712959509336392635890027020720920576206204415033063345558757610481997790985920529025415791853477103287";
    array_src42[83]="7389283927039031088670889550211117658985857852986627077417020042567850509125859081082538229102862748521299002442744,6692410214022585934679281655695418265419268783572215186041372481027948022249247445977391218074452291099484568867322";
    array_src51[83]="3557156080892514065869775576455221969099126331255683319412109722173546721245671286727465115376081922581007472500526,27192140349242313599883762137422752638637584987353303806119631873257237174042023224228409616132251990418059913341886";
    array_src52[83]="10635206876909711380727601436386738224449740099401620094145018061931512810807693959617754058191377751919565746741176,140658753913384287088688163449404955227368237856505855150933691034500235319667838680588920341463692463299771745263";
    array_src61[83]="28783581102819526838010315006771615158704736910239957777923435812873841121758123356572209617418441950718802171421052,31572567008003755294471812282171721242995171507702473651573270363663281470282715022608161979836207022534238009747856";
    array_src62[83]="30356540621091145563693946604260874048302882462964753271681256535940278808397962816277857445885755482420057937135105,36140710295867737714822497282391790021467938323866201524557059203768823843013323393465642553547095240175980652708632";
    array_n[83]="7159755893333245337267786769364641206323232323510799877006720907451492757089710374068857727101162997338824990787214";
    array_src11[84]="32796138946916250397011894196675729121536090606795263299059282726149484653468414493851150518554370321910749417385741,15467992594417446953659748356249942459893391772446709154732937704672388223193054988249023785174710076479339743211325";
    array_src12[84]="8291337517630494274408917525906855713940164995880579780258432908656349561189276520326668308481969633419755841595525,8217411385513819548441890525258110728520976493050429908451121391889002626003146735509499442532853383289043686967908";
    array_src21[84]="13648995492021687502352434737458680643984132876538200080778870850855671565609594771412136923343905642449039602117143,30211658768329451137533506253776708424299565540490873795419298882351426812104190149259600658929057533952743443633848";
    array_src22[84]="18111083848413995424060830555467391707386620295351896746517264660108902629115369711001149204753767944790038003087723,6868770316526835915654042411516542537448316142674827592243163919114514135638341606124236781638516766369623462535444";
    array_src31[84]="32466933330295441856535384668066661940559445541625095055996998849606703836521994817117628881712552385905262965620978,2323682118869962744842771182381088764418698598970377511432407228828161244523332243558385325062067444255975430422685";
    array_src32[84]="13404363093947099659734636881541021937988144031128066889544534382317113009895127474305976964930107870047941834288946,7559591538703632834360914532688293438101654853513679153125793812979743663652145623258814414711557542638197753609046";
    array_src41[84]="5302052161241105081716117817695287710502142114840960228689319522605149971444968187363117741984329392354779511921670,4990755928078308573940796405460574044621287152743025552213773053376063840343963338757881695232623508597362103800520";
    array_src42[84]="24190456298624268394291232797993406873628334181834403137221553221500501950944843874666956051830597742586954617106250,35376327288813931607401606093036222221976994093967467897412418522343985551130896201139040298999573710484173809712355";
    array_src51[84]="21790566465429114017648172032673432867265641985390402167335344838664850512310494513961926689922394899079347274237452,27756042660730227123440053272878622594177906598035277571321600545973145512612029966782342864322122006589543064857387";
    array_src52[84]="812062318803248803450779046274620774196654444088435508713096262044754521303416018348925490347312912457335578996645,5853858611287100133377569081167748787253744393488074312515587513173128538325536101633587057776765394289061579732839";
    array_src61[84]="1738305904831958969428398157749125810991385990215595736975743600308175857454909097533688822440825658597098082592878,10297751729955074682820628467583109777120409962001261190289445282597603225579182985176043305334484089710867065384941";
    array_src62[84]="5135750151908476397505423958069292289141363670066614189316783821239916218770339116923636804246422771381442216377838,8519100969533849546878792415225896559828811049145325104681872759090200855828789104886355361201294485127070479767840";
    array_n[84]="37132365583712555629865890983900057077687258046684251160813360273312099482300323543020312541345516216713096376113031";
    array_src11[85]="14365688436572238877572058716556998324742816302895596686458106072338544876078954037467051385512186828912782453471748,19523614764027938521247907037447417789467298947719273415254471040480099622545649035129407435079754912480598564969155";
    array_src12[85]="19886082034689946278993770512660582883627402619279304449863265107567890542412639906627326576609151626725088472926675,23911989477325437041152218073960649659593593108104606968570217987203568289258532481622284617786960790980914040031706";
    array_src21[85]="32120873488702895862114779839588503127524163673402648134576952291658861296814150989495604579416994920592623771309795,24420717663388923077316301847623093971837167630766824943152574334041560115052917659261833832165844227329135337613054";
    array_src22[85]="28687896986749724646821076515010746306991354165877663652888771198542286640449203010047087691931936992724788886021046,25249605429370073264223313671783425196656637303089300860010679605073242782260922119565039232334313787546760698721891";
    array_src31[85]="14552865604113927334031821173383505923896589794624051376204841836894411099228641290346788690705744609314793400120141,11400500558849451223055285671570384908629358992383343954575516305969251376176606772749495955686090975517853729941503";
    array_src32[85]="6126435637064786840320708995475328853489519186553102135886202383146323597378735376296863872226969672533987150032145,29638434747193819154933565529161401585989993031683361540389827634018694480781386402306855557724058453665603281837123";
    array_src41[85]="2644786254376229113701126573176376458950499964652937268673539448775057782200993957162454345992102138255061338295378,1908268628671908236405346370782427095119064836824842873377059929414060908884222086661645434451204426299246932935415";
    array_src42[85]="11921220919112674098330833011016784003483757543007134583054886136401545106754495541167401287782145269910357848841778,26991251161423307961578259977268871626051839093732561288865660684934851149516083759897955368609959460559157646653824";
    array_src51[85]="12586757533241977508869321128257456256243949274691971928920191017731908763418595605028739034255755195448419371726295,2044331639876420558403372252183882705384167184623392970865766354136624329579377126912272154869301977711663111879073";
    array_src52[85]="8000934961845393926099412700400367652764139819380676438488301469473172646943151775660215572081751185444011534447529,9895001977033216660325093373160012772246202436791342465887165832528793655811588030665519886795106617323894970657681";
    array_src61[85]="4491318171656459147727719792089193815498344496676247533236205380959325842556819256012304463422348526155914310885745,30288995656228481205541671832472020241703678793543295743752321207769219010529222715906229625969015013907354466284624";
    array_src62[85]="15846011880670390020697623628338662553036240606516955832911970407803715227955784541551642569142276826966552995120892,34909880635078272735663594113595351362033892949477928075290571851409280167566852701601048630997717312803228506665555";
    array_n[85]="16604530346561035256107290370927764687678839930925084317521244872414221201485081321795440143136316791140891284823029";
    array_src11[86]="37433891774230061387947036981310707430323473167086010723161799427509782310176890419114977554550883417984337215084986,4453030091409789361402781086228635332828093548111577742211932135350395459615019423176353403160067990062540502353604";
    array_src12[86]="30659827669471343185666843329555131998154642045367251787172629041851535113431123351506809755434797268135730672779962,9382373578989472258947117074120505303033388127768085914418995296245190671439462667595453406904643238296357033633341";
    array_src21[86]="31029997438268409302151846034612938742215257568121559605536579401352552203030113884810054773112187558057836841087021,12007249568940478735041689779996323686059672481121597849034450056807509245410079170660292648055477668345753609658693";
    array_src22[86]="21847772258141914731710829894265377941651739145951622850735572751762909396991766607335835345364090931889304092748044,28621572997455595143074327428793206930997758624879661564982458016155318106368302004413933694368357622198254323686447";
    array_src31[86]="8065367338066231999835074997057092289059579200274618770632657987783541272631079251225337454356628585824660409155979,28259735021343727944295962828535756137119644576238387519963676964990436558307927952077164047768070519753156405174911";
    array_src32[86]="12996275077392562181129815997687176545408110494572800190193969891252166076957805479352913034496761596325515381223733,11679711647747885762553443900949006756618480081688869803428570818841859978184344351741449651476344198940434563706722";
    array_src41[86]="26492664092294321456567090030670981774118746352781947157574481079101389461484555533504971012876393132127782814128039,28600011885423044461785633840686081647006295693300253060764344650474457508589524828232755362627224277817159912217219";
    array_src42[86]="7926170858744148776075621953237440178067544119289043359517023959106285721724890267866202553855112835328949626419563,5122118350741188546762909253837696453524898926319248951337248966688643887948548170426529437490839300441678472509948";
    array_src51[86]="24191368735713065853387912726185857285344069242883429834899347563420126232528591306374795572927113050986349793714432,37124746790808109443116577629711801597520619301016016381799272349677308803243748320220477667416113794031709357093818";
    array_src52[86]="4060899542357793599917391827556274799112951714307007375230122318202782047416960094890829663395008388894678017124858,23249568455212257241979572957691858334436632950832989444905108038413597618646794941261152058574608460763463714566320";
    array_src61[86]="2329387890329795906755240892756411374478456016439598648234312545614836395316042617106830684089982410473971483365426,5843794901423384262432415539717642794851991164877020876050011628358531219149276848428670198702016167146836878706091";
    array_src62[86]="20417172983640072080465811190341913154892411871229599485116755293710371095083041348647205832777199884696978486713494,28922300193227570769727346866336534946940973494600198916195176834103717360608894274633948785353328544446918557097605";
    array_n[86]="38175139297793392266674466762254085884187409617181342866411427541355451597493938572442402296738648711705322259605096";
    array_src11[87]="13454621681980082453982775253496062566234437781764106199509977645809232045948130654634538340190393222929259300006877,7512958176167913765657987410820301030390959433709871628853477581756062165277584885163009597743471262593969491995175";
    array_src12[87]="15873865331419741016307320160729207769645616417781172998571730229032733261185310058880075301288927692393714667822197,3106094734080906851231856176225758976544665475785809371149639289468384501529933912956646147281773637716988322705221";
    array_src21[87]="29125312448081482987719667773174428373633465518046012436563497886116209962888280064342706019901403699115716091326902,34926428258819559730998504704847960402557162732257712864908317595064195651954227683656941738089463188373238489922407";
    array_src22[87]="16669853310862956393122469263919226137075013281047457808024890959783426386397324848597000936947834884408484449302356,27054423753731620098863259394758820020147210831097295714709741601663072091588304314441952179331176447233382522222730";
    array_src31[87]="8250221829877640476579935676083449281666063575377258752147030521020496028760385713814619388531659157868738758907179,11446241009679738681180455595324062239816871528455079588273300963155461790976631519224270257752144995656936729021946";
    array_src32[87]="3191403030323410729522897181488730525039027265023968605598150945420633655061899774969786567356386210987628760377151,6830699168620066213569698411173782928755472717647106679609374747651640646841223215414148861970327461733681176204013";
    array_src41[87]="12500864833212909150248653997277726713925480465987727008702831885609829255124721643954920323347327003487643957340364,21943348947849604430039484839409291779643163397966974565874401267635436776826066545770487119840419592192457290344018";
    array_src42[87]="36010864067180108209842992360157749925779606601795489235921856816731540129301570060309878633185006544544947223812585,19033422028231144360109033722359128834173718221252617458111331585479709882173523751892613909557022576926612533132178";
    array_src51[87]="1075920108609440058428770451626216147033811752454439099376116679457663472004533056553392984238261799423392301317961,33701146266823757902025016398757626214066872117353511208632467184028907540536423374818613283311414376498498551831906";
    array_src52[87]="34311685976921751445655726783483449843837377669193509905341382907569385561391471019105377095891155371840923402832606,35357369350798871477306676985588599359841023336613151274021972704115442688047670136491733998832675171227694985810294";
    array_src61[87]="11239291159996977799934565902714741229574897085418892882636053546080768095366747196167795527891928767618995031712040,25875956193077595149495330618522639529344614454255677495879699353682810591311823041625189798794408648656603532345938";
    array_src62[87]="13238831063365758727679937846020445710141551634725868528312246260207871281663627475081710422290145961773326595132348,29484247557006225232162088213691363410330323715127486299292690347086228354193004461584582493862794227883900771108178";
    array_n[87]="9236016080907380052250192437443557061666059627088109042844402157346868949762277863286447704323105914812331634999025";
    array_src11[88]="15267184598699817661221406281947739849999042052181727575395584190491354013598584994437314128620415256909198597074503,12378443808974579023897578921211104731280083784207954770170541459039978322219159065294465700161013058213209179159178";
    array_src12[88]="33712823873483288730061246391960436497533697628090563535297441693180510430897781407550186024948391876456921422386805,7753643720591323210357191829907908697436712554342086765881731874723251755942157273457012383895363722104857000364093";
    array_src21[88]="12151144525562751233304125990872004345714058741599875687293904353251392737162630613512972231295207699673747371093914,22698752142897893032908970853471336729562545151042973931314896165553626937754946668212429804779022002279592355577770";
    array_src22[88]="13885365458433105751778999067015195120579022369170647680281905229692096525662123166988262380616805970523526809827609,24036424887112426447715984689091621104680073548781842705458601545154014733415439929852038017872739920076351794812247";
    array_src31[88]="36228591936197400846848462387713775799588718667986464607668833530008306823459220146615204987526783498391409113501736,13808533753268775405648140376372217812550774616039643297063908345504904600258703335737516827570014966592219366752702";
    array_src32[88]="23139706493268329431044696290292954705314151832150283244063135274628688564072606201517597670517015974036256430916126,38037515491041750080113418862534540856710954400936865739846402864046815353185489759587449202327237078655882468076814";
    array_src41[88]="28089783538797921719469503519388085252828493968108446019757209418903975261116145017662885901618982240516280315491605,12656603652228622110195687055581335935763327733129512736927259683837862105260060252511821297090464399330964707754187";
    array_src42[88]="32697321705551300551408921796847512789535268152649400505354957606755692690974115421082478765735778405026673617205887,4397376386890880810866198342950411078518144806095219121645017910417914773659373865145670143034266103939755991706655";
    array_src51[88]="37381565793513215469794250306611565898631975001552662853417519809807788079853297590595918003156917880019344126847811,8534134990747255084943509225165142907542929397165446204571055029482996577285921162451508853873194926320784780465315";
    array_src52[88]="11228887742825564837819134483947473315455374009338425631556842328647099109045876055929863010763634740348634615763721,21876267902582868366475826632192165904241833333098821892537234453382438826099847660595430492815657242758094834938455";
    array_src61[88]="22690073374231218793992061007747539876706693612441582296751055063552345810364743644941536423633761602453970012850201,30998772793038991834103852968763271014245701927657341489956135594420259888436261783517342813705169381784965224763279";
    array_src62[88]="30560058287952141191557746248602777304158855240664392652301376445911609908393786169052871091712031021235390234409005,28093829292241798226847225621967986136809945441261415694256240222256142386339076355567657774367813789749685849143287";
    array_n[88]="11688675555754744430961784434294828330610838540414710790236343677703773067354164482297531140479604862215412343775817";
    array_src11[89]="16196038305706238636237823244637244389107435132146012688973128467692293983869423339165315379828863565801023111776144,33063639281764372688493388239060623770049824288249574722443883860081485461255735814342026380842746511794439970620363";
    array_src12[89]="10268922976113199433471742952844444394155893291217121537332636730521484791522209555743699042706433315628829975029088,20315258986513407096616292529460792600548681977578756739034370895327520936410186579350870695470527355598495004061975";
    array_src21[89]="24586770101998345352979948194259525316415043150763660738940736095056898140994146353303722864457676554954481671659462,6020602002813892589826361033159418955210671967491322855482654494544170871999866739660686047309691431233443490668218";
    array_src22[89]="15588909728201740153956955923472200407915512381541877660874792862617480568871205815041995535695559500090277130903489,9049033036834465190938537170021228642398081751700559279220988700382045026599730065764040529046512408278711326728801";
    array_src31[89]="35305135287485466484672418736106409049959395835065525230407488845628574219984252121707781396286430363076328804807550,35638782345404208157580739351005650188523915786238759181988348947073795437439253605196442190237524279053388144108234";
    array_src32[89]="3108742813020963857904036821568714407916579401989786757354994281024279090125847080830514972592948832431448364745325,26750008216523105098836702808476177887827271979942353172705799811605843702909916952290141821809808721966782324147727";
    array_src41[89]="21917646139427782394686049612200217146513889693047813042439479075220838032026091175782392114476024159509787761941468,29675032456110878868547810454619920477102621848656495749352898226521466528475077953164594155266680082302812845780817";
    array_src42[89]="13054702422565121285661345713644557556528425116294761910108852502936337367839841431997417729291066176286537027700244,6350029400317591125009830371102986780042556987702541916856221746389809226073687491986471574991459640809736714751585";
    array_src51[89]="33131328302433262177677758484008237213253054726268141968427230823496199618392860508364360148456632818461637788981761,22237223081059924944206084007204007970821285877642616228324463573257462441438963076483546963034140126545282258616127";
    array_src52[89]="8846231640064607995088148400863533633861964323963448252950862938686295965579033379307743281182700535211778453896860,38079329070856864424608219596058714091186151411845728183243363432098840812182010459619092511783855334376552732916520";
    array_src61[89]="14212784261042771285884442568328498048257370328900634061719001552970042000637766932233926206857965641959437854291353,20973936492946264885913459902094121580256016378160006090769971819851427713341011999270821330424797376911416090759174";
    array_src62[89]="8585412937565878150950717341385639208598685098840986069638242699375922846228867976841326459294041371901294905844719,9883585053807039840896744710970009221891493253495707359853897542009312070029234291389368128096770514330128508454887";
    array_n[89]="7669512696359212505399407292283398615989679231009560822998697733021969331657814382812585508154259733126237082973513";
    array_src11[90]="23089717238344727907709804179598304032629175914840148308477571024368719885434914631502555716216411625381241766762659,19117292929407915169914375037274457281193209512439273228060209355258939163895101845894478988584632653827601911396863";
    array_src12[90]="4627174333107263913855794736264677737316812239839634853551895915173150996613363555888140646462440938409159791988919,2599486539817044666888363866968218298590634211838571500913716879451686583588505498243666935345389847643862700876554";
    array_src21[90]="10691339032348172164173224653912896432305617235702733261806920810340043199779834493746648934885321536501023890667345,7781710087297969201259141232254672653577358975494514647411661449496119820113123083282856297647200880425157805850676";
    array_src22[90]="18324296832681317794292494104737862225942992637153224602604500209865487719065533850200582042006728000425700663148364,31395430288772755170213043853075672290022246294843179742681389180745578770866136633753551935766656865875047018827640";
    array_src31[90]="14440870972141514362649928466644269129042097606838503675971754611697018142931325522722516962569495852753046956319741,26830805410630328670850867385669801525850051716043827983352737522941919911652842408942112626573053756763864675334585";
    array_src32[90]="7515608631128981335765125262288489282457338834183069017913960118611482247286864759169050091042404133247168112713900,29269932955646737671653088645924379899885683158702599455761246480746986047900916355437110728957936555351001568586037";
    array_src41[90]="18104990642470565196400577762855458815923578931444677554971508348272733289597377342149840767789433412159580814411417,38565476536988554020580290445637503420465265875094087641979581388266359418217903971426546353561747805365236057545893";
    array_src42[90]="31276328410042791829829432674323012706459227147378553082750660350237077579925952417809259872762738742247180728376798,10481477422590134557266895482736914882457463662844263918437421713602565544926903205057474948406406027985610343888488";
    array_src51[90]="7103482769621376022696060167844682043140299956717263944219749776053648993278367877340955237302592007231738831558006,31159562808385826878327312396347165020149499055413535970503227653537244281665397832352005728671257027302779196970536";
    array_src52[90]="13835017115499475140733243199866451719768758597784778307451542148848274641088025468003955977293627973016168164083291,15344208401326717033076821754077092062980418286825290948338465735044812659292219431637788970654270427435810719501640";
    array_src61[90]="5068163183338633492696443377152594715283177367448410527925845581319994449293616181985837676714083393047512482960370,31068558721502745401917235399637163420000898486434287724357414561815151200858559168885366465330252049187502676429384";
    array_src62[90]="10251826990951460593647160680453510250901462771974823553964514300817565921446204969703677693225639975941056205922530,37295671818745387998724332261407445367551490372587331194823576692722143344678227010610961182762336664278210906123902";
    array_n[90]="29206710623559710245817669194255804264310131234434320370884564982789528023369499870604499582483432943477336003333466";
    array_src11[91]="1970963485323418607061050708732540405374146940586420504720551532427148129363622358972670561027380131641656575795794,18320774234317080785028674926409449312589756047628220442320619655062599711638669196383766162576831941760985932536184";
    array_src12[91]="3814655788313241440704050033860768173078782263862783735860465115242053687381352272895382328606416554357866433336783,14416046986428045203532530924662623152460355172817676046196273871532128017271681211245026057011393641777045427634143";
    array_src21[91]="1930414595239816607215043645879664470532823506840451428303107914500024287764607636892111332297714564315408892104829,13979788737480761460414951797155492150401283835806624619409050591380653246551493236264292408360394853101854485167344";
    array_src22[91]="281823221869201196460215136206011471944135578943789022796296069604106999762939400457380844912940890798277998114989,12289213127403694403127942694671500460789024973819093478527029345718719313510678399835978166809411092728745309349163";
    array_src31[91]="34331157017451340483388979959300171244796879475035793840799423133786439023746644721286991848861236323901709140953113,5204204267076803643750235654653018146310593161316351264230765423754132933376939949546320424190045308529875010886573";
    array_src32[91]="25466640191220667060215462814656431180316521101881309026748312512399653079116999356059820424735880561318681821757426,18058993626197120479421744402828410736642315623791885162628370115894204009379016033088977635081803020636963519691603";
    array_src41[91]="12915196661574558441564211508981503423218729903817615724990467997337027965313657611555946965121119829965144431882138,27931106344411089026820721387953841994087877040189437857317182284513084515308884572424255450272402121016918691042974";
    array_src42[91]="32116796219711379574362228566804657049286028427675672016702165239261707368377370370800872779748606364810453570638559,23046334475639818098539659180816326814837617060360337181999614611127269402295784216382665237072998944946509683293649";
    array_src51[91]="5337240190245300081327159792838567226601158673319051473174927290729840326972740371789273421343435268002386050589714,27674494081831147975520681958710435312484350701679183814746278478069128787441071135834480460812139602754400248326758";
    array_src52[91]="3771735163900633235973342104391402288960057434893978415119649402591636176531795548607537020342672925732714443663218,6833080273116562056026352521815780517277875336248127601485040277040002587455921301171358123463204054200637626998186";
    array_src61[91]="38412799343876173364233855779201218193624510315831559325787449002882224225064754462828703044216828836916615757868944,3622544972129409918519754234049293802804491739500692997307353628543209004747132417348862605268776309828964939557216";
    array_src62[91]="39255238309176652480528622490181924466454809014025071574786882613898135195343505225350646914498708856117403889699035,27704624983180625583282448219291330947500695983462766982487880393213773599621385816033266072178541201817875423985479";
    array_n[91]="1996820136061299212011337040506724037496722748678448293503938028344641188372864781633833378330220789515772159931898";
    array_src11[92]="21878418098127778906633587510328355619298818812188278740026414497496499011310640707335401554176759888515425562115355,17484143695275980855677383243348374391413529774917738221805452940014143737368121081838085483984136649362796637857779";
    array_src12[92]="10163115663859905641314790835229613349226402066597747853094919651594093020812879946856784516502966422923859591062496,30638914028432067675225380210785106370306087154272431749890833301034691005700547972468809507319173726450295519721218";
    array_src21[92]="28118152339722389802365891571255518527434881687378292934138223607134800174428604885980393258113219220421372432147355,27654972123818489095654118880230865057843481236185664694149243132076935773489399899726529065784988686702903496625174";
    array_src22[92]="39145697609930393985167716395343250978287992776982210147320097425424546080837774317378248672514866752725030918794957,36048683288393138170746789180015126977841247228888305475571360365806598774145252107307717418166238655568429884204712";
    array_src31[92]="16937006652290848420390256941815527653696028055417564353546428505111263723239831521723204869941358874100369968729304,19849957434699120343519706526470238231763424041872435818413023083267609465573007897746702077196201832103779234551556";
    array_src32[92]="22726508858911915202811666448875043797629783415828012556063098904581757577881842454125392234076175281605356879171892,9587070821746913434124837958804622286140740045038184041584335563288108212219196566905928565402441575100014763427353";
    array_src41[92]="1154254388681088654557892798684182436296376414991226786746233206183055645788616762617429267433256283187167744707267,15309281271702347544180173013227576365994348044802699988712982615774253834451918965742060061744202910464115653174134";
    array_src42[92]="28869831357320750533460791761946320766448919480043571326123049345149711362338779134075381566850937367864361056333504,33491511445977890670037386253639545528952147746544966981659225322835512078207532765717032935048943813280824727101240";
    array_src51[92]="867530385658051151843556475258545575107416723369899658591520316965245725755636831928959916622611027166960857678183,26212427436829793268058558343224217209789983532277379587396006440584215111895477116506029019620142602924880315083022";
    array_src52[92]="24984338616483722853208486689220695246966017132528650687775471741590131689103984338164425158739503907914743579674376,2733701014123051859028140359679639602361191296872932309988089071285488954463966305218784303542411606800480478167686";
    array_src61[92]="33117256650255827008614085023297771791308249706622021713480933825521024028846037013784866087531086438812156227371674,31875835014472733076170587982631223267217865402829685820192816561347386201717940889665807398832329033055878001857065";
    array_src62[92]="10471858649620044289230358181195019340971117262977637186199766121335859127176393570029407545384074929447327649818046,8939022234604523260035406620794630752016153014503186175386017558792222393891702055060361879972724986869376954056403";
    array_n[92]="32954225507664226466932650394065209249165331740209216167426266536045082760783522520966476134641269523061789865783868";
    array_src11[93]="25060111065547575075434871867402886700850608697158350290668108987201957211740867802125572110879243445824127784905257,23643293090334592990702246097760738408032283335424933704640533805623445055789687791519183561087951343949059148954863";
    array_src12[93]="29621244167773185871305198713796363482531645252017756747757227361817480146640553562377476299842478649134275003735197,131369431568233922597771079593274760768420293766377401026511296559150281789054051227588481369625805406555355219880";
    array_src21[93]="16939529631468756188936428976716594135053370039457867782752602124817875151742234196641208237869734455313509120264248,26112296093504623748380307140106319930634376386367957173178365044951895723095367160102243418431558697837350142499704";
    array_src22[93]="14955928288748607920561899008956632237433152641199428262524259344650508325528712770311425855548108452181300145640794,6534433539306219840971154220227723906408780024466682695785720116282677179410249815269377350833375371924625971754590";
    array_src31[93]="37355547388698738324285538528876253705537858110521976650210506460229670727705741547861075211804440315144413739165406,27457350868239142710172706613874576670865757434357024241077776658402265244899215116582235314982271680194674322564915";
    array_src32[93]="1298868559136026058441819179963056143802473852466299470960618538972007747952792822752669318749320913431015513958549,2609926524556025118461805347400930551621920988139779570028490556724468502168304213643149109860812183630187592045067";
    array_src41[93]="8265580549252439113476893632343469075540075469959839671027667596919486167688926676774748946066254982718878718214100,6305976176592421844956225095458937010395788066328536658433215515855255335263203611836385155739177498553975426288791";
    array_src42[93]="7033149487591191056167479082686618692228740322087041340502584736954405187037762941970224622952721846683549218226499,37161910494031601010717662632929002939142499877866430743151423494496820203221054994881614270042200609418920414064848";
    array_src51[93]="17600838881384673469806286542387878497432386376768835365068684057439674095501080497226527119387623800401117747913909,22577586714120833498258915130343183447797253277293248984698158945078189583304311073513473305085951438195782684751481";
    array_src52[93]="12156539944975036899627856037189064818824355073900605597094273634605153488063303331225306308419011894788026785673290,23924978805968009166755544116944226841550470492897149666295072574591128324026687700436600909296637260500246661885140";
    array_src61[93]="19275478386336271674348416374355650521046846803510208648621236554039264035627405730180297195521888393200391774898935,3307674463807337060646179677047573597093013155538596456751324749593490592012706117756118992161566815634960334571685";
    array_src62[93]="12017058718964510435398312094408742168489413641569891613455659387970728621759034015479682441744508921049448065649105,32798253313272723507277086306823391277701325680816266196163223841007724415513319557049099666059793780512036653794552";
    array_n[93]="3341926293080548315972710007437026951501005067126322966922083869521286282743187895178971081385270789652089989896711";
    array_src11[94]="12524216972903168634017789189224391243000738332430709504776065921981707374690296361494454453789017070074271624576680,12307549183967502564631271338296929429183414888236047600159847826097768790520286433840666778419399999285120915492633";
    array_src12[94]="11424907457911778591680025296104130360717944482325839771235036647563220727914055210243008368147125571215824107825962,231318277229493565930815780304303552640882003027102424190516597650898991592358639104086940035678010703445356145062";
    array_src21[94]="10984163227459684899604861431011423189624453879616256289407399113741353072285232282082859845918769289360060688103994,11258453146959616206712866816965763407332461868196220837078084794967645897244994511911842981816504721678165522626414";
    array_src22[94]="9586770227160071365584195880936566532425172408291184167945206749852696404389507960496111486371760297515060450176361,448061391793929635901028654875447234070347452278077045725757041284325656926832993769857436894861379437317285351569";
    array_src31[94]="4714413402035326267613912497955586969358624808600853477682895851101281306976458475240835576053273948531606207965801,23297497584080833411724272390180674101124445708619795695977439191363466912978160215696614398047163072675779033980367";
    array_src32[94]="36568537748229293325658916830382142831457337736738381067970198152165366766909474641630319170696211735361356289131101,13970273938648065797515214686984454901395186245364683736249543858979484025481805044542653362477938097090274229187053";
    array_src41[94]="8990876138606317710797314475246945464396301399150417714829765192432400234002653729852314223383533362009162591635878,15576480340966845475928250370294951931933718442211419768582532122324196442411476249241225685205017928298050620841819";
    array_src42[94]="33965089795212619368010352366380712493899118119705990162429998456799017710007154608818463448731678222251128876715271,3511417454914279908113711895932511461968544651951615170152812381554564180691715274967564677773953749527789112285755";
    array_src51[94]="6597810336292907825560466092347093580074975473886425286551532854692830275726278738094361670781341013735474379394726,28255722801679998107934837413258162777749795619898748420139188491907641456321846433723132748486050168329606119704564";
    array_src52[94]="15683554427528199707443363606644084089830860918186545042608521124059290814946409838850394522545568519587087601384409,34537405374759727727726385039320537164521353646396238529746958960925564172849843045391915806494088085766661003909581";
    array_src61[94]="9807975813437692674261139124181938547172618805110926082938894886307219002772297848184928205509829829391274262170296,38911695808686037218298605949789307448403646852910814133085071556014426134037491519425008491336945604290607351997659";
    array_src62[94]="6436906728758867385810356600713591075397250420205554531497171870958239732441961024899375966951180169658196556193178,24795557314104759537443878460354835213004857186730138061802684401603984083966889625344340391445789271281805561514226";
    array_n[94]="6912229060935171002574627418966488043449998787824235871304922241900681381175867669682662844843759717172884006401088";
    array_src11[95]="4966018952667159161338392926787125534329351794788024659128986575164518618242476799589758196172772343141091269645395,32641252056747195120249675046339997526669123558794503891437172763098260537888798824229979328113987687681390343564050";
    array_src12[95]="23889825078317560476634590824022422669213898356601943412825345027353760316143190704713305512471509419701329626694558,22748905010224143748978210884674142816048047539030718221754164523074996558945741088926237139350141458977665096038623";
    array_src21[95]="12962316546304972820077174863472311447820205866602797431254532484840248523651860996406914434713408822905706951919501,7355720944341257887401092023211858050238221519666089268984160771361909383035296490920052927103645212951908142478336";
    array_src22[95]="8520354246328704728405195906569521310971841173127218646070300411666822027028626581555110554224149199228495133794265,26315404562074876626211188288166770260941814850602481988659425826845938681230776173495971260524700814622656252639468";
    array_src31[95]="4376852001281789873360167205089553164444294349300761716654822608231887254567002540631283389564335518331233785466761,18206169504218671509524608923304362526365703858122528017568541807985725332774189723765107814280280940654268172146904";
    array_src32[95]="32177464854132449159289385908016016412489337091446020988284300671353100433371626987506522883329597643853572334372597,29351313306851483002911061294889243892311741514916685767036992031196054456414408085444160602338891710853964514703386";
    array_src41[95]="26636549182763324816909140890620663884047470689429657725353869942179216994272911125770594187133761598883276350466361,35482524514765897757996720007446075892144496699446328407850759031523266785862520315716021523366515618855957698504670";
    array_src42[95]="37554099152490709088320269772765996816501256360814848865031760317884936818031154169386058570869686476978537470358556,29625986963502646304289146842877494613083719095216599260964304221588992964685975347817900793508967272650554748713764";
    array_src51[95]="18374650520093604890360955357042239994189457211743622539051700782056562985416369437296314923665875637998681322648600,5715176097735670919264427935082003740933026823997003144175493822713816902822965256591597950515046529732885027761967";
    array_src52[95]="24521746518771665304784582422289458791558915383300040145350466270136384711593536038536331757978878706364289186641573,7800021333458002900287109079022087307959784877884124201215162207454198214438848112481743679800303163637233532580642";
    array_src61[95]="28799124299290018035942629700631896584578864829991775955663865835335790830758526029085662670844162386101750031358123,5740861597880893035034489019100299148423343355371349670562569886666116729815255666360188670925685004183063524297817";
    array_src62[95]="19330862438078994996592810803132206788709550542763800874205315517740145118297957487258167884823539893942010926068142,28556791099329983336516508888252178644823427396826048883253873307336218862836552596841515250728876697284245887361777";
    array_n[95]="10940917197831986509523416569082088475368674504398666105073808843934466587021836294273643505311717686452995711138343";
    array_src11[96]="36676931042343189720025901542098914207580451292132232361575078098803268178903904844360629587678501160263170622141980,34665735666591776328921212892384462662220260910325223010216034389630894772989433998861239411789363511559494251748613";
    array_src12[96]="18728870528491083109425484782919118174452487784527667476377572711893209707575509734153110484203190919043711311079632,31655427881255663910047308677494834322516356026355246935620925411385029172124398776553990227871783434430274980642315";
    array_src21[96]="12139658288796486127322501973809584156811572007576973626981444594131264356987980460160950484314599846730244831874276,9952791168094679962662764839423120021972156979098574278769996632611872795974304286659495237556600775303994361079405";
    array_src22[96]="23797098194567996792722064909928577193784896562602657024787731639531852167731653954720833891977367838028418223990473,39398705764313379220567558384185430144409205590380680212742700555916273812588626265525708281410825289053687964613348";
    array_src31[96]="13288659760160214781471441836015858146265845472025942685244038405724045293942095370891070856467897535611239991159424,535802728684443701350041195220196781450641229561169603034814529473355242460312180447600603016151112991796790433338";
    array_src32[96]="5632450967681627815988886644746129025339849690991026861859754675554845177703045859918568757216145036127999127111993,15815816298764520809286433495324780125619613410505244484120237914386134123066503525436794740669940007140724760624952";
    array_src41[96]="32916384275115619640978823032408126895567973487398207628230128286369628247243841942496245451022392436480997847725080,33884224392094647124672426992748407116646072854068655214622399330289077761456747334551813718768325393516328768510337";
    array_src42[96]="26625698576744568927566030822737351883483863345900636268358222681427809871300459832577336055286814095209035022111863,6532701361525511894569263338156333907125885497828740675157456647926055033136219087211547595157854710213855720209381";
    array_src51[96]="16611152270009828041914081172838352968481033875488469983164612924123043024431231638209853888882494614706138621276632,32706659663609933302573431296147497413880309972831458726030155937696100202353208642677022162074397060937803809063830";
    array_src52[96]="23281292164553797758787063199662250305703706242760748045302606547841742988000308787135392159026776139985621083330141,35406234528607980067524737444776729979857490021089609827294031240040915333637776064215942063872891814284467310054999";
    array_src61[96]="35542494433569751851855093507260874298335979177825658771490225148000328277740301426020610269143513472788140357647552,24950938450530125532848004742688356011102583012215521224358197059784858973666365890181162328411946316413435011711174";
    array_src62[96]="2172167858876367803410853488158512750298205981818433900312060638187706853534033819537703309549161489145080022007217,33756580562317301047463816184626539434522860099472065579864073539852544883255563598919323594438704374783651916702278";
    array_n[96]="26184003692178851872410875065019895309198431859393646425612538010281729422718942002573697187379768301329351053095737";
    array_src11[97]="6013959732452396086815661331143336380536084512649827791195379981194739280882017174837015357917851782995998039228873,31486260003311755183661541618267731675829709681944029309629730259552262679733847930297083151172936633910016421208866";
    array_src12[97]="23066369519650277449586606328363220502029858427445437149087838286534182036166695929758708468168881333529957348941415,24226109966696492949285369929582257461166248585379390562487552060178581162489269114661529800965493177594830514647379";
    array_src21[97]="36571231896828000885453265300975283078814096666115699699859785442106135601803337862288009216066358431843369770908767,22204791331110259347504246550321386552587467263191685648395538396538661262095258726706781771997918195462956277038720";
    array_src22[97]="14273266855442207757075554608921957057861481456601546622637988515414070539951007342690204542097653430253402061919867,9597086747435810487553070299287915969090691633053791518462903269991667118521015704325439879730249503466611513295604";
    array_src31[97]="37212963921334192995092102076675478931957298365560674038914109161387058917388659520130251362217632236683677027550142,20412997731287032131877418159791654646744946395962986055328790437364246354047571124082811192725825649368563835355152";
    array_src32[97]="14927297114497594652330185286612648977343209764873075935677181208649804459364251085423241779478321177276497205551221,2101755102861072484239368456363944145566366004608525785684230924387068400938931170440570628964558541205316837846596";
    array_src41[97]="6721319566580146911798528679848832569540800900188543891312316979430464681071871801941426299398271178836615293709523,17559957200613243212114453419373580921625374230636495989033065286488376369025844631743003683035639706231170726521461";
    array_src42[97]="20912680155122249156014729192741632402537708376819753194879466305062675903952083708968302173814899252325892497759469,17142253327919111647535099714238334343642481941674576809656888416342565985936437845804326436602538781969777361461811";
    array_src51[97]="35787054411720959879538404742256188498370576493717770876446366987776356648766267455728331602849659606509197186923972,13532976604273466167967388795140985569330827130111959220361948075984743916679678703047227840416081583867768091009853";
    array_src52[97]="31794612920127228270092148370922326649650244625133722244929780426156205304893705693418986819080270853604302454119597,22121151395101824957370305731696034868731162181254761497203117398624544296865750384552368564823790653222074168588820";
    array_src61[97]="20339918305162629222446544813314110581505330396350998831625027873620124421354712972203735597985663113047979393966327,28659469938817817238965251379952498372019506098653830899799547197258235143768386409571757557193334584638669123209130";
    array_src62[97]="5726940782009185555083403183213971955778935185023759281191095614669328973285010750950952912105999210824731691519806,36352430045705717424510374828575834874128534144505486999426293662350038653855601490678436542911990416039252146593372";
    array_n[97]="9079343261755955922529723840239442959539741067875519518523024616021874895771493599936281479308751559963446908625683";
    array_src11[98]="6310002911602259712238506763546872996717881699513875075066839031163555924586170905749756081625011468840939835339234,37361059077831233688550694327981271654796928316911868769010749639167393858273269368715791704621616652564560164120201";
    array_src12[98]="20438350182505375514131980885003806826743499497917214691584448898749072928259598410189042162688555616099274766998145,21158258425349916227053753854692059910313471868453604915217899944010428982390878988299732707640803075892238325141594";
    array_src21[98]="8692582098984405839043575175895383328679778701216724673990733718153790062383242144529009781261221696278338285025360,26914326145091476672890359014687306607946831449279770444566139040670526129907150209229242423486545138302157887863280";
    array_src22[98]="11989673203001940612510933343973334245725700193595426036439539060134690258875633560583833939163696032591499000389150,29721809208466074601179528770084087561872685386671046032647704232056589262540905528342857278463214643205447265328992";
    array_src31[98]="36352844981125541653876598134201794033336563295977655734592599108594413407661087787637662511714919963979591107897216,17832950266516259050266285053432257338507058391727708036166660871539023210746854508024736812739469684636998395836116";
    array_src32[98]="19092754132043404069138542916413043304236757529851355101973419157904126043655539232915317980612563804446023819743456,34737767209211530205008548296129173776086664018130339130117273278806478931069425767725463985743926457816146676425585";
    array_src41[98]="36534845453549994387342761835748162366056288276445446626809900867447451353373687579132825855729926525894826174356453,6214572619330982251331926479774971805890435031341861892168292711929470986216132392243699986271738767371434670523106";
    array_src42[98]="23937266280619322419115280461111882127767452625730033747696729891423882954522640520691765023131210778583642934819038,16347616549533859956924549918724981593398795221078846432319621103194442470671471475180079801578692907237417159238734";
    array_src51[98]="15476419268508854829700462083591899094472009688062589237034573914602670593274916996452877068018715708371197378426225,5658656731073881568622608223605031240350177472281725158913249693765862043773851082480574605826675760978060983587661";
    array_src52[98]="274912774400250256686660965170427585874376119275986264779048793892793935533823747310940640747143809077639678107252,26401615042097681358226038733369513396958196704632697123047297772366592506938907021906236647610345575955152479559316";
    array_src61[98]="855208135253515047395744292293705480678601660755052712521858911492082706194033737352231622681513875203438983206796,8402021471810013340058738650104975429466284312098909019622967661963473666812135315954946862045280139411732364070517";
    array_src62[98]="2534355335660060159517095311281324582267377386752725294756115838751480629518536489744261164953143154000547169081306,32573773611575687594871836646801414947071727131845080325323475235432558506231069548960101537198579050922466902578872";
    array_n[98]="34389964155299774204459829293787582180128013764285399903684912832812627976060647981721777828641203362777306599251334";
    array_src11[99]="3782096894952528546553182899908869959170964859092431072417967726141783592547540146985302019307072097892137149367398,17214510128683489289068539834644405876562696661793837343587142996017023032016877569724161423647051315792780979175174";
    array_src12[99]="20595456551401274008244714329503481801516781238457403710180893508083663111071410506271916807855231449365784154669067,32233529865527313876956298396474784903916779850541650473198376691769466147998069211928654371311515224511427159751856";
    array_src21[99]="10288631476858492247436901344442761071195959112164609892435032286762542167822839942315251281311129054286925844539717,20427403601284380724882505003983448554250789900885332158248566191958321266245843865328920728522779638908107977231724";
    array_src22[99]="16415136753755282386638771881057565226471387037444935993523164046888698207355829220475738779459181310485771150548221,6972378564413870452249420301981139931638496540830823123968573220707855990719169689914901046654689234649045671947205";
    array_src31[99]="15382587722198529030417513162741455895267900824598273483284848809376209924098935755264240256419161764250168890523023,38619895556999333841987546317625803711455446081956516495056579931050838479000562189055032665601229125109538762765689";
    array_src32[99]="1021340762706642947551099056011097347226456803435262744080374617757281081740054889367220345083959198839816395785085,35566442419658407060474891647079823418108407081053745014766122458341451361924430880798463076161210062410779068516379";
    array_src41[99]="32252691847897157241018404232886712356863009607035379750073165909545954796009541595308649496790294084450019318991490,28669294512399627236482170108737520013649496889426577178021009489674990081232683393369016121502095060698741630042925";
    array_src42[99]="15076816339032627968488390619010753518653393713008965148567979224060740356679161206485162833843373306702082704504106,16532395756425409218323307129245461047988484627032890827514464227559684534153273310903506891170382781250251858346896";
    array_src51[99]="32092890103436333417499966209176133614673807516945045831561438045471243426661928441053017359643325368093420971208319,14830003263561255957435495955142335703366962058629267270083098119364352919288569106953659388915919688287760874336700";
    array_src52[99]="7070920184219100947203508353517103640321217179817953726202740368938495795147699068388986851658860516904308883077005,37341701805764800526163258102734621946443998538600312548032688058257155809379935778786816112824133544167837023100278";
    array_src61[99]="33221407091977394414094134874581349874872388772664236665780931949406528128660428564755208567144741843797599812968011,15853589104365235198318840676799756573434373744372297283873066583686598923615704610224046633020521893427823092900970";
    array_src62[99]="7723102201888906903666847773915568257177406674166855926559619386886004387232836520011027984907671206249993108112146,6742826285034258099660213418677069569567565985504864715395358109147963465538121678776046193971854248196358221831961";
    array_n[99]="30712117707147068847726055626976706253590645991561005900675604896968075032567855116925332066406621913466965271171938";

    
    for(i=0; i<nb_iteration; i++)
    {
        /* Set element from strings */
        ec_point_set_aff_str(P1, array_src11[i], array_src12[i], false, 10, WEIERSTRASS, k2, STACK_1);
        ec_point_set_aff_str(P2, array_src21[i], array_src22[i], false, 10, WEIERSTRASS, k2, STACK_1);
        ec_point_set_aff_str(P3, array_src31[i], array_src32[i], false, 10, WEIERSTRASS, k2, STACK_1);
        ec_point_set_aff_str(P4, array_src41[i], array_src42[i], false, 10, WEIERSTRASS, k2, STACK_1);
        ec_point_set_aff_str(P5, array_src51[i], array_src52[i], false, 10, WEIERSTRASS, k2, STACK_1);
        ec_point_set_aff_str(P6, array_src61[i], array_src62[i], false, 10, WEIERSTRASS, k2, STACK_1);
        number_set_str(n, array_n[i], 10);
        
        /* Add */
        
        ec_point_add(res, P1, P2, E, STACK_1);
        if(ec_point_are_equal(res, P3, E, STACK_1)==false)
        {
            test=false;
            printf("res:"); ec_point_print(res, 10, true, k, STACK_1); printf("\n"); 
            printf("P3:"); ec_point_print(P3, 10, true, k, STACK_1); printf("\n"); 
            return -1;
        }
        
        /* Dbl */
        
        ec_point_dbl(res, P1, E, STACK_1);
        if(ec_point_are_equal(res, P4, E, STACK_1)==false)
        {
            test=false;
            printf("res:"); ec_point_print(res, 10, true, k, STACK_1); printf("\n"); 
            printf("P4:"); ec_point_print(P4, 10, true, k, STACK_1); printf("\n"); 
            return -1;
        }
        
        /* Sub */
        
        ec_point_sub(res, P1, P2, E, STACK_1);
        if(ec_point_are_equal(res, P5, E, STACK_1)==false)
        {
            test=false;
            printf("res:"); ec_point_print(res, 10, true, k, STACK_1); printf("\n"); 
            printf("P5:"); ec_point_print(P5, 10, true, k, STACK_1); printf("\n"); 
            return -1;
        }
        
        /* Mul */
        
        ec_point_mul(res, n, P1, E, STACK_1);
        if(ec_point_are_equal(res, P6, E, STACK_1)==false)
        {
            test=false;
            printf("res:"); ec_point_print(res, 10, true, k, STACK_1); printf("\n"); 
            printf("P6:"); ec_point_print(P6, 10, true, k, STACK_1); printf("\n"); 
            return -1;
        }
    }
    
    
    /* Free allocated memory */
    
    ec_point_free(P1, k2);
    ec_point_free(P2, k2);
    ec_point_free(P3, k2);
    ec_point_free(P4, k2);
    ec_point_free(P5, k2);
    ec_point_free(P6, k2);
    ec_point_free(res, k2);
    
    number_free(&p);
    number_free(&n);
    field_elt_free(&non_residue, k);
    field_free(k);
    ec_free(E);
    field_free(k2);
    free_mphell();
    
    /* Result */
    
    if (test)
    {
        printf("SUCCESS \n");
        return 0;
    }
    else
    {
        printf("ERROR \n");
        return -1;
    }
}
